#
# for Windows + MinGW + MSYS
#
# export GUI_TYPE
# export BUILDDIR
# export LIBS
# export LDFLAGS
# export CDEFS
# export CFLAGS
# export INSTALLDIR

CC=gcc
CXX=g++
LD=g++

EXEFILE=bml3mk5

DATADIR=data

SRCDIR=src
SRCVM=$(SRCDIR)/vm
SRCFMGEN=$(SRCVM)/fmgen
SRCDEP=$(SRCVM)/bml3mk5
SRCVID=$(SRCDIR)/video
SRCVIDWAV=$(SRCVID)/wave
SRCVIDVFW=$(SRCVID)/vfw
SRCVIDFFM=$(SRCVID)/ffmpeg
SRCVIDWIN=$(SRCVID)/windows
SRCGUIAGAR=$(SRCDIR)/gui/agar
SRCGUIWIN=$(SRCDIR)/gui/windows
ifeq ($(GUI_TYPE),GUI_TYPE_AGAR)
	SRCGUI=$(SRCGUIAGAR)
endif
ifeq ($(GUI_TYPE),GUI_TYPE_WINDOWS)
	SRCGUI=$(SRCGUIWIN)
endif

RESDIR=res
SRCRES=$(SRCDIR)/$(RESDIR)/common

LOCALEDIR=locale
SRCLOCALE=$(LOCALEDIR)

EXE=$(BUILDDIR)/$(EXEFILE)

EMUOBJS=$(SRCDIR)/config.o \
	$(SRCDIR)/fifo.o \
	$(SRCDIR)/fileio.o \
	$(SRCDIR)/logging.o \
	$(SRCDIR)/emu.o \
	$(SRCDIR)/emumsg.o \
	$(SRCDIR)/sdl_input.o \
	$(SRCDIR)/sdl_input_keysym.o \
	$(SRCDIR)/sdl_screen.o \
	$(SRCDIR)/sdl_sound.o \
	$(SRCDIR)/sdl_timer.o \
	$(SRCDIR)/sdl_utils.o \
	$(SRCDIR)/sdl_debugger.o \
	$(SRCDIR)/msgboard.o \
	$(SRCDIR)/sdl_socket.o \
	$(SRCDIR)/sdl_main.o \
	$(SRCDIR)/depend.o \
	$(SRCDIR)/clocale.o \
	$(SRCDIR)/parseopt.o \
	$(SRCDIR)/cchar.o \
	$(SRCDIR)/cmutex.o \
	$(SRCDIR)/csurface.o \
	$(SRCDIR)/cbitmap.o \
	$(SRCDIR)/utility.o
#

VMOBJS= $(SRCVM)/device.o \
	$(SRCVM)/event.o \
	$(SRCVM)/hd46505.o \
	$(SRCVM)/ay38910.o \
	$(SRCVM)/disk.o \
	$(SRCVM)/mc6809.o \
	$(SRCVM)/debugger.o \
	$(SRCVM)/mc6809dasm.o

FMGENOBJS=$(SRCFMGEN)/fmgen.o \
	$(SRCFMGEN)/fmtimer.o \
	$(SRCFMGEN)/opm.o \
	$(SRCFMGEN)/opna.o \
	$(SRCFMGEN)/psg.o
#

DEPOBJS=$(SRCDEP)/acia.o \
	$(SRCDEP)/bml3mk5.o \
	$(SRCDEP)/cmt.o \
	$(SRCDEP)/comm.o \
	$(SRCDEP)/display.o \
	$(SRCDEP)/kanji.o \
	$(SRCDEP)/keyboard.o \
	$(SRCDEP)/memory.o \
	$(SRCDEP)/pia.o \
	$(SRCDEP)/printer.o \
	$(SRCDEP)/sound.o \
	$(SRCDEP)/parsewav.o \
	$(SRCDEP)/mb8866.o \
	$(SRCDEP)/mc6843.o \
	$(SRCDEP)/floppy.o \
	$(SRCDEP)/via.o \
	$(SRCDEP)/psgc.o \
	$(SRCDEP)/psg9c.o \
	$(SRCDEP)/board.o \
	$(SRCDEP)/keyrecord.o \
	$(SRCDEP)/timer.o 

GUIOBJSAGAR=$(SRCGUIAGAR)/ag_gui_base.o \
	$(SRCGUIAGAR)/ag_gui_config.o \
	$(SRCGUIAGAR)/ag_dlg.o \
	$(SRCGUIAGAR)/ag_file_dlg.o \
	$(SRCGUIAGAR)/ag_volume_dlg.o \
	$(SRCGUIAGAR)/ag_config_dlg.o \
	$(SRCGUIAGAR)/ag_keybind_dlg.o \
	$(SRCGUIAGAR)/ag_recaud_dlg.o \
	$(SRCGUIAGAR)/ag_recvid_dlg.o \
	$(SRCGUIAGAR)/ag_seldrv_dlg.o \
	$(SRCGUIAGAR)/ag_dir_dlg.o \
	$(SRCGUIAGAR)/ag_gui.o \
	$(SRCGUIWIN)/win_ledbox.o \
	$(SRCGUIWIN)/win_vkeyboard.o \
	$(SRCGUIWIN)/win_gui.o

GUIOBJSWIN=$(SRCGUIWIN)/win_dialogbox.o \
	$(SRCGUIWIN)/win_aboutbox.o \
	$(SRCGUIWIN)/win_filebox.o \
	$(SRCGUIWIN)/win_folderbox.o \
	$(SRCGUIWIN)/win_fontbox.o \
	$(SRCGUIWIN)/win_volumebox.o \
	$(SRCGUIWIN)/win_configbox.o \
	$(SRCGUIWIN)/win_keybindbox.o \
	$(SRCGUIWIN)/win_keybindctrl.o \
	$(SRCGUIWIN)/win_key_trans.o \
	$(SRCGUIWIN)/win_recaudbox.o \
	$(SRCGUIWIN)/win_recvidbox.o \
	$(SRCGUIWIN)/win_seldrvbox.o \
	$(SRCGUIWIN)/winfont.o \
	$(SRCGUIWIN)/win_ledbox.o \
	$(SRCGUIWIN)/win_vkeyboard.o \
	$(SRCGUIWIN)/win_gui.o

VIDOBJSWAV=$(SRCVIDWAV)/wav_rec_audio.o
VIDOBJSVFW=$(SRCVIDVFW)/vfw_rec_video.o
VIDOBJSFFM=$(SRCVIDFFM)/ffm_loadlib.o $(SRCVIDFFM)/ffm_rec_audio.o $(SRCVIDFFM)/ffm_rec_video.o
VIDOBJSWIN=$(SRCVIDWIN)/win_rec_video.o $(SRCVIDWIN)/win_bitmap.o

VIDOBJS=$(VIDOBJSWAV) $(VIDOBJSVFW) $(VIDOBJSFFM) $(VIDOBJSWIN) $(SRCVID)/rec_audio.o $(SRCVID)/rec_video.o

ifeq ($(GUI_TYPE),GUI_TYPE_AGAR)
	GUIOBJS=$(GUIOBJSAGAR)
endif
ifeq ($(GUI_TYPE),GUI_TYPE_WINDOWS)
	GUIOBJS=$(GUIOBJSWIN)
endif

GUIOBJS:=$(GUIOBJS) $(SRCDIR)/gui/gui_base.o \
	$(SRCDIR)/gui/ledbox.o \
	$(SRCDIR)/gui/sdl2_ledbox.o \
	$(SRCDIR)/gui/vkeyboard.o \
	$(SRCDIR)/gui/gui_keybinddata.o

SRCWINRES=$(SRCDIR)/$(RESDIR)/windows
WINRESOBJS=$(SRCWINRES)/bml3mk5.res
ifeq ($(GUI_TYPE),GUI_TYPE_WINDOWS)
	WINRESOBJS=$(SRCWINRES)/bml3mk5_gui.res
endif

OBJS_BASE=$(DEPOBJS) $(VMOBJS) $(FMGENOBJS) $(EMUOBJS) $(GUIOBJS) $(VIDOBJS) $(WINRESOBJS)
DEPS_BASE=$(DEPOBJS) $(VMOBJS) $(FMGENOBJS) $(EMUOBJS) $(GUIOBJS) $(VIDOBJS) 
OBJS=$(OBJS_BASE:%=$(BUILDDIR)/%)
DEPS=$(DEPS_BASE:%.o=$(BUILDDIR)/%.d)

WINDRES=windres.exe

MAKEFILEDEP=$(BUILDDIR)/Makefile.dep

#
#
#

all: exe

exe: $(BUILDDIR) $(EXE)

depend: $(MAKEFILEDEP)

$(MAKEFILEDEP): $(BUILDDIR) $(DEPS)
	cat $(DEPS) > $@

$(EXE): $(OBJS)
	$(LD) -o $@ $(OBJS) $(LIBS) $(LDFLAGS)

$(BUILDDIR)/%.o: %.cpp
	$(CXX) $(CDEFS) $(CFLAGS) -c $< -o $@

$(BUILDDIR)/%.d: %.cpp
	-$(CC) $(CDEFS) $(CFLAGS) -MM $< | sed 's#^.*\.o:#$@:#' | sed 's/\.d:/.o:/' > $@

$(BUILDDIR)/%.o: %.c
	$(CC) $(CDEFS) $(CFLAGS) -c $< -o $@

$(BUILDDIR)/%.d: %.c
	-$(CC) $(CDEFS) $(CFLAGS) -MM $< | sed 's#^.*\.o:#$@:#' | sed 's/\.d:/.o:/' > $@

$(BUILDDIR)/%.res: %.rc
	$(WINDRES) $< -O coff -o $@

install: exe
	mkdir -p $(INSTALLDIR)
	cp -p $(EXE) $(INSTALLDIR)
#	cp -p $(DATADIR)/?*.* $(INSTALLDIR)
	mkdir -p $(INSTALLDIR)/$(RESDIR)
	cp -p $(SRCRES)/*.* $(INSTALLDIR)/$(RESDIR)
	mkdir -p $(INSTALLDIR)/$(LOCALEDIR)
	for i in $(LOCALEDIR)/*/*; do if [ -d $$i ]; then \
		mkdir -p $(INSTALLDIR)/$$i; cp -p $$i/*.mo $(INSTALLDIR)/$$i; \
	fi; done

$(BUILDDIR):
	mkdir -p $(BUILDDIR)/$(SRCFMGEN)
	mkdir -p $(BUILDDIR)/$(SRCDEP)
	mkdir -p $(BUILDDIR)/$(SRCGUIAGAR)
	mkdir -p $(BUILDDIR)/$(SRCGUIWIN)
	mkdir -p $(BUILDDIR)/$(SRCVIDWAV)
	mkdir -p $(BUILDDIR)/$(SRCVIDVFW)
	mkdir -p $(BUILDDIR)/$(SRCVIDFFM)
	mkdir -p $(BUILDDIR)/$(SRCVIDWIN)
	mkdir -p $(BUILDDIR)/$(SRCWINRES)

clean:
	rm -rf $(BUILDDIR)

include $(MAKEFILEDEP)
