/** @file clocale.h

	Skelton for retropc emulator

	@author Sasaji
	@date   2014.12.01

	@brief [i18n with gettext]
	
	@note char code policy
		Win32: MBCS (maybe shift jis on Japanese Edition)
		Mac  : utf-8
		Linux: should be utf-8  
*/

#ifndef _CLOCALE_H_
#define _CLOCALE_H_

#include "depend.h"

#ifdef USE_GETTEXT

#if defined(_WIN32) && !defined(__MINGW32__)

#include "libintl.h"
#if defined(UNICODE) || defined(_UNICODE)
#ifdef _DEBUG
#pragma comment(lib, "libintlud.lib")
#else
#pragma comment(lib, "libintlu.lib")
#endif
#include "wlibintl.h"
#else
#ifdef _DEBUG
#pragma comment(lib, "libintld.lib")
#else
#pragma comment(lib, "libintl.lib")
#endif
#endif

#else

#include <locale.h>
#include <libintl.h>

#endif

#if defined(UNICODE) || defined(_UNICODE)
/* if variable string, use _tgettext instead of _(). */
#define _(text) wgettext(L##text)
#define _tgettext wgettext
#define _tdgettext dwgettext
#define _tdcgettext dcwgettext
#define _tngettext nwgettext
#define _tdngettext dnwgettext
#define _tdcngettext dcnwgettext

#define _tbindtextdomain wbindtextdomain
#define _tbind_textdomain_codeset wbind_textdomain_codeset

#define _ttextdomain wtextdomain

#define libintl_tfreeres libintl_wfreeres
#else /* MBCS */
#define _(text) gettext(text)
#define _tgettext gettext
#define _tdgettext dgettext
#define _tdcgettext dcgettext
#define _tngettext ngettext
#define _tdngettext dngettext
#define _tdcngettext dcngettext

#define _tbindtextdomain bindtextdomain
#define _tbind_textdomain_codeset wbind_textdomain_codeset

#define _ttextdomain textdomain

#define libintl_tfreeres libintl_freeres
#endif

#else	// ! USE_GETTEXT

#include <locale.h>

#if defined(UNICODE) || defined(_UNICODE)
#define _(text) (L##text)
#define _tgettext
#else
#define _(text) (text)
#define _tgettext
#endif

#endif	// USE_GETTEXT

/// Specify a text that xgettext should get and translate.
/// (When use xgettext, set --keyword=_TX.)
#define _TX _T

#ifdef _WIN32
#include <windows.h>
#endif

/// locale class with gnu gettext
class CLocale
{
private:
	_TCHAR *package_name;
	char *locale_name;
#ifdef _WIN32
	LCID  lcid;
#endif

	void Clear();

public:
	CLocale();
	CLocale(const _TCHAR *new_app_path, const _TCHAR *new_package_name, const char *new_locale);
//	CLocale(const wchar_t *new_package_name, const wchar_t *new_locale);
	~CLocale();
	bool SetLocale(const _TCHAR *new_app_path, const _TCHAR *new_package_name, const char *new_locale);
//	bool SetLocale(const wchar_t *new_package_name, const wchar_t *new_locale);
	bool UnsetLocale();
	bool IsOk() { return (package_name != NULL && locale_name != NULL); }
	const _TCHAR *GetText(const _TCHAR *text) { return (const _TCHAR *)_tgettext(text); }

#ifdef _WIN32
	LCID GetLcid() { return lcid; }
#endif
	bool SetThreadLocale();

private:
	/// Cannot copy
	CLocale(const CLocale &) {}
	CLocale &operator=(const CLocale &) { return *this; }

#ifdef _WIN32
	static BOOL CALLBACK enum_lcids_proc(LPSTR lcid_string);
	LCID get_lcid(const char *locale);
#endif
};

#endif /* _CLOCALE_H_ */
