/** @file common.h

	Skelton for retropc emulator

	@author Takeda.Toshiya
	@date   2006.08.18 -

	@brief [ common header ]
*/

#ifndef _COMMON_H_
#define _COMMON_H_

#if defined(_MSC_VER) && (_MSC_VER <= 1500)
#include <SDL.h>
#else
#include <stdint.h>
#endif

// variable scope of 'for' loop for microsoft visual c++ 6.0 and embedded visual c++ 4.0
//#if defined(_MSC_VER) && (_MSC_VER == 1200)
//#define for if(0);else for
//#endif

/* endian */
#if (defined(__BYTE_ORDER) && (__BYTE_ORDER == __BIG_ENDIAN)) \
 || (defined(BYTE_ORDER) && (BYTE_ORDER == BIG_ENDIAN))
	/* big endian */
#define USE_BIG_ENDIAN 1
#endif

typedef union {
#ifdef USE_BIG_ENDIAN
	struct {
		uint8_t h3, h2, h, l;
	} b;
	struct {
		int8_t h3, h2, h, l;
	} sb;
	struct {
		uint16_t h, l;
	} w;
	struct {
		int16_t h, l;
	} sw;
#else
	struct {
		uint8_t l, h, h2, h3;
	} b;
	struct {
		uint16_t l, h;
	} w;
	struct {
		int8_t l, h, h2, h3;
	} sb;
	struct {
		int16_t l, h;
	} sw;
#endif
	uint32_t d;
	int32_t sd;
} pair;

typedef union {
	struct {
#ifdef USE_BIG_ENDIAN
		uint8_t h, l;
#else
		uint8_t l, h;
#endif
	} b;
	uint16_t u16;
	int16_t s16;
} pair16;

typedef union {
	struct {
#ifdef USE_BIG_ENDIAN
		uint8_t h3, h2, h, l;
#else
		uint8_t l, h, h2, h3;
#endif
	} b;
	uint32_t u32;
	int32_t s32;
} pair32;

typedef union {
	struct {
#ifdef USE_BIG_ENDIAN
		uint8_t h7, h6, h5, h4, h3, h2, h, l;
#else
		uint8_t l, h, h2, h3, h4, h5, h6, h7;
#endif
	} b;
	uint64_t u64;
	int64_t s64;
} pair64;

// rgb color
#define _RGB888    32

#if defined(_RGB555)
//#define RGB_COLOR(r, g, b) ((uint16_t)(((uint16_t)(r) & 0xf8) << 7) | (uint16_t)(((uint16_t)(g) & 0xf8) << 2) | (uint16_t)(((uint16_t)(b) & 0xf8) >> 3))
typedef uint16_t scrntype;
#elif defined(_RGB565)
//#define RGB_COLOR(r, g, b) ((uint16_t)(((uint16_t)(r) & 0xf8) << 8) | (uint16_t)(((uint16_t)(g) & 0xfc) << 3) | (uint16_t)(((uint16_t)(b) & 0xf8) >> 3))
typedef uint16_t scrntype;
#elif defined(_RGB888)
//#define RGB_COLOR(r, g, b) (((uint32_t)(r) << 16) | ((uint32_t)(g) << 8) | ((uint32_t)(b) << 0))
typedef uint32_t scrntype;
#endif


typedef struct stVmRectWH {
	int x;
	int y;
	int w;
	int h;
} VmRectWH;

typedef struct stVmPoint {
	int x;
	int y;
} VmPoint;

typedef struct stVmSize {
	int cx;
	int cy;
} VmSize;

typedef struct stVmRect {
	int left;
	int top;
	int right;
	int bottom;
} VmRect;

#include "depend.h"

#endif
