/** @file depend.h

	Skelton for retropc emulator


	@author Sasaji
	@date   2012.03.01

	@brief [ dependency definition ]
*/

#ifndef _DEPEND_H_
#define _DEPEND_H_

// disable warnings C4189, C4995 and C4996 for microsoft visual c++ 2005
#if defined(_MSC_VER) && (_MSC_VER >= 1400)
#pragma warning( disable : 4819 )
#pragma warning( disable : 4995 )
#pragma warning( disable : 4996 )
#endif

// check memory leaks
#if defined(_DEBUG) && defined(_WIN32) && !defined(__MINGW32__)
//#define _CRTDBG_MAP_ALLOC
#undef _malloca
#include <stdlib.h>
#include <crtdbg.h>
#define malloc(s) _malloc_dbg(s, _NORMAL_BLOCK, __FILE__, __LINE__)
#if _MSC_VER > 1500
#define new new(_NORMAL_BLOCK, __FILE__, __LINE__)
#endif
#endif

#if defined(USE_SDL) || defined(USE_SDL2)
#include <SDL.h>
#if defined(USE_GTK)
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>
#if defined(USE_OPENGL)
#if !GTK_CHECK_VERSION(3,16,0)
#undef USE_OPENGL
#endif
#endif /* USE_OPENGL */
#endif /* USE_GTK */
#endif /* USE_SDL || USE_SDL2 */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "common.h"

#ifdef USE_BIG_ENDIAN
inline uint16_t swap16(uint16_t x);
inline uint32_t swap32(uint32_t x);
inline uint64_t swap64(uint64_t x);
#define Int16_LE(x) (int16_t)swap16((uint16_t)(x))
#define Int32_LE(x) (int32_t)swap32((uint32_t)(x))
#define Int64_LE(x) (int64_t)swap64((uint64_t)(x))
#define Uint16_LE(x) swap16(x)
#define Uint32_LE(x) swap32(x)
#define Uint64_LE(x) swap64(x)
#else
#define Int16_LE(x) (x)
#define Int32_LE(x) (x)
#define Int64_LE(x) (x)
#define Uint16_LE(x) (x)
#define Uint32_LE(x) (x)
#define Uint64_LE(x) (x)
#endif

uint16_t conv_to_uint16_le(uint8_t *);
uint32_t conv_to_uint32_le(uint8_t *);
void conv_from_uint16_le(uint8_t *, uint16_t);
void conv_from_uint32_le(uint8_t *, uint32_t);

#ifndef MAX
#define MAX(a,b) ((a) > (b) ? (a) : (b))
#endif
#ifndef MIN
#define MIN(a,b) ((a) < (b) ? (a) : (b))
#endif

#ifdef _WIN32
/* windows */
#include <windows.h>
#include <tchar.h>

#ifdef __MINGW32__
#include <getopt.h>
#endif

#define USE_REC_VIDEO
#define USE_REC_VIDEO_VFW
#define USE_REC_VIDEO_FFMPEG	1
#define USE_REC_VIDEO_WIN		1

#define USE_REC_AUDIO
#define USE_REC_AUDIO_WAVE
#define USE_REC_AUDIO_FFMPEG	2

#elif (defined(__APPLE__) && defined(__MACH__))
/* macosx */
#include <unistd.h>
#include "extra/tchar.h"
#define __stdcall

#define USE_REC_VIDEO
#define USE_REC_VIDEO_AVKIT		0x01
#define USE_REC_VIDEO_QTKIT		0x02
#define USE_REC_VIDEO_FFMPEG	1
#define USE_REC_VIDEO_COCOA		1

#define USE_REC_AUDIO
#define USE_REC_AUDIO_WAVE
#define USE_REC_AUDIO_AVKIT		0x10
#define USE_REC_AUDIO_FFMPEG	2

#else
/* linux (Unix) */
#include <unistd.h>
#include "extra/tchar.h"
#define __stdcall

#define USE_LIB_GTK
#ifdef GUI_TYPE_AGAR
#define USE_X11_LEDBOX
#define USE_X11_VKEYBOARD
#else
#define USE_GTK_LEDBOX
#define USE_GTK_VKEYBOARD
#endif

#define USE_REC_VIDEO
#define USE_REC_VIDEO_FFMPEG	1
#define USE_REC_VIDEO_LIBPNG	1

#define USE_REC_AUDIO
#define USE_REC_AUDIO_WAVE
#define USE_REC_AUDIO_FFMPEG	2

#endif

#ifndef _MAX_PATH
#define _MAX_PATH	(260)
#endif

#if defined(USE_WIN)
/* WIN API */

#define CDelay(ms) Sleep(ms)

#elif defined(USE_SDL) || defined(USE_WX)
/* SDL1 */

#define CDelay(ms) SDL_Delay(ms)

# define SDL_Keycode		SDLKey

# define SDL_WINDOW_FULLSCREEN	SDL_FULLSCREEN
# define SDL_WINDOW_OPENGL		SDL_OPENGL

#elif defined(USE_SDL2)
/* SDL2 */

#define CDelay(ms) SDL_Delay(ms)

# if defined(GUI_TYPE_GTK_X11)
//#  define GUI_USE_FOREIGN_WINDOW
# endif

#endif /* USE_SDL2 */

/* Support Outside Ledbox */
#if defined(_WIN32) \
 ||(defined(__APPLE__) && defined(__MACH__)) \
 || defined(USE_SDL2_LEDBOX) \
 || defined(USE_X11_LEDBOX) \
 || defined(USE_GTK_LEDBOX)
#define USE_OUTSIDE_LEDBOX 1
#endif


#ifdef _UNICODE
	/* _UNICODE */

#ifndef _WIN32
wchar_t *_wgetenv(const wchar_t *);
int _wsystem(const wchar_t *);
int swprintf(wchar_t *ws, const wchar_t *format, ...);
int vswprintf(wchar_t *ws, const wchar_t *format, va_list arg);

#  ifdef _tcsicmp
#  undef _tcsicmp
#  endif
#  define _tcsicmp wcscasecmp

# endif /* !_WIN32 */

#else	/* !_UNICODE */

# ifndef _WIN32

#  ifdef _tcsicmp
#  undef _tcsicmp
#  endif
#  define _tcsicmp strcasecmp

# endif /* !_WIN32 */

#endif	/* _UNICODE */

#endif	/* _DEPEND_H_ */
