/** @file gtk_input_keysym.cpp

	Skelton for retropc emulator
	GTK+ + SDL edition

	@author Sasaji
	@date   2017.01.27

	@brief [ gtk input keysym ]
*/

#include "emu.h"
#include "vm/vm.h"
#include "gui/gui.h"
#include "gtk_input.h"

/// notify accel key
int EMU::system_key_down(int code)
{
	if (vm) {
		code = translate_global_key(code);
		vm->system_key_down(code);
	}
	return code;
}
void EMU::system_key_up(int code)
{
	if (vm) {
		code = translate_global_key(code);
		vm->system_key_up(code);
	}
}

uint8_t EMU::translate_keysym(uint8_t type, int code, long status, int *new_code, bool *new_keep_frames)
{
	short scan_code = (short)((status & 0x1ff0000) >> 16);
	return translate_keysym(type, code, scan_code, new_code, new_keep_frames);
}

uint8_t EMU::translate_keysym(uint8_t type, int code, short scan_code, int *new_code, bool *new_keep_frames)
{
	int n_code = 0;
	bool n_keep_frames = false;
	uint8_t n_type = type;

#ifdef USE_GTK
	if (code < 128) {
		n_code = gtkcode2keycode1[code & 0x7f];
	}
    if ((code & 0xffff00) == 0xff00) {
		n_code = gtkcode2keycode2[code & 0xff];
	}
	if (n_code == 0) {
		for(const sc2kc_t *p = scancode2keycode; p->sc >= 0; p++) {
			if (p->sc == scan_code) {
				n_code = p->kc;
				break;
			}
		}
	}
#endif /* USE_GTK */

//	out_debugf(_T("trans_key: code:%02x -> %02x scan:%02x"),code,n_code,scan_code);

	if (new_code) *new_code = n_code;
	if (new_keep_frames) *new_keep_frames = n_keep_frames;
	
	return n_type;
}

/// execute accel key
bool EMU::execute_global_keys(int code, uint32_t status)
{
	if (gui) {
		code = translate_global_key(code);
		return gui->ExecuteGlobalKeys(code, status);
	}
	return false;
}

/// release accel key
bool EMU::release_global_keys(int code, uint32_t status)
{
	if (gui) {
		code = translate_global_key(code);
		return gui->ReleaseGlobalKeys(code, status);
	}
	return false;
}

//
int EMU::translate_global_key(int code)
{
	if (code == KEYCODE_RETURN) {
		code = GLOBALKEY_RETURN;
	} else if (code == KEYCODE_LCTRL || code == KEYCODE_RCTRL) {
			code = GLOBALKEY_CONTROL;
	} else if (code == KEYCODE_0) {
		code = GLOBALKEY_0;
	} else if (KEYCODE_1 <= code && code <= KEYCODE_9) {
		code = code + GLOBALKEY_1 - KEYCODE_1;
	} else if (KEYCODE_A <= code && code <= KEYCODE_Z) {
		code = code + GLOBALKEY_A - KEYCODE_A;
	} else if (KEYCODE_F1 <= code && code <= KEYCODE_F12) {
		code = code + GLOBALKEY_F1 - KEYCODE_F1;
	} else if (KEYCODE_F13 <= code && code <= KEYCODE_F15) {
		code = code + GLOBALKEY_F13 - KEYCODE_F13;
	}
	return code;
}
