/** @file ag_config_dlg.cpp

	Skelton for retropc emulator
	SDL + Ager edition

	@author Sasaji
	@date   2012.04.08

	@brief [ ag_config_dlg ]
*/

#include "ag_config_dlg.h"
#include <agar/core/types.h>
#include "../gui.h"
//#include "../../config.h"
#include "ag_gui_base.h"
//#include "../../clocale.h"
#include "../../msgboard.h"

namespace GUI_AGAR
{

#ifdef USE_IOPORT_FDD
#define IOPORT_STARTNUM 0
#else
#define IOPORT_STARTNUM 2
#endif

AG_CONFIG_DLG::AG_CONFIG_DLG(EMU *parent_emu, AG_GUI_BASE *parent_gui) : AG_DLG(parent_emu, parent_gui)
{
	memset(&param, 0, sizeof(param));
	param.fdd_type  = config.fdd_type;
	param.io_port = config.io_port;

	param.wav_reverse = 0;
	param.wav_half = 1;
	param.wav_correct = 0;
	param.wav_correct_type = 0;

	param.wav_sample_rate = 3;
	param.wav_sample_bits = 0;

	param.disptmg_skew = config.disptmg_skew;
	param.curdisp_skew = config.curdisp_skew;

	param.gl_filter_type = config.gl_filter_type;
	param.led_show = 0;
	param.led_pos = config.led_pos;
	param.capture_type = config.capture_type;

	paramtmp = NULL;

	dirdlg = new AG_DIR_DLG(parent_emu, parent_gui);
}

AG_CONFIG_DLG::~AG_CONFIG_DLG()
{
	delete dirdlg;
	delete paramtmp;
}

/**
 * create config dialog
 */
void AG_CONFIG_DLG::Create()
{
	param.disptmg_skew = config.disptmg_skew;
	param.curdisp_skew = config.curdisp_skew;
	param.use_opengl = config.use_opengl;
	param.gl_filter_type = config.gl_filter_type;
	param.led_show = gui->GetLedBoxPhase(-1);
	param.led_pos = config.led_pos;
	param.capture_type = config.capture_type;
	param.exram_size_num = config.exram_size_num;

	param.use_power_off = (config.use_power_off ? 1 : 0);
	param.dipswitch = config.dipswitch;
	param.fdd_type = emu->get_parami(VM::ParamFddType);
	param.mount_fdd = config.mount_fdd;
	param.delayfd1 = FLG_DELAY_FDSEARCH ? 1 : 0;
	param.delayfd2 = FLG_DELAY_FDSEEK ? 1 : 0;
	param.io_port = emu->get_parami(VM::ParamIOPort);
	param.wav_reverse = config.wav_reverse ? 1 : 0;
	param.wav_half = config.wav_half ? 1 : 0;
	param.wav_correct = config.wav_correct ? 1 : 0;
	param.wav_correct_type = config.wav_correct_type;
	param.wav_sample_rate = config.wav_sample_rate;
	param.wav_sample_bits = config.wav_sample_bits;

	memcpy(param.snap_path, config.snapshot_path.Get(), _MAX_PATH);
	memcpy(param.rom_path, config.rom_path.Get(), _MAX_PATH);
	memcpy(param.font_path, config.font_path.Get(), _MAX_PATH);
	memcpy(param.msg_font_name, config.msgboard_msg_fontname.Get(), _MAX_PATH);
	sprintf(param.msg_font_size, "%d", config.msgboard_msg_fontsize);
	memcpy(param.info_font_name, config.msgboard_info_fontname.Get(), _MAX_PATH);
	sprintf(param.info_font_size, "%d", config.msgboard_info_fontsize);
	memcpy(param.menu_font_name, config.menu_fontname.Get(), _MAX_PATH);
	sprintf(param.menu_font_size, "%d", config.menu_fontsize);

	for(int i=0; i<MAX_PRINTER; i++) {
		memcpy(param.prn_host[i], config.printer_server_host[i].Get(), _MAX_PATH);
		sprintf(param.prn_port[i], "%d", config.printer_server_port[i]);
	}
	for(int i=0; i<MAX_COMM; i++) {
		memcpy(param.com_host[i], config.comm_server_host[i].Get(), _MAX_PATH);
		sprintf(param.com_port[i], "%d", config.comm_server_port[i]);
		param.com_dipsw[i] = config.comm_dipswitch[i];
	}
#ifdef USE_DEBUGGER
	memcpy(param.dbgr_host, config.debugger_server_host.Get(), _MAX_PATH);
	sprintf(param.dbgr_port, "%d", config.debugger_server_port);
#endif

	param.showmsg_undefop = FLG_SHOWMSG_UNDEFOP ? 1 : 0;

	if (paramtmp == NULL) {
		paramtmp = new ConfigDlgParam;
	}
	*paramtmp = param;

	AG_Window *win = AG_WindowNew(AG_WINDOW_MODAL | AG_WINDOW_DIALOG | AG_WINDOW_NORESIZE);
	AG_WindowSetCaptionS(win, _("Configure"));

	AG_Box *vbox_base;
	AG_Box *hbox_base;
	AG_Box *vbox_left;
	AG_Box *vbox_right;
	AG_Box *vbox;
	AG_Box *hbox;
	AG_Radio *rad;
	AG_Checkbox *cbox;
	AG_Notebook *nb;
	AG_NotebookTab *tabs[5];

	int i;
	char name[128];

	nb = AG_NotebookNew(win, AG_NOTEBOOK_HFILL);

	//
	// 0 Mode
	tabs[0] = AG_NotebookAddTab(nb, _("Mode"), AG_BOX_HORIZ);

	vbox_base = AG_BoxNewVert(tabs[0], AG_BOX_HFILL | AG_BOX_VFILL);
	hbox_base = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL | AG_BOX_HOMOGENOUS);

	// left
	vbox_left = AG_BoxNewVert(hbox_base, AG_BOX_VFILL);

	// Mode switch
	vbox = AG_BoxNewVert(vbox_left, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, _("DIP Switch (*)"));

	set_label(chkDipSwitchBase[0],_("MODE Switch"));
	if (config.dipswitch & 0x04) {
		chkDipSwitchBase[0][0] = '>';
	}
	cbox = AG_CheckboxNewFlag(vbox, 0, chkDipSwitchBase[0], (Uint *)&paramtmp->dipswitch, 0x04);

	// FDD type
	vbox = AG_BoxNewVert(vbox_left, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, _("FDD Type (*)"));

	set_label(radFddTypeBase[0],_("No FDD"));
	set_label(radFddTypeBase[1],_("3inch compact FDD"));
	set_label(radFddTypeBase[2],_("5.25inch mini FDD"));
	set_label(radFddTypeBase[3],_("8inch standard FDD"));
	for(i=0; i<4; i++) {
		radFddType[i] = radFddTypeBase[i];
		if (i == config.fdd_type) {
			radFddTypeBase[i][0] = '>';
		}
	}
	radFddType[4]=NULL;

	rad = AG_RadioNewInt(vbox, AG_RADIO_EXPAND, (const char **)radFddType, &paramtmp->fdd_type);
	AG_AddEvent(rad, "radio-changed", OnChangeFddType, "%Cp", this);

	AG_CheckboxNewInt(vbox_left, 0, _("Enable the state of power off"), &paramtmp->use_power_off);

	// right
	vbox_right = AG_BoxNewVert(hbox_base, AG_BOX_VFILL);

	// I/O Port address
	vbox = AG_BoxNewVert(vbox_right, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, _("I/O Port Address (*)"));

	set_label(chkIOPortBase[0],_("5.25inch FDC  $FF00 - $FF04"));
	set_label(chkIOPortBase[1],_("3inch FDC  $FF18 - $FF20"));
	set_label(chkIOPortBase[2],_("6voice PSG  $FF30 - $FF33"));
	set_label(chkIOPortBase[3],_("Ex LPT Port  $FF3C - $FF3F"));
	set_label(chkIOPortBase[4],_("Ex COM Port  $FF40 - $FF41"));
	set_label(chkIOPortBase[5],_("9voice PSG  $FF70 - $FF7F"));
	set_label(chkIOPortBase[6],_("KANJI ROM  $FF75 - $FF76"));
	for(i=IOPORT_STARTNUM; i<IOPORT_NUMS; i++) {
		if (config.io_port & (1 << i)) {
			chkIOPortBase[i][0] = '>';
		}
		cbox = AG_CheckboxNewFlag(vbox, 0, chkIOPortBase[i], (Uint *)&paramtmp->io_port, (1 << i));
		AG_AddEvent(cbox, "checkbox-changed", OnChangeIOPort, "%Cp %i", this, i);
		chkIOPort[i] = cbox;
	}
	chkIOPort[7] = NULL;

	//
	AG_LabelNewS(vbox_base, AG_LABEL_HFILL, _("* Need restart program or \"PowerOn\"."));


	//
	// 1 screen
	tabs[1] = AG_NotebookAddTab(nb, _("Screen"), AG_BOX_HORIZ);

	vbox_base = AG_BoxNewVert(tabs[1], AG_BOX_EXPAND);
	hbox_base = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL | AG_BOX_HOMOGENOUS);

#ifdef USE_OPENGL
	// GL Filter Type
	vbox = AG_BoxNewVert(hbox_base, AG_BOX_FRAME);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, _("OpenGL"));

	const char *gl_use_list[] = {
		_("No"),
		_("Yes(sync)"),
		_("Yes(async)"),
		NULL
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, _("Use (*)"));
	UComboNew(hbox, gl_use_list, config.use_opengl, OnChangeUseOpenGL);

	const char *gl_filter_list[] = {
		_("Nearest Neighbour"),
		_("Linear"),
		NULL
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, _("Filter Type"));
	UComboNew(hbox, gl_filter_list, config.gl_filter_type, OnChangeGLFilterType);
#endif

	// CRTC
	vbox = AG_BoxNewVert(hbox_base, AG_BOX_FRAME);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, _("CRTC"));

	const char *skew_list[] = {
		"0",
		"1",
		"2",
		NULL
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, _("Disptmg Skew"));
	UComboNew(hbox, skew_list, config.disptmg_skew, OnChangeDisptmgSkew);

	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, _("Curdisp Skew"));
	UComboNew(hbox, skew_list, config.curdisp_skew, OnChangeCurdispSkew);

	// LED
	hbox_base = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL | AG_BOX_HOMOGENOUS);

	const char *led_show_list[] = {
		_("Hide"),
		_("Show Inside"),
		_("Show Outside"),
		NULL
	};
	vbox = AG_BoxNewVert(hbox_base, 0);
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, _("LED"));
	UComboNew(hbox, led_show_list, paramtmp->led_show, OnChangeLedShow);

	const char *led_pos_list[] = {
		_("LeftTop"),
		_("RightTop"),
		_("LeftBottom"),
		_("RightBottom"),
		NULL
	};
	vbox = AG_BoxNewVert(hbox_base, 0);
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, _("Position"));
	UComboNew(hbox, led_pos_list, config.led_pos, OnChangeLedPos);
	
	// capture type
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	const char *cap_type_list[] = {
		"BMP",
#if defined(USE_REC_VIDEO_WIN) || defined(USE_REC_VIDEO_COCOA) || defined(USE_REC_VIDEO_LIBPNG)
		"PNG",
#endif
		NULL
	};
	AG_LabelNewS(hbox, 0, _("Capture Type"));
	UComboNew(hbox, cap_type_list, config.capture_type, OnChangeCaptureType);

	// snapshot path
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	TextboxNew(hbox, _("Snapshot Path"), 40, paramtmp->snap_path, sizeof(paramtmp->snap_path)-1);
	AG_ButtonNewFn(hbox, 0, _("Folder..."), OnShowDirDlg, "%Cp %Cp %i", this, win, SnapShotPath);

	// font path
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	TextboxNew(hbox, _("Font Path"), 40, paramtmp->font_path, sizeof(paramtmp->font_path)-1);
	AG_ButtonNewFn(hbox, 0, _("Folder..."), OnShowDirDlg, "%Cp %Cp %i", this, win, FontPath);

	// msgboard font
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	TextboxNew(hbox, _("Message Font"), 16, paramtmp->msg_font_name, sizeof(paramtmp->msg_font_name)-1);
	TextboxNew(hbox, _(" Size"), 4, paramtmp->msg_font_size, sizeof(paramtmp->msg_font_size)-1);

	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	TextboxNew(hbox, _("Info Font"), 16, paramtmp->info_font_name, sizeof(paramtmp->info_font_name)-1);
	TextboxNew(hbox, _(" Size"), 4, paramtmp->info_font_size, sizeof(paramtmp->info_font_size)-1);

	// menu font
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	TextboxNew(hbox, _("Menu Font (*)"), 16, paramtmp->menu_font_name, sizeof(paramtmp->menu_font_name)-1);
	TextboxNew(hbox, _(" Size"), 4, paramtmp->menu_font_size, sizeof(paramtmp->menu_font_size)-1);

	//
	AG_LabelNewS(vbox_base, AG_LABEL_HFILL, _("* Need restart program."));

	//
	// 2 tape
	tabs[2] = AG_NotebookAddTab(nb, _("Tape, FDD"), AG_BOX_HORIZ);

	vbox_base = AG_BoxNewVert(tabs[2], AG_BOX_EXPAND);
	hbox_base = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL | AG_BOX_HOMOGENOUS);

	// Load wav file
	vbox = AG_BoxNewVert(hbox_base, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, _("Load Wav File from Tape"));

	AG_CheckboxNewInt(vbox, 0, _("Reverse Wave"), &paramtmp->wav_reverse);
	AG_CheckboxNewInt(vbox, 0, _("Half Wave"), &paramtmp->wav_half);
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	cbox = AG_CheckboxNewInt(hbox, 0, _("Correct"), &paramtmp->wav_correct);

	const char *radCorrect[] = {
		_("COS Wave"),
		_("SIN Wave"),
		NULL
	};
	rad = AG_RadioNewInt(hbox, AG_RADIO_EXPAND, radCorrect, &paramtmp->wav_correct_type);

	AG_AddEvent(cbox, "checkbox-changed", OnChangeCorrect, "%Cp %Cp", this, rad);
//	if (paramtmp->wav_correct == 0) {
//		AG_WidgetDisable(rad);
//	} else {
//		AG_WidgetEnable(rad);
//	}

	// Save wav file
	vbox = AG_BoxNewVert(hbox_base, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, _("Save Wav File to Tape"));

	const char *rate_list[] = {
		"11025",
		"22050",
		"44100",
		"48000",
		NULL
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, _("Sample Rate"));
	UComboNew(hbox, rate_list, paramtmp->wav_sample_rate, OnChangeSampleRate);

	const char *bits_list[] = {
		"8",
		"16",
		NULL
	};
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, _("Sample Bits"));
	UComboNew(hbox, bits_list, paramtmp->wav_sample_bits, OnChangeSampleBits);

	// FDD
	vbox = AG_BoxNewVert(vbox_base, AG_BOX_FRAME | AG_BOX_HFILL);
	AG_LabelNewS(vbox, AG_LABEL_HFILL, _("Floppy Disk Drive"));

	// fdd mount
	hbox = AG_BoxNewHoriz(vbox, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, _("When start up, mount disk at:"));
	for(i=0; i<MAX_DRIVE; i++) {
		sprintf(name, "%d  ", i);
		AG_CheckboxNewFlag(hbox, 0, name, (Uint *)&paramtmp->mount_fdd, (1 << i));
	}
	// delay
	AG_CheckboxNewInt(vbox, 0, _("Ignore delays to find sector."), &paramtmp->delayfd1);
	AG_CheckboxNewInt(vbox, 0, _("Ignore delays to seek track."), &paramtmp->delayfd2);

	//
	// 3 network
	tabs[3] = AG_NotebookAddTab(nb, _("Network"), AG_BOX_HORIZ);

	vbox_base = AG_BoxNewVert(tabs[3], AG_BOX_EXPAND);

	// LPT
	for(i=0; i<MAX_PRINTER; i++) {
		hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
		sprintf(name, _("LPT%d Hostname"), i);
		TextboxNew(hbox, name, 15, paramtmp->prn_host[i], sizeof(paramtmp->prn_host[i])-1);
		TextboxNew(hbox, _(" Port"), 6, paramtmp->prn_port[i], sizeof(paramtmp->prn_port[i])-1);
	}
	// COM
	const char *comm_baud_list[] = {
		_("S:300baud/F:1200baud"),
		_("S:600baud/F:2400baud"),
		_("S:1200baud/F:4800baud"),
		_("S:2400baud/F:9600baud"),
		NULL
	};
	for(i=0; i<MAX_COMM; i++) {
		hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
		sprintf(name, _("COM%d Hostname"), i);
		TextboxNew(hbox, name, 15, paramtmp->com_host[i], sizeof(paramtmp->com_host[i])-1);
		TextboxNew(hbox, _(" Port"), 6, paramtmp->com_port[i], sizeof(paramtmp->com_port[i])-1);

		UComboNew(hbox, comm_baud_list, paramtmp->com_dipsw[i]-1, OnChangeCommBaud, i);
	}
#ifdef USE_DEBUGGER
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	AG_LabelNewS(hbox, 0, _("Connectable host to Debugger"));
	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	TextboxNew(hbox, _T(" "), 15, paramtmp->dbgr_host, sizeof(paramtmp->dbgr_host)-1);
	TextboxNew(hbox, _(" Port"), 6, paramtmp->dbgr_port, sizeof(paramtmp->dbgr_port)-1);
#endif

	//
	// 4 CPU, Memory
	tabs[4] = AG_NotebookAddTab(nb, _("CPU, Memory"), AG_BOX_HORIZ);

	vbox_base = AG_BoxNewVert(tabs[4], AG_BOX_EXPAND);

	hbox = AG_BoxNewHoriz(vbox_base, AG_BOX_HFILL);
	TextboxNew(hbox, _("ROM Path (*)"), 40, paramtmp->rom_path, sizeof(paramtmp->rom_path)-1);
	AG_ButtonNewFn(hbox, 0, _("Folder..."), OnShowDirDlg, "%Cp %Cp %i", this, win, RomPath);
	//
	AG_CheckboxNewFlag(vbox_base, 0, _("Use Extended Memory (64KB)"), (Uint *)&paramtmp->exram_size_num, 1);
	//
	AG_CheckboxNewInt(vbox_base, 0, _("Show message when the CPU fetches undefined opcode."), &paramtmp->showmsg_undefop);

	//
	AG_LabelNewS(vbox_base, AG_LABEL_HFILL, _("* Need restart program or \"PowerOn\"."));

	//
	// button
	hbox_base = AG_BoxNewHoriz(win, AG_BOX_EXPAND);
	AG_ButtonNewFn(hbox_base, 0, _("OK"), OnOk, "%Cp %Cp", this, win);
	AG_ButtonNewFn(hbox_base, 0, _("Cancel"), OnClose, "%Cp %Cp", this, win);

	AG_SetEvent(win, "window-close", OnClose, "%Cp %Cp", this, win);

	gui->PostEtSystemPause(true);
	AG_WindowShow(win);

	//
	change_fdd_type(paramtmp->fdd_type);
}

void AG_CONFIG_DLG::change_fdd_type(int index)
{
	if (index == 0) {
		paramtmp->io_port = paramtmp->io_port & ~IOPORT_MSK_FDDALL;
#ifdef USE_IOPORT_FDD
		AG_WidgetEnable(chkIOPort[0]);
		AG_WidgetEnable(chkIOPort[1]);
#endif
	} else if (index == 1) {
		paramtmp->io_port = (paramtmp->io_port & ~IOPORT_MSK_FDDALL) | IOPORT_MSK_3FDD;
#ifdef USE_IOPORT_FDD
		AG_WidgetDisable(chkIOPort[0]);
		AG_WidgetDisable(chkIOPort[1]);
#endif
	} else if (index == 2 || index == 3) {
		paramtmp->io_port = (paramtmp->io_port & ~IOPORT_MSK_FDDALL) | IOPORT_MSK_5FDD;
#ifdef USE_IOPORT_FDD
		AG_WidgetDisable(chkIOPort[0]);
		AG_WidgetDisable(chkIOPort[1]);
#endif
	}
}

void AG_CONFIG_DLG::change_io_port(int index, int status)
{
#ifdef USE_IOPORT_FDD
	if (index == 0 && status & (1 << 0)) {
		paramtmp->io_port &= ~(1 << 1);
	} else if (index == 1 && status & (1 << 1)) {
		paramtmp->io_port &= ~(1 << 0);
	}
#endif
	if (index == 5 && status & (1 << 5)) {
		paramtmp->io_port &= ~(1 << 6);
	} else if (index == 6 && status & (1 << 6)) {
		paramtmp->io_port &= ~(1 << 5);
	}
}

void AG_CONFIG_DLG::set_label(char *label, const char *str)
{
	strcpy(label,"  ");
	strcat(label, str);
}

void AG_CONFIG_DLG::Close(AG_Window *win)
{
	AG_ObjectDetach(win);
	gui->PostEtSystemPause(false);
}

int AG_CONFIG_DLG::SetData(AG_Window *win)
{
	param = *paramtmp;

	config.disptmg_skew = param.disptmg_skew;
	config.curdisp_skew = param.curdisp_skew;
	config.use_opengl = (uint8_t)param.use_opengl;
	config.gl_filter_type = (uint8_t)param.gl_filter_type;
	config.led_pos = param.led_pos;
	config.capture_type = param.capture_type;
	config.exram_size_num = param.exram_size_num;

	config.use_power_off = (param.use_power_off != 0);
	config.dipswitch = (param.dipswitch & 0xff);

	emu->set_parami(VM::ParamFddType, param.fdd_type);
#ifndef USE_IOPORT_FDD
	param.io_port &= ~IOPORT_MSK_FDDALL;
	param.io_port |= (param.fdd_type == 1 ? IOPORT_MSK_3FDD : (param.fdd_type >= 2 ? IOPORT_MSK_5FDD : 0));
#endif
	config.mount_fdd = param.mount_fdd;
	config.delay_fdd = (param.delayfd1 ? MSK_DELAY_FDSEARCH : 0)
					| (param.delayfd2 ? MSK_DELAY_FDSEEK : 0);

	emu->set_parami(VM::ParamIOPort, param.io_port);

	config.wav_reverse = (param.wav_reverse == 1);
	config.wav_half = (param.wav_half == 1);
	config.wav_correct = ( param.wav_correct == 1);
	config.wav_correct_type = (uint8_t)param.wav_correct_type;
	config.wav_sample_rate = (uint8_t)param.wav_sample_rate;
	config.wav_sample_bits = (uint8_t)param.wav_sample_bits;

	int val = 0;
	for(int i=0; i<MAX_PRINTER; i++) {
		config.printer_server_host[i].Set(param.prn_host[i]);
		val = 0;
		if (sscanf(param.prn_port[i], "%d", &val) == 1 && val > 0 && val < 65535) {
			config.printer_server_port[i] = val;
		}
	}
	for(int i=0; i<MAX_COMM; i++) {
		config.comm_server_host[i].Set(param.com_host[i]);
		val = 0;
		if (sscanf(param.com_port[i], "%d", &val) == 1 && val > 0 && val < 65535) {
			config.comm_server_port[i] = val;
		}
		config.comm_dipswitch[i] = param.com_dipsw[i];
	}
#ifdef USE_DEBUGGER
	config.debugger_server_host.Set(param.dbgr_host);
	val = 0;
	if (sscanf(param.dbgr_port, "%d", &val) == 1 && val > 0 && val < 65535) {
		config.debugger_server_port = val;
	}
#endif

	config.snapshot_path.Set(param.snap_path);
	config.font_path.Set(param.font_path);
	config.msgboard_msg_fontname.Set(param.msg_font_name);
	config.msgboard_info_fontname.Set(param.info_font_name);
	val = 0;
	if (sscanf(param.msg_font_size, "%d", &val) == 1 && val >= 6 && val <= 60) {
		config.msgboard_msg_fontsize = val;
	}
	val = 0;
	if (sscanf(param.info_font_size, "%d", &val) == 1 && val >= 6 && val <= 60) {
		config.msgboard_info_fontsize = val;
	}
	config.menu_fontname.Set(param.menu_font_name);
	val = 0;
	if (sscanf(param.menu_font_size, "%d", &val) == 1 && val >= 6 && val <= 60) {
		config.menu_fontsize = val;
	}

	config.rom_path.Set(param.rom_path);

	config.show_dlg = (param.showmsg_undefop ? config.show_dlg | MSK_SHOWMSG_UNDEFOP : config.show_dlg & ~MSK_SHOWMSG_UNDEFOP);

	// set message font
#ifdef USE_MESSAGE_BOARD
	MsgBoard *msgboard = emu->get_msgboard();
	if (msgboard) {
		msgboard->SetFont();
	}
#endif
	// set menu font
//	gui->ConfigLoad();
	gui->ChangeLedBox(param.led_show);
	gui->ChangeLedBoxPosition(config.led_pos);
	config.save();
#ifdef USE_OPENGL
	emu->change_opengl_attr();
#endif

	delete paramtmp;
	paramtmp = NULL;

	return 1;
}

void AG_CONFIG_DLG::show_dir_dlg(AG_Window *win, DirDlgType type)
{
	switch(type) {
	case RomPath:
		dirdlg->CreateLoad(_("Rom Path"), paramtmp->rom_path, sizeof(paramtmp->rom_path), win);
		break;
	case SnapShotPath:
		dirdlg->CreateLoad(_("Snapshot Path"), paramtmp->snap_path, sizeof(paramtmp->snap_path), win);
		break;
	case FontPath:
		dirdlg->CreateLoad(_("Font Path"), paramtmp->font_path, sizeof(paramtmp->font_path), win);
		break;
	}
}

/*
 * Event Handler (static)
 */
void AG_CONFIG_DLG::OnOk(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_Window *win = (AG_Window *)AG_PTR(2);

	if (dlg->SetData(win)) {
		dlg->Close(win);
	}
}

void AG_CONFIG_DLG::OnClose(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_Window *win = (AG_Window *)AG_PTR(2);

	dlg->Close(win);
}

void AG_CONFIG_DLG::OnChangeCorrect(AG_Event *event)
{
//	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
//	AG_Radio *rad = (AG_Radio *)AG_PTR(2);
//
//	if (dlg->paramtmp->wav_correct == 0) {
//		AG_WidgetDisable(rad);
//	} else {
//		AG_WidgetEnable(rad);
//	}
}

void AG_CONFIG_DLG::OnChangeSampleRate(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->wav_sample_rate = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeSampleBits(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->wav_sample_bits = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeFddType(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	int index = (int)AG_INT(2);

	dlg->change_fdd_type(index);
}

void AG_CONFIG_DLG::OnChangeIOPort(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	int index = AG_INT(2);
	int status = AG_INT(3);

	dlg->change_io_port(index, status);
}

void AG_CONFIG_DLG::OnChangeDisptmgSkew(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->disptmg_skew = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeCurdispSkew(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->curdisp_skew = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeUseOpenGL(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->use_opengl = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeGLFilterType(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->gl_filter_type = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeLedShow(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->led_show = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeLedPos(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);

	if (state) {
		dlg->paramtmp->led_pos = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeCaptureType(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(2);
	int state = AG_INT(3);
	
	if (state) {
		dlg->paramtmp->capture_type = item->label;
	}
}

void AG_CONFIG_DLG::OnChangeCommBaud(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	int num   = AG_INT(2);
	AG_TlistItem *item = (AG_TlistItem *)AG_PTR(3);
	int state = AG_INT(4);

	if (state) {
		dlg->paramtmp->com_dipsw[num] = item->label + 1;
	}
}

void AG_CONFIG_DLG::OnShowDirDlg(AG_Event *event)
{
	AG_CONFIG_DLG *dlg = (AG_CONFIG_DLG *)AG_PTR(1);
	AG_Window *win = (AG_Window *)AG_PTR(2);
	DirDlgType type = (DirDlgType)AG_INT(3);

	dlg->show_dir_dlg(win, type);
}

}; /* namespace GUI_AGAR */
