/** @file ag_volume_dlg.cpp

	Skelton for retropc emulator
	SDL + Ager edition

	@author Sasaji
	@date   2012.4.5

	@brief [ ag_volume_dlg ]
*/

#include "ag_volume_dlg.h"
//#include "../../config.h"
#include "../gui.h"

namespace GUI_AGAR
{

AG_VOLUME_DLG::AG_VOLUME_DLG(EMU *parent_emu, AG_GUI_BASE *parent_gui) : AG_DLG(parent_emu, parent_gui)
{
	// volume dialog
	volmax = 100;
	volmin = 0;
}

AG_VOLUME_DLG::~AG_VOLUME_DLG()
{
}
/*
 * create volume dialog
 */
void AG_VOLUME_DLG::Create()
{
	AG_Window *win = AG_WindowNew(AG_WINDOW_MODAL | AG_WINDOW_DIALOG | AG_WINDOW_NORESIZE);
	AG_WindowSetCaptionS(win, _("Volume"));

	AG_Box *hbox_base;
	AG_Fixed *fix;
	AG_Fixed *fix2;
	AG_Label *lbl;
	AG_Slider *sli;
	AG_Checkbox *cbox;
	AG_Separator *sep;

	int w = 65;
	int h[3] = {32,100,20};
	int padd = 5;
	const char *t[8] = {_("Master"),_("Beep"),_("6voice\nPSG"),_("9voice\nPSG"),_("Relay"),_("CMT"),_("FDD"),NULL};

	get_volume();


	hbox_base = AG_BoxNewHoriz(win, AG_BOX_EXPAND);

	fix = AG_FixedNew(hbox_base, AG_FIXED_EXPAND);

	int x = 0;
	int y = 0;
	for(int i=0; i<7; i++) {
		y = padd;
		lbl = AG_LabelNewS(fix, AG_LABEL_HFILL, t[i]);
		AG_LabelJustify(lbl, AG_TEXT_CENTER);
		AG_FixedMove(fix, lbl, x, y);
		AG_FixedSize(fix, lbl, w, h[0]);
		y += padd + h[0];
		fix2 = AG_FixedNew(fix, AG_FIXED_HFILL | AG_FIXED_FRAME);
		AG_FixedMove(fix, fix2, x, y+h[1]/2);
		AG_FixedSize(fix, fix2, w, 1);
		sli = AG_SliderNewInt(fix, AG_SLIDER_VERT, AG_SLIDER_EXPAND, &volume[i], &volmin, &volmax);
		AG_SetEvent(sli, "slider-changed", OnChange, "%Cp %i", this, i);
		AG_FixedMove(fix, sli, x+4, y);
		AG_FixedSize(fix, sli, w-8, h[1]);
		y += padd + h[1];
		cbox = AG_CheckboxNewInt(fix, 0, _("Mute"), &mute[i]);
		AG_SetEvent(cbox, "checkbox-changed", OnChange, "%Cp %i", this, i);
		AG_FixedMove(fix, cbox, x, y);
		AG_FixedSize(fix, cbox, w, h[2]);
		y += padd + h[2];

		x += (padd + w);

		if (i==0) {
			// sep
			sep = AG_SeparatorNew(fix, AG_SEPARATOR_VERT);
			AG_FixedMove(fix, sep, x, padd);
			AG_FixedSize(fix, sep, 12, y - padd);
			x += padd + 12;
		}
	}
	x += padd;
	y += padd;

	AG_WindowSetGeometryAligned(win, AG_WINDOW_MC, x, y + 60);

	AG_ButtonNewFn(win, AG_BUTTON_HFILL, _("Close"), OnClose, "%Cp %Cp", this, win);

	AG_SetEvent(win, "window-close", OnClose, "%Cp %Cp", this, win);
	AG_AddEvent(win, "key-up", OnCancel, "%Cp", win);

	AG_WindowShow(win);
}

void AG_VOLUME_DLG::Change()
{
	set_volume();
	emu->set_volume(0);
}

void AG_VOLUME_DLG::Close(AG_Window *win)
{
	Change();
	AG_ObjectDetach(win);
}

void AG_VOLUME_DLG::get_volume()
{
	volume[0] = volmax - config.volume;
	volume[1] = volmax - config.beep_volume;
	volume[2] = volmax - config.psg6_volume;
	volume[3] = volmax - config.psg9_volume;
	volume[4] = volmax - config.relay_volume;
	volume[5] = volmax - config.cmt_volume;
	volume[6] = volmax - config.fdd_volume;

	mute[0] = config.mute ? 1 : 0;
	mute[1] = config.beep_mute ? 1 : 0;
	mute[2] = config.psg6_mute ? 1 : 0;
	mute[3] = config.psg9_mute ? 1 : 0;
	mute[4] = config.relay_mute ? 1 : 0;
	mute[5] = config.cmt_mute ? 1 : 0;
	mute[6] = config.fdd_mute ? 1 : 0;
}

void AG_VOLUME_DLG::set_volume()
{
	config.volume = volmax - volume[0];
	config.beep_volume = volmax - volume[1];
	config.psg6_volume = volmax - volume[2];
	config.psg9_volume = volmax - volume[3];
	config.relay_volume = volmax - volume[4];
	config.cmt_volume = volmax - volume[5];
	config.fdd_volume = volmax - volume[6];
	config.mute = (mute[0] != 0);
	config.beep_mute = (mute[1] != 0);
	config.psg6_mute = (mute[2] != 0);
	config.psg9_mute = (mute[3] != 0);
	config.relay_mute = (mute[4] != 0);
	config.cmt_mute = (mute[5] != 0);
	config.fdd_mute = (mute[6] != 0);
}

/*
 * Event Handler (static)
 */
void AG_VOLUME_DLG::OnClose(AG_Event *event)
{
	AG_VOLUME_DLG *dlg = (AG_VOLUME_DLG *)AG_PTR(1);
	AG_Window *win = (AG_Window *)AG_PTR(2);

	dlg->Close(win);
}

// close dialog
void AG_VOLUME_DLG::OnCancel(AG_Event *event)
{
	AG_Window *win = (AG_Window *)AG_PTR(1);

	if (strcmp(event->name, "key-up") == 0) {
		int key = AG_INT(2);
		if (key != AG_KEY_ESCAPE) {
			return;
		}
	}

	AG_ObjectDetach(win);
}

// slider or checkbox changed
void AG_VOLUME_DLG::OnChange(AG_Event *event)
{
	AG_VOLUME_DLG *dlg = (AG_VOLUME_DLG *)AG_PTR(1);
//	int num = AG_INT(2);

	dlg->Change();
}

}; /* namespace GUI_AGAR */
