/** @file cocoa_basepanel.mm
 
 HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
 Skelton for retropc emulator
 SDL edition + Cocoa GUI
 
 @author Sasaji
 @date   2015.04.30 -
 
 @brief [ panel ]
 */

#import "cocoa_basepanel.h"

@implementation CocoaTextSize
+ (NSSize)size:(NSControl *)ctrl constTo:(NSSize)constraint
{
	NSFont *font = [ctrl font];
	
	NSTextStorage *textStorage = [[NSTextStorage alloc] initWithString:self];
	NSTextContainer *textContainer = [[NSTextContainer alloc] initWithContainerSize:constraint];
	
	NSLayoutManager *layoutManager = [[NSLayoutManager alloc] init];
	[layoutManager addTextContainer:textContainer];
	[textStorage addLayoutManager:layoutManager];
	[textStorage addAttribute:NSFontAttributeName value:font
						range:NSMakeRange(0, [textStorage length])];
	[textContainer setLineFragmentPadding:0.0];
	
	(void) [layoutManager glyphRangeForTextContainer:textContainer];
	return [layoutManager
			usedRectForTextContainer:textContainer].size;
}
@end

@implementation CocoaLabel
+ (CocoaLabel *)create:(NSRect)re title:(const char *)title
{
	return [CocoaLabel createWithFit:&re title:title align:NSLeftTextAlignment];
}
+ (CocoaLabel *)create:(NSRect)re title:(const char *)title align:(NSTextAlignment)align
{
	return [CocoaLabel createWithFit:&re title:title align:align];
}
+ (CocoaLabel *)createWithFitW:(NSRect *)re title:(const char *)title
{
	return [CocoaLabel createWithFitW:re title:title align:NSLeftTextAlignment];
}
+ (CocoaLabel *)createWithFitW:(NSRect *)re title:(const char *)title align:(NSTextAlignment)align
{
	NSRect rew = *re;
	CocoaLabel *obj = [CocoaLabel createWithFit:&rew title:title align:align];
	re->size.width = rew.size.width;
	[obj setFrame:*re];
	return obj;
}
+ (CocoaLabel *)createWithFitH:(NSRect *)re title:(const char *)title
{
	return [CocoaLabel createWithFitH:re title:title align:NSLeftTextAlignment];
}
+ (CocoaLabel *)createWithFitH:(NSRect *)re title:(const char *)title align:(NSTextAlignment)align
{
	NSRect reh = *re;
	CocoaLabel *obj = [CocoaLabel createWithFit:&reh title:title align:align];
	re->size.height = reh.size.height;
	[obj setFrame:*re];
	return obj;
}
+ (CocoaLabel *)createWithFit:(NSRect *)re title:(const char *)title
{
	return [CocoaLabel createWithFit:re title:title align:NSLeftTextAlignment];
}
+ (CocoaLabel *)createWithFit:(NSRect *)re title:(const char *)title align:(NSTextAlignment)align
{
	CocoaLabel *me = [CocoaLabel alloc];
	
	[me initWithFrame:*re];
	[me setStringValue:[NSString stringWithUTF8String:title]];
	[me setEditable:FALSE];
	[me setBezeled:FALSE];
	[me setDrawsBackground:FALSE];
	[me setSelectable:FALSE];
	[me setAlignment:align];
	[me sizeToFit];
	*re = [me frame];

	return me;
}
@end

@implementation CocoaTextField
+ (CocoaTextField *)create:(NSRect)re text:(const char *)text action:(SEL)action
{
	return [CocoaTextField create:re text:text action:action align:NSLeftTextAlignment];
}
+ (CocoaTextField *)create:(NSRect)re text:(const char *)text action:(SEL)action align:(NSTextAlignment)align
{
	CocoaTextField *me = [CocoaTextField alloc];
	
	[me initWithFrame:re];
	[me setStringValue:[NSString stringWithUTF8String:text]];
	[me setEditable:YES];
	[me setBezeled:YES];
	[me setDrawsBackground:YES];
	[me setSelectable:YES];
	[me setAction:action];
	[me setAlignment:align];
/*	[[me cell] setUsesSingleLineMode:YES]; */
	
	return me;
}
+ (CocoaTextField *)create:(NSRect)re num:(int)num action:(SEL)action
{
	return [CocoaTextField create:re num:num action:action align:NSLeftTextAlignment];
}
+ (CocoaTextField *)create:(NSRect)re num:(int)num action:(SEL)action align:(NSTextAlignment)align
{
	CocoaTextField *me = [CocoaTextField alloc];
	
	[me initWithFrame:re];
	[me setStringValue:[[NSNumber numberWithInt:num] stringValue]];
	[me setEditable:YES];
	[me setBezeled:YES];
	[me setDrawsBackground:YES];
	[me setSelectable:YES];
	[me setAction:action];
	[me setAlignment:align];
/*	[[me cell] setUsesSingleLineMode:YES]; */
	
	return me;
}
@end

@implementation CocoaObjectStructure
@synthesize obj1;
@synthesize obj2;
+ (CocoaObjectStructure *)create:(id)newobj1 :(id)newobj2
{
	CocoaObjectStructure *me = [CocoaObjectStructure alloc];
	[me init];
	me.obj1=newobj1;
	me.obj2=newobj2;
	
	return me;
}
@end

@implementation CocoaButton
@synthesize relatedObject;
+ (CocoaButton *)create:(NSRect)re title:(const char *)title action:(SEL)action
{
	CocoaButton *me = [CocoaButton alloc];

	[me initWithFrame:re];
	[me setTitle:[NSString stringWithUTF8String:title]];
	[me setBezelStyle:NSRoundedBezelStyle];
	[me setAction:action];
	[me setRelatedObject:nil];

	return me;
}
@end

@implementation CocoaPopUpButton
+ (CocoaPopUpButton *)create:(NSRect)re items:(const char **)items action:(SEL)action selidx:(int)selidx
{
	CocoaPopUpButton *me = [CocoaPopUpButton alloc];
	int i;

	[me initWithFrame:re pullsDown:NO];
	for(i=0; items[i] != NULL; i++) {
		[me addItemWithTitle:[NSString stringWithUTF8String:items[i]]];
	}
	[me setAction:action];
	if (selidx < i) {
		[me selectItemAtIndex:selidx];
	}
	return me;
}
@end

@implementation CocoaCheckBox
@synthesize index;
+ (CocoaCheckBox *)create:(NSRect)re title:(const char *)title action:(SEL)action value:(bool)value
{
	return [CocoaCheckBox create:re title:title index:0 action:action value:value];
	
}
+ (CocoaCheckBox *)create:(NSRect)re title:(const char *)title index:(int)index action:(SEL)action value:(bool)value
{
	CocoaCheckBox *me = [CocoaCheckBox alloc];
	[me initWithFrame:re];
	[me setTitle:[NSString stringWithUTF8String:title]];
	[me setButtonType:NSSwitchButton];
	[me setAction:action];
	[me setState:value ? NSOnState : NSOffState];
	me.index = index;

	return me;
}
@end

@implementation CocoaRadioGroup
+ (CocoaRadioGroup *)create:(NSRect)re rows:(int)rows cols:(int)cols titles:(const char **)titles action:(SEL)action selidx:(int)selidx
{
	CocoaRadioGroup *me = [CocoaRadioGroup alloc];
	[me initWithFrame:re mode:NSRadioModeMatrix cellClass:NSButtonCell.class numberOfRows:rows numberOfColumns:cols];
	[me setCellSize:NSMakeSize(re.size.width / cols,re.size.height / rows)];
	
	int i = 0;
	int row;
	int col;
	for(row=0; row<rows && titles[i] != NULL; row++) {
		for(col=0; col<cols && titles[i] != NULL; col++) {
			NSButtonCell *cell=[me cellAtRow:row column:col];
			[cell setButtonType:NSRadioButton];
			[cell setTitle:[NSString stringWithUTF8String:titles[i]]];
			if (i == selidx) {
				[me selectCell:cell];
			}
			i++;
		}
	}
	
	[me setAction:action];

	return me;
}
@end

@implementation CocoaSlider
@synthesize index;
+ (CocoaSlider *)create:(NSRect)re action:(SEL)action value:(int)value
{
	return [CocoaSlider create:re index:0 action:action value:value];
}
+ (CocoaSlider *)create:(NSRect)re index:(int)index action:(SEL)action value:(int)value
{
	CocoaSlider *me = [CocoaSlider alloc];
	
	[me initWithFrame:re];
	[me setAction:action];
	[me setMaxValue:100.0];
	[me setMinValue:0.0];
	[me setIntValue:value];
	me.index = index;

	return me;
}
@end

@implementation CocoaTabView
+ (CocoaTabView *)create:(NSRect)re
{
	CocoaTabView *me = [CocoaTabView alloc];
	
	[me initWithFrame:re];
	[me setTabViewType:NSTopTabsBezelBorder];
	return me;
}
+ (CocoaTabView *)create:(NSRect)re tabs:(const char **)tabs
{
	CocoaTabView *me = [CocoaTabView alloc];
	int i;

	[me initWithFrame:re];
	[me setTabViewType:NSTopTabsBezelBorder];
	
	for(i=0; tabs[i] != NULL; i++) {
		[me addTabItem:tabs[i]];
	}
	
	return me;
}
- (NSTabViewItem *)addTabItem:(const char *)label
{
	NSString *str = [NSString stringWithUTF8String:label];
	NSTabViewItem *item = [[NSTabViewItem alloc] init];
	CocoaView *view = [[CocoaView alloc] init];
	[item setLabel:str];
	[item setView:view];
	[self addTabViewItem:item];
	return item;
}
- (NSRect)adjustFrameSize:(NSRect)re
{
	NSUInteger i;
	int max_right = 0;
	int max_bottom = 0;
	int max_margin = 0;

	NSArray *items = [self tabViewItems];
	NSUInteger count = [items count];
	for(i=0; i < count; i++) {
		NSTabViewItem *item = [items objectAtIndex:i];
		CocoaView *view = (CocoaView *)[item view];
		if (max_right < view.max_right) max_right = view.max_right;
		if (max_bottom < view.max_bottom) max_bottom = view.max_bottom;
		if (max_margin < view.margin) max_margin = view.margin;
	}
	max_right += max_margin + 10;
	max_bottom += max_margin + 48;
	[self setFrameSize:NSMakeSize(max_right, max_bottom)];
	re.size.width = max_right;
	re.size.height = max_bottom;
	return re;
}
@end

@implementation CocoaBox
+ (CocoaBox *)create:(NSRect)re title:(const char *)title
{
	CocoaBox *me = [CocoaBox alloc];

	[me initWithFrame:re];
	CocoaView *view = [[CocoaView alloc]init];
	[me setContentView:view];
	[me setTitle:[NSString stringWithUTF8String:title]];
	
	return me;
}
- (NSRect)adjustFrameSize:(NSRect)re
{
#if 1
	CocoaView *view = [self contentView];
	view.max_right += view.margin + 4;
	view.max_bottom += view.margin + 22;
	[self setFrameSize:NSMakeSize(view.max_right, view.max_bottom)];
	re.size.width = view.max_right;
	re.size.height = view.max_bottom;
#else
	[self sizeToFit];
	NSRect fre = [self frame];
	re.size.width = fre.size.width;
	re.size.height = fre.size.height;
#endif
	return re;
}
@end



@implementation CocoaView
@synthesize max_bottom;
@synthesize max_right;
@synthesize margin;
- (id)init
{
	[super init];
	//	[self setAutoresizesSubviews:YES];
	//	[self setAutoresizingMask:NSViewHeightSizable];
	[self setAutoresizesSubviews:NO];
	[self setAutoresizingMask:NSViewNotSizable];
	
	max_right = 0;
	max_bottom = 0;
	
	margin = 10;
	
	return self;
}
- (BOOL)isFlipped
{
	return YES;
}
- (NSRect)makeRect:(int)left :(int)top :(int)w :(int)h
{
	if (max_right < left + w) max_right = left + w;
	if (max_bottom < top + h) max_bottom = top + h;
	
	return NSMakeRect(left,top,w,h);
}
- (NSRect)remakeSize:(NSRect)re :(int)w :(int)h
{
	int left = re.origin.x;
	int top = re.origin.y;

	return [self makeRect:left:top:w:h];
}
- (NSRect)remakePosition:(NSRect)re :(int)left :(int)top :(int)new_w :(int)new_h
{
	return [self makeRect:left:top:new_w:new_h];
}
- (NSRect)remakeLeft:(NSRect)re :(int)left :(int)new_w :(int)new_h
{
	int top = re.origin.y;
	
	return [self makeRect:left:top:new_w:new_h];
}
- (NSRect)remakeLeft:(NSRect)re :(int)left
{
	return [self remakeLeft:re:left:re.size.width:re.size.height];
}
- (NSRect)remakeTop:(NSRect)re :(int)top :(int)new_w :(int)new_h
{
	int left = re.origin.x;

	return [self makeRect:left:top:new_w:new_h];
}
- (NSRect)remakeTop:(NSRect)re :(int)top
{
	return [self remakeTop:re:top:re.size.width:re.size.height];
}
- (NSRect)addPosition:(NSRect)re :(int)offsetx :(int)offsety :(int)new_w :(int)new_h
{
	int left = re.origin.x;
	int top = re.origin.y;
	
	return [self makeRect:left+offsetx:top+offsety:new_w:new_h];
}
- (NSRect)addWidth:(NSRect)re :(int)new_w :(int)new_h
{
	return [self addPosition:re:re.size.width:0:new_w:new_h];
}
- (NSRect)addHeight:(NSRect)re :(int)new_w :(int)new_h
{
	return [self addPosition:re:0:re.size.height:new_w:new_h];
}
+ (int)topPos:(NSRect)re
{
	return re.origin.y;
}
+ (int)bottomPos:(NSRect)re
{
	return re.origin.y + re.size.height;
}
- (void)adjustFrameSize
{
	max_right = max_right + margin;
	max_bottom = max_bottom + margin;
}
@end



@implementation CocoaBasePanel
- (id)init
{
	[super init];
	
	CocoaView *view = [[CocoaView alloc] init];
	[self setContentView:view];

	return self;
}
- (int)margin
{
	CocoaView *view = [self contentView];
	return view.margin;
}
- (void)adjustFrame
{
	CocoaView *view = [self contentView];
	[view adjustFrameSize];
	[self setFrame:NSMakeRect(0,0,view.max_right,view.max_bottom + 32) display:YES];
}

@end
