/** @file cocoa_configpanel.mm
 
 HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
 Skelton for retropc emulator
 SDL edition + Cocoa GUI
 
 @author Sasaji
 @date   2015.04.30 -
 
 @brief [ config panel ]
 */

#import "cocoa_gui.h"
#import "cocoa_configpanel.h"
#import "../../config.h"
#import "../../emu.h"
#import "../gui.h"
#import "../../clocale.h"
#import "../../msgboard.h"
#import "cocoa_fontpanel.h"

extern EMU *emu;
extern GUI *gui;

#ifdef USE_IOPORT_FDD
#define IOPORT_STARTNUM 0
#else
#define IOPORT_STARTNUM 2
#endif

@implementation CocoaConfigPanel
- (id)init
{
	const char *tab_labels[] = { _("Mode"),_("Screen"),_("Tape, FDD"),_("Network"),_("CPU, Memory"),NULL };
	const char *fdd_labels[] = {
		_("No FDD"),
		_("3inch compact FDD"),
		_("5.25inch mini FDD"),
		_("8inch standard FDD"),
	NULL };
	const char *io_labels[] = {
		_("5.25inch FDC  $FF00 - $FF04"),
		_("3inch FDC  $FF18 - $FF20"),
		_("6voice PSG  $FF30 - $FF33"),
		_("Ex LPT Port  $FF3C - $FF3F"),
		_("Ex COM Port  $FF40 - $FF41"),
		_("9voice PSG  $FF70 - $FF7F"),
		_("KANJI ROM  $FF75 - $FF76"),
	NULL };
	const char *gl_use_labels[] = { _("No"),_("Yes(sync)"),_("Yes(async)"),NULL };
	const char *gl_filter_labels[] = { _("Nearest Neighbour"),_("Linear"),NULL };
	const char *led_show_labels[] = { _("Hide"),_("Show Inside"),_("Show Outside"),NULL };
	const char *led_pos_labels[] = { _("LeftTop"),_("RightTop"),_("LeftBottom"),_("RightBottom"),NULL };
	const char *skew_labels[] = { "0","1","2",NULL };
	const char *corr_labels[] = { _("COS Wave"),_("SIN Wave"),NULL };
	const char *rate_labels[] = { "11025", "22050", "44100", "48000", NULL };
	const char *bits_labels[] = { "8", "16", NULL };
	const char *comm_baud_labels[] = {
		_("S:300baud/F:1200baud"),
		_("S:600baud/F:2400baud"),
		_("S:1200baud/F:4800baud"),
		_("S:2400baud/F:9600baud"),
		NULL
	};
	const char *cap_type_labels[] = {
		"BMP",
#if defined(USE_REC_VIDEO_COCOA)
		"PNG",
#endif
		NULL
	};

	[super init];

	self.title = [NSString stringWithUTF8String:_("Configure")];
	[self setShowsResizeIndicator:FALSE];
	
	CocoaView *view = [self contentView];
	NSRect re_tab;
	int top = view.margin;
	int left = view.margin;
	
	re_tab = [view makeRect:left:top:300:200];
	
	CocoaTabView *tabView = [CocoaTabView create:re_tab tabs:tab_labels];
	[view addSubview:tabView];

	NSRect re;
	NSTabViewItem *tab;
	CocoaView *tab_view;
	NSRect re_box;
	CocoaBox *box;
	CocoaView *box_view;
	CocoaLabel *lbl;
	CocoaButton *btn;

	int bleft = view.margin;
	int btop = view.margin;
	int i;
	int center_left = 0;
	int middle_top = 0;

	// Mode tab
	tab = [tabView tabViewItemAtIndex:0];
	tab_view = (CocoaView *)[tab view];

	//
	re_box = [tab_view makeRect:bleft:btop:260:80];
	box = [CocoaBox create:re_box title:_("DIP Switch (*)")];
	box_view = [box contentView];
	
	re = [box_view makeRect:bleft:btop:16:22];
	lbl = [CocoaLabel create:re title:(config.dipswitch & 4) ? ">" : " "];
	[box_view addSubview:lbl];
	
	re = [box_view addWidth:re:240:24];
	chkDipswitch = [CocoaCheckBox create:re title:_("MODE Switch") action:nil value:(config.dipswitch & 4)];
	[box_view addSubview:chkDipswitch];
	
	re_box = [box adjustFrameSize:re_box];
	if (center_left < box_view.max_right) center_left = box_view.max_right;
	[tab_view addSubview:box];

	//
	re_box = [tab_view addHeight:re_box:260:200];
	box = [CocoaBox create:re_box title:_("FDD Type (*)")];
	box_view = [box contentView];
	
	re = [box_view makeRect:bleft:btop+2:16:24];
	lbl = [CocoaLabel create:re title:(config.fdd_type == 0) ? ">" : " "];
	[box_view addSubview:lbl];
	re = [box_view addHeight:re:16:24];
	lbl = [CocoaLabel create:re title:(config.fdd_type == 1) ? ">" : " "];
	[box_view addSubview:lbl];
	re = [box_view addHeight:re:16:24];
	lbl = [CocoaLabel create:re title:(config.fdd_type == 2) ? ">" : " "];
	[box_view addSubview:lbl];
	re = [box_view addHeight:re:16:24];
	lbl = [CocoaLabel create:re title:(config.fdd_type == 3) ? ">" : " "];
	[box_view addSubview:lbl];
	
	re = [box_view remakeTop:re:btop];
	re = [box_view addWidth:re:240:96];
	radFddType = [CocoaRadioGroup create:re rows:4 cols:1 titles:fdd_labels action:@selector(selectFddType:) selidx:emu->get_parami(VM::ParamFddType)];
	[box_view addSubview:radFddType];

	re_box = [box adjustFrameSize:re_box];
	if (center_left < box_view.max_right) center_left = box_view.max_right;
	[tab_view addSubview:box];

	re_box = [tab_view addHeight:re_box:260:24];
	chkPowerOff = [CocoaCheckBox create:re_box title:_("Enable the state of power off") action:nil value:config.use_power_off];
	[tab_view addSubview:chkPowerOff];
	middle_top = [CocoaView bottomPos:re_box];
	
	//
	re_box = [tab_view makeRect:center_left+bleft:btop:320:200];
	box = [CocoaBox create:re_box title:_("I/O Port Address (*)")];
	box_view = [box contentView];

	re = [box_view makeRect:bleft:btop:0:0];
	for(i=IOPORT_STARTNUM; i<IOPORT_NUMS; i++) {
		re = [box_view remakeLeft:re:bleft];
		re = [box_view addHeight:re:16:24];
		lbl = [CocoaLabel create:re title:(config.io_port & (1 << i)) ? ">" : " "];
		[box_view addSubview:lbl];
		
		re = [box_view addWidth:re:270:24];
		chkIOPort[i] = [CocoaCheckBox create:re title:io_labels[i] index:i action:@selector(selectIO:) value:(emu->get_parami(VM::ParamIOPort) & (1 << i))];
		[box_view addSubview:chkIOPort[i]];
		[self selectIO:chkIOPort[i]];
	}
	re_box = [box adjustFrameSize:re_box];
	if (middle_top < [CocoaView bottomPos:re_box]) middle_top = [CocoaView bottomPos:re_box];
	[tab_view addSubview:box];

	//
//	re_box = [tab_view addHeight:re_box:260:32];
	re_box = [tab_view makeRect:bleft:middle_top:260:32];
	re_box = [tab_view remakeLeft:re_box:bleft];
	lbl = [CocoaLabel createWithFitW:&re_box title:_("* Need restart program or \"PowerOn\".")];
	[tab_view addSubview:lbl];


	// Screen tab
	tab = [tabView tabViewItemAtIndex:1];
	tab_view = (CocoaView *)[tab view];
	
	//
	re_box = [tab_view makeRect:bleft:btop:260:100];
	box = [CocoaBox create:re_box title:_("OpenGL")];
	box_view = [box contentView];

	re = [box_view makeRect:bleft:btop:80:24];
	lbl = [CocoaLabel create:re title:_("Use (*)")];
	[box_view addSubview:lbl];
	
	re = [box_view addWidth:re:180:24];
	popUseOpenGL = [CocoaPopUpButton create:re items:gl_use_labels action:nil selidx:config.use_opengl];
	[box_view addSubview:popUseOpenGL];
	
	re = [box_view remakeLeft:re:bleft];
	re = [box_view addHeight:re:80:24];
	lbl = [CocoaLabel create:re title:_("Filter Type")];
	[box_view addSubview:lbl];
	
	re = [box_view addWidth:re:180:24];
	popGLFilter = [CocoaPopUpButton create:re items:gl_filter_labels action:nil selidx:config.gl_filter_type];
	[box_view addSubview:popGLFilter];

	re_box = [box adjustFrameSize:re_box];
	[tab_view addSubview:box];

	//
	re_box = [tab_view addWidth:re_box:240:100];
	box = [CocoaBox create:re_box title:_("CRTC")];
	box_view = [box contentView];

	re = [box_view makeRect:bleft:btop:180:24];
	lbl = [CocoaLabel create:re title:_("Disptmg Skew")];
	[box_view addSubview:lbl];

	re = [box_view addWidth:re:60:24];
	popDisptmgSkew = [CocoaPopUpButton create:re items:skew_labels action:nil selidx:config.disptmg_skew];
	[box_view addSubview:popDisptmgSkew];

	re = [box_view remakeLeft:re:bleft];
	re = [box_view addHeight:re:180:24];
	lbl = [CocoaLabel create:re title:_("Curdisp Skew")];
	[box_view addSubview:lbl];

	re = [box_view addWidth:re:60:24];
	popCurdispSkew = [CocoaPopUpButton create:re items:skew_labels action:nil selidx:config.curdisp_skew];
	[box_view addSubview:popCurdispSkew];

	re_box = [box adjustFrameSize:re_box];
	[tab_view addSubview:box];
	
	//
	re_box = [tab_view remakeLeft:re_box:bleft];
	re_box = [tab_view addHeight:re_box:60:24];
	lbl = [CocoaLabel create:re_box title:_("LED")];
	[tab_view addSubview:lbl];

	re_box = [tab_view addWidth:re_box:160:24];
	int led_show = gui->GetLedBoxPhase(-1);
	popLEDShow = [CocoaPopUpButton create:re_box items:led_show_labels action:nil selidx:led_show];
	[tab_view addSubview:popLEDShow];
	
	re_box = [tab_view addWidth:re_box:20:24];
	re_box = [tab_view addWidth:re_box:60:24];
	lbl = [CocoaLabel create:re_box title:_("Position")];
	[tab_view addSubview:lbl];
	
	re_box = [tab_view addWidth:re_box:160:24];
	popLEDPosition = [CocoaPopUpButton create:re_box items:led_pos_labels action:nil selidx:config.led_pos];
	[tab_view addSubview:popLEDPosition];

	//
	re_box = [tab_view remakeLeft:re_box:bleft];
	re_box = [tab_view addHeight:re_box:140:26];
	lbl = [CocoaLabel create:re_box title:_("Capture Type")];
	[tab_view addSubview:lbl];

	re_box = [tab_view addWidth:re_box:160:26];
	popCaptureType = [CocoaPopUpButton create:re_box items:cap_type_labels action:nil selidx:config.capture_type];
	[tab_view addSubview:popCaptureType];

	//
	re_box = [tab_view remakeLeft:re_box:bleft];
	re_box = [tab_view addHeight:re_box:140:26];
	lbl = [CocoaLabel create:re_box title:_("Snapshot Path")];
	[tab_view addSubview:lbl];
	
	re_box = [tab_view addWidth:re_box:360:26];
	txtSnapPath = [CocoaTextField create:re_box text:config.snapshot_path action:nil];
	[tab_view addSubview:txtSnapPath];
	
	re_box = [tab_view addWidth:re_box:100:26];
	btn = [CocoaButton create:re_box title:_("Folder...") action:@selector(showFolderPanel:)];
	[btn setRelatedObject:txtSnapPath];
	[tab_view addSubview:btn];

	//
	re_box = [tab_view remakeLeft:re_box:bleft];
	re_box = [tab_view addHeight:re_box:140:26];
	lbl = [CocoaLabel create:re_box title:_("Font Path")];
	[tab_view addSubview:lbl];
	
	re_box = [tab_view addWidth:re_box:360:26];
	txtFontPath = [CocoaTextField create:re_box text:config.font_path action:nil];
	[tab_view addSubview:txtFontPath];
	
	re_box = [tab_view addWidth:re_box:100:26];
	btn = [CocoaButton create:re_box title:_("Folder...") action:@selector(showFolderPanel:)];
	[btn setRelatedObject:txtFontPath];
	[tab_view addSubview:btn];

	//
	re_box = [tab_view remakeLeft:re_box:bleft];
	re_box = [tab_view addHeight:re_box:140:24];
	lbl = [CocoaLabel create:re_box title:_("Message Font")];
	[tab_view addSubview:lbl];
	
	re_box = [tab_view addWidth:re_box:200:24];
	txtMsgFontName = [CocoaTextField create:re_box text:config.msgboard_msg_fontname action:nil];
	[tab_view addSubview:txtMsgFontName];
	
	re_box = [tab_view addWidth:re_box:100:24];
	lbl = [CocoaLabel createWithFit:&re_box title:_(" Size")];
	[tab_view addSubview:lbl];
	
	re_box = [tab_view addWidth:re_box:100:24];
	txtMsgFontSize = [CocoaTextField create:re_box num:config.msgboard_msg_fontsize action:nil];
	[tab_view addSubview:txtMsgFontSize];

	re_box = [tab_view addWidth:re_box:100:26];
	btn = [CocoaButton create:re_box title:_("File...") action:@selector(showFilePanel:)];
	[btn setRelatedObject:txtMsgFontName];
	[tab_view addSubview:btn];

	//
	re_box = [tab_view remakeLeft:re_box:bleft];
	re_box = [tab_view addHeight:re_box:140:24];
	lbl = [CocoaLabel create:re_box title:_("Info Font")];
	[tab_view addSubview:lbl];
	
	re_box = [tab_view addWidth:re_box:200:24];
	txtInfoFontName = [CocoaTextField create:re_box text:config.msgboard_info_fontname action:nil];
	[tab_view addSubview:txtInfoFontName];
	
	re_box = [tab_view addWidth:re_box:100:24];
	lbl = [CocoaLabel createWithFitW:&re_box title:_(" Size")];
	[tab_view addSubview:lbl];
	
	re_box = [tab_view addWidth:re_box:100:24];
	txtInfoFontSize = [CocoaTextField create:re_box num:config.msgboard_info_fontsize action:nil];
	[tab_view addSubview:txtInfoFontSize];

	re_box = [tab_view addWidth:re_box:100:26];
	btn = [CocoaButton create:re_box title:_("File...") action:@selector(showFilePanel:)];
	[btn setRelatedObject:txtInfoFontName];
	[tab_view addSubview:btn];

	//
	re_box = [tab_view remakeLeft:re_box:bleft];
	re_box = [tab_view addHeight:re_box:300:24];
	lbl = [CocoaLabel create:re_box title:_("* Need restart program.")];
	[tab_view addSubview:lbl];

	// Tape tab
	tab = [tabView tabViewItemAtIndex:2];
	tab_view = (CocoaView *)[tab view];

	//
	re_box = [tab_view makeRect:bleft:btop:300:200];
	box = [CocoaBox create:re_box title:_("Load Wav File from Tape")];
	box_view = [box contentView];

	re = [box_view makeRect:bleft:btop:260:24];
	chkReverseWave = [CocoaCheckBox create:re title:_("Reverse Wave") action:nil value:config.wav_reverse];
	[box_view addSubview:chkReverseWave];

	re = [box_view addHeight:re:260:24];
	chkHalfWave = [CocoaCheckBox create:re title:_("Half Wave") action:nil value:config.wav_half];
	[box_view addSubview:chkHalfWave];

	re = [box_view addHeight:re:260:24];
	chkCorrectWave = [CocoaCheckBox create:re title:_("Correct") action:@selector(selectCorrect:) value:config.wav_correct];
	[box_view addSubview:chkCorrectWave];

	re = [box_view addHeight:re:260:24];
	radCorrect = [CocoaRadioGroup create:re rows:1 cols:2 titles:corr_labels action:nil selidx:config.wav_correct_type ];
	[box_view addSubview:radCorrect];
	[self selectCorrect:chkCorrectWave];

	re_box = [box adjustFrameSize:re_box];
	[tab_view addSubview:box];
	int tp_bottom = [CocoaView bottomPos:re_box];

	//
	re_box = [tab_view addWidth:re_box:300:200];
	box = [CocoaBox create:re_box title:_("Save Wav File to Tape")];
	box_view = [box contentView];
	
	re = [box_view makeRect:bleft:btop:120:24];
	lbl = [CocoaLabel create:re title:_("Sample Rate")];
	[box_view addSubview:lbl];
	
	re = [box_view addWidth:re:120:24];
	popSampleRate = [CocoaPopUpButton create:re items:rate_labels action:nil selidx:config.wav_sample_rate];
	[box_view addSubview:popSampleRate];
	
	re = [box_view remakeLeft:re:bleft];
	re = [box_view addHeight:re:120:24];
	lbl = [CocoaLabel create:re title:_("Sample Bits")];
	[box_view addSubview:lbl];
	
	re = [box_view addWidth:re:120:24];
	popSampleBits = [CocoaPopUpButton create:re items:bits_labels action:nil selidx:config.wav_sample_bits];
	[box_view addSubview:popSampleBits];

	re_box = [box adjustFrameSize:re_box];
	[tab_view addSubview:box];
	tp_bottom = [CocoaView bottomPos:re_box] > tp_bottom ? [CocoaView bottomPos:re_box] : tp_bottom;
	tp_bottom += 10;

	// FDD
	re_box = [tab_view makeRect:bleft:tp_bottom:600:200];
	box = [CocoaBox create:re_box title:_("Floppy Disk Drive")];
	box_view = [box contentView];

	// mount fdd
	re = [box_view makeRect:bleft:btop:200:24];
	lbl = [CocoaLabel createWithFitW:&re title:_("When start up, mount disk at:")];
	[box_view addSubview:lbl];
	for(i=0; i<MAX_DRIVE; i++) {
		char name[4];
		sprintf(name, "%d ", i);
		re = [box_view addWidth:re:32:24];
		chkFddMount[i] = [CocoaCheckBox create:re title:name action:nil value:(config.mount_fdd & (1 << i))];
		[box_view addSubview:chkFddMount[i]];
	}

	re = [box_view remakeLeft:re:bleft];
	re = [box_view addHeight:re:300:24];
	chkDelayFd1 = [CocoaCheckBox create:re title:_("Ignore delays to find sector.") action:nil value:(FLG_DELAY_FDSEARCH != 0)];
	[box_view addSubview:chkDelayFd1];

	re = [box_view addHeight:re:300:24];
	chkDelayFd2 = [CocoaCheckBox create:re title:_("Ignore delays to seek track.") action:nil value:(FLG_DELAY_FDSEEK != 0)];
	[box_view addSubview:chkDelayFd2];

	re_box = [box adjustFrameSize:re_box];
	[tab_view addSubview:box];

	// Network tab
	tab = [tabView tabViewItemAtIndex:3];
	tab_view = (CocoaView *)[tab view];
	
	re_box = [tab_view makeRect:bleft:btop:0:0];
	for(i=0; i<MAX_PRINTER; i++) {
		char name[128];
		sprintf(name, _("LPT%d Hostname"), i);
		re_box = [tab_view remakeLeft:re_box:bleft];
		re_box = [tab_view addHeight:re_box:120:24];
		lbl = [CocoaLabel create:re_box title:name];
		[tab_view addSubview:lbl];
		
		re_box = [tab_view addWidth:re_box:140:24];
		txtLPTHost[i] = [CocoaTextField create:re_box text:config.printer_server_host[i] action:nil];
		[tab_view addSubview:txtLPTHost[i]];
		
		re_box = [tab_view addWidth:re_box:60:24];
		lbl = [CocoaLabel createWithFitW:&re_box title:_(" Port") align:NSRightTextAlignment];
		[tab_view addSubview:lbl];
		
		re_box = [tab_view addWidth:re_box:80:24];
		txtLPTPort[i] = [CocoaTextField create:re_box num:config.printer_server_port[i] action:nil align:NSRightTextAlignment];
		[tab_view addSubview:txtLPTPort[i]];
	}
	for(i=0; i<MAX_COMM; i++) {
		char name[128];
		sprintf(name, _("COM%d Hostname"), i);
		re_box = [tab_view remakeLeft:re_box:bleft];
		re_box = [tab_view addHeight:re_box:120:24];
		lbl = [CocoaLabel create:re_box title:name];
		[tab_view addSubview:lbl];
		
		re_box = [tab_view addWidth:re_box:140:24];
		txtCOMHost[i] = [CocoaTextField create:re_box text:config.comm_server_host[i] action:nil];
		[tab_view addSubview:txtCOMHost[i]];
		
		re_box = [tab_view addWidth:re_box:60:24];
		lbl = [CocoaLabel createWithFitW:&re_box title:_(" Port") align:NSRightTextAlignment];
		[tab_view addSubview:lbl];
		
		re_box = [tab_view addWidth:re_box:80:24];
		txtCOMPort[i] = [CocoaTextField create:re_box num:config.comm_server_port[i] action:nil align:NSRightTextAlignment];
		[tab_view addSubview:txtCOMPort[i]];

		re_box = [tab_view addWidth:re_box:180:24];
		popCOMDipswitch[i] = [CocoaPopUpButton create:re_box items:comm_baud_labels action:nil selidx:(config.comm_dipswitch[i]-1)];
		[tab_view addSubview:popCOMDipswitch[i]];
	}
#ifdef USE_DEBUGGER
	re_box = [tab_view remakeLeft:re_box:bleft];
	re_box = [tab_view addHeight:re_box:240:24];
	lbl = [CocoaLabel create:re_box title:_("Connectable host to Debugger")];
	[tab_view addSubview:lbl];
	
	re_box = [tab_view remakeLeft:re_box:bleft];
	re_box = [tab_view addHeight:re_box:120:24];
	re_box = [tab_view addWidth:re_box:140:24];
	txtDbgrHost = [CocoaTextField create:re_box text:config.debugger_server_host action:nil];
	[tab_view addSubview:txtDbgrHost];
	
	re_box = [tab_view addWidth:re_box:60:24];
	lbl = [CocoaLabel createWithFitW:&re_box title:_(" Port") align:NSRightTextAlignment];
	[tab_view addSubview:lbl];
	
	re_box = [tab_view addWidth:re_box:80:24];
	txtDbgrPort = [CocoaTextField create:re_box num:config.debugger_server_port action:nil align:NSRightTextAlignment];
	[tab_view addSubview:txtDbgrPort];
#endif

	// CPU, Memory tab
	tab = [tabView tabViewItemAtIndex:4];
	tab_view = (CocoaView *)[tab view];

	re_box = [tab_view makeRect:bleft:btop:100:26];
	lbl = [CocoaLabel create:re_box title:_("ROM Path (*)")];
	[tab_view addSubview:lbl];
	
	re_box = [tab_view addWidth:re_box:380:26];
	txtROMPath = [CocoaTextField create:re_box text:config.rom_path action:nil];
	[tab_view addSubview:txtROMPath];
	
	re_box = [tab_view addWidth:re_box:100:26];
	btn = [CocoaButton create:re_box title:_("Folder...") action:@selector(showFolderPanel:)];
	[btn setRelatedObject:txtROMPath];
	[tab_view addSubview:btn];

	re_box = [tab_view remakeLeft:re_box:bleft];
	re_box = [tab_view addHeight:re_box:260:24];
	chkExMem = [CocoaCheckBox create:re_box title:_("Use Extended Memory (64KB)") action:nil value:(config.exram_size_num == 1)];
	[tab_view addSubview:chkExMem];

	re_box = [tab_view remakeLeft:re_box:bleft];
	re_box = [tab_view addHeight:re_box:360:24];
	chkUndefOp = [CocoaCheckBox create:re_box title:_("Show message when the CPU fetches undefined opcode.") action:nil value:FLG_SHOWMSG_UNDEFOP];
	[tab_view addSubview:chkUndefOp];

	re_box = [tab_view addHeight:re_box:0:24];
	
	re_box = [tab_view remakeLeft:re_box:bleft];
	re_box = [tab_view addHeight:re_box:300:24];
	lbl = [CocoaLabel create:re_box title:_("* Need restart program or \"PowerOn\".")];
	[tab_view addSubview:lbl];

	re_tab = [tabView adjustFrameSize:re_tab];

	// button
	
	re = [view makeRect:left+re_tab.size.width-120:top+re_tab.size.height:120:32];
	CocoaButton *btnCancel = [CocoaButton create:re title:_("Cancel") action:@selector(dialogCancel:)];
	[view addSubview:btnCancel];

	re = [view addPosition:re:-120-view.margin:0:120:32];
	CocoaButton *btnOK = [CocoaButton create:re title:_("OK") action:@selector(dialogOk:)];
	[view addSubview:btnOK];
	
	
	[self adjustFrame];
	
	[self selectFddType:radFddType];
	
	return self;
}

- (NSInteger)runModal
{
	return [NSApp runModalForWindow:self];
}

- (void)close
{
	[NSApp stopModalWithCode:NSCancelButton];
	[super close];
}

- (void)dialogOk:(id)sender
{
	int i;
	int valuel;
	int val;

	// set data
	config.use_power_off = ([chkPowerOff state] == NSOnState);
	if ([chkDipswitch state] == NSOnState) {
		config.dipswitch |= 4;
	} else {
		config.dipswitch &= ~4;
	}
	emu->set_parami(VM::ParamFddType,(int)[radFddType selectedRow]);

	for(i=0;i<MAX_DRIVE;i++) {
		config.mount_fdd = (([chkFddMount[i] state] == NSOnState) ? config.mount_fdd | (1 << i) : config.mount_fdd & ~(1 << i));
	}
	config.delay_fdd = ([chkDelayFd1 state] == NSOnState ? MSK_DELAY_FDSEARCH : 0)
					 | ([chkDelayFd2 state] == NSOnState ? MSK_DELAY_FDSEEK : 0);

	for(i=IOPORT_STARTNUM;i<IOPORT_NUMS;i++) {
		val = emu->get_parami(VM::ParamIOPort);
		if ([chkIOPort[i] state] == NSOnState) {
			val |= (1 << i);
		} else {
			val &= ~(1 << i);
		}
		emu->set_parami(VM::ParamIOPort, val);
	}
#ifndef USE_IOPORT_FDD
	int fdd_type = emu->get_parami(VM::ParamFddType);
	val = emu->get_parami(VM::ParamIOPort);
	val &= ~IOPORT_MSK_FDDALL;
	val |= (fdd_type == 1 ? IOPORT_MSK_3FDD : (fdd_type >= 2 ? IOPORT_MSK_5FDD : 0));
	emu->set_parami(VM::ParamIOPort, val);
#endif

	emu->set_parami(VM::ParamUseOpenGL, (int)[popUseOpenGL indexOfSelectedItem]);
	config.gl_filter_type = [popGLFilter indexOfSelectedItem];

	config.led_pos = [popLEDPosition indexOfSelectedItem];
	config.capture_type = [popCaptureType indexOfSelectedItem];
	
	config.curdisp_skew = [popCurdispSkew indexOfSelectedItem];
	config.disptmg_skew = [popDisptmgSkew indexOfSelectedItem];
	
	config.snapshot_path.Set([[txtSnapPath stringValue] UTF8String]);
	config.font_path.Set([[txtFontPath stringValue] UTF8String]);
	
	config.msgboard_msg_fontname.Set([[txtMsgFontName stringValue] UTF8String]);
	config.msgboard_info_fontname.Set([[txtInfoFontName stringValue] UTF8String]);
	valuel = [txtMsgFontSize intValue];
	if (1 <= valuel && valuel <= 60) {
		config.msgboard_msg_fontsize = (int)valuel;
	}
	valuel = [txtInfoFontSize intValue];
	if (1 <= valuel && valuel <= 60) {
		config.msgboard_info_fontsize = (int)valuel;
	}

	config.wav_reverse = ([chkReverseWave state] == NSOnState);
	config.wav_half = ([chkHalfWave state] == NSOnState);
	config.wav_correct = ([chkCorrectWave state] == NSOnState);
	config.wav_correct_type = [radCorrect selectedColumn];

	config.wav_sample_rate = [popSampleRate indexOfSelectedItem];
	config.wav_sample_bits = [popSampleBits indexOfSelectedItem];
	
	for(i=0;i<MAX_PRINTER;i++) {
		config.printer_server_host[i].Set([[txtLPTHost[i] stringValue] UTF8String]);
		valuel = [txtLPTPort[i] intValue];
		if (0 <= valuel && valuel <= 65535) {
			config.printer_server_port[i] = (int)valuel;
		}
	}
	for(i=0;i<MAX_COMM;i++) {
		config.comm_server_host[i].Set([[txtCOMHost[i] stringValue] UTF8String]);
		valuel = [txtCOMPort[i] intValue];
		if (0 <= valuel && valuel <= 65535) {
			config.comm_server_port[i] = (int)valuel;
		}
		config.comm_dipswitch[i] = (int)[popCOMDipswitch[i] indexOfSelectedItem] + 1;
	}
#ifdef USE_DEBUGGER
	config.debugger_server_host.Set([[txtDbgrHost stringValue] UTF8String]);
	valuel = [txtDbgrPort intValue];
	if (0 <= valuel && valuel <= 65535) {
		config.debugger_server_port = (int)valuel;
	}
#endif
	
	config.rom_path.Set([[txtROMPath stringValue] UTF8String]);

	config.exram_size_num = ([chkExMem state] == NSOnState) ? 1 : 0;
	config.show_dlg = ([chkUndefOp state] == NSOnState ? config.show_dlg | MSK_SHOWMSG_UNDEFOP : config.show_dlg & ~MSK_SHOWMSG_UNDEFOP);

	// set message font
	MsgBoard *msgboard = emu->get_msgboard();
	if (msgboard) {
		msgboard->SetFont();
	}

	gui->ChangeLedBox((int)[popLEDShow indexOfSelectedItem]);
	gui->ChangeLedBoxPosition(config.led_pos);
	config.save();
	emu->change_opengl_attr();

    // OK button is pushed
	[NSApp stopModalWithCode:NSOKButton];
	[super close];
}

- (void)dialogCancel:(id)sender
{
    // Cancel button is pushed
	[self close];
}

- (void)showFolderPanel:(id)sender
{
	NSInteger result;

	CocoaTextField *text = [sender relatedObject];
	if (text == nil) return;
	
	NSOpenPanel *panel = [NSOpenPanel openPanel];

	// cannot select file
	[panel setCanChooseFiles:NO];
	[panel setCanChooseDirectories:YES];
	
	// set current folder
	[panel setDirectoryURL:[NSURL fileURLWithPath:[text stringValue]]];
	
	// Display modal dialog
	result = [panel runModal];
	
	if(result == NSOKButton) {
		// get file path (use NSURL)
		NSURL *filePath = [panel URL];

		[text setStringValue:[filePath path]];
	}
}

- (void)showFilePanel:(id)sender
{
	NSInteger result;
	
	CocoaTextField *text = [sender relatedObject];
	if (text == nil) return;
	
	NSOpenPanel *panel = [NSOpenPanel openPanel];
	
	// can select file
	[panel setCanChooseFiles:YES];
	[panel setCanChooseDirectories:NO];
	// set font folder
	NSURL *url = nil;
	BOOL exist = false;
	url = [NSURL fileURLWithPath:@"/System/Library/Fonts/"];
	exist = [url checkResourceIsReachableAndReturnError:nil];
	if (!exist) {
		url = [NSURL fileURLWithPath:@"/Library/Fonts/"];
	}
	[panel setDirectoryURL:url];

	// Display modal dialog
	result = [panel runModal];
	
	if(result == NSOKButton) {
		// get file path (use NSURL)
		NSURL *filePath = [panel URL];
		NSArray *arr = [filePath pathComponents];
		[text setStringValue:[arr lastObject]];
	}
}

- (void)showFontPanel:(id)sender
{
	NSInteger result;
	
	CocoaObjectStructure *obj = [sender relatedObject];
	if (obj == nil) return;
	CocoaTextField *fname = [obj obj1];
	if (fname == nil) return;
	CocoaTextField *fsize = [obj obj2];
	if (fsize == nil) return;
	
	// set font
	NSFont *font = [NSFont fontWithName:[fname stringValue] size:[fsize integerValue]];

	CocoaFontPanel *panel = [[CocoaFontPanel alloc] init];
	
	[panel setPanelFont:font isMultiple:NO];
	
	// Display modal dialog
	result = [panel runModal];

	if(result == NSOKButton) {
	}
//	[[NSFontManager sharedFontManager]orderFrontFontPanel:self];
}

- (void)changeFont:(id)sender
{

}

- (void)selectFddType:(CocoaRadioGroup *)sender
{
#ifdef USE_IOPORT_FDD
	int num = (int)[sender selectedRow];
	switch(num) {
		case 0:
			// No FDD
			[chkIOPort[0] setState:NSOffState];
			[chkIOPort[1] setState:NSOffState];
			[chkIOPort[0] setEnabled:YES];
			[chkIOPort[1] setEnabled:YES];
			break;
		case 1:
			// 3inch FDD
			[chkIOPort[0] setState:NSOffState];
			[chkIOPort[1] setState:NSOnState];
			[chkIOPort[0] setEnabled:NO];
			[chkIOPort[1] setEnabled:NO];
			break;
		case 2:
		case 3:
			// 5inch FDD
			[chkIOPort[0] setState:NSOnState];
			[chkIOPort[1] setState:NSOffState];
			[chkIOPort[0] setEnabled:NO];
			[chkIOPort[1] setEnabled:NO];
			break;
		default:
			break;
	}
#endif
}

- (void)selectIO:(CocoaCheckBox *)sender
{
	int num = [sender index];
	switch(num) {
#ifdef USE_IOPORT_FDD
		case 0:
			// 5inch FDD
			if ([sender state] == NSOnState) {
				[chkIOPort[1] setState:NSOffState];
			}
			break;
		case 1:
			// 3inch FDD
			if ([sender state] == NSOnState) {
				[chkIOPort[0] setState:NSOffState];
			}
			break;
#endif
		case 5:
			// 9psg
			if ([sender state] == NSOnState) {
				[chkIOPort[6] setState:NSOffState];
			}
			break;
		case 6:
			// Kanji
			if ([sender state] == NSOnState) {
				[chkIOPort[5] setState:NSOffState];
			}
			break;
		default:
			break;
	}
}

- (void)selectCorrect:(CocoaCheckBox *)sender
{
//	if ([sender state] == NSOnState) {
//		[radCorrect setEnabled:YES];
//	} else {
//		[radCorrect setEnabled:NO];
//	}
}

@end
