/** @file cocoa_gui.mm

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator
	SDL edition + Cocoa GUI

	@author Sasaji
	@date   2015.04.24 -

	@brief [ gui for macosx ]
*/

#import <Cocoa/Cocoa.h>
#import "cocoa_gui.h"
#import "../../emu.h"
#import "../gui.h"
#import "cocoa_volumepanel.h"
#import "cocoa_keybindpanel.h"
#import "cocoa_configpanel.h"
#import "cocoa_recvidpanel.h"
#import "cocoa_recaudpanel.h"
#import "cocoa_seldrvpanel.h"
#import "cocoa_ledbox.h"
#ifdef USE_VKEYBOARD
#import "cocoa_vkeyboard.h"
#endif
#import "../../config.h"
#import "../../clocale.h"
#import "../../version.h"
#import "../../sdl_main.h"
#import <SDL_syswm.h>

extern EMU *emu;

static NSWindow *get_main_window()
{
	NSWindow *window = nil;
#ifndef USE_SDL2
	NSArray *windows = [NSApp windows];
	for(int i=0; i<[windows count]; i++) {
		NSWindow *nw = [windows objectAtIndex:i];
		NSRange rg = [[nw className] rangeOfString:@"SDL"];
		if (rg.length > 0) {
			window = nw;
			break;
		}
	}
#else
	SDL_SysWMinfo info;
	SDL_VERSION(&info.version);
	SDL_GetWindowWMInfo(emu->get_window(), &info);
	window = info.info.cocoa.window;
#endif
	return window;
}

//
@implementation NSApplication (SDLApplication)
- (void)orderFrontStandardAboutPanel:(id)sender
{
	GUI *gui = emu->get_gui();
	char name[256],ver[256],libver[256];
	sprintf(name, "%s Emulator", DEVICE_NAME);
	sprintf(ver, "Version: %s \"%s\""
			,APP_VERSION,PLATFORM);
#ifdef USE_DEBUGGER
	strcat(ver, " with Debugger");
#endif
#ifdef _DEBUG
	strcat(ver, " (DEBUG Version)");
#endif
	gui->GetLibVersionString(libver, 256, ", ");

	NSAttributedString *cre = [[NSAttributedString alloc] initWithString:
							   [NSString stringWithUTF8String:libver]];
	
	NSDictionary *dic = [NSDictionary dictionaryWithObjectsAndKeys:
		[NSString stringWithUTF8String:name],@"ApplicationName"
		,[NSString stringWithUTF8String:ver],@"ApplicationVersion"
		,@"",@"Version"
		,cre,@"Credits"
		,nil];

	[self orderFrontStandardAboutPanelWithOptions:dic];
}
@end

#ifdef GUI_TYPE_COCOA

@implementation CocoaMenuItem
@synthesize drv;
@synthesize num;

- (id)initWithTitle:(NSString *)new_title action:(SEL)new_action drv:(int)new_drv num:(int)new_num keyEquivalent:(NSString *)new_accl
{
	self.drv = new_drv;
	self.num = new_num;
	[super initWithTitle:new_title action:new_action keyEquivalent:new_accl];
	return self;
}

@end


@implementation CocoaMenu
+ (CocoaMenu *)create_menu:(const char *)new_title
{
	CocoaMenu *menu = [[CocoaMenu alloc] initWithTitle:[NSString stringWithUTF8String:new_title]];
	return menu;
}
+ (CocoaMenu *)create_menu:(const char *)new_title :(id)new_delegate
{
	CocoaMenu *menu = [[CocoaMenu alloc] initWithTitle:[NSString stringWithUTF8String:new_title]];
	[menu setDelegate:new_delegate];
	return menu;
}
- (CocoaMenuItem *)add_menu_item:(const char *)new_title :(NSObject *)new_target :(SEL)new_action :(int)new_drv :(int)new_num :(unichar)new_accl
{
	CocoaMenuItem *menuItem;
	NSString *title = [NSString stringWithUTF8String:(new_title)];
	NSString *accl;
	if (new_accl != 0) {
		accl = [NSString stringWithCharacters:&new_accl length:1];
	} else {
		accl = @"";
	}
	
	menuItem = [[CocoaMenuItem alloc] initWithTitle:title action:new_action drv:new_drv num:new_num keyEquivalent:accl];
	
	[menuItem setKeyEquivalentModifierMask:(NSAlternateKeyMask)];
	[menuItem setTarget:new_target];
    [self addItem:menuItem];
	//  [menuItem release];
	//	[accl release];
	//	[title release];
	return menuItem;
}
- (void)add_sub_menu:(CocoaMenu *)submenu :(const char *)new_title
{
	[self add_sub_menu:submenu:new_title:nil:nil:0:0];
}
- (void)add_sub_menu:(CocoaMenu *)submenu :(const char *)new_title :(NSObject *)new_target :(SEL)new_action :(int)new_drv :(int)new_num
{
	CocoaMenuItem *menuItem;
	NSString *title = [[NSString alloc] initWithUTF8String:(new_title)];
	
	menuItem = [[CocoaMenuItem alloc] initWithTitle:title action:new_action drv:new_drv num:new_num keyEquivalent:@""];
	if (new_target != nil) [menuItem setTarget:new_target];
	[menuItem setSubmenu:submenu];
	[self addItem:menuItem];
	//	[submenu release];
	//	[menuItem release];
}

@end


@implementation CocoaController
@synthesize gui;

- (void)UpdateRecentFiles:(CocoaMenuItem *)menuItem :(CRecentPathList &)list :(int)drv :(SEL)action
{
	CocoaMenu *menu = (CocoaMenu *)[menuItem submenu];
	if (menu != nil) {
		bool flag = false;
		char str[_MAX_PATH];
		[menu removeAllItems];
		for(int i = 0; i < list.Count(); i++) {
			if (!gui->GetRecentFileStr(list[i]->path, list[i]->num, str, 64)) break;
			[menu add_menu_item:str:self:action:drv:i:0];
			flag = true;
		}
		if (!flag) {
			[menu add_menu_item:_("None"):nil:nil:0:0:0];
		}
	}
}

// Control

- (void)Reset:(id)sender
{
	gui->PostEtReset();
}
- (void)SpecialReset:(id)sender
{
	gui->PostEtSpecialReset();
}
- (void)WarmReset:(id)sender
{
	gui->PostEtWarmReset(-1);
}
- (void)Dipswitch:(id)sender
{
	gui->Dipswitch([sender num]);
}
- (void)TogglePause:(id)sender
{
	gui->TogglePause();
}
- (void)CPUPower:(id)sender
{
	gui->PostEtCPUPower([sender num]);
}
#ifdef _MBS1
- (void)ChangeSystemMode:(id)sender
{
	gui->ChangeSystemMode([sender num]);
}
#endif
- (void)ChangeFddType:(id)sender
{
	gui->ChangeFddType([sender num]);
}
- (void)ToggleSyncIRQ:(id)sender
{
	gui->PostEtToggleSyncIRQ();
}
#ifdef _MBS1
- (void)ToggleMemNoWait:(id)sender
{
	gui->ToggleMemNoWait();
}
#endif
- (void)ShowOpenAutoKeyDialog:(id)sender
{
	gui->ShowOpenAutoKeyDialog();
}
- (void)StartAutoKey:(id)sender
{
	gui->PostEtStartAutoKeyMessage();
}
- (void)StopAutoKey:(id)sender
{
	gui->PostEtStopAutoKeyMessage();
}
- (void)ShowPlayRecKeyDialog:(id)sender
{
	gui->ShowPlayRecKeyDialog();
}
- (void)StopPlayRecKey:(id)sender
{
	gui->StopPlayRecKey();
}
- (void)ShowRecordRecKeyDialog:(id)sender
{
	gui->ShowRecordRecKeyDialog();
}
- (void)ShowRecordStateAndRecKeyDialog:(id)sender
{
	if (gui->ShowSaveStateDialog(true)) {
		gui->ShowRecordRecKeyDialog();
	}
}
- (void)StopRecordRecKey:(id)sender
{
	gui->StopRecordRecKey();
}
- (void)ShowLoadStateDialog:(id)sender
{
	gui->ShowLoadStateDialog();
}
- (void)ShowSaveStateDialog:(id)sender
{
	gui->ShowSaveStateDialog(false);
}
- (void)LoadRecentState:(id)sender
{
	gui->PostEtLoadRecentStatusMessage([sender num]);
}
- (void)UpdateRecentStateList:(id)sender
{
}

// Tape

- (void)ShowLoadDataRecDialog:(id)sender
{
	gui->ShowLoadDataRecDialog();
}
- (void)ShowSaveDataRecDialog:(id)sender
{
	gui->ShowSaveDataRecDialog();
}
- (void)RewindDataRec:(id)sender
{
	gui->PostEtRewindDataRecMessage();
}
- (void)FastForwardDataRec:(id)sender
{
	gui->PostEtFastForwardDataRecMessage();
}
- (void)CloseDataRec:(id)sender
{
	gui->PostEtCloseDataRecMessage();
}
- (void)ToggleRealModeDataRec:(id)sender
{
	gui->PostEtToggleRealModeDataRecMessage();
}
- (void)LoadRecentDataRec:(id)sender
{
	gui->PostEtLoadRecentDataRecMessage([sender num]);
}
- (void)UpdateRecentDataRecList:(id)sender
{
}

// FDD

- (void)ShowOpenFloppyDiskDialog:(id)sender
{
	int drv = [sender drv];

	gui->ShowOpenFloppyDiskDialog(drv);
}
- (void)ChangeSideFloppyDisk:(id)sender
{
	int drv = [sender drv];
	gui->PostEtChangeSideFloppyDisk(drv);
}
- (void)CloseFloppy:(id)sender
{
	int drv = [sender drv];
	gui->PostEtCloseFloppyMessage(drv);
}
- (void)ToggleWriteProtectFloppyDisk:(id)sender
{
	int drv = [sender drv];
	gui->PostEtToggleWriteProtectFloppyDisk(drv);
}
- (void)OpenRecentFloppy:(id)sender
{
	int drv = [sender drv];
	int num = [sender num];
	gui->PostEtOpenRecentFloppyMessage(drv, num);
}
- (void)OpenFloppySelectedVolume:(id)sender
{
	int drv = [sender drv];
	int num = [sender num];
	gui->PostEtOpenFloppySelectedVolume(drv, num);
}
- (void)UpdateRecentFloppyList:(id)sender
{
}
- (void)UpdateVolumeFloppyList:(id)sender
{
}

// Screen

- (void)ChangeFrameRate:(id)sender
{
	int num = [sender num];
	gui->ChangeFrameRate(num);
}
- (void)ResizeRecordVideoSurface:(id)sender
{
	int num = [sender num];
	gui->PostEtResizeRecordVideoSurface(num);
}
- (void)ChangeRecordVideoFrameRate:(id)sender
{
	int num = [sender num];
	gui->PostEtStartRecordVideo(num);
}
- (void)ShowRecordVideoDialog:(id)sender
{
	int num = [sender num];
	gui->ShowRecordVideoAndAudioDialog(num);
}
- (void)StopRecordVideo:(id)sender
{
	gui->PostEtStopRecordVideo();
}
- (void)CaptureScreen:(id)sender
{
	gui->PostEtCaptureScreen();
}
- (void)ChangeWindowMode:(id)sender
{
	int num = [sender num];
	gui->ChangeWindowMode(num);
}
- (void)ChangeFullScreenMode:(id)sender
{
	int num = [sender num];
	gui->ChangeFullScreenMode(num);
}
- (void)ToggleStretchScreen:(id)sender
{
	int num = [sender num];
	gui->ChangeStretchScreen(num);
}
//- (void)ToggleCutoutScreen:(id)sender
//{
//	gui->ChangeStretchScreen(2);
//}
- (void)ChangePixelAspectMode:(id)sender
{
	int num = [sender num];
	gui->ChangePixelAspect(num);
}
- (void)ChangeScanLine:(id)sender
{
	int num = [sender num];
	gui->PostEtChangeDrawMode(num);
}
- (void)ChangeAfterImage:(id)sender
{
	int num = [sender num];
	gui->PostEtChangeAfterImage(num);
}
#ifdef USE_KEEPIMAGE
- (void)ChangeKeepImage:(id)sender
{
	int num = [sender num];
	gui->PostEtChangeKeepImage(num);
}
#endif
#ifdef _MBS1
- (void)ChangeRGBType:(id)sender
{
	int num = [sender num];
	gui->ChangeRGBType(num);
}
#endif
- (void)ChangeUseOpenGL:(id)sender
{
	int num = [sender num];
	gui->ChangeUseOpenGL(num);
}
- (void)ChangeOpenGLFilter:(id)sender
{
	gui->ChangeOpenGLFilter(-1);
}

// Sound

- (void)ShowVolumeDialog:(id)sender
{
	gui->ShowVolumeDialog();
}
- (void)ShowRecordAudioDialog:(id)sender
{
	gui->ShowRecordAudioDialog();
}
- (void)StopRecordSound:(id)sender
{
	gui->PostEtStopRecordSound();
}
- (void)ChangeSoundFrequency:(id)sender
{
	int num = [sender num];
	gui->ChangeSoundFrequency(num);
}
- (void)ChangeSoundLatency:(id)sender
{
	int num = [sender num];
	gui->ChangeSoundLatency(num);
}

// Devices

- (void)ShowSavePrinterDialog:(id)sender
{
	int drv = [sender drv];

	gui->ShowSavePrinterDialog(drv);
}
- (void)PrintPrinter:(id)sender
{
	int drv = [sender drv];
	gui->PostEtPrintPrinterMessage(drv);
}
- (void)ClearPrinterBuffer:(id)sender
{
	int drv = [sender drv];
	gui->PostEtClearPrinterBufferMessage(drv);
}
- (void)EnablePrinterDirect:(id)sender
{
	int drv = [sender drv];
	gui->PostEtEnablePrinterDirectMessage(drv);
}
- (void)EnableCommServer:(id)sender
{
	int drv = [sender drv];
//	gui->PostEtEnableCommServerMessage(drv);
	gui->ToggleEnableCommServer(drv);
}
- (void)ConnectComm:(id)sender
{
	int drv = [sender drv];
//	gui->PostEtConnectCommMessage(drv);
	gui->ToggleConnectComm(drv);
}
- (void)CommThroughMode:(id)sender
{
	int drv = [sender drv];
	gui->ToggleCommThroughMode(drv);
}
- (void)CommBinaryMode:(id)sender
{
	int drv = [sender drv];
	gui->ToggleCommBinaryMode(drv);
}
- (void)SendCommTelnetCommand:(id)sender
{
	int drv = [sender drv];
	int num = [sender num];
	gui->SendCommTelnetCommand(drv, num);
}

// Options

- (void)ToggleLedBox:(id)sender
{
	int num = [sender num];
	switch(num) {
	case 1:
		gui->ToggleShowLedBox();
		break;
	case 2:
		gui->ToggleInsideLedBox();
		break;
	default:
	gui->ToggleLedBox();
		break;
	}
}
#if 0
- (void)ToggleShowLedBox:(id)sender
{
	gui->ToggleShowLedBox();
}
- (void)ToggleInsideLedBox:(id)sender
{
	gui->ToggleInsideLedBox();
}
#endif
- (void)ToggleMessageBoard:(id)sender
{
	gui->ToggleMessageBoard();
}
- (void)ChangeUseJoypad:(id)sender
{
	int num = [sender num];
	gui->ChangeUseJoypad(num);
}
#ifdef USE_LIGHTPEN
- (void)ToggleEnableLightpen:(id)sender
{
	gui->ToggleEnableLightpen();
}
#endif
#ifdef USE_MOUSE
- (void)ToggleUseMouse:(id)sender
{
	gui->PostMtToggleUseMouse();
}
#endif
- (void)ShowKeybindDialog:(id)sender
{
	gui->ShowKeybindDialog();
}
- (void)ShowConfigureDialog:(id)sender
{
	gui->ShowConfigureDialog();
}
- (void)ShowVirtualKeyboard:(id)sender
{
	gui->ShowVirtualKeyboard();
}

#ifdef USE_DEBUGGER
- (void)OpenDebugger:(id)sender
{
	int num = [sender num];
	
	gui->OpenDebugger(num);
}
- (void)CloseDebugger:(id)sender
{
	gui->CloseDebugger();
}
#endif

/// called from emu thread
- (void)PerformUpdateScreen
{
	gui->UpdateScreen();
}

///
/// Menu state and enable check
///
- (BOOL)validateMenuItem:(CocoaMenuItem *)menuItem
{
	int state = NSOffState;
	BOOL enable = TRUE;
	SEL act = [menuItem action];
	int drv = [menuItem drv];

	if (act == @selector(Reset:)) {
		if (!config.now_power_off) {
			state = NSOnState;
		}
#ifdef _BML3MK5
	} else if (act == @selector(Dipswitch:)) {
		int num = (1 << [menuItem num]);
		if (config.dipswitch & num) {
			state = NSOnState;
		}
#endif
#ifdef _MBS1
	} else if (act == @selector(ChangeSystemMode:)) {
		if ([menuItem num] == gui->GetSystemMode()) {
			state = NSOnState;
		}
#endif
	} else if (act == @selector(TogglePause:)) {
		if (gui->NowPause()) {
			state = NSOnState;
		}
	} else if (act == @selector(ToggleSyncIRQ:)) {
		if (gui->NowSyncIRQ()) {
			state = NSOnState;
		}
#ifdef _MBS1
	} else if (act == @selector(ToggleMemNoWait:)) {
		if (gui->NowMemNoWait()) {
			state = NSOnState;
		}
#endif
	} else if (act == @selector(CPUPower:)) {
		if ([menuItem num] == config.cpu_power) {
			state = NSOnState;
		}
	} else if (act == @selector(ChangeFddType:)) {
		if ([menuItem num] == gui->NextFddType()) {
			state = NSOnState;
		}
	} else if (act == @selector(ShowOpenAutoKeyDialog:)
		    || act == @selector(StartAutoKey:)) {
		if (gui->IsRunningAutoKey()) {
			enable = FALSE;
		}
//	} else if (act == @selector(StopAutoKey:)) {
//		if (!gui->IsRunningAutoKey()) {
//			enable = FALSE;
//		}
	} else if (act == @selector(ShowPlayRecKeyDialog:)) {
		if (gui->NowPlayingRecKey()) {
			state = NSOnState;
		}
//	} else if (act == @selector(StopPlayRecKey:)) {
//		if (!gui->NowPlayingRecKey()) {
//			enable = FALSE;
//		}
	} else if (act == @selector(ShowRecordStateAndRecKeyDialog:)) {
		if (gui->NowRecordingRecKey()) {
			state = NSOnState;
		}
	} else if (act == @selector(UpdateRecentStateList:)) {
		[self UpdateRecentFiles:menuItem:config.recent_state_path:0:@selector(LoadRecentState:)];
//	} else if (act == @selector(StopRecordRecKey:)) {
//		if (!gui->NowRecordingRecKey()) {
//			enable = FALSE;
//		}
	} else if (act == @selector(ShowLoadDataRecDialog:)) {
		if (gui->IsOpenedLoadDataRecFile()) {
			state = NSOnState;
		}
	} else if (act == @selector(ShowSaveDataRecDialog:)) {
		if (gui->IsOpenedSaveDataRecFile()) {
			state = NSOnState;
		}
	} else if (act == @selector(ToggleRealModeDataRec:)) {
		if (config.realmode_datarec) {
			state = NSOnState;
		}
	} else if (act == @selector(RewindDataRec:)
			|| act == @selector(FastForwardDataRec:)
			|| act == @selector(CloseDataRec:)) {
		if (!(gui->IsOpenedSaveDataRecFile() || gui->IsOpenedLoadDataRecFile())) {
			enable = FALSE;
		}
	} else if (act == @selector(UpdateRecentDataRecList:)) {
		[self UpdateRecentFiles:menuItem:config.recent_datarec_path:0:@selector(LoadRecentDataRec:)];
	} else if (act == @selector(ShowOpenFloppyDiskDialog:)) {
		if (gui->InsertedFloppyDisk(drv)) {
			state = NSOnState;
		}
	} else if (act == @selector(ChangeSideFloppyDisk:)) {
		char new_title[64];
		int side = gui->GetSideFloppyDisk(drv);
		if (side == 1 && config.fdd_type == 1) {
			strcpy(new_title, _("Change Side to A"));
		} else {
			strcpy(new_title, _("Change Side to B"));
		}
		[menuItem setTitle:[NSString stringWithUTF8String:new_title]];
		if (side < 0 || config.fdd_type != 1) {
			enable = FALSE;
		}
	} else if (act == @selector(CloseFloppy:)) {
		if (!gui->InsertedFloppyDisk(drv)) {
			enable = FALSE;
		}
	} else if (act == @selector(ToggleWriteProtectFloppyDisk:)) {
		if (!gui->InsertedFloppyDisk(drv)) {
			enable = FALSE;
		} else if (gui->WriteProtectedFloppyDisk(drv)) {
			state = NSOnState;
		}
	} else if (act == @selector(UpdateRecentFloppyList:)) {
		[self UpdateRecentFiles:menuItem:config.recent_disk_path[drv]:drv:@selector(OpenRecentFloppy:)];
	} else if (act == @selector(UpdateVolumeFloppyList:)) {
		CocoaMenu *menu = (CocoaMenu *)[menuItem submenu];
		if (menu != nil) {
			bool flag = false;
			char str[32];
			[menu removeAllItems];
			d88_file_t *d88_file = gui->GetD88File(drv);
			for(int num = 0; num < d88_file->bank_nums; num++) {
				gui->GetMultiVolumeStr(num, d88_file->bank[num].name, str);
				[menu add_menu_item:str:self:@selector(OpenFloppySelectedVolume:):drv:num:0];
				flag = true;
			}
			if (!flag) {
				[menu add_menu_item:_("None"):nil:nil:0:0:0];
			}
		}
	} else if (act == @selector(OpenFloppySelectedVolume:)) {
		int num = [menuItem num];
		d88_file_t *d88_file = gui->GetD88File(drv);
		if (d88_file->cur_bank == num) {
			state = NSOnState;
		}
		if (d88_file->bank_nums == 1) {
			enable = FALSE;
		}
	} else if (act == @selector(ChangeFrameRate:)) {
		int num = [menuItem num];
		if (gui->GetFrameRateNum() == num) {
			state = NSOnState;
		}
	} else if (act == @selector(ResizeRecordVideoSurface:)) {
		int num = [menuItem num];
		if (gui->GetRecordVideoSurfaceNum() == num) {
			state = NSOnState;
		}
		if (gui->NowRecordingVideo()) {
			enable = FALSE;
		}
	} else if (act == @selector(ShowRecordVideoDialog:)) {
		int num = [menuItem num];
		if (gui->GetRecordVideoFrameNum() == num) {
			state = NSOnState;
		}
		if (gui->NowRecordingVideo()) {
			enable = FALSE;
		}
//	} else if (act == @selector(StopRecordVideo:)) {
//		if (!gui->NowRecordingVideo()) {
//			enable = FALSE;
//		}
	} else if (act == @selector(ChangeWindowMode:)) {
		int num = [menuItem num];
		if (gui->GetWindowMode() == num) {
			state = NSOnState;
		}
	} else if (act == @selector(ChangeFullScreenMode:)) {
		int num = [menuItem num];
		if (gui->IsFullScreen()) {
			if (gui->GetFullScreenMode() == num) {
				state = NSOnState;
			}
			enable = FALSE;
		}
	} else if (act == @selector(ToggleStretchScreen:)) {
		int num = [menuItem num];
		if (gui->GetStretchScreen() == num) {
			state = NSOnState;
		}
//	} else if (act == @selector(ToggleCutoutScreen:)) {
//		if (gui->GetStretchScreen() == 2) {
//			state = NSOnState;
//		}
	} else if (act == @selector(ChangePixelAspectMode:)) {
		int num = [menuItem num];
		if (gui->GetPixelAspectMode() == num) {
			state = NSOnState;
		}
	} else if (act == @selector(ChangeScanLine:)) {
		int num = [menuItem num];
		if (gui->GetDrawMode() == num) {
			state = NSOnState;
		}
	} else if (act == @selector(ChangeAfterImage:)) {
		int num = [menuItem num];
		if (gui->GetAfterImageMode() == num) {
			state = NSOnState;
		}
#ifdef USE_KEEPIMAGE
	} else if (act == @selector(ChangeKeepImage:)) {
		int num = [menuItem num];
		if (gui->GetKeepImageMode() == num) {
			state = NSOnState;
		}
#endif
#ifdef _MBS1
	} else if (act == @selector(ChangeRGBType:)) {
		int num = [menuItem num];
		if (gui->GetRGBTypeMode() == num) {
			state = NSOnState;
		}
#endif
	} else if (act == @selector(ChangeUseOpenGL:)) {
		int num = [menuItem num];
		if (gui->GetOpenGLMode() == num) {
			state = NSOnState;
		}
	} else if (act == @selector(ChangeOpenGLFilter:)) {
		int num = [menuItem num];
		if (gui->GetOpenGLFilter() == num) {
			state = NSOnState;
		}
	} else if (act == @selector(ShowRecordAudioDialog:)) {
		if (gui->NowRecordingSound()) {
			state = NSOnState;
			enable = FALSE;
		}
//	} else if (act == @selector(StopRecordSound:)) {
//		if (!gui->NowRecordingSound()) {
//			enable = FALSE;
//		}
	} else if (act == @selector(ChangeSoundFrequency:)) {
		int num = [menuItem num];
		if (gui->GetSoundFrequencyNum() == num) {
			state = NSOnState;
		}
	} else if (act == @selector(ChangeSoundLatency:)) {
		int num = [menuItem num];
		if (gui->GetSoundLatencyNum() == num) {
			state = NSOnState;
		}
	} else if (act == @selector(ShowSavePrinterDialog:) || act == @selector(PrintPrinter:)) {
		int drv = [menuItem drv];
		if (gui->GetPrinterBufferSize(drv) <= 0) {
			enable = FALSE;
		}
	} else if (act == @selector(EnablePrinterDirect:)) {
		int drv = [menuItem drv];
		if (gui->IsEnablePrinterDirect(drv)) {
			state = NSOnState;
		}
	} else if (act == @selector(EnableCommServer:)) {
		int drv = [menuItem drv];
		if (gui->IsEnableCommServer(drv)) {
			state = NSOnState;
		}
	} else if (act == @selector(ConnectComm:)) {
		int drv = [menuItem drv];
		if (gui->NowConnectingComm(drv)) {
			state = NSOnState;
		}
	} else if (act == @selector(CommThroughMode:)) {
		int drv = [menuItem drv];
		if (gui->NowCommThroughMode(drv)) {
			state = NSOnState;
		}
	} else if (act == @selector(CommBinaryMode:)) {
		int drv = [menuItem drv];
		if (gui->NowCommBinaryMode(drv)) {
			state = NSOnState;
		}
	} else if (act == @selector(ToggleLedBox:)) {
		int num = [menuItem num];
		switch(num) {
		case 1:
		if (gui->IsShownLedBox()) {
			state = NSOnState;
		}
			break;
		case 2:
		if (gui->IsInsidedLedBox()) {
			state = NSOnState;
		}
			break;
		}
//	} else if (act == @selector(ToggleInsideLedBox:)) {
//		if (gui->IsInsidedLedBox()) {
//			state = NSOnState;
//		}
	} else if (act == @selector(ToggleMessageBoard:)) {
		if (gui->IsShownMessageBoard()) {
			state = NSOnState;
		}
	} else if (act == @selector(ChangeUseJoypad:)) {
		int num = [menuItem num];
		if (gui->IsEnableJoypad(num)) {
			state = NSOnState;
		}
#ifdef USE_LIGHTPEN
	} else if (act == @selector(ToggleEnableLightpen:)) {
		if (gui->IsEnableLightpen()) {
			state = NSOnState;
		}
#endif
#ifdef USE_MOUSE
	} else if (act == @selector(ToggleUseMouse:)) {
		if (gui->IsEnableMouse()) {
			state = NSOnState;
		}
#endif
	} else if (act == @selector(ShowVirtualKeyboard:)) {
		if (gui->IsShownVirtualKeyboard()) {
			state = NSOnState;
		}
#ifdef USE_DEBUGGER
	} else if (act == @selector(OpenDebugger:)) {
		int num = [menuItem num];
		if (gui->IsDebuggerOpened(num)) {
			enable = FALSE;
		}
#endif
	}
	[menuItem setState:state];
	return enable;
}

@end

#ifdef USE_DELEGATE
@implementation CocoaRecentDataRecList
- (id)initWithTarget:(CocoaController *)new_target
{
	[super init];
	target = new_target;
	return self;
}

- (void)menuWillOpen:(CocoaMenu *)menu
{
	if (menu == nil) return;
		
	for(int i = 0; i < MAX_HISTORY; i++) {
		if (strlen(config.recent_datarec_path[i]) == 0) break;
		[Util add_menu_item:menu:config.recent_datarec_path[i]:target:@selector(LoadRecentDataRec:):0:i:0];
	}
}
- (void)menuDidClose:(CocoaMenu *)menu
{
	if (menu == nil) return;

	[menu removeAllItems];
}
@end

@implementation CocoaRecentFloppyDiskList
- (id)initWithTarget:(CocoaController *)new_target :(int)new_drv
{
	[super init];
	target = new_target;
	drv = new_drv;
	return self;
}

- (void)menuWillOpen:(CocoaMenu *)menu
{
	if (menu == nil) return;
	
	for(int i = 0; i < MAX_HISTORY; i++) {
		if (strlen(config.recent_disk_path[drv][i]) == 0) break;
		[Util add_menu_item:menu:config.recent_disk_path[drv][i]:target:@selector(OpenRecentFloppy:):drv:i:0];
	}
}
- (void)menuDidClose:(CocoaMenu *)menu
{
	if (menu == nil) return;
	
	[menu removeAllItems];
}
@end

@implementation CocoaVolumeFloppyDiskList
- (id)initWithTarget:(CocoaController *)new_target :(int)new_drv
{
	[super init];
	target = new_target;
	drv = new_drv;
	return self;
}

- (void)menuWillOpen:(CocoaMenu *)menu
{
	if (menu == nil) return;
	
	GUI *gui = target.gui;
	
	for(int num = 0; num < gui->d88_file[drv].bank_num; num++) {
		char name[32];
		sprintf(name, "%02d: %s", num+1
			, strlen(gui->d88_file[drv].bank[num].name) > 0 ? gui->d88_file[drv].bank[num].name : _("(no label)"));;
		CocoaMenuItem *item = [Util add_menu_item:menu:name:target:@selector(OpenFloppySelectedVolume:):drv:num:0];
		if (item && gui->d88_file[drv].cur_bank == num) {
			[item setState:NSOnState];
		}
	}
}
- (void)menuDidClose:(CocoaMenu *)menu
{
	if (menu == nil) return;
	
	[menu removeAllItems];
}
@end
#endif

void add_main_menu(CocoaMenu *submenu, const char *new_title)
{
	CocoaMenuItem *menuItem;
	NSString *title = [[NSString alloc] initWithUTF8String:(new_title)];
	
	menuItem = [[CocoaMenuItem alloc] initWithTitle:title action:nil drv:0 num:0 keyEquivalent:@""];
	[menuItem setSubmenu:submenu];
	[[NSApp mainMenu] addItem:menuItem];
//	[submenu release];
//	[menuItem release];
}


GUI::GUI(int argc, char **argv, EMU *new_emu) : GUI_BASE(argc, argv, new_emu)
{
	recv = [[CocoaController alloc] init];
	[recv setGui:this];
}

GUI::~GUI()
{
	[recv release];
}

int GUI::CreateWidget(SDL_Surface *screen, int width, int height)
{
	set_delegate_to_sdl_window(this);
	return 0;
}

int GUI::CreateMenu()
{
	GUI_BASE::CreateMenu();

    remove_window_menu();
	translate_apple_menu();
    setup_menu();
    return 0;
}
void GUI::ShowMenu()
{
//	if ([NSMenu menuBarVisible] != YES) {
//		[NSMenu setMenuBarVisible:YES];
//	}
}
void GUI::HideMenu()
{
//	if ([NSMenu menuBarVisible] != NO) {
//		[NSMenu setMenuBarVisible:NO];
//	}
}
void GUI::PreProcessEvent()
{
	// nothing to do
}
int GUI::ProcessEvent(SDL_Event *e)
{
	int rc = GUI_BASE::ProcessEvent(e);
	if (rc <= 0) return rc;
	
	// TODO: cannot visible menubar in fullscreen
#if 0
	if (e->type == SDL_MOUSEBUTTONDOWN) {
		SDL_MouseButtonEvent *me = (SDL_MouseButtonEvent *)e;
		if (me->button == SDL_BUTTON_RIGHT) {
			ShowMenu();
			[NSMenu popUpContextMenu:popupMenu withEvent:nil forView:nil];
		}
	} else
#endif
#if 0
	if (e->type == SDL_MOUSEMOTION) {
		SDL_MouseMotionEvent *me = (SDL_MouseMotionEvent *)e;
		if (IsFullScreen()) {
			if(me->y == 0) {
				CGReleaseAllDisplays();
				NSWindow *w = get_main_window();
				if (w) {
					[w setLevel:0];
				}
			}
			else if(me->y > 20) {
				CGCaptureAllDisplays();
				NSWindow *w = get_main_window();
				if (w) {
					long lv = CGShieldingWindowLevel();
					[w setLevel:lv];
				}
			}
			return 0;
		}
	}
#endif
	return 1;
}

void GUI::ScreenModeChanged(bool fullscreen)
{
}

void GUI::SetFocusToMainWindow()
{
#ifndef USE_SDL2
	NSWindow *win = get_main_window();
	if (win) {
		[win makeKeyAndOrderFront:nil];
	}
#else
	SDL_RaiseWindow(emu->get_window());
#endif
}

/* Create menu */
void GUI::setup_menu(void)
{
	int i;
	int drv;
	char name[128];

	// control menu

    CocoaMenu *controlMenu = [CocoaMenu create_menu:_("Control")];

	[controlMenu add_menu_item:_("PowerOn"):recv:@selector(Reset:):0:0:NSF3FunctionKey];
#ifdef _BML3MK5
	[controlMenu add_menu_item:_("Mode Switch"):recv:@selector(Dipswitch:):0:2:'m'];
#endif
	[controlMenu add_menu_item:_("Reset Switch"):recv:@selector(WarmReset:):0:0:'r'];

	[controlMenu addItem:[NSMenuItem separatorItem]];

#ifdef _MBS1
	CocoaMenu *systemMenu = [CocoaMenu create_menu:_("System Mode")];
	[systemMenu add_menu_item:_("A Mode (S1)"):recv:@selector(ChangeSystemMode:):0:1:'m'];
	[systemMenu add_menu_item:_("B Mode (L3)"):recv:@selector(ChangeSystemMode:):0:0:'m'];
	[controlMenu add_sub_menu:systemMenu:_("System Mode")];

	[controlMenu addItem:[NSMenuItem separatorItem]];
#endif

	CocoaMenu *fddTypeMenu = [CocoaMenu create_menu:_("FDD Type")];
	[fddTypeMenu add_menu_item:_("No FDD"):recv:@selector(ChangeFddType:):0:0:'f'];
	[fddTypeMenu add_menu_item:_("3inch compact FDD"):recv:@selector(ChangeFddType:):0:1:'f'];
#ifdef _BML3MK5
	[fddTypeMenu add_menu_item:_("5.25inch mini FDD"):recv:@selector(ChangeFddType:):0:2:'f'];
	[fddTypeMenu add_menu_item:_("8inch standard FDD"):recv:@selector(ChangeFddType:):0:3:'f'];
#endif
#ifdef _MBS1
	[fddTypeMenu add_menu_item:_("5.25inch mini FDD (2D Type)"):recv:@selector(ChangeFddType:):0:2:'f'];
	[fddTypeMenu add_menu_item:_("5.25inch mini FDD (2HD Type)"):recv:@selector(ChangeFddType:):0:3:'f'];
#endif
	[controlMenu add_sub_menu:fddTypeMenu:_("FDD Type")];

	[controlMenu addItem:[NSMenuItem separatorItem]];

	[controlMenu add_menu_item:_("Pause"):recv:@selector(TogglePause:):0:0:'q'];

	[controlMenu addItem:[NSMenuItem separatorItem]];

	CocoaMenu *cpuSpeedMenu = [CocoaMenu create_menu:_("CPU Speed")];
	[cpuSpeedMenu add_menu_item:_("CPU x0.5"):recv:@selector(CPUPower:):0:0:'9'];
	[cpuSpeedMenu add_menu_item:_("CPU x1"):recv:@selector(CPUPower:):0:1:'1'];
	[cpuSpeedMenu add_menu_item:_("CPU x2"):recv:@selector(CPUPower:):0:2:'2'];
	[cpuSpeedMenu add_menu_item:_("CPU x4"):recv:@selector(CPUPower:):0:3:'3'];
	[cpuSpeedMenu add_menu_item:_("CPU x8"):recv:@selector(CPUPower:):0:4:'4' ];
	[cpuSpeedMenu add_menu_item:_("CPU x16"):recv:@selector(CPUPower:):0:5:'5'];
	[cpuSpeedMenu addItem:[NSMenuItem separatorItem]];
	[cpuSpeedMenu add_menu_item:_("Sync With CPU Speed"):recv:@selector(ToggleSyncIRQ:):0:0:'0'];
	[controlMenu add_sub_menu:cpuSpeedMenu:_("CPU Speed")];

	[controlMenu addItem:[NSMenuItem separatorItem]];

#ifdef _MBS1
//	[controlMenu add_menu_item:_("Memory No Wait"):recv:@selector(ToggleMemNoWait:):0:0:'0'];

//	[controlMenu addItem:[NSMenuItem separatorItem]];
#endif

	CocoaMenu *autoKeyMenu = [CocoaMenu create_menu:_("Auto Key")];
	[autoKeyMenu add_menu_item:_("Open..."):recv:@selector(ShowOpenAutoKeyDialog:):0:0:0];
	[autoKeyMenu add_menu_item:_("Paste"):recv:@selector(StartAutoKey:):0:0:0];
	[autoKeyMenu add_menu_item:_("Stop"):recv:@selector(StopAutoKey:):0:0:0];
	[controlMenu add_sub_menu:autoKeyMenu:_("Auto Key")];

	[controlMenu addItem:[NSMenuItem separatorItem]];

	CocoaMenu *recKeyMenu = [CocoaMenu create_menu:_("Record Key")];
	[recKeyMenu add_menu_item:_("Play..."):recv:@selector(ShowPlayRecKeyDialog:):0:0:'e'];
	[recKeyMenu add_menu_item:_("Stop Playing"):recv:@selector(StopPlayRecKey:):0:0:0];
	[recKeyMenu addItem:[NSMenuItem separatorItem]];
	[recKeyMenu add_menu_item:_("Record..."):recv:@selector(ShowRecordStateAndRecKeyDialog:):0:1:0];
	[recKeyMenu add_menu_item:_("Stop Recording"):recv:@selector(StopRecordRecKey:):0:0:0];
	[controlMenu add_sub_menu:recKeyMenu:_("Record Key")];

	[controlMenu addItem:[NSMenuItem separatorItem]];

	[controlMenu add_menu_item:_("Load State..."):recv:@selector(ShowLoadStateDialog:):0:0:'o'];
	[controlMenu add_menu_item:_("Save State..."):recv:@selector(ShowSaveStateDialog:):0:0:0];

	[controlMenu addItem:[NSMenuItem separatorItem]];

#ifdef USE_DELEGATE
	CocoaMenu *stateRecentMenu = [CocoaMenu create_menu:_("Recent State Files"):[[CocoaRecentStateList alloc] initWithTarget:recv]];
	[controlMenu add_sub_menu:stateRecentMenu:_("Recent State Files")];
#else
	CocoaMenu *stateRecentMenu = [CocoaMenu create_menu:_("Recent State Files")];
	[controlMenu add_sub_menu:stateRecentMenu:_("Recent State Files"):recv:@selector(UpdateRecentStateList:):0:0];
#endif

	/* Put menu into the menubar */
	add_main_menu(controlMenu,_("Control"));

	// tape menu

	CocoaMenu *tapeMenu = [CocoaMenu create_menu:_("Tape")];

	[tapeMenu add_menu_item:_("Play..."):recv:@selector(ShowLoadDataRecDialog:):0:0:NSF7FunctionKey];
	[tapeMenu add_menu_item:_("Rec..."):recv:@selector(ShowSaveDataRecDialog:):0:0:NSF6FunctionKey];
	[tapeMenu add_menu_item:_("Rewind"):recv:@selector(RewindDataRec:):0:0:NSF5FunctionKey];
	[tapeMenu add_menu_item:_("F.F."):recv:@selector(FastForwardDataRec:):0:0:NSF8FunctionKey];
	[tapeMenu add_menu_item:_("Eject"):recv:@selector(CloseDataRec:):0:0:0];

	[tapeMenu addItem:[NSMenuItem separatorItem]];

	[tapeMenu add_menu_item:_("Real Mode"):recv:@selector(ToggleRealModeDataRec:):0:0:0];

	[tapeMenu addItem:[NSMenuItem separatorItem]];

#ifdef USE_DELEGATE
	CocoaMenu *tapeRecentMenu = [CocoaMenu create_menu:_("Recent Files"):[[CocoaRecentDataRecList alloc] initWithTarget:recv]];
	[tapeMenu add_sub_menu:tapeRecentMenu:_("Recent Files")];
#else
	CocoaMenu *tapeRecentMenu = [CocoaMenu create_menu:_("Recent Files")];
	[tapeMenu add_sub_menu:tapeRecentMenu:_("Recent Files"):recv:@selector(UpdateRecentDataRecList:):0:0];
#endif
	
	/* Put menu into the menubar */
	add_main_menu(tapeMenu,_("Tape"));
	
	// fdd menu

	for(drv = 0; drv < USE_DRIVE; drv++) {
		sprintf(name, _("FDD%d"), drv);

		CocoaMenu *fddMenu = [CocoaMenu create_menu:name];
		
		[fddMenu add_menu_item:_("Open..."):recv:@selector(ShowOpenFloppyDiskDialog:):drv:0:(NSF9FunctionKey + drv)];
		[fddMenu add_menu_item:_("Change Side to A"):recv:@selector(ChangeSideFloppyDisk:):drv:0:0];
		[fddMenu add_menu_item:_("Close"):recv:@selector(CloseFloppy:):drv:0:0];
		
		[fddMenu addItem:[NSMenuItem separatorItem]];
		
		[fddMenu add_menu_item:_("Write Protect"):recv:@selector(ToggleWriteProtectFloppyDisk:):drv:0:0];

		[fddMenu addItem:[NSMenuItem separatorItem]];
		
#ifdef USE_DELEGATE
		CocoaMenu *fddVolumeMenu = [CocoaMenu create_menu:_("Multi Volume"):[[CocoaVolumeFloppyDiskList alloc] initWithTarget:recv:drv]];
		[fddMenu add_sub_menu:fddVolumeMenu:_("Multi Volume")];
#else
		CocoaMenu *fddVolumeMenu = [CocoaMenu create_menu:_("Multi Volume")];
		[fddMenu add_sub_menu:fddVolumeMenu:_("Multi Volume"):recv:@selector(UpdateVolumeFloppyList:):drv:0];
#endif
		[fddMenu addItem:[NSMenuItem separatorItem]];
		
#ifdef USE_DELEGATE
		CocoaMenu *fddRecentMenu = [CocoaMenu create_menu:_("Recent Files"):[[CocoaRecentFloppyDiskList alloc] initWithTarget:recv:drv]];
		[fddMenu add_sub_menu:fddRecentMenu:_("Recent Files")];
#else
		CocoaMenu *fddRecentMenu = [CocoaMenu create_menu:_("Recent Files")];
		[fddMenu add_sub_menu:fddRecentMenu:_("Recent Files"):recv:@selector(UpdateRecentFloppyList:):drv:0];
#endif
		/* Put menu into the menubar */
		add_main_menu(fddMenu,name);
	}
	
	// screen menu

	CocoaMenu *screenMenu = [CocoaMenu create_menu:_("Screen")];
	
	CocoaMenu *frameRateMenu = [CocoaMenu create_menu:_("Frame Rate")];
	[frameRateMenu add_menu_item:_("Auto"):recv:@selector(ChangeFrameRate:):0:-1:0];
	[frameRateMenu add_menu_item:_("60fps"):recv:@selector(ChangeFrameRate:):0:0:0];
	[frameRateMenu add_menu_item:_("30fps"):recv:@selector(ChangeFrameRate:):0:1:0];
	[frameRateMenu add_menu_item:_("20fps"):recv:@selector(ChangeFrameRate:):0:2:0];
	[frameRateMenu add_menu_item:_("15fps"):recv:@selector(ChangeFrameRate:):0:3:0];
	[frameRateMenu add_menu_item:_("12fps"):recv:@selector(ChangeFrameRate:):0:4:0];
	[frameRateMenu add_menu_item:_("10fps"):recv:@selector(ChangeFrameRate:):0:5:0];
	[screenMenu add_sub_menu:frameRateMenu:_("Frame Rate")];
	
	[screenMenu addItem:[NSMenuItem separatorItem]];
	
	CocoaMenu *recScreenMenu = [CocoaMenu create_menu:_("Record Screen")];
	for(i = 0; i < 2; i++) {
		GetRecordVideoSizeStr(i, name);
		[recScreenMenu add_menu_item:name:recv:@selector(ResizeRecordVideoSurface:):0:i:0];
	}
	[recScreenMenu addItem:[NSMenuItem separatorItem]];
	[recScreenMenu add_menu_item:_("Rec 60fps"):recv:@selector(ShowRecordVideoDialog:):0:0:0];
	[recScreenMenu add_menu_item:_("Rec 30fps"):recv:@selector(ShowRecordVideoDialog:):0:1:0];
	[recScreenMenu add_menu_item:_("Rec 20fps"):recv:@selector(ShowRecordVideoDialog:):0:2:0];
	[recScreenMenu add_menu_item:_("Rec 15fps"):recv:@selector(ShowRecordVideoDialog:):0:3:0];
	[recScreenMenu add_menu_item:_("Rec 12fps"):recv:@selector(ShowRecordVideoDialog:):0:4:0];
	[recScreenMenu add_menu_item:_("Rec 10fps"):recv:@selector(ShowRecordVideoDialog:):0:5:0];
	[recScreenMenu add_menu_item:_("Stop"):recv:@selector(StopRecordVideo:):0:0:0];
	[recScreenMenu addItem:[NSMenuItem separatorItem]];
	[recScreenMenu add_menu_item:_("Capture"):recv:@selector(CaptureScreen:):0:0:0];
	[screenMenu add_sub_menu:recScreenMenu:_("Record Screen")];
	
	[screenMenu addItem:[NSMenuItem separatorItem]];
	
	CocoaMenu *windowModeMenu = [CocoaMenu create_menu:_("Window")];
	for(i = 0; i < GetWindowModeCount(); i++) {
		GetWindowModeStr(i, name);
		[windowModeMenu add_menu_item:name:recv:@selector(ChangeWindowMode:):0:i:0];
	}
	[screenMenu add_sub_menu:windowModeMenu:_("Window")];
	
	CocoaMenu *screenModeMenu = [CocoaMenu create_menu:_("Fullscreen")];
	[screenModeMenu add_menu_item:_("Stretch Screen"):recv:@selector(ToggleStretchScreen:):0:1:'x'];
	[screenModeMenu add_menu_item:_("Cutout Screen"):recv:@selector(ToggleStretchScreen:):0:2:'x'];
	[screenModeMenu addItem:[NSMenuItem separatorItem]];
	for(i = 0; i < GetFullScreenModeCount(); i++) {
		GetFullScreenModeStr(i, name);
		[screenModeMenu add_menu_item:name:recv:@selector(ChangeFullScreenMode:):0:i:0];
	}
	[screenMenu add_sub_menu:screenModeMenu:_("Fullscreen")];

	CocoaMenu *pixelAspectMenu = [CocoaMenu create_menu:_("Aspect Ratio")];
	for(i = 0; i < GetPixelAspectModeCount(); i++) {
		GetPixelAspectModeStr(i, name);
		[pixelAspectMenu add_menu_item:name:recv:@selector(ChangePixelAspectMode:):0:i:0];
	}
	[screenMenu add_sub_menu:pixelAspectMenu:_("Aspect Ratio")];

	[screenMenu addItem:[NSMenuItem separatorItem]];

	CocoaMenu *drawingMenu = [CocoaMenu create_menu:_("Drawing Mode")];
		[drawingMenu add_menu_item:_("Full Draw"):recv:@selector(ChangeScanLine:):0:0:'s'];
		[drawingMenu add_menu_item:_("Scanline"):recv:@selector(ChangeScanLine:):0:1:'s'];
		[drawingMenu add_menu_item:_("Stripe"):recv:@selector(ChangeScanLine:):0:2:'s'];
		[drawingMenu add_menu_item:_("Checker"):recv:@selector(ChangeScanLine:):0:3:'s'];
	[screenMenu add_sub_menu:drawingMenu:_("Drawing Mode")];

	[screenMenu addItem:[NSMenuItem separatorItem]];

	[screenMenu add_menu_item:_("Afterimage1"):recv:@selector(ChangeAfterImage:):0:1:'t'];
	[screenMenu add_menu_item:_("Afterimage2"):recv:@selector(ChangeAfterImage:):0:2:'t'];

	[screenMenu addItem:[NSMenuItem separatorItem]];

#ifdef USE_KEEPIMAGE
	[screenMenu add_menu_item:_("Keepimage1"):recv:@selector(ChangeKeepImage:):0:1:0];
	[screenMenu add_menu_item:_("Keepimage2"):recv:@selector(ChangeKeepImage:):0:2:0];
#endif
#ifdef _MBS1
	[screenMenu add_menu_item:_("Digital RGB"):recv:@selector(ChangeRGBType:):0:0:0];
	[screenMenu add_menu_item:_("Analog RGB"):recv:@selector(ChangeRGBType:):0:1:0];
#endif

#ifdef USE_OPENGL
	[screenMenu addItem:[NSMenuItem separatorItem]];
	[screenMenu add_menu_item:_("Use OpenGL(Sync)"):recv:@selector(ChangeUseOpenGL:):0:1:'y'];
	[screenMenu add_menu_item:_("Use OpenGL(Async)"):recv:@selector(ChangeUseOpenGL:):0:2:'y'];
	CocoaMenu *glFilterMenu = [CocoaMenu create_menu:_("OpenGL Filter")];
		[glFilterMenu add_menu_item:_("Nearest Neighbour"):recv:@selector(ChangeOpenGLFilter:):0:0:'u'];
		[glFilterMenu add_menu_item:_("Linear"):recv:@selector(ChangeOpenGLFilter:):0:1:'u'];
	[screenMenu add_sub_menu:glFilterMenu:_("OpenGL Filter")];
#endif

	/* Put menu into the menubar */
	add_main_menu(screenMenu,_("Screen"));
	
	// Sound menu
	
	CocoaMenu *soundMenu = [CocoaMenu create_menu:_("Sound")];

	[soundMenu add_menu_item:_("Volume..."):recv:@selector(ShowVolumeDialog:):0:0:'v'];
	[soundMenu addItem:[NSMenuItem separatorItem]];
	CocoaMenu *recSoundMenu = [CocoaMenu create_menu:_("Record Sound")];
	[recSoundMenu add_menu_item:_("Start Recording"):recv:@selector(ShowRecordAudioDialog:):0:0:0];
	[recSoundMenu add_menu_item:_("Stop Recording"):recv:@selector(StopRecordSound:):0:0:0];
	[soundMenu add_sub_menu:recSoundMenu:_("Record Sound")];
	[soundMenu addItem:[NSMenuItem separatorItem]];
	[soundMenu add_menu_item:_("8000Hz"):recv:@selector(ChangeSoundFrequency:):0:2:0];
	[soundMenu add_menu_item:_("11025Hz"):recv:@selector(ChangeSoundFrequency:):0:3:0];
	[soundMenu add_menu_item:_("22050Hz"):recv:@selector(ChangeSoundFrequency:):0:4:0];
	[soundMenu add_menu_item:_("44100Hz"):recv:@selector(ChangeSoundFrequency:):0:5:0];
	[soundMenu add_menu_item:_("48000Hz"):recv:@selector(ChangeSoundFrequency:):0:6:0];
	[soundMenu add_menu_item:_("96000Hz"):recv:@selector(ChangeSoundFrequency:):0:7:0];
	[soundMenu addItem:[NSMenuItem separatorItem]];
	[soundMenu add_menu_item:_("50msec"):recv:@selector(ChangeSoundLatency:):0:0:0];
	[soundMenu add_menu_item:_("100msec"):recv:@selector(ChangeSoundLatency:):0:1:0];
	[soundMenu add_menu_item:_("200msec"):recv:@selector(ChangeSoundLatency:):0:2:0];
	[soundMenu add_menu_item:_("300msec"):recv:@selector(ChangeSoundLatency:):0:3:0];
	[soundMenu add_menu_item:_("400msec"):recv:@selector(ChangeSoundLatency:):0:4:0];

	/* Put menu into the menubar */
	add_main_menu(soundMenu,_("Sound"));
	
	// Devices menu
	
	CocoaMenu *deviceMenu = [CocoaMenu create_menu:_("Devices")];
	for(drv = 0; drv < MAX_PRINTER; drv++) {
		sprintf(name, _("LPT%d"), drv);
		CocoaMenu *lptMenu = [CocoaMenu create_menu:name];
		[lptMenu add_menu_item:_("Save..."):recv:@selector(ShowSavePrinterDialog:):drv:0:0];
		[lptMenu add_menu_item:_("Print to mpprinter"):recv:@selector(PrintPrinter:):drv:0:0];
		[lptMenu add_menu_item:_("Clear"):recv:@selector(ClearPrinterBuffer:):drv:0:0];
		[lptMenu addItem:[NSMenuItem separatorItem]];
		[lptMenu add_menu_item:_("Direct Send to mpprinter"):recv:@selector(EnablePrinterDirect:):drv:0:0];
		[deviceMenu add_sub_menu:lptMenu:name];
	}
	[deviceMenu addItem:[NSMenuItem separatorItem]];
	for(drv = 0; drv < MAX_COMM; drv++) {
		sprintf(name, _("COM%d"), drv);
		CocoaMenu *comMenu = [CocoaMenu create_menu:name];
		[comMenu add_menu_item:_("Enable Server"):recv:@selector(EnableCommServer:):drv:0:0];
		[comMenu add_menu_item:_("Connect"):recv:@selector(ConnectComm:):drv:0:0];
		[comMenu addItem:[NSMenuItem separatorItem]];
		[comMenu add_menu_item:_("Comm With Byte Data"):recv:@selector(CommThroughMode:):drv:0:0];
		[comMenu addItem:[NSMenuItem separatorItem]];
		CocoaMenu *telnetMenu = [CocoaMenu create_menu:_("Options For Telnet")];
			[telnetMenu add_menu_item:_("Binary Mode"):recv:@selector(CommBinaryMode:):drv:0:0];
			[telnetMenu addItem:[NSMenuItem separatorItem]];
			[telnetMenu add_menu_item:_("Send WILL ECHO"):recv:@selector(SendCommTelnetCommand:):drv:1:0];
		[comMenu add_sub_menu:telnetMenu:_("Options For Telnet")];
		[deviceMenu add_sub_menu:comMenu:name];
	}
	/* Put menu into the menubar */
	add_main_menu(deviceMenu,_("Devices"));

	// Options menu
	
	CocoaMenu *optionMenu = [CocoaMenu create_menu:_("Options")];
	
	[optionMenu add_menu_item:_("Show LED"):recv:@selector(ToggleLedBox:):0:1:'l'];
	[optionMenu add_menu_item:_("Inside LED"):recv:@selector(ToggleLedBox:):0:2:'l'];
	[optionMenu add_menu_item:_("Show Message"):recv:@selector(ToggleMessageBoard:):0:0:'z'];
	[optionMenu addItem:[NSMenuItem separatorItem]];
	[optionMenu add_menu_item:_("Use Joypad"):recv:@selector(ChangeUseJoypad:):0:1:'j'];
#ifdef USE_PIAJOYSTICK
	[optionMenu add_menu_item:_("Use Joypad (PIA Type)"):recv:@selector(ChangeUseJoypad:):0:2:'j'];
#endif
#ifdef USE_LIGHTPEN
	[optionMenu add_menu_item:_("Enable Lightpen"):recv:@selector(ToggleEnableLightpen:):0:0:0];
#endif
#ifdef USE_MOUSE
	[optionMenu add_menu_item:_("Use Mouse"):recv:@selector(ToggleUseMouse:):0:0:0];
#endif
	[optionMenu add_menu_item:_("Keybind..."):recv:@selector(ShowKeybindDialog:):0:0:'k'];
	[optionMenu add_menu_item:_("Virtual Keyboard"):recv:@selector(ShowVirtualKeyboard:):0:0:0];
#ifdef USE_DEBUGGER
	[optionMenu addItem:[NSMenuItem separatorItem]];
	[optionMenu add_menu_item:_("Start Debugger"):recv:@selector(OpenDebugger:):0:0:'d'];
	[optionMenu add_menu_item:_("Stop Debugger"):recv:@selector(CloseDebugger:):0:0:0];
#endif
	[optionMenu addItem:[NSMenuItem separatorItem]];
	[optionMenu add_menu_item:_("Configure..."):recv:@selector(ShowConfigureDialog:):0:0:'c'];
	
	/* Put menu into the menubar */
	add_main_menu(optionMenu,_("Options"));
	
	/* create popup menu */
//	popupMenu = [CocoaMenu create_menu:_("AAA")];
//	[popupMenu add_menu_item:_("BBB"):nil:nil:0:0:0];
}

bool GUI::NeedUpdateScreen()
{
//	emu->out_logf(LOG_DEBUG, "GUI::NeedUpdateScreen: %d", need_update_screen);
	if (need_update_screen > 0) {
		need_update_screen = 0;
		// post message to main thread
		[recv performSelectorOnMainThread:@selector(PerformUpdateScreen) withObject:nil waitUntilDone:FALSE];
		// send signal to main loop
		need_update = true;
		SDL_CondSignal(cond_allow_update_screen);
		return true;
	} else {
		return false;
	}
}
void GUI::DecreaseUpdateScreenCount(void)
{
	// nothing to do
}

bool GUI::ShowLoadDataRecDialog(void)
{
	NSInteger	result;
	NSOpenPanel *panel;
	
	PostEtSystemPause(true);
	GoWindowMode();

	panel = [NSOpenPanel openPanel];
	// title
	[panel setTitle:[NSString stringWithUTF8String:_("Play Data Recorder Tape")]];
	// filtering file types
	NSArray *fileTypes = [NSArray arrayWithObjects:@"l3",@"l3b",@"l3c",@"wav",@"t9x",nil];
	[panel setAllowedFileTypes:fileTypes];
	[panel setAllowsOtherFileTypes:TRUE];
	// set current folder
	[panel setDirectoryURL:[NSURL fileURLWithPath:[NSString stringWithUTF8String:config.initial_datarec_path]]];
	
	// Display modal dialog
	result = [panel runModal];
	
	if(result == NSOKButton) {
		// get file path (use NSURL)
		NSURL *filePath = [panel URL];
		
		PostEtLoadDataRecMessage([[filePath path] UTF8String]);
	} else {
		PostEtSystemPause(false);
	}
	SetFocusToMainWindow();
	return (result == NSOKButton);
}

bool GUI::ShowSaveDataRecDialog(void)
{
	NSInteger	result;
	NSSavePanel *panel;
	
	PostEtSystemPause(true);
	GoWindowMode();

	panel = [NSSavePanel savePanel];
	// title
	[panel setTitle:[NSString stringWithUTF8String:_("Record Data Recorder Tape")]];
	// filtering file types
	NSArray *fileTypes = [NSArray arrayWithObjects:@"l3",@"l3b",@"l3c",@"wav",@"t9x",nil];
	[panel setAllowedFileTypes:fileTypes];
	[panel setAllowsOtherFileTypes:TRUE];
	// set current folder
	[panel setDirectoryURL:[NSURL fileURLWithPath:[NSString stringWithUTF8String:config.initial_datarec_path]]];
	
	// Display modal dialog
	result = [panel runModal];
	
	if(result == NSOKButton) {
		// get file path (use NSURL)
		NSURL *filePath = [panel URL];
		
		PostEtSaveDataRecMessage([[filePath path] UTF8String]);
	} else {
		PostEtSystemPause(false);
	}
	SetFocusToMainWindow();
	return (result == NSOKButton);
}

bool GUI::ShowOpenFloppyDiskDialog(int drv)
{
	NSInteger	result;
	NSOpenPanel *panel;
	
	PostEtSystemPause(true);
	GoWindowMode();

	panel = [NSOpenPanel openPanel];
	char title[128];
	sprintf(title,_("Open Floppy Disk %d"),drv);
	// title
	[panel setTitle:[NSString stringWithUTF8String:title]];
	// filtering file types
	NSArray *fileTypes = [NSArray arrayWithObjects:@"d88",nil];
	[panel setAllowedFileTypes:fileTypes];
	[panel setAllowsOtherFileTypes:TRUE];
	// set current folder
	[panel setDirectoryURL:[NSURL fileURLWithPath:[NSString stringWithUTF8String:config.initial_disk_path]]];
	
	// Display modal dialog
	result = [panel runModal];
	
	if(result == NSOKButton) {
		// get file path (use NSURL)
		NSURL *filePath = [panel URL];
		
		PostEtOpenFloppyMessage(drv, [[filePath path] UTF8String], 0, 0, true);
	} else {
		PostEtSystemPause(false);
	}
	SetFocusToMainWindow();
	return (result == NSOKButton);
}

int GUI::ShowSelectFloppyDriveDialog(int drv)
{
	CocoaSelDrvPanel *panel;
	
//	PostEtSystemPause(true);
	GoWindowMode();

	panel = [[CocoaSelDrvPanel alloc] initWithPrefix:drv prefix:_("FDD")];
	
	// Display modal dialog
	[panel runModal];
	int new_drv = [panel selectedDrive];
	
	[panel release];
	
	PostEtSystemPause(false);
	SetFocusToMainWindow();
	return new_drv;
}

bool GUI::ShowLoadStateDialog(void)
{
	NSInteger	result;
	NSOpenPanel *panel;
	
	PostEtSystemPause(true);
	GoWindowMode();

	panel = [NSOpenPanel openPanel];
	// title
	[panel setTitle:[NSString stringWithUTF8String:_("Load Status Data")]];
	// filtering file types
	NSArray *fileTypes = [NSArray arrayWithObjects:@"l3r",nil];
	[panel setAllowedFileTypes:fileTypes];
	
	// set current folder
	[panel setDirectoryURL:[NSURL fileURLWithPath:[NSString stringWithUTF8String:config.initial_state_path]]];
	
	// Display modal dialog
	result = [panel runModal];
	
	if(result == NSOKButton) {
		// get file path (use NSURL)
		NSURL *filePath = [panel URL];
		
		PostEtLoadStatusMessage([[filePath path] UTF8String]);
	} else {
		PostEtSystemPause(false);
	}
	SetFocusToMainWindow();
	return (result == NSOKButton);
}

bool GUI::ShowSaveStateDialog(bool cont)
{
	NSInteger	result;
	NSSavePanel *panel;
	
	PostEtSystemPause(true);
	GoWindowMode();

	panel = [NSSavePanel savePanel];
	// title
	[panel setTitle:[NSString stringWithUTF8String:_("Save Status Data")]];
	// filtering file types
	NSArray *fileTypes = [NSArray arrayWithObjects:@"l3r",nil];
	[panel setAllowedFileTypes:fileTypes];
	
	// set current folder
	[panel setDirectoryURL:[NSURL fileURLWithPath:[NSString stringWithUTF8String:config.initial_state_path]]];
	
	// Display modal dialog
	result = [panel runModal];
	
	if(result == NSOKButton) {
		// get file path (use NSURL)
		NSURL *filePath = [panel URL];
		
		PostEtSaveStatusMessage([[filePath path] UTF8String], cont);
	} else {
		PostEtSystemPause(false);
	}
	SetFocusToMainWindow();
	return (result == NSOKButton);
}

bool GUI::ShowOpenAutoKeyDialog(void)
{
	NSInteger	result;
	NSOpenPanel *panel;
	
	PostEtSystemPause(true);
	GoWindowMode();

	panel = [NSOpenPanel openPanel];
	// title
	[panel setTitle:[NSString stringWithUTF8String:_("Open Text File")]];
	// filtering file types
	NSArray *fileTypes = [NSArray arrayWithObjects:@"txt",@"bas",@"lpt",nil];
	[panel setAllowedFileTypes:fileTypes];
	
	// set current folder
	[panel setDirectoryURL:[NSURL fileURLWithPath:[NSString stringWithUTF8String:config.initial_autokey_path]]];
	
	// Display modal dialog
	result = [panel runModal];
	
	if(result == NSOKButton) {
		// get file path (use NSURL)
		NSURL *filePath = [panel URL];
		
		PostEtLoadAutoKeyMessage([[filePath path] UTF8String]);
	} else {
		PostEtSystemPause(false);
	}
	SetFocusToMainWindow();
	return (result == NSOKButton);
}

bool GUI::ShowPlayRecKeyDialog(void)
{
	NSInteger	result;
	NSOpenPanel *panel;
	
	PostEtSystemPause(true);
	GoWindowMode();

	panel = [NSOpenPanel openPanel];
	// title
	[panel setTitle:[NSString stringWithUTF8String:_("Play Recorded Keys")]];
	// filtering file types
	NSArray *fileTypes = [NSArray arrayWithObjects:@"l3k",nil];
	[panel setAllowedFileTypes:fileTypes];
	// set current folder
	[panel setDirectoryURL:[NSURL fileURLWithPath:[NSString stringWithUTF8String:config.initial_state_path]]];
	
	// Display modal dialog
	result = [panel runModal];
	
	if(result == NSOKButton) {
		// get file path (use NSURL)
		NSURL *filePath = [panel URL];
		
		PostEtLoadRecKeyMessage([[filePath path] UTF8String]);
	} else {
		PostEtSystemPause(false);
	}
	SetFocusToMainWindow();
	return (result == NSOKButton);
}

bool GUI::ShowRecordRecKeyDialog(void)
{
	NSInteger	result;
	NSSavePanel *panel;
	
	PostEtSystemPause(true);
	GoWindowMode();
	
	panel = [NSSavePanel savePanel];
	// title
	[panel setTitle:[NSString stringWithUTF8String:_("Record Input Keys")]];
	// filtering file types
	NSArray *fileTypes = [NSArray arrayWithObjects:@"l3k",nil];
	[panel setAllowedFileTypes:fileTypes];
	// set current folder
	[panel setDirectoryURL:[NSURL fileURLWithPath:[NSString stringWithUTF8String:config.initial_state_path]]];
	
	// Display modal dialog
	result = [panel runModal];
	
	if(result == NSOKButton) {
		// get file path (use NSURL)
		NSURL *filePath = [panel URL];
		
		PostEtSaveRecKeyMessage([[filePath path] UTF8String], false);
	} else {
		PostEtSystemPause(false);
	}
	SetFocusToMainWindow();
	return (result == NSOKButton);
}

bool GUI::ShowSavePrinterDialog(int drv)
{
	NSInteger	result;
	NSSavePanel *panel;
	
	PostEtSystemPause(true);
	GoWindowMode();

	panel = [NSSavePanel savePanel];
	// title
	[panel setTitle:[NSString stringWithUTF8String:_("Save Printing Data")]];
	// filtering file types
	NSArray *fileTypes = [NSArray arrayWithObjects:@"lpt",@"txt",@"bas",nil];
	[panel setAllowedFileTypes:fileTypes];
	[panel setAllowsOtherFileTypes:TRUE];
	// set current folder
	[panel setDirectoryURL:[NSURL fileURLWithPath:[NSString stringWithUTF8String:config.initial_printer_path]]];
	
	// Display modal dialog
	result = [panel runModal];
	
	if(result == NSOKButton) {
		// get file path (use NSURL)
		NSURL *filePath = [panel URL];
		
		PostEtSavePrinterMessage(drv, [[filePath path] UTF8String]);
	} else {
		PostEtSystemPause(false);
	}
	SetFocusToMainWindow();
	return (result == NSOKButton);
}

bool GUI::ShowRecordVideoDialog(int fps_num)
{
	NSInteger result;
	CocoaRecVideoPanel *panel;
	
	PostEtSystemPause(true);
	GoWindowMode();

	panel = [[CocoaRecVideoPanel alloc] initWithCont:false];
	
	// Display modal dialog
	result = [panel runModal];
	
	[panel release];
	
	if (result == NSOKButton) {
		PostEtStartRecordVideo(fps_num);
	} else {
		PostEtSystemPause(false);
	}
	SetFocusToMainWindow();
	return (result == NSOKButton);
}

bool GUI::ShowRecordAudioDialog(void)
{
	NSInteger result;
	CocoaRecAudioPanel *panel;
	
	PostEtSystemPause(true);
	GoWindowMode();

	panel = [[CocoaRecAudioPanel alloc] init];
	
	// Display modal dialog
	result = [panel runModal];
	
	[panel release];
	
	if (result == NSOKButton) {
		PostEtStartRecordSound();
	} else {
		PostEtSystemPause(false);
	}
	SetFocusToMainWindow();
	return (result == NSOKButton);
}

bool GUI::ShowRecordVideoAndAudioDialog(int fps_num)
{
	NSInteger result;
	
	PostEtSystemPause(true);
	GoWindowMode();

	CocoaRecVideoPanel *panelv;
	panelv = [[CocoaRecVideoPanel alloc] initWithCont:true];
	
	// Display modal dialog
	result = [panelv runModal];
	
	[panelv release];
	
	if (result != NSOKButton) {
		PostEtSystemPause(false);
		return false;
	}

	CocoaRecAudioPanel *panela;
	panela = [[CocoaRecAudioPanel alloc] init];
	
	// Display modal dialog
	result = [panela runModal];

	[panela release];

	if (result == NSOKButton) {
		// start video and audio
		PostEtStartRecordVideo(fps_num);
	} else {
		PostEtSystemPause(false);
	}
	SetFocusToMainWindow();
	return (result == NSOKButton);
}

bool GUI::ShowVolumeDialog(void)
{
	CocoaVolumePanel *panel;

	GoWindowMode();
	
	panel = [[CocoaVolumePanel alloc] init];
	[panel runModal];
	//	[panel orderFront:self];
	[panel release];
	SetFocusToMainWindow();
	
	return true;
}

bool GUI::ShowKeybindDialog(void)
{
	NSInteger	result;
	CocoaKeybindPanel *panel;
	
	PostEtSystemPause(true);
	GoWindowMode();

	panel = [[CocoaKeybindPanel alloc] init];
	
	// Display modal dialog
	result = [panel runModal];
	
	[panel release];
	PostEtSystemPause(false);
	SetFocusToMainWindow();

	return (result == NSOKButton);
}

bool GUI::ShowConfigureDialog(void)
{
	NSInteger	result;
	CocoaConfigPanel *panel;
	
	PostEtSystemPause(true);
	GoWindowMode();

	panel = [[CocoaConfigPanel alloc] init];
	
	// Display modal dialog
	result = [panel runModal];
	
	[panel release];
	PostEtSystemPause(false);
	SetFocusToMainWindow();

	return (result == NSOKButton);
}

void GUI::GoWindowMode(void)
{
	if (IsFullScreen()) {
		ChangeWindowMode(-1);
	}
}

#ifdef USE_MOUSE
/// Change Mouse Cursor
void GUI::ToggleUseMouse(void)
{
	GUI_BASE::ToggleUseMouse();
	if (IsEnableMouse()) {
		CGDisplayHideCursor(kCGDirectMainDisplay);
	} else {
		CGDisplayShowCursor(kCGDirectMainDisplay);
	}
}
#endif

#endif /* GUI_TYPE_COCOA */

void remove_window_menu(void)
{
    NSMenu *mainMenu;
    NSInteger i;
    
    mainMenu = [NSApp mainMenu];
	
    i = [mainMenu indexOfItemWithTitle:@"Window"];
    if (i >= 0) [mainMenu removeItemAtIndex:i];
    i = [mainMenu indexOfItemWithTitle:@"View"];
    if (i >= 0) [mainMenu removeItemAtIndex:i];
}

#define APPLE_MENU_STRING _TX("About bml3mk5"),_TX("Hide bml3mk5"),_TX("Hide Others"),_TX("Show All"),_TX("Quit bml3mk5"),_TX("Services"),_TX("Preferences…")

void translate_apple_menu(void)
{
	NSMenu *appleMenu;
	NSArray *items;
    NSInteger i;
    NSInteger count;
	
    appleMenu = [[[NSApp mainMenu] itemAtIndex:0] submenu];
	items = [appleMenu itemArray];
	count = [items count];
	for(i=0; i<count; i++) {
		NSMenuItem *item = [items objectAtIndex:(NSUInteger)i];
		if ([item isSeparatorItem]) continue;
		if ([[item keyEquivalent] isEqual: @"q"]) {
			const unichar c[2] = { NSF4FunctionKey, 0 };
			[item setKeyEquivalent:[NSString stringWithCharacters:c length:1]];
			[item setKeyEquivalentModifierMask:NSAlternateKeyMask];
		} else {
			[item setKeyEquivalent:@""];
		}
		[item setTitle:[NSString stringWithUTF8String:_tgettext([[item title] UTF8String])]];
	}
}

@implementation CocoaWindowDelegate
@synthesize parentObject;
@synthesize guiObject;
+ (id)allocWithParent:(id)new_obj gui:(GUI_BASE *)new_gui
{
	id me = [[CocoaWindowDelegate alloc] init];
	[me setParentObject:new_obj];
	[me setGuiObject:new_gui];
	return me;
}
- (BOOL)windowShouldClose:(id)sender
{
	return parentObject != nil ? [parentObject windowShouldClose:sender] : NO;
}
- (void)windowDidBecomeKey:(NSNotification *)aNotification
{
	if (parentObject != nil) [parentObject windowDidBecomeKey:aNotification];
}
- (void)windowDidResignKey:(NSNotification *)aNotification
{
	if (parentObject != nil) [parentObject windowDidResignKey:aNotification];
}
-(NSDragOperation)draggingEntered:(id<NSDraggingInfo>)sender
{
    return NSDragOperationCopy;
}
-(NSDragOperation)draggingUpdated:(id<NSDraggingInfo>)sender
{
    return NSDragOperationCopy;
}
- (void)performDragOperation:(id<NSDraggingInfo>)sender
{
	NSPasteboard *pb = [sender draggingPasteboard];
	NSArray *files = [pb propertyListForType:NSFilenamesPboardType];
	for(int i=0; i<[files count]; i++) {
		NSString *file = (NSString *)[files objectAtIndex:i];
		guiObject->OpenFileByExtention([file UTF8String]);
	}
}
- (void)windowDidMiniaturize:(NSNotification *)aNotification
{
#ifdef USE_SDL2
	if (parentObject != nil) [parentObject windowDidMiniaturize:aNotification];
#endif
	// another window also miniaturize
	NSWindow *me = [aNotification object];
	NSArray *windows;
	windows = [NSApp windows];
	for(int i=0; i<[windows count]; i++) {
		NSWindow *window = (NSWindow *)[windows objectAtIndex:i];
		if (window != me && [window isVisible]) [window miniaturize:self];
	}
}
- (void)windowDidDeminiaturize:(NSNotification *)aNotification
{
#ifdef USE_SDL2
	if (parentObject != nil) [parentObject windowDidDeminiaturize:aNotification];
#endif
	// another window also de-miniaturize
	NSWindow *me = [aNotification object];
	NSArray *windows;
	windows = [NSApp windows];
	for(int i=0; i<[windows count]; i++) {
		NSWindow *window = (NSWindow *)[windows objectAtIndex:i];
		if (window != me && [window isMiniaturized]) [window deminiaturize:self];
	}
}
- (void)windowDidMove:(NSNotification *)aNotification
{
#ifdef USE_SDL2
	if (parentObject != nil) [parentObject windowDidMove:aNotification];
#else
	NSWindow *me = [aNotification object];
	NSArray *windows;
	windows = [NSApp windows];
	for(int i=0; i<[windows count]; i++) {
		NSWindow *window = (NSWindow *)[windows objectAtIndex:i];
		if (window != me && [[window className] isEqualToString:@"CocoaLedBox"]) [window performSelector:@selector(move)];
	}
#endif
}
- (void)windowDidBecomeMain:(NSNotification *)aNotification
{
	NSWindow *me = [aNotification object];
	NSArray *windows;
	windows = [NSApp windows];
	for(int i=0; i<[windows count]; i++) {
		NSWindow *window = (NSWindow *)[windows objectAtIndex:i];
		if (window != me && [window isVisible]) {
			[window setLevel:NSFloatingWindowLevel];
		}
	}
}
- (void)windowDidResignMain:(NSNotification *)aNotification
{
	NSWindow *me = [aNotification object];
	NSArray *windows;
	windows = [NSApp windows];
	for(int i=0; i<[windows count]; i++) {
		NSWindow *window = (NSWindow *)[windows objectAtIndex:i];
		if (window != me && [window isVisible]) {
			[window setLevel:NSNormalWindowLevel];
			[window orderWindow:NSWindowAbove relativeTo:[me windowNumber]];
		}
	}
}
#ifdef USE_SDL2
-(void) windowDidExpose:(NSNotification *) aNotification
{
	if (parentObject != nil) [parentObject windowDidExpose:aNotification];
}
-(void) windowDidResize:(NSNotification *) aNotification
{
	if (parentObject != nil) [parentObject windowDidResize:aNotification];
}
-(void) windowWillEnterFullScreen:(NSNotification *) aNotification
{
	if (parentObject != nil) [parentObject windowWillEnterFullScreen:aNotification];
}
-(void) windowDidEnterFullScreen:(NSNotification *) aNotification
{
	if (parentObject != nil) [parentObject windowDidEnterFullScreen:aNotification];
}
-(void) windowWillExitFullScreen:(NSNotification *) aNotification
{
	if (parentObject != nil) [parentObject windowWillExitFullScreen:aNotification];
}
-(void) windowDidExitFullScreen:(NSNotification *) aNotification
{
	if (parentObject != nil) [parentObject windowDidExitFullScreen:aNotification];
}
-(NSApplicationPresentationOptions)window:(NSWindow *)window willUseFullScreenPresentationOptions:(NSApplicationPresentationOptions)proposedOptions
{
	return parentObject != nil ? [parentObject window:window willUseFullScreenPresentationOptions:proposedOptions] : 0;
}
#endif
@end

void set_delegate_to_sdl_window(GUI_BASE *new_gui)
{
	NSWindow *sdl_window = NULL;
	NSArray *windows = [NSApp windows];
	for(int i=0; i<[windows count]; i++) {
		NSWindow *window = (NSWindow *)[windows objectAtIndex:i];
		NSString *name = [window className];
emu->out_debugf("COCOA window %d: %s %llx", i, [name UTF8String], window);
		NSRange ra = [name rangeOfString:@"SDL"];
		if (ra.length > 0) {
			NSObject *delegate = [window delegate];
emu->out_debugf("COCOA delegete: %s", [[delegate className] UTF8String]);
			// set delegate wrapper (if fullscreen, delegate is nil)
			if (delegate && ![[delegate className] isEqualToString:@"CocoaWindowDelegate"]) {
				[window setDelegate:[CocoaWindowDelegate allocWithParent:delegate gui:new_gui]];
				[window registerForDraggedTypes:[NSArray arrayWithObject:NSFilenamesPboardType]];
emu->out_debugf("COCOA delegete wrapped");
			}
			sdl_window = window;
		}
	}
#if !(defined(USE_SDL2) && defined(USE_SDL2_LEDBOX))
	// send window object to ledbox
	ledbox_set_owner_window(sdl_window);
#endif
}

#ifdef GUI_TYPE_AGAR
#include "../agar/ag_gui.h"
#endif
bool GUI::StartAutoKey(void)
{
	bool rc = false;
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	NSPasteboard *pb = [NSPasteboard generalPasteboard];
	NSString *type = [pb availableTypeFromArray:[NSArray arrayWithObject:NSStringPboardType]];
	if (type != nil) {
		NSString *str = [pb stringForType:NSStringPboardType];
		rc = emu->start_auto_key([str UTF8String]);
	}
	[pool release];
	return rc;
}

bool GUI::ShowVirtualKeyboard(void)
{
#ifdef USE_VKEYBOARD
	if (!vkeyboard) {
		vkeyboard = new Vkbd::VKeyboard();

		uint8_t *buf;
		int siz;
		emu->get_vm_key_buffer(&buf, &siz);
		vkeyboard->SetStatusBufferPtr(buf, siz);
		vkeyboard->Create(emu->resource_path());
		vkeyboard->Show();
	} else {
		vkeyboard->Close();
	}
	return true;
#else
	return false;
#endif
}
