/** @file cocoa_keybindpanel.mm
 
 HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
 Skelton for retropc emulator
 SDL edition + Cocoa GUI
 
 @author Sasaji
 @date   2015.04.30 -
 
 @brief [ keybind panel ]
 */

#import <Carbon/Carbon.h>
#import "cocoa_gui.h"
#import "cocoa_keybindpanel.h"
#import "../../config.h"
#import "../../emu.h"
#import "../../clocale.h"
#import "cocoa_key_trans.h"

extern EMU *emu;

@implementation CocoaTableData
- (id)initWithValue:(int)new_tabnum
{
	[super init];
	
	kbdata.Init(emu, new_tabnum, new_tabnum == 0 ? 0 : 1, new_tabnum == 2 ? 1 : 0);

	// get parameters
	selected.row = -1;
	selected.col = -1;
	
	return self;
}
- (int)tab_num
{
	return kbdata.tab_num;
}
- (int)selectedCol
{
	return selected.col;
}
- (void)setSelectCol:(int)col row:(int)row
{
	selected.col = col;
	selected.row = row;
}
- (Uint32)combi
{
	return kbdata.GetCombi();
}
- (void)setCombi:(Uint32)value
{
	kbdata.SetCombi(value);
}
- (NSInteger)numberOfRowsInTableView:(NSTableView *)tableView
{
	return kbdata.GetNumberOfRows();
}
- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row
{
	char label[128];
	int col = 0;
	if ([[tableColumn identifier] isEqualToString:@"1"]) {
		col++;
	}
	if([[tableColumn identifier] isEqualToString:@"vmkey"]) {
		col=-1;
	}
	kbdata.GetCellString((int)row,col,label);
	return [NSString stringWithUTF8String:label];
}
- (void)tableView:(NSTableView *)tableView setObjectValue:(id)obj forTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row
{
}

- (void)SetVmKeyMap:(Uint16 *)vmKeyMap :(int)size
{
	kbdata.SetVmKeyMap(vmKeyMap,size);
}

- (void)SetVmKey:(int)idx :(Uint16)code
{
	kbdata.SetVmKey(idx,code);
}

- (bool)SetVmKeyCode:(int)idx :(Uint16)code
{
	return kbdata.SetVmKeyCode(idx,code);
}

- (void)SetVkKeyMap:(Uint32 *)vkKeyMap
{
	kbdata.SetVkKeyMap(vkKeyMap);
}

- (void)SetVkKeyDefMap:(Uint32 *)vkKeyDefMap :(int)rows :(int)cols
{
	kbdata.SetVkKeyDefMap(vkKeyDefMap,rows,cols);
}

- (void)SetVkKeyPresetMap:(Uint32 *)vkKeyMap :(int)idx
{
	kbdata.SetVkKeyPresetMap(vkKeyMap,idx);
}

- (bool)SetVkKeyCode:(codecols_t *)obj :(Uint32)code :(char *)label
{
	return kbdata.SetVkKeyCode(obj,code,label);
}

- (bool)SetVkKeyCode:(int)row :(int)col :(Uint32)code :(char *)label
{
	return kbdata.SetVkKeyCode(row,col,code,label);
}

- (bool)SetVkKeyCode:(Uint32)code :(char *)label
{
	if (selected.row < 0 || selected.col < 0) return false;
	return kbdata.SetVkKeyCode(selected.row,selected.col-1,code,label);
}

- (bool)SetVkJoyCode:(Uint32)code :(char *)label
{
	if (selected.row < 0 || selected.col < 0) return false;
	return kbdata.SetVkJoyCode(selected.row,selected.col-1,code,label);
}

- (void)loadDefaultPreset
{
	kbdata.LoadDefaultPreset();
}

- (void)loadPreset:(int)idx
{
	kbdata.LoadPreset(idx);
}

- (void)savePreset:(int)idx
{
	kbdata.SavePreset(idx);
}

- (void)SetData
{
	kbdata.SetData();
}

- (void)onClickCell:(NSTableView *)sender
{
#ifdef USE_NSCELL
	int row = (int)[sender clickedRow];
	int col = (int)[sender clickedColumn];
	
	selected.row = row;
	selected.col = col;

	if (col > 0) {
		[sender editColumn:col row:row withEvent:nil select:YES];
	}
#endif
}

- (void)onDoubleClickCell:(NSTableView *)sender
{
}
@end

#ifndef USE_NSCELL
@implementation CocoaNText
- (id)initWithDataSource:(NSTableView *)new_tbl data:(CocoaTableData *)new_data col:(NSTableColumn *)tableColumn row:(int)new_row
{
	[super init];
//	[self setEditable:YES];
//	[self setSelectable:YES];
	dataSource = new_data;

	int new_col = 0;
	if ([[tableColumn identifier] isEqualToString:@"1"]) {
		new_col++;
	}
	if([[tableColumn identifier] isEqualToString:@"vmkey"]) {
		new_col=-1;
	}
	col = new_col;
	row = new_row;
	
	if ([dataSource tab_num] != 0) {
		// update joystick status per 0.1 sec
		joy_stat = emu->joy_buffer();
		NSRunLoop *loop = [NSRunLoop currentRunLoop];
		timer = [NSTimer timerWithTimeInterval:0.1 target:self selector:@selector(updateJoy) userInfo:nil repeats:YES];
		[loop addTimer:timer forMode:NSRunLoopCommonModes];
	} else {
		joy_stat = NULL;
		timer = nil;
	}
//	[self setFocusRingType:NSFocusRingTypeDefault];
	[self setString:[dataSource tableView:new_tbl objectValueForTableColumn:tableColumn row:row]];
	
	selClick = FALSE;

	return self;
}
- (void)dealloc
{
	if (timer != nil) [timer invalidate];
	[super dealloc];
}
- (BOOL)acceptsFirstResponder
{
	return YES;
}
- (void)keyDown:(NSEvent *)event
{
	char label[128];
	int tab = [dataSource tab_num];
	if (tab == 0) {
		int code = SDL_QZ_HandleKeyEvents(event);
		[dataSource SetVkKeyCode:code:label];
		[self setString:[NSString stringWithUTF8String:label]];
		selClick = TRUE;
		[self setNeedsDisplay:YES];
	}
}
- (void)flagsChanged:(NSEvent *)event
{
	char label[128];
	int tab = [dataSource tab_num];
	if (tab == 0) {
		int code = SDL_QZ_HandleKeyEvents(event);
		[dataSource SetVkKeyCode:code:label];
		[self setString:[NSString stringWithUTF8String:label]];
		selClick = TRUE;
		[self setNeedsDisplay:YES];
	}
}
- (void)keyUp:(NSEvent *)event
{
	char label[128];
	int tab = [dataSource tab_num];
	if (tab == 0) {
		int code = SDL_QZ_HandleKeyEvents(event);
		[dataSource SetVkKeyCode:code:label];
		[self setString:[NSString stringWithUTF8String:label]];
		selClick = TRUE;
		[self setNeedsDisplay:YES];
	}
}
- (void)mouseDown:(NSEvent *)event
{
	int count = (int)[event clickCount];
	[dataSource setSelectCol:col+1 row:row];
	selClick = TRUE;
	[self setNeedsDisplay:YES];
	if (count != 2) return;
	char label[128];
	int tab = [dataSource tab_num];
	if (tab != 0) {
		[dataSource SetVkJoyCode:0:label];
	} else {
		[dataSource SetVkKeyCode:0:label];
	}
	[self setString:[NSString stringWithUTF8String:label]];
}
- (void)updateJoy
{
	if (joy_stat == NULL) return;
	
	char label[128];
	
	if (col >= 0) {
		emu->reset_joystick();
		if (joy_stat[col]) {
			[dataSource SetVkJoyCode:joy_stat[col]:label];
			[self setString:[NSString stringWithUTF8String:label]];
			[self setNeedsDisplay:YES];
		}
	}
}
- (void)setString:(NSString *)aString
{
	str = [[NSAttributedString alloc] initWithString:(aString ? aString : @"")];
}
- (void)drawRect:(NSRect)theRect
{
	NSRect re = [self bounds];
	
	[[NSColor whiteColor] set];
	NSRectFill(re);

	if (col >= 0) {
		if (selClick && self == [NSView focusView]) {
			NSSetFocusRingStyle(NSFocusRingBelow);
		}
		
		[[NSColor grayColor] set];
		NSFrameRect(re);
	}

	[str drawInRect: NSMakeRect(
		re.origin.x + 6.0, re.origin.y - 1.0,
		re.size.width, re.size.height)];
	
	[self setNeedsDisplay:NO];
}
//- (void)resetCursorRects
//{
//	NSRect rect = [self bounds];
//	NSCursor* cursor = [NSCursor IBeamCursor];
//	[self addCursorRect:rect cursor:cursor];
//}
@end
#else /* USE_NSCELL */
@implementation CocoaText
- (id)initWithDataSource:(CocoaTableData *)new_data
{
	[super init];
	[self setEditable:YES];
	[self setSelectable:YES];
//	[self setBordered:YES];
	dataSource = new_data;
	
	if ([dataSource tab_num] != 0) {
		// update joystick status per 0.1 sec
		joy_stat = emu->joy_buffer();
		NSRunLoop *loop = [NSRunLoop currentRunLoop];
		timer = [NSTimer timerWithTimeInterval:0.1 target:self selector:@selector(updateJoy) userInfo:nil repeats:YES];
		[loop addTimer:timer forMode:NSRunLoopCommonModes];
	} else {
		joy_stat = NULL;
		timer = nil;
	}

	return self;
}
- (void)dealloc
{
	if (timer != nil) [timer invalidate];
	[super dealloc];
}
- (void)keyDown:(NSEvent *)event
{
	char label[128];
	int tab = [dataSource tab_num];
	if (tab == 0) {
		int code = SDL_QZ_HandleKeyEvents(event);
		[dataSource SetVkKeyCode:code:label];
		[self setString:[NSString stringWithUTF8String:label]];
	}
}
- (void)flagsChanged:(NSEvent *)event
{
	char label[128];
	int tab = [dataSource tab_num];
	if (tab == 0) {
		int code = SDL_QZ_HandleKeyEvents(event);
		[dataSource SetVkKeyCode:code:label];
		[self setString:[NSString stringWithUTF8String:label]];
	}
}
- (void)keyUp:(NSEvent *)event
{
	char label[128];
	int tab = [dataSource tab_num];
	if (tab == 0) {
		int code = SDL_QZ_HandleKeyEvents(event);
		[dataSource SetVkKeyCode:code:label];
		[self setString:[NSString stringWithUTF8String:label]];
	}

}
- (BOOL)performKeyEquivalent:(NSEvent *)event
{
	return NO;
}
- (void)mouseDown:(NSEvent *)event
{
	int count = (int)[event clickCount];
	if (count != 2) return;
	char label[128];
	int tab = [dataSource tab_num];
	if (tab != 0) {
		[dataSource SetVkJoyCode:0:label];
	} else {
		[dataSource SetVkKeyCode:0:label];
	}
	[self setString:[NSString stringWithUTF8String:label]];
}
- (void)updateJoy
{
	if (joy_stat == NULL) return;
	
	int col = [dataSource selectedCol];
	char label[128];
	
    if (col > 0) {
        emu->reset_joystick();
        if (joy_stat[col-1]) {
            [dataSource SetVkJoyCode:joy_stat[col-1]:label];
			[self setString:[NSString stringWithUTF8String:label]];
        }
    }
}

@end

@implementation CocoaTextCell
- (id)init
{
	[super init];
	[self setEditable:YES];
	[self setSelectable:YES];
	[self setBordered:YES];
	return self;
}
- (id)initWithCoder:(NSCoder *)decoder
{
	[super initWithCoder:decoder];
	[self setEditable:YES];
	[self setSelectable:YES];
	[self setBordered:YES];
	return self;
}
- (id)initTextCell:(NSString *)aString
{
//	[super init];
	[super initTextCell:aString];
	[self setEditable:YES];
	[self setSelectable:YES];
	[self setBordered:YES];
	return self;
}
- (id)initImageCell:(NSImage *)anImage
{
//	[super init];
	[super initImageCell:anImage];
	[self setEditable:YES];
	[self setSelectable:YES];
	[self setBordered:YES];
	return self;
}
- (void)selectWithFrame:(NSRect)re inView:(NSView *)ctlView editor:(NSText *)text delegate:(id)obj start:(NSInteger)start length:(NSInteger)length
{
	NSTableView *view = (NSTableView *)ctlView;
	CocoaText *ntext = [[CocoaText alloc] initWithDataSource:[view dataSource]];
	[ntext setString:[text string]];
	[super selectWithFrame:re inView:ctlView editor:ntext delegate:obj start:start length:0];
}
@end
#endif /* USE_NSCELL */

//@implementation CocoaTableFieldView
//- (void)keyDown:(NSEvent *)event
//{
//	int i=0;
//}
//- (BOOL)textShouldBeginEditing:(NSText *)textObject
//{
//	return YES;
//}
//@end

@implementation CocoaTableView
+ (CocoaTableView *)create:(NSRect)re tabnum:(int)tab_num
{
	NSRect re_ctl;

	CocoaTableView *me = [CocoaTableView alloc];
	[me initWithFrame:re];
	[me setBorderType:NSBezelBorder];

	re_ctl = NSMakeRect(0,0,re.size.width,24);
	NSScroller *hs = [[NSScroller alloc] initWithFrame:re_ctl];
//	[me addSubview:hs];
	[me setHorizontalScroller:hs];
	[me setHasHorizontalScroller:YES];
	re_ctl = NSMakeRect(0,0,24,re.size.height);
	NSScroller *vs = [[NSScroller alloc] initWithFrame:re_ctl];
//	[me addSubview:vs];
	[me setVerticalScroller:vs];
	[me setHasVerticalScroller:YES];

	[me setAutohidesScrollers:NO];
//	[me setAutoresizesSubviews:YES];

	re_ctl = NSMakeRect(10,10,re.size.width,24);
	NSTableHeaderView *hv = [[NSTableHeaderView alloc] init];
	
	re_ctl = NSMakeRect(10,128,re.size.width, re.size.height);
	NSTableView *table = [[NSTableView alloc] init];
//	[me addSubview:table];
	[table setHeaderView:hv];

	[table setAllowsColumnReordering:NO];
	[table setAllowsColumnResizing:NO];
	[table setAllowsMultipleSelection:NO];
	[table setAllowsEmptySelection:YES];
//	[table setGridStyleMask:(NSTableViewSolidVerticalGridLineMask | NSTableViewSolidHorizontalGridLineMask)];
//	[table setSelectionHighlightStyle:NSTableViewSelectionHighlightStyleNone];
	char label[128];

	NSTableColumn *col;
	col = [[NSTableColumn alloc] initWithIdentifier:@"vmkey"];
	if (tab_num == 2) strcpy(label, _("PIA on L3"));
	else strcpy(label, _("Level3 Key"));
	[col.headerCell setStringValue:[NSString stringWithUTF8String:label]];
	[col setWidth:120.0];
	[col setEditable:NO];
	[table addTableColumn:col];
	for(int i=0; i<2; i++) {
		sprintf(label, "%d", i);
		col = [[NSTableColumn alloc] initWithIdentifier:[NSString stringWithUTF8String:label]];
		if (tab_num != 0) sprintf(label, _("Joypad%d"), i+1);
		else sprintf(label, _("Bind%d"), i+1);
		[col.headerCell setStringValue:[NSString stringWithUTF8String:label]];
#ifdef USE_NSCELL
		[col setDataCell:[[CocoaTextCell alloc] init]];
#endif
		[col setWidth:120.0];
		[col setEditable:YES];
		[table addTableColumn:col];
	}
	[me setDocumentView:table];
#ifndef USE_NSCELL
	[table setDelegate:[[CocoaTableViewDelegate alloc] init]];
#endif
	CocoaTableData *data = [[CocoaTableData alloc] initWithValue:tab_num];
	[table setDataSource:data];
	[table setTarget:data];
	[table setAction:@selector(onClickCell:)];
//	[table setDoubleAction:@selector(onDoubleClickCell:)];

	return me;
}
- (CocoaTableData *)data
{
	return (CocoaTableData *)[[self documentView] dataSource];
}
- (void)deleteColumns
{
	NSTableView *table = [self documentView];
	NSArray *cols = [table tableColumns];
	NSUInteger count = [cols count];
	NSUInteger i;
	for  (i=0; i < count; i++) {
		[table removeTableColumn:[[table tableColumns] objectAtIndex:0]];
	}
}
@end

#ifndef USE_NSCELL
@implementation CocoaTableViewDelegate
- (id)init
{
	[super init];
	cells = [[NSMutableDictionary alloc] init];
	return self;
}
- (NSView *)tableView:(NSTableView *)tableView viewForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row
{
	CocoaTableData *data = [tableView dataSource];
	NSMutableString *idt = [NSMutableString stringWithString:[tableColumn identifier]];
	[idt appendFormat:@"-%d", (int)row];
	CocoaNText *ntext = [cells objectForKey:idt];
	if (ntext == nil) {
		ntext = [[CocoaNText alloc] initWithDataSource:tableView data:data col:tableColumn row:(int)row];
		[cells setObject:ntext forKey:idt];
	}
	return ntext;
}
//- (void)tableView:(NSTableView *)tableView willDisplayCell:(id)cell forTableColumn:(NSTableColumn *) tableColumn row:(NSInteger)row
//{
//	NSLog(@"%@:%d",[tableColumn identifier], row);
//}
- (BOOL)tableView:(NSTableView *)tableView shouldSelectRow:(NSInteger)row
{
	return NO;
}
- (CGFloat)tableView:(NSTableView *)tableView heightOfRow:(NSInteger)row
{
	return 20.0;
}
@end
#endif

@implementation CocoaButtonAttr
@synthesize tabNum;
@synthesize idx;
- (id)initWithValue:(int)new_tabnum :(int)new_idx
{
	[super init];
	tabNum = new_tabnum;
	idx = new_idx;
	return self;
}
@end

@implementation CocoaKeybindPanel
- (id)init
{
	const char *tab_labels[] = { _("Keyboard"),_("Joypad"),
#if (KEYBIND_MAX_NUM >= 3)
		_("Joypad (PIA Type)"),
#endif
		NULL };

	[super init];

	SDL_QZ_InitOSKeymap();
	
	self.title = [NSString stringWithUTF8String:_("Keybind")];
	[self setShowsResizeIndicator:FALSE];
	
	CocoaView *view = [self contentView];
	NSRect re_tab;
	NSRect re_table;
	int top = view.margin;
	int left = view.margin;
	

	re_tab = [view makeRect:left:top:500:400];
	CocoaTabView *tabView = [CocoaTabView create:re_tab tabs:tab_labels];
	[view addSubview:tabView];
	CocoaView *tab_view;
	
	NSRect re;
	CocoaButton *btn;
	char name[64];
	NSTabViewItem *tab;

	//
	for(int tab_num=0; tab_num<KEYBIND_MAX_NUM; tab_num++) {
		tab = [tabView tabViewItemAtIndex:tab_num];
		tab_view = (CocoaView *)[tab view];

		// table

		re_table = [tab_view makeRect:left:top:390:400];
		tableView[tab_num] = [CocoaTableView create:re_table tabnum:tab_num];
		[tab_view addSubview:tableView[tab_num]];
		
		// checkbox

		if (tab_num == 1) {
			re = [tab_view addHeight:re_table:400:32];
			chkCombi = [CocoaCheckBox create:re title:_("Recognize as another key when pressed two buttons") action:nil value:[[tableView[tab_num] data] combi]];
			[tab_view addSubview:chkCombi];
		}
		
		// button (right side)

		re = [tab_view addWidth:re_table:180:32];
		btn = [CocoaButton create:re title:_("Load Default") action:@selector(loadDefaultPreset:)];
		[tab_view addSubview:btn];
		[btn setRelatedObject:[[CocoaButtonAttr alloc]initWithValue:tab_num:-1]];
		re = [tab_view addHeight:re:180:32];
		for(int i=0; i<KEYBIND_PRESETS; i++) {
			re = [tab_view addHeight:re:180:32];
			sprintf(name, _("Load Preset %d"), i+1);
			btn = [CocoaButton create:re title:name action:@selector(loadPreset:)];
			[tab_view addSubview:btn];
			[btn setRelatedObject:[[CocoaButtonAttr alloc]initWithValue:tab_num:i]];
			
		}
		re = [tab_view addHeight:re:180:32];
		for(int i=0; i<KEYBIND_PRESETS; i++) {
			re = [tab_view addHeight:re:180:32];
			sprintf(name, _("Save Preset %d"), i+1);
			btn = [CocoaButton create:re title:name action:@selector(savePreset:)];
			[tab_view addSubview:btn];
			[btn setRelatedObject:[[CocoaButtonAttr alloc]initWithValue:tab_num:i]];
		}
	}

	re_tab = [tabView adjustFrameSize:re_tab];

	// button
	
	re = [view makeRect:re_tab.origin.x+re_tab.size.width-120:re_tab.origin.y+re_tab.size.height:120:32];
	CocoaButton *btnCancel = [CocoaButton create:re title:_("Cancel") action:@selector(dialogCancel:)];
	[view addSubview:btnCancel];

	re = [view addPosition:re:-120-view.margin:0:120:32];
	CocoaButton *btnOK = [CocoaButton create:re title:_("OK") action:@selector(dialogOk:)];
	[view addSubview:btnOK];
	
	[self adjustFrame];
	
	return self;
}

- (NSInteger)runModal
{
	return [NSApp runModalForWindow:self];
}

- (void)close
{
	[NSApp stopModalWithCode:NSCancelButton];
	[super close];
}

- (void)dialogOk:(id)sender
{
    // OK button is pushed
	for(int tab=0; tab<KEYBIND_MAX_NUM; tab++) {
		CocoaTableData *data = [tableView[tab] data];
		[data SetData];
		if (tab == 1) {
			[data setCombi:[chkCombi state] == NSOnState ? 1 : 0];
		}
	}

	emu->save_keybind();
	
	[NSApp stopModalWithCode:NSOKButton];
	[super close];
}

- (void)dialogCancel:(id)sender
{
    // Cancel button is pushed
	[self close];
}

- (void)loadDefaultPreset:(id)sender
{
	CocoaButtonAttr *attr = (CocoaButtonAttr *)[sender relatedObject];
	NSTableView *view = [tableView[attr.tabNum] documentView];
	CocoaTableData *data = [view dataSource];
	[view editColumn:0 row:[view selectedRow] withEvent:nil select:YES];
	[data loadDefaultPreset];
	if (attr.tabNum == 1) {
		[chkCombi setState:[data combi]];
	}
	[view reloadData];
}

- (void)loadPreset:(id)sender
{
	CocoaButtonAttr *attr = (CocoaButtonAttr *)[sender relatedObject];
	NSTableView *view = [tableView[attr.tabNum] documentView];
	CocoaTableData *data = [view dataSource];
	[view editColumn:0 row:[view selectedRow] withEvent:nil select:YES];
	[data loadPreset:attr.idx];
	if (attr.tabNum == 1) {
		[chkCombi setState:[data combi]];
	}
	[view reloadData];
}

- (void)savePreset:(id)sender
{
	CocoaButtonAttr *attr = (CocoaButtonAttr *)[sender relatedObject];
	NSTableView *view = [tableView[attr.tabNum] documentView];
	CocoaTableData *data = [view dataSource];
	[view editColumn:0 row:[view selectedRow] withEvent:nil select:YES];
	if (attr.tabNum == 1) {
		[data setCombi:[chkCombi state] == NSOnState ? 1 : 0];
	}
	[data savePreset:attr.idx];
	[view reloadData];
}

@end
