/** @file cocoa_recaudpanel.mm
 
 HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
 Skelton for retropc emulator
 SDL edition + Cocoa GUI
 
 @author Sasaji
 @date   2016.11.05 -
 
 @brief [ record audio panel ]
 */

#import "cocoa_gui.h"
#import "cocoa_recaudpanel.h"
#import "../../video/rec_audio.h"
#import "../../config.h"
#import "../../emu.h"
#import "../../clocale.h"

extern EMU *emu;

static const char *type_label[] = {
#ifdef USE_REC_AUDIO
#ifdef USE_REC_AUDIO_AVKIT
	_TX("avkit"),
#endif
#ifdef USE_REC_AUDIO_WAVE
	_TX("wave"),
#endif
#ifdef USE_REC_AUDIO_FFMPEG
	_TX("ffmpeg"),
#endif
#endif
	NULL };
static const int type_ids[] = {
#ifdef USE_REC_AUDIO
#ifdef USE_REC_AUDIO_AVKIT
	RECORD_AUDIO_TYPE_AVKIT,
#endif
#ifdef USE_REC_AUDIO_WAVE
	RECORD_AUDIO_TYPE_WAVE,
#endif
#ifdef USE_REC_AUDIO_FFMPEG
	RECORD_AUDIO_TYPE_FFMPEG,
#endif
#endif
	0 };

@implementation CocoaRecAudioPanel
- (id)init
{
	int i;
	for(i=0; i < COCOA_RECAUDIO_LIBS; i++) {
		codnums[i] = 0;
		quanums[i] = 0;
		enables[i] = false;
		codbtn[i] = nil;
		quabtn[i] = nil;
	}
	
	int codnum = emu->get_parami(VM::ParamRecAudioCodec);
//	int quanum = emu->get_parami(VM::ParamRecAudioQuality);
	typnum = 0;
	for(i=0; type_ids[i] != 0; i++) {
		if (type_ids[i] == emu->get_parami(VM::ParamRecAudioType)) {
			typnum = i;
			codnums[i] = codnum;
//			quanums[i] = quanum;
			break;
		}
	}

	[super init];
	
	self.title = [[NSString alloc] initWithUTF8String:_("Record Sound")];
	[self setShowsResizeIndicator:NO];
	
	NSRect re;
	CocoaView *view = [self contentView];
	int top = view.margin;
	int left = view.margin;

	NSRect re_tab = [view makeRect:left:top:300:100];
	tabView = [CocoaTabView create:re_tab];
	[view addSubview:tabView];
	CocoaRecAudioTabViewDelegate *dele = [[CocoaRecAudioTabViewDelegate alloc] initWithPanel:self];
	[tabView setDelegate:dele];
	
	NSTabViewItem *tab;
	CocoaView *tab_view;
	CocoaLabel *label;

	for(i=0; type_ids[i] != 0; i++) {
		tab = [tabView addTabItem:_tgettext(type_label[i])];
		tab_view = (CocoaView *)[tab view];
		int btop  = tab_view.margin;
		int bleft = tab_view.margin;
		enables[i] = emu->rec_sound_enabled(type_ids[i]);

		switch(type_ids[i]) {
		case RECORD_AUDIO_TYPE_WAVE:
			re = [tab_view makeRect:bleft:btop:360:24];
			label = [CocoaLabel create:re title:_("Select a sample rate on sound menu in advance.")];
			[tab_view addSubview:label];

			break;
		default:
			re = [tab_view makeRect:bleft:btop:120:24];
			label = [CocoaLabel create:re title:_("Codec Type")];
			[tab_view addSubview:label];

			const char **codlbl = emu->get_rec_sound_codec_list(type_ids[i]);
			re = [tab_view addWidth:re:160:24];
			codbtn[i] = [CocoaPopUpButton create:re items:codlbl action:nil selidx:codnums[i]];
			[tab_view addSubview:codbtn[i]];

//			re = [tab_view remakeLeft:re:bleft];
//			re = [tab_view addHeight:re:120:24];
//			label = [CocoaLabel create:re title:_("Quality")];
//			[tab_view addSubview:label];

//			const char **qualbl = emu->get_rec_audio_quality_list(type_ids[i]);
//			re = [tab_view addWidth:re:160:24];
//			quabtn[i] = [CocoaPopUpButton create:re items:qualbl action:nil selidx:quanums[i]];
//			[tab_view addSubview:quabtn[i]];
			break;
		}

		if (!enables[i]) {
			re = [tab_view remakeLeft:re:bleft];
			re = [tab_view addHeight:re:280:24];
			label = [CocoaLabel create:re title:_("! Need install library.")];
			[tab_view addSubview:label];
		}
		
		re = [tab_view addPosition:re:0:0:tab_view.margin:tab_view.margin];
	}
	re_tab = [tabView adjustFrameSize:re_tab];

	// button
	
	re = [view makeRect:re_tab.origin.x+re_tab.size.width-120:re_tab.origin.y+re_tab.size.height+view.margin:120:32];
	CocoaButton *btnCancel = [CocoaButton create:re title:_("Cancel") action:@selector(dialogCancel:)];
	[view addSubview:btnCancel];

	re = [view addPosition:re:-120-view.margin:0:120:32];
	btnOK = [CocoaButton create:re title:_("Start") action:@selector(dialogOk:)];
	[view addSubview:btnOK];

	[self adjustFrame];
	
	return self;
}

- (NSInteger)runModal
{
	return type_ids[0] ? [NSApp runModalForWindow:self] : NSCancelButton;
}

- (void)close
{
	[NSApp stopModalWithCode:NSCancelButton];
	[super close];
}

- (void)dialogOk:(id)sender
{
	typnum = (int)[tabView indexOfTabViewItem:[tabView selectedTabViewItem]];
	int codnum = (int)[codbtn[typnum] indexOfSelectedItem];
//	int quanum = (int)[quabtn[typnum] indexOfSelectedItem];
	emu->set_parami(VM::ParamRecAudioType, type_ids[typnum]);
	emu->set_parami(VM::ParamRecAudioCodec, codnum);
//	emu->set_parami(VM::ParamRecAudioQuality, quanum);

    // OK button is pushed
	[NSApp stopModalWithCode:NSOKButton];
	[super close];
}

- (void)dialogCancel:(id)sender
{
    // Cancel button is pushed
	[self close];
}
- (void)selectTab:(int)num
{
	[btnOK setEnabled:enables[num]];
}

@end

@implementation CocoaRecAudioTabViewDelegate
- (id)initWithPanel:(CocoaRecAudioPanel *)new_panel
{
	[super init];
	panel = new_panel;
	return self;
}
- (void)tabView:(NSTabView *)tabView didSelectTabViewItem:(NSTabViewItem *)tabViewItem
{
	[panel selectTab:(int)[tabView indexOfTabViewItem:tabViewItem]];
}

@end

