/** @file cocoa_recvidpanel.mm
 
 HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
 Skelton for retropc emulator
 SDL edition + Cocoa GUI
 
 @author Sasaji
 @date   2015.05.21 -
 
 @brief [ record video panel ]
 */

#import "cocoa_gui.h"
#import "cocoa_recvidpanel.h"
#import "../../video/rec_video.h"
#import "../../config.h"
#import "../../emu.h"
#import "../../clocale.h"

extern EMU *emu;

static const char *type_label[] = {
#ifdef USE_REC_VIDEO
#ifdef USE_REC_VIDEO_AVKIT
	_TX("avkit"),
#endif
#ifdef USE_REC_VIDEO_QTKIT
	_TX("qtkit"),
#endif
#ifdef USE_REC_VIDEO_FFMPEG
	_TX("ffmpeg"),
#endif
#endif
	NULL };
static const int type_ids[] = {
#ifdef USE_REC_VIDEO
#ifdef USE_REC_VIDEO_AVKIT
	RECORD_VIDEO_TYPE_AVKIT,
#endif
#ifdef USE_REC_VIDEO_QTKIT
	RECORD_VIDEO_TYPE_QTKIT,
#endif
#ifdef USE_REC_VIDEO_FFMPEG
	RECORD_VIDEO_TYPE_FFMPEG,
#endif
#endif
	0 };

@implementation CocoaRecVideoPanel
- (id)initWithCont:(bool)cont
{
	int i;
	for(i=0; i < COCOA_RECVIDEO_LIBS; i++) {
		codnums[i] = 0;
		quanums[i] = 0;
		enables[i] = false;
		codbtn[i] = nil;
		quabtn[i] = nil;
	}
	
	int codnum = emu->get_parami(VM::ParamRecVideoCodec);
	int quanum = emu->get_parami(VM::ParamRecVideoQuality);
	typnum = 0;
	for(i=0; type_ids[i] != 0; i++) {
		if (type_ids[i] == emu->get_parami(VM::ParamRecVideoType)) {
			typnum = i;
			codnums[i] = codnum;
			quanums[i] = quanum;
			break;
		}
	}

	[super init];
	
	self.title = [[NSString alloc] initWithUTF8String:_("Record Screen")];
	[self setShowsResizeIndicator:NO];
	
	NSRect re;
	CocoaView *view = [self contentView];
	int top = view.margin;
	int left = view.margin;

	NSRect re_tab = [view makeRect:left:top:300:100];
	tabView = [CocoaTabView create:re_tab];
	[view addSubview:tabView];
	CocoaRecVideoTabViewDelegate *dele = [[CocoaRecVideoTabViewDelegate alloc] initWithPanel:self];
	[tabView setDelegate:dele];
	
	NSTabViewItem *tab;
	CocoaView *tab_view;
	CocoaLabel *label;

	for(i=0; type_ids[i] != 0; i++) {
		tab = [tabView addTabItem:_tgettext(type_label[i])];
		tab_view = (CocoaView *)[tab view];
		int btop  = tab_view.margin;
		int bleft = tab_view.margin;
		enables[i] = emu->rec_video_enabled(type_ids[i]);

		re = [tab_view makeRect:bleft:btop:120:24];
		label = [CocoaLabel create:re title:_("Codec Type")];
		[tab_view addSubview:label];

		const char **codlbl = emu->get_rec_video_codec_list(type_ids[i]);
		re = [tab_view addWidth:re:160:24];
		codbtn[i] = [CocoaPopUpButton create:re items:codlbl action:nil selidx:codnums[i]];
		[tab_view addSubview:codbtn[i]];

		re = [tab_view remakeLeft:re:bleft];
		re = [tab_view addHeight:re:120:24];
		label = [CocoaLabel create:re title:_("Quality")];
		[tab_view addSubview:label];

		const char **qualbl = emu->get_rec_video_quality_list(type_ids[i]);
		re = [tab_view addWidth:re:160:24];
		quabtn[i] = [CocoaPopUpButton create:re items:qualbl action:nil selidx:quanums[i]];
		[tab_view addSubview:quabtn[i]];

		if (!enables[i]) {
			re = [tab_view remakeLeft:re:bleft];
			re = [tab_view addHeight:re:280:24];
			label = [CocoaLabel create:re title:_("! Need install library.")];
			[tab_view addSubview:label];
		}
		
		re = [tab_view addPosition:re:0:0:tab_view.margin:tab_view.margin];
	}
	re_tab = [tabView adjustFrameSize:re_tab];

	// button
	
	re = [view makeRect:re_tab.origin.x+re_tab.size.width-120:re_tab.origin.y+re_tab.size.height+view.margin:120:32];
	CocoaButton *btnCancel = [CocoaButton create:re title:_("Cancel") action:@selector(dialogCancel:)];
	[view addSubview:btnCancel];

	re = [view addPosition:re:-120-view.margin:0:120:32];
	btnOK = [CocoaButton create:re title:cont ? _("Next") : _("Start") action:@selector(dialogOk:)];
	[view addSubview:btnOK];

	[self adjustFrame];
	
	return self;
}

- (NSInteger)runModal
{
	return type_ids[0] ? [NSApp runModalForWindow:self] : NSCancelButton;
}

- (void)close
{
	[NSApp stopModalWithCode:NSCancelButton];
	[super close];
}

- (void)dialogOk:(id)sender
{
	typnum = (int)[tabView indexOfTabViewItem:[tabView selectedTabViewItem]];
	int codnum = (int)[codbtn[typnum] indexOfSelectedItem];
	int quanum = (int)[quabtn[typnum] indexOfSelectedItem];
	emu->set_parami(VM::ParamRecVideoType, type_ids[typnum]);
	emu->set_parami(VM::ParamRecVideoCodec, codnum);
	emu->set_parami(VM::ParamRecVideoQuality, quanum);

    // OK button is pushed
	[NSApp stopModalWithCode:NSOKButton];
	[super close];
}

- (void)dialogCancel:(id)sender
{
    // Cancel button is pushed
	[self close];
}
- (void)selectTab:(int)num
{
	[btnOK setEnabled:enables[num]];
}

@end

@implementation CocoaRecVideoTabViewDelegate
- (id)initWithPanel:(CocoaRecVideoPanel *)new_panel
{
	[super init];
	panel = new_panel;
	return self;
}
- (void)tabView:(NSTabView *)tabView didSelectTabViewItem:(NSTabViewItem *)tabViewItem
{
	[panel selectTab:(int)[tabView indexOfTabViewItem:tabViewItem]];
}

@end

