/** @file cocoa_volumepanel.mm
 
 HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
 Skelton for retropc emulator
 SDL edition + Cocoa GUI
 
 @author Sasaji
 @date   2015.04.30 -
 
 @brief [ volume panel ]
 */

#import "cocoa_gui.h"
#import "cocoa_volumepanel.h"
#import "../../config.h"
#import "../../emu.h"
#import "../../clocale.h"

extern EMU *emu;

@implementation CocoaVolumePanel
- (id)init
{
	const char *t[8] = {_("Master"),_("Beep"),_("6voice\nPSG"),_("9voice\nPSG"),_("Relay"),_("CMT"),_("FDD"),NULL};

	[super init];
	
	self.title = [NSString stringWithUTF8String:_("Volume")];
	[self setShowsResizeIndicator:NO];
	
	NSRect re;
	CocoaView *view = [self contentView];
	int top = view.margin;
	int left = view.margin;
	
	for(int i=0; t[i] != NULL; i++) {
		re = [view makeRect:left:top:80:32];
		CocoaLabel *label = [CocoaLabel create:re title:t[i] align:NSCenterTextAlignment];
		[view addSubview:label];

		re = [view addHeight:re:80:128];
		CocoaSlider *slider = [CocoaSlider create:re index:i action:@selector(changeSlider:) value:[self volume:i]];
		[view addSubview:slider];
		
		re = [view addHeight:re:80:32];
		CocoaCheckBox *check = [CocoaCheckBox create:re title:_("Mute") index:i action:@selector(changeMute:) value:[self mute:i]];
		[view addSubview:check];
		
		left += re.size.width;
		
		if (i == 0) {
			// separator
			re = [view makeRect:left:top:2:184];
			
			NSBox *sep = [[NSBox alloc] initWithFrame:re];

			[view addSubview:sep];
			
			left += 16;
		}
	}

	re = [view addHeight:re:80:32];
	CocoaButton *btnClose = [CocoaButton create:re title:_("Close") action:@selector(dialogClose:)];
	[view addSubview:btnClose];
	
	[self adjustFrame];
	
	return self;
}

- (NSInteger)runModal
{
	return [NSApp runModalForWindow:self];
}

- (void)close
{
	[NSApp stopModalWithCode:NSOKButton];
	[super close];
}

- (void)dialogClose:(id)sender
{
	[self close];
}

- (bool)mute:(int)idx
{
	switch(idx) {
		case 0:
			return config.mute;
		case 1:
			return config.beep_mute;
		case 2:
			return config.psg6_mute;
		case 3:
			return config.psg9_mute;
		case 4:
			return config.relay_mute;
		case 5:
			return config.cmt_mute;
		case 6:
			return config.fdd_mute;
		default:
			break;
	}
	return false;
}

- (int)volume:(int)idx
{
	switch(idx) {
		case 0:
			return config.volume;
		case 1:
			return config.beep_volume;
		case 2:
			return config.psg6_volume;
		case 3:
			return config.psg9_volume;
		case 4:
			return config.relay_volume;
		case 5:
			return config.cmt_volume;
		case 6:
			return config.fdd_volume;
		default:
			break;
	}
	return 0;
}

- (void)changeSlider:(CocoaSlider *)sender
{
	switch(sender.index) {
		case 0:
			config.volume = [sender intValue];
			break;
		case 1:
			config.beep_volume = [sender intValue];
			break;
		case 2:
			config.psg6_volume = [sender intValue];
			break;
		case 3:
			config.psg9_volume = [sender intValue];
			break;
		case 4:
			config.relay_volume = [sender intValue];
			break;
		case 5:
			config.cmt_volume = [sender intValue];
			break;
		case 6:
			config.fdd_volume = [sender intValue];
			break;
		default:
			break;
	}
	emu->set_volume(0);
}

- (void)changeMute:(CocoaCheckBox *)sender
{
	switch(sender.index) {
		case 0:
			config.mute = ([sender state] == NSOnState);
			break;
		case 1:
			config.beep_mute = ([sender state] == NSOnState);
			break;
		case 2:
			config.psg6_mute = ([sender state] == NSOnState);
			break;
		case 3:
			config.psg9_mute = ([sender state] == NSOnState);
			break;
		case 4:
			config.relay_mute = ([sender state] == NSOnState);
			break;
		case 5:
			config.cmt_mute = ([sender state] == NSOnState);
			break;
		case 6:
			config.fdd_mute = ([sender state] == NSOnState);
			break;
		default:
			break;
	}
	emu->set_volume(0);
}

@end
