/** @file gtk_configbox.cpp

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2015.05.10 -

	@brief [ config box ]
*/

#include "gtk_configbox.h"
#include "gtk_folderbox.h"
#include "../../emu.h"
#include "../../config.h"
#include "../../clocale.h"
#include "../../msgboard.h"
#include "../../utility.h"
#include "gtk_x11_gui.h"

extern EMU *emu;

namespace GUI_GTK_X11
{

#ifdef USE_IOPORT_FDD
#define IOPORT_STARTNUM 0
#else
#define IOPORT_STARTNUM 2
#endif

ConfigBox::ConfigBox(GUI *new_gui) : DialogBox(new_gui)
{
	memset(chkIO, 0, sizeof(chkIO));
}

ConfigBox::~ConfigBox()
{
}

bool ConfigBox::Show(GtkWidget *parent_window)
{
	DialogBox::Show(parent_window);

	if (dialog) return true;

	int flags = GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT;
	dialog = gtk_dialog_new_with_buttons(_("Configure"), GTK_WINDOW(window),
		(GtkDialogFlags)flags,
		_("OK"), GTK_RESPONSE_ACCEPT,
		_("Cancel"), GTK_RESPONSE_REJECT,
		NULL);
	GtkWidget *cont = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
	GtkWidget *nb;
	GtkWidget *vboxall;
	GtkWidget *hboxall;
	GtkWidget *vboxl;
	GtkWidget *vboxr;
	GtkWidget *hbox;
	GtkWidget *vbox;
//	GtkWidget *frm;
//	GtkWidget *lbl;
//	GtkWidget *btn;

	char buf[128];

	// create notebook tab
	nb = create_notebook(cont);

	// 0 Mode

	vboxall = create_vbox(NULL);
	add_note(nb, vboxall, _("Mode"));

	hboxall = create_hbox(vboxall);
	vboxl = create_vbox(hboxall);

	create_frame(vboxl, _("DIP Switch (*)"), &vbox, &hbox);
	create_label(hbox, (config.dipswitch & 4) ? ">" : " ");
	chkMODE = create_check_box(hbox, _("MODE Switch"), config.dipswitch & 4);

	int fdd_type = emu->get_parami(VM::ParamFddType);
	const char *fdd_type_label[] = {
		_("No FDD"),
		_("3inch compact FDD"),
		_("5.25inch mini FDD"),
		_("8inch standard FDD"),
		NULL
	};
	create_frame(vboxl, _("FDD Type (*)"), &vbox);
	for(int i=0; i<4; i++) {
		hbox = create_hbox(vbox);
		create_label(hbox, config.fdd_type == i ? ">" : " ");
		radFDD[i] = create_radio_box(hbox,radFDD,fdd_type_label[i],i,fdd_type, G_CALLBACK(OnChangedFDD));
	}

	chkPowerOff = create_check_box(vboxl,_("Enable the state of power off"),config.use_power_off);

	vboxr = create_vbox(hboxall);

	int io_port = emu->get_parami(VM::ParamIOPort);
	const char *io_port_label[] = {
		_("5.25inch FDC  $FF00 - $FF04"),
		_("3inch FDC  $FF18 - $FF20"),
		_("6voice PSG  $FF30 - $FF33"),
		_("Ex LPT Port  $FF3C - $FF3F"),
		_("Ex COM Port  $FF40 - $FF41"),
		_("9voice PSG  $FF70 - $FF7F"),
		_("KANJI ROM  $FF75 - $FF76"),
		NULL
	};
	create_frame(vboxr, _("I/O Port Address (*)"), &vbox);
	for(int i=IOPORT_STARTNUM; i<IOPORT_NUMS; i++) {
		hbox = create_hbox(vbox);
		create_label(hbox, config.io_port & (1 << i) ? ">" : " ");
		chkIO[i] = create_check_box(hbox,io_port_label[i],io_port & (1 << i),i,G_CALLBACK(OnChangedIO));
	}
	ChangeFDDType(get_radio_state_idx(radFDD,4));

	hbox = create_hbox(vboxall);
	create_label(hbox, _("* Need restart program or \"PowerOn\"."));

	// 1 screen

	vboxall = create_vbox(NULL);
	add_note(nb, vboxall, _("Screen"));

	hboxall = create_hbox(vboxall);

	const char *gl_use_label[] = {
		_("No"),
		_("Yes(sync)"),
		_("Yes(async)"),
		NULL
	};
	const char *gl_filter_label[] = {
		_("Nearest Neighbour"),
		_("Linear"),
		NULL
	};
	int use_opengl = 0;
	int gl_filter_type = 0;
#ifdef USE_OPENGL
	use_opengl = config.use_opengl;
	gl_filter_type = config.gl_filter_type;
#endif
	create_frame(hboxall, _("OpenGL"), &vbox, &hbox);
	comUseOpenGL = create_combo_box(hbox,_("Use (*)"),gl_use_label,use_opengl);
	hbox = create_hbox(vbox);
	comGLFilter = create_combo_box(hbox,_("Filter Type"),gl_filter_label,gl_filter_type);
#ifndef USE_OPENGL
	gtk_widget_set_sensitive(comUseOpenGL, FALSE);
	gtk_widget_set_sensitive(comGLFilter, FALSE);
#endif

	const char *skew_label[] = {
		"0",
		"1",
		"2",
		NULL
	};
	create_frame(hboxall, _("CRTC"), &vbox, &hbox);
	comDispSkew = create_combo_box(hbox,_("Disptmg Skew"),skew_label,config.disptmg_skew);
	hbox = create_hbox(vbox);
	comCurdSkew = create_combo_box(hbox,_("Curdisp Skew"),skew_label,config.curdisp_skew);

	const char *led_show_label[] = {
		_("Hide"),
		_("Show Inside"),
		_("Show Outside"),
		NULL
	};
	const char *led_pos_label[] = {
		_("LeftTop"),
		_("RightTop"),
		_("LeftBottom"),
		_("RightBottom"),
		NULL
	};
	int led_show = gui->GetLedBoxPhase(-1);
	hbox = create_hbox(vboxall);
	comLEDShow = create_combo_box(hbox,_("LED"),led_show_label,led_show);
	comLEDPos = create_combo_box(hbox,_("Position"),led_pos_label,config.led_pos);

	const char *cap_type_label[] = {
		"BMP",
		"PNG",
		NULL
	};
	hbox = create_hbox(vboxall);
	comCapType = create_combo_box(hbox,_("Capture Type"),cap_type_label,config.capture_type);

	hbox = create_hbox(vboxall);
	txtSnapPath = create_text_with_label(hbox, _("Snapshot Path"), config.snapshot_path, 40);
	create_button(hbox,_("Folder..."), G_CALLBACK(OnSelectSnapPath));

	hbox = create_hbox(vboxall);
	txtFontPath = create_text_with_label(hbox, _("Font Path"), config.font_path, 40);
	create_button(hbox, _("Folder..."), G_CALLBACK(OnSelectFontPath));

	hbox = create_hbox(vboxall);
	txtMsgFontName = create_text_with_label(hbox, _("Message Font"), config.msgboard_msg_fontname, 15);
	sprintf(buf, "%d", config.msgboard_msg_fontsize);
	txtMsgFontSize = create_text_with_label(hbox, _(" Size"), buf, 3);

	hbox = create_hbox(vboxall);
	txtInfoFontName = create_text_with_label(hbox, _("Info Font"), config.msgboard_info_fontname, 15);
	sprintf(buf, "%d", config.msgboard_info_fontsize);
	txtInfoFontSize = create_text_with_label(hbox, _(" Size"), buf, 3);

	hbox = create_hbox(vboxall);
	create_label(hbox, _("* Need restart program."));

	// 2 tape, FDD

	vboxall = create_vbox(NULL);
	add_note(nb, vboxall, _("Tape, FDD"));

	hboxall = create_hbox(vboxall);

	create_frame(hboxall, _("Load Wav File from Tape"), &vbox, &hbox);
	chkReverseWave = create_check_box(hbox, _("Reverse Wave"), config.wav_reverse);
	hbox = create_hbox(vbox);
	chkHalfWave = create_check_box(hbox, _("Half Wave"), config.wav_half);
	hbox = create_hbox(vbox);
	chkCorrectWave = create_check_box(hbox, _("Correct"), config.wav_correct);
	const char *correct_type_label[] = {
		_("COS Wave"),
		_("SIN Wave"),
		NULL
	};
	hbox = create_hbox(vbox);
	create_radio_box(hbox, correct_type_label, 2, radCorrectType, config.wav_correct_type);

	create_frame(hboxall, _("Save Wav File to Tape"), &vbox, &hbox);
	const char *rate_label[] = {
		"11025",
		"22050",
		"44100",
		"48000",
		NULL
	};
	comSampleRate = create_combo_box(hbox,_("Sample Rate"),rate_label,config.wav_sample_rate);
	const char *bits_label[] = {
		"8",
		"16",
		NULL
	};
	hbox = create_hbox(vbox);
	comSampleBits = create_combo_box(hbox,_("Sample Bits"),bits_label,config.wav_sample_bits);

	// FDD

	hboxall = create_hbox(vboxall);

	// fdd mount
	create_frame(hboxall, _("Floppy Disk Drive"), &vbox, &hbox);
	create_label(hbox, _("When start up, mount disk at:"));
	for(int i=0; i<MAX_DRIVE; i++) {
		sprintf(buf, "%d  ", i);
		chkFDMount[i] = create_check_box(hbox, buf, config.mount_fdd & (1 << i));
	}
	hbox = create_hbox(vbox);
	chkDelayFd1 = create_check_box(hbox, _("Ignore delays to find sector."), FLG_DELAY_FDSEARCH);
	hbox = create_hbox(vbox);
	chkDelayFd2 = create_check_box(hbox, _("Ignore delays to seek track."), FLG_DELAY_FDSEEK);

	// 3 network

	vboxall = create_vbox(NULL);
	add_note(nb, vboxall, _("Network"));

	// LPT
	for(int drv=0; drv<MAX_PRINTER; drv++) {
		hbox = create_hbox(vboxall);
		sprintf(buf, _("LPT%d Hostname"), drv);
		txtLPTHost[drv] = create_text_with_label(hbox, buf, config.printer_server_host[drv], 20);
		sprintf(buf, "%d", config.printer_server_port[drv]);
		txtLPTPort[drv] = create_text_with_label(hbox, _(" Port"), buf, 5);
	}
	// COM
	const char *comm_baud_label[] = {
		_("S:300baud/F:1200baud"),
		_("S:600baud/F:2400baud"),
		_("S:1200baud/F:4800baud"),
		_("S:2400baud/F:9600baud"),
		NULL
	};
	for(int drv=0; drv<MAX_COMM; drv++) {
		hbox = create_hbox(vboxall);
		sprintf(buf, _("COM%d Hostname"), drv);
		txtCOMHost[drv] = create_text_with_label(hbox, buf, config.comm_server_host[drv], 20);
		sprintf(buf, "%d", config.comm_server_port[drv]);
		txtCOMPort[drv] = create_text_with_label(hbox, _(" Port"), buf, 5);
		comCOMBaud[drv] = create_combo_box(hbox, " ", comm_baud_label, config.comm_dipswitch[drv]);
	}
#ifdef USE_DEBUGGER
	hbox = create_hbox(vboxall);
	create_label(hbox, _("Connectable host to Debugger"));
	hbox = create_hbox(vboxall);
	txtDbgrHost = create_text_with_label(hbox, _T(" "), config.debugger_server_host, 20);
	sprintf(buf, "%d", config.debugger_server_port);
	txtDbgrPort = create_text_with_label(hbox, _(" Port"), buf, 5);
#endif

	// 4 CPU, Memory

	vboxall = create_vbox(NULL);
	add_note(nb, vboxall, _("CPU, Memory"));

	hbox = create_hbox(vboxall);
	txtROMPath = create_text_with_label(hbox, _("ROM Path (*)"), config.rom_path, 40);
	create_button(hbox,_("Folder..."), G_CALLBACK(OnSelectROMPath));

	hbox = create_hbox(vboxall);
	chkExMem = create_check_box(hbox, _("Use Extended Memory (64KB)"), config.exram_size_num);

	hbox = create_hbox(vboxall);
	chkUndefOp = create_check_box(hbox, _("Show message when the CPU fetches undefined opcode."), FLG_SHOWMSG_UNDEFOP != 0);

	hbox = create_hbox(vboxall);
	create_label(hbox, _("* Need restart program or \"PowerOn\"."));

	//

	gtk_widget_show_all(dialog);
	g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(OnResponse), (gpointer)this);
//	gint rc = gtk_dialog_run(GTK_DIALOG(dialog));

	emu->set_pause(1, true);

	return true;
}

void ConfigBox::Hide()
{
	DialogBox::Hide();
	emu->set_pause(1, false);
}

bool ConfigBox::SetData()
{
	int val = 0;

	config.use_power_off = (get_check_state(chkPowerOff));
	if (get_check_state(chkMODE)) {
		config.dipswitch |= 4;
	} else {
		config.dipswitch &= ~4;
	}
	emu->set_parami(VM::ParamFddType, get_radio_state_idx(radFDD, 4));
	emu->set_parami(VM::ParamIOPort, get_check_state_num(chkIO, IOPORT_NUMS));
#ifndef USE_IOPORT_FDD
	int fdd_type = emu->get_parami(VM::ParamFddType);
	val = emu->get_parami(VM::ParamIOPort);
	val &= ~IOPORT_MSK_FDDALL;
	val |= (fdd_type == 1 ? IOPORT_MSK_3FDD : (fdd_type >= 2 ? IOPORT_MSK_5FDD : 0));
	emu->set_parami(VM::ParamIOPort, val);
#endif

	config.mount_fdd = get_check_state_num(chkFDMount, MAX_DRIVE);
	config.delay_fdd = (get_check_state(chkDelayFd1) ? MSK_DELAY_FDSEARCH : 0)
					| (get_check_state(chkDelayFd2) ? MSK_DELAY_FDSEEK : 0);

#ifdef USE_OPENGL
	config.use_opengl = (uint8_t)get_combo_sel_num(comUseOpenGL);
	config.gl_filter_type = (uint8_t)get_combo_sel_num(comGLFilter);
#endif

	config.disptmg_skew = get_combo_sel_num(comDispSkew);
	config.curdisp_skew = get_combo_sel_num(comCurdSkew);

	int led_show = get_combo_sel_num(comLEDShow);
	config.led_pos = get_combo_sel_num(comLEDPos);

	config.capture_type = get_combo_sel_num(comCapType);

	config.snapshot_path.Set(get_text(txtSnapPath));
	config.font_path.Set(get_text(txtFontPath));
	config.msgboard_msg_fontname.Set(get_text(txtMsgFontName));
	config.msgboard_info_fontname.Set(get_text(txtInfoFontName));
	val = 0;
	if (sscanf(get_text(txtMsgFontSize), "%d", &val) == 1 && val >= 6 && val <= 60) {
		config.msgboard_msg_fontsize = val;
	}
	val = 0;
	if (sscanf(get_text(txtInfoFontSize), "%d", &val) == 1 && val >= 6 && val <= 60) {
		config.msgboard_info_fontsize = val;
	}

	config.wav_reverse = get_check_state(chkReverseWave);
	config.wav_half = get_check_state(chkHalfWave);
	config.wav_correct = get_check_state(chkCorrectWave);
	config.wav_correct_type = (uint8_t)get_radio_state_idx(radCorrectType, 2);
	config.wav_sample_rate = (uint8_t)get_combo_sel_num(comSampleRate);
	config.wav_sample_bits = (uint8_t)get_combo_sel_num(comSampleBits);

	for(int i=0; i<MAX_PRINTER; i++) {
		config.printer_server_host[i].Set(get_text(txtLPTHost[i]));
		val = 0;
		if (sscanf(get_text(txtLPTPort[i]), "%d", &val) == 1 && val > 0 && val < 65535) {
			config.printer_server_port[i] = val;
		}
	}
	for(int i=0; i<MAX_COMM; i++) {
		config.comm_server_host[i].Set(get_text(txtCOMHost[i]));
		val = 0;
		if (sscanf(get_text(txtCOMPort[i]), "%d", &val) == 1 && val > 0 && val < 65535) {
			config.comm_server_port[i] = val;
		}
		config.comm_dipswitch[i] = get_combo_sel_num(comCOMBaud[i]);
	}
#ifdef USE_DEBUGGER
	config.debugger_server_host.Set(get_text(txtDbgrHost));
	val = 0;
	if (sscanf(get_text(txtDbgrPort), "%d", &val) == 1 && val > 0 && val < 65535) {
		config.debugger_server_port = val;
	}
#endif

	config.rom_path.Set(get_text(txtROMPath));
	config.exram_size_num = get_check_state(chkExMem);
	config.show_dlg = get_check_state(chkUndefOp) ? config.show_dlg | MSK_SHOWMSG_UNDEFOP : config.show_dlg & ~MSK_SHOWMSG_UNDEFOP;

	// set message font
	MsgBoard *msgboard = emu->get_msgboard();
	if (msgboard) {
		msgboard->SetFont();
	}

	gui->ChangeLedBox(led_show);
	gui->ChangeLedBoxPosition(config.led_pos);
	config.save();
#ifdef USE_OPENGL
	emu->change_opengl_attr();
#endif

	return true;
}

void ConfigBox::ShowFolderBox(const char *title, GtkWidget *entry)
{
	FolderBox fbox(dialog);
	const char *path = gtk_entry_get_text(GTK_ENTRY(entry));
	if (fbox.Show(title, path)) {
		gtk_entry_set_text(GTK_ENTRY(entry),fbox.GetPath());
	}
}

void ConfigBox::ChangeFDDType(int index)
{
#ifdef USE_IOPORT_FDD
//	int index = get_radio_state_idx(radFDD,3);
//	g_print("ChangeFDDType: %d\n", index);
	if (chkIO[0] == NULL) return;
	if (index == 0) {
		set_check_state(chkIO[0],false);
		set_check_state(chkIO[1],false);
		gtk_widget_set_sensitive(chkIO[0], true);
		gtk_widget_set_sensitive(chkIO[1], true);
	} else if (index == 1) {
		set_check_state(chkIO[0],false);
		set_check_state(chkIO[1],true);
		gtk_widget_set_sensitive(chkIO[0], false);
		gtk_widget_set_sensitive(chkIO[1], false);
	} else if (index == 2) {
		set_check_state(chkIO[0],true);
		set_check_state(chkIO[1],false);
		gtk_widget_set_sensitive(chkIO[0], false);
		gtk_widget_set_sensitive(chkIO[1], false);
	}
#endif
}

void ConfigBox::ChangeIOPort(int index)
{
//	g_print("ChangeIOPort: %d\n", index);
	if (chkIO[2] == NULL) return;
	bool status = get_check_state(chkIO[index]);
#ifdef USE_IOPORT_FDD
	if (index == 0 && status) {
		set_check_state(chkIO[index+1],false);
	} else if (index == 1 && status) {
		set_check_state(chkIO[index-1],false);
	}
#endif
	if (index == 5 && status) {
		set_check_state(chkIO[index+1],false);
	} else if (index == 6 && status) {
		set_check_state(chkIO[index-1],false);
	}
}

void ConfigBox::OnChangedFDD(GtkWidget *widget, gpointer user_data)
{
	ConfigBox *obj = (ConfigBox *)user_data;
	int index = (int)(intptr_t)g_object_get_data(G_OBJECT(widget), "num");
	obj->ChangeFDDType(index);
}

void ConfigBox::OnChangedIO(GtkWidget *widget, gpointer user_data)
{
	ConfigBox *obj = (ConfigBox *)user_data;
	int index = (int)(intptr_t)g_object_get_data(G_OBJECT(widget), "num");
	obj->ChangeIOPort(index);
}

void ConfigBox::OnSelectSnapPath(GtkWidget *widget, gpointer user_data)
{
	ConfigBox *obj = (ConfigBox *)user_data;
	obj->ShowFolderBox(_("Snapshot Path"), obj->txtSnapPath);
}

void ConfigBox::OnSelectFontPath(GtkWidget *widget, gpointer user_data)
{
	ConfigBox *obj = (ConfigBox *)user_data;
	obj->ShowFolderBox(_("Font Path"), obj->txtFontPath);
}

void ConfigBox::OnSelectROMPath(GtkWidget *widget, gpointer user_data)
{
	ConfigBox *obj = (ConfigBox *)user_data;
	obj->ShowFolderBox(_("Rom Path"), obj->txtROMPath);
}

void ConfigBox::OnResponse(GtkWidget *widget, gint response_id, gpointer user_data)
{
	ConfigBox *obj = (ConfigBox *)user_data;
	if (response_id == GTK_RESPONSE_ACCEPT) {
		obj->SetData();
	}
//	g_print("OnResponse: %d\n",response_id);
	obj->Hide();
}

}; /* namespace GUI_GTK_X11 */


