/** @file gtk_volumebox.cpp

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2015.05.10 -

	@brief [ volume box ]
*/

#include "gtk_volumebox.h"
#include "../../emu.h"
#include "../../clocale.h"

extern EMU *emu;

namespace GUI_GTK_X11
{

VolumeBox::VolumeBox(GUI *new_gui) : DialogBox(new_gui)
{
}

VolumeBox::~VolumeBox()
{
}

bool VolumeBox::Show(GtkWidget *parent_window)
{
	DialogBox::Show(parent_window);

	if (dialog) return true;

	int flags = GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT;
	dialog = gtk_dialog_new_with_buttons(_("Volume"), GTK_WINDOW(window),
		(GtkDialogFlags)flags,
		_("Close"), GTK_RESPONSE_REJECT,
		NULL);
	GtkWidget *cont = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
	GtkWidget *hboxall;
	GtkWidget *vbox;
	GtkWidget *vol;
	GtkWidget *lbl;

	const char *t[8] = {_("Master"),_("Beep"),_("6voice\nPSG"),_("9voice\nPSG"),_("Relay"),_("CMT"),_("FDD"),NULL};

	//

	hboxall = create_hbox(cont);

	for(int i=0; i<7; i++) {
		vbox = create_vbox(hboxall);
		lbl = create_label(vbox, t[i]);
		gtk_widget_set_size_request(lbl, 64, 48);
		vol = create_volume_box(vbox, GetVolume(i), i, G_CALLBACK(OnChangeVolume));
		gtk_widget_set_size_request(vol, 64, 128);
		create_check_box(vbox, _("Mute"), GetMute(i), i, G_CALLBACK(OnChangeMute));
	}

	//

	gtk_widget_show_all(dialog);
	g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(OnResponse), (gpointer)this);

	return true;
}

void VolumeBox::SetVolume(int index, int value)
{
	switch(index) {
	case 0:
		config.volume = value;
		break;
	case 1:
		config.beep_volume = value;
		break;
	case 2:
		config.psg6_volume = value;
		break;
	case 3:
		config.psg9_volume = value;
		break;
	case 4:
		config.relay_volume = value;
		break;
	case 5:
		config.cmt_volume = value;
		break;
	case 6:
		config.fdd_volume = value;
		break;
	default:
		break;
	}
	emu->set_volume(0);
}

int VolumeBox::GetVolume(int index)
{
	int value = 0;
	switch(index) {
	case 0:
		value = config.volume;
		break;
	case 1:
		value = config.beep_volume;
		break;
	case 2:
		value = config.psg6_volume;
		break;
	case 3:
		value = config.psg9_volume;
		break;
	case 4:
		value = config.relay_volume;
		break;
	case 5:
		value = config.cmt_volume;
		break;
	case 6:
		value = config.fdd_volume;
		break;
	default:
		break;
	}
	return value;
}

void VolumeBox::SetMute(int index, bool value)
{
	switch(index) {
	case 0:
		config.mute = value;
		break;
	case 1:
		config.beep_mute = value;
		break;
	case 2:
		config.psg6_mute = value;
		break;
	case 3:
		config.psg9_mute = value;
		break;
	case 4:
		config.relay_mute = value;
		break;
	case 5:
		config.cmt_mute = value;
		break;
	case 6:
		config.fdd_mute = value;
		break;
	default:
		break;
	}
	emu->set_volume(0);
}

bool VolumeBox::GetMute(int index)
{
	int value = 0;
	switch(index) {
	case 0:
		value = config.mute;
		break;
	case 1:
		value = config.beep_mute;
		break;
	case 2:
		value = config.psg6_mute;
		break;
	case 3:
		value = config.psg9_mute;
		break;
	case 4:
		value = config.relay_mute;
		break;
	case 5:
		value = config.cmt_mute;
		break;
	case 6:
		value = config.fdd_mute;
		break;
	default:
		break;
	}
	return value;
}

void VolumeBox::OnChangeVolume(GtkWidget *widget, gpointer user_data)
{
//	g_print("OnChangeVolume\n");
	VolumeBox *obj = (VolumeBox *)user_data;
	int index = (int)(intptr_t)g_object_get_data(G_OBJECT(widget), "num");
	obj->SetVolume(index, (int)obj->get_volume_value(widget));
}

void VolumeBox::OnChangeMute(GtkWidget *widget, gpointer user_data)
{
//	g_print("OnChangeMute\n");
	VolumeBox *obj = (VolumeBox *)user_data;
	int index = (int)(intptr_t)g_object_get_data(G_OBJECT(widget), "num");
	obj->SetMute(index, obj->get_check_state(widget));
}

void VolumeBox::OnResponse(GtkWidget *widget, gint response_id, gpointer user_data)
{
	g_print("OnResponse\n");
	VolumeBox *obj = (VolumeBox *)user_data;
	obj->Hide();
}

}; /* namespace GUI_GTK_X11 */


