/** @file win_configbox.cpp

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2012.03.24 -

	@brief [ config box ]
*/
#include "win_configbox.h"
#include "../../emu.h"
#ifdef USE_MESSAGE_BOARD
#include "../../msgboard.h"
#endif
#include "win_folderbox.h"
#include "win_filebox.h"
#include "win_fontbox.h"
#include "../../res/resource.h"
#include "../../config.h"
#include "win_gui.h"

namespace GUI_WIN
{

#ifdef USE_IOPORT_FDD
#define IOPORT_STARTNUM 0
#else
#define IOPORT_STARTNUM 2
#endif

static const _TCHAR *rate_list[] = {
	_T("11025"), _T("22050"), _T("44100"), _T("48000"), NULL
};
static const _TCHAR *bits_list[] = {
	_T("8"), _T("16"), NULL
};
static const _TCHAR *skew_list[] = {
	_T("0"), _T("1"), _T("2"), NULL
};
#ifdef USE_DIRECT3D
static const _TCHAR *d3d_use_list[] = {
	_TX("No"), _TX("Yes(sync)"), _TX("Yes(async)"), NULL
};
static const _TCHAR *d3d_filter_list[] = {
	_TX("None"), _TX("Point"), _TX("Linear"), NULL
};
#endif
#ifdef USE_OPENGL
static const _TCHAR *opengl_use_list[] = {
	_TX("No"), _TX("Yes(sync)"), _TX("Yes(async)"), NULL
};
static const _TCHAR *opengl_filter_list[] = {
	_TX("Nearest Neighbour"), _TX("Linear"), NULL
};
#endif
static const _TCHAR *led_show_list[] = {
	_TX("Hide"), _TX("Show Inside"), _TX("Show Outside"), NULL
};
static const _TCHAR *led_pos_list[] = {
	_TX("LeftTop"), _TX("RightTop"), _TX("LeftBottom"), _TX("RightBottom"), NULL
};
static const _TCHAR *cap_type_list[] = {
	_T("BMP"),
#ifdef USE_REC_VIDEO_WIN
	_T("PNG"),
#endif
	NULL
};
static const _TCHAR *comm_baud_list[] = {
	_TX("S:300baud/F:1200baud"), _TX("S:600baud/F:2400baud"), _TX("S:1200baud/F:4800baud"), _TX("S:2400baud/F:9600baud"), NULL
};

ConfigBox::ConfigBox(HINSTANCE hInst, CFont *new_font, EMU *new_emu, GUI *new_gui)
	: CDialogBox(hInst, IDD_CONFIGURE, new_font, new_emu, new_gui)
{
	hInstance = hInst;

	fdd_type  = config.fdd_type;
	io_port = config.io_port;

	selected_tabctrl = 0;
}

ConfigBox::~ConfigBox()
{
}

INT_PTR ConfigBox::onInitDialog(UINT message, WPARAM wParam, LPARAM lParam)
{
	HWND hCtrl;
	RECT re;
	RECT re_box;

	//
	CDialogBox::onInitDialog(message, wParam, lParam);

	fdd_type = emu->get_parami(VM::ParamFddType);
	io_port = emu->get_parami(VM::ParamIOPort);

	// power off
	CheckDlgButton(hDlg, IDC_CHK_POWEROFF, config.use_power_off);

	// MODE switch
	CheckDlgButton(hDlg, ID_DIPSWITCH3, (config.dipswitch & 4) != 0);

	// FDD type select
	CheckDlgButton(hDlg, IDC_RADIO_NOFDD, fdd_type == 0);
	CheckDlgButton(hDlg, IDC_RADIO_3FDD, fdd_type == 1);
	CheckDlgButton(hDlg, IDC_RADIO_5FDD, fdd_type == 2);
	CheckDlgButton(hDlg, IDC_RADIO_8FDD, fdd_type == 3);
	for(int i=0; i<MAX_DRIVE; i++) {
		CheckDlgButton(hDlg, IDC_CHK_FDD_MOUNT0 + i, (config.mount_fdd & (1 << i)) != 0);
	}
	CheckDlgButton(hDlg, IDC_CHK_DELAYFD1, FLG_DELAY_FDSEARCH);
	CheckDlgButton(hDlg, IDC_CHK_DELAYFD2, FLG_DELAY_FDSEEK);

	// I/O port address
	for (int i=IOPORT_STARTNUM; i<IOPORT_NUMS; i++) {
		CheckDlgButton(hDlg, IDC_CHK_IOPORT1 + i, (io_port & (1 << i)) != 0);
	}

	// Load wave file
	CheckDlgButton(hDlg, IDC_CHK_REVERSE, config.wav_reverse);
	CheckDlgButton(hDlg, IDC_CHK_HALFWAVE, config.wav_half);

	CheckDlgButton(hDlg, IDC_RADIO_COSW, config.wav_correct_type == 0);
	CheckDlgButton(hDlg, IDC_RADIO_SINW, config.wav_correct_type == 1);
	CheckDlgButton(hDlg, IDC_CHK_CORRECT, config.wav_correct);
//	if (config.wav_correct) {
//		CheckDlgButton(hDlg, IDC_CHK_CORRECT, 1);
//		hCtrl = GetDlgItem(hDlg, IDC_RADIO_COSW);
//		EnableWindow(hCtrl, true);
//		hCtrl = GetDlgItem(hDlg, IDC_RADIO_SINW);
//		EnableWindow(hCtrl, true);
//	} else {
//		CheckDlgButton(hDlg, IDC_CHK_CORRECT, 0);
//		hCtrl = GetDlgItem(hDlg, IDC_RADIO_COSW);
//		EnableWindow(hCtrl, false);
//		hCtrl = GetDlgItem(hDlg, IDC_RADIO_SINW);
//		EnableWindow(hCtrl, false);
//	}

	// Save wave file
	SetComboBoxItems(IDC_COMBO_SRATE, rate_list, config.wav_sample_rate);
	SetComboBoxItems(IDC_COMBO_SBITS, bits_list, config.wav_sample_bits);

	// crtc
	SetComboBoxItems(IDC_COMBO_DISPTMG, skew_list, config.disptmg_skew);
	SetComboBoxItems(IDC_COMBO_CURDISP, skew_list, config.curdisp_skew);

#ifdef USE_DIRECT3D
	// d3d use
	SetComboBoxItems(IDC_COMBO_D3D_USE, d3d_use_list, config.use_direct3d, true);

	// d3d filter type
	SetComboBoxItems(IDC_COMBO_D3D_FILTER, d3d_filter_list, config.d3d_filter_type, true);
#endif
#ifdef USE_OPENGL
	// opengl use
	SetComboBoxItems(IDC_COMBO_OPENGL_USE, opengl_use_list, config.use_opengl, true);

	// opengl filter type
	SetComboBoxItems(IDC_COMBO_OPENGL_FILTER, opengl_filter_list, config.gl_filter_type, true);
#endif

	// led show
	int led_show = gui->GetLedBoxPhase(-1);
	SetComboBoxItems(IDC_COMBO_LED_SHOW, led_show_list, led_show, true);
	// led pos
	SetComboBoxItems(IDC_COMBO_LED_POS, led_pos_list, config.led_pos, true);

	// capture type
	SetComboBoxItems(IDC_COMBO_CAPTURE_TYPE, cap_type_list, config.capture_type, true);

	// comm baud
	for(int j=0; j<MAX_COMM; j++) {
		SetComboBoxItems(IDC_COMBO_COM0 + j, comm_baud_list, config.comm_dipswitch[j] - 1, true);
	}

	// snapshot path
	SetDlgItemText(hDlg, IDC_SNAP_PATH, config.snapshot_path);
	// font file
	SetDlgItemText(hDlg, IDC_FONT_FILE, config.font_path);
	// message font name
	SetDlgItemText(hDlg, IDC_MSG_FONT_NAME_L, config.msgboard_info_fontname);
	SetDlgItemText(hDlg, IDC_MSG_FONT_NAME_S, config.msgboard_msg_fontname);
	// message font size
	SetDlgItemInt(hDlg, IDC_MSG_FONT_SIZE_L, config.msgboard_info_fontsize, FALSE);
	SetDlgItemInt(hDlg, IDC_MSG_FONT_SIZE_S, config.msgboard_msg_fontsize, FALSE);

	// hostname, port
	for(int i=0; i<MAX_PRINTER; i++) {
		SetDlgItemText(hDlg, IDC_HOSTNAME_LPT0 + i, config.printer_server_host[i]);
		SetDlgItemInt(hDlg, IDC_PORT_LPT0 + i, config.printer_server_port[i], TRUE);
	}
	for(int i=0; i<MAX_COMM; i++) {
		SetDlgItemText(hDlg, IDC_HOSTNAME_COM0 + i, config.comm_server_host[i]);
		SetDlgItemInt(hDlg, IDC_PORT_COM0 + i, config.comm_server_port[i], TRUE);
	}
#ifdef USE_DEBUGGER
	SetDlgItemText(hDlg, IDC_HOSTNAME_DBGR, config.debugger_server_host);
	SetDlgItemInt(hDlg, IDC_PORT_DBGR, config.debugger_server_port, TRUE);
#endif

	// rom path
	SetDlgItemText(hDlg, IDC_ROM_PATH, config.rom_path);

	// extended memory
	CheckDlgButton(hDlg, IDC_CHK_EXMEM, config.exram_size_num == 1);

	// undef opcode
	CheckDlgButton(hDlg, IDC_CHK_UNDEFOP, FLG_SHOWMSG_UNDEFOP != 0);

	// layout
	// ^uRg[
	HWND hTabCtrl = GetDlgItem(hDlg, IDC_TAB1);

	TCITEM tcitm;
	tcitm.mask = TCIF_TEXT;

	tcitm.pszText = (LPSTR)_("Mode");
	TabCtrl_InsertItem(hTabCtrl , 0 , &tcitm);
	tcitm.pszText = (LPSTR)_("Screen");
	TabCtrl_InsertItem(hTabCtrl , 1 , &tcitm);
	tcitm.pszText = (LPSTR)_("Tape, FDD");
	TabCtrl_InsertItem(hTabCtrl , 2 , &tcitm);
	tcitm.pszText = (LPSTR)_("Network");
	TabCtrl_InsertItem(hTabCtrl , 3 , &tcitm);
	tcitm.pszText = (LPSTR)_("CPU, Memory");
	TabCtrl_InsertItem(hTabCtrl , 4 , &tcitm);
	TabCtrl_SetCurSel(hTabCtrl, selected_tabctrl);

	int h = font->GetHeight();
	RECT re_tab;
	GetClientRect(hTabCtrl, &re_tab);
	TabCtrl_AdjustRect(hTabCtrl, FALSE, &re_tab);
	re_tab.top += margin + margin;
	re_tab.left += margin + margin;

	// 0:Mode
	int lay_r_left = 0;

	// DIP switch
	re_box.top = re_tab.top;
	re_box.left = re_tab.left;
	re_box.right = 0;
	re_box.bottom = 0;

	re.top = re_box.top + h + padding;
	re.left = re_box.left + margin;
	AdjustStatic(IDC_STATIC_MODE_SW, &re);
	re.left = re.right + padding;
	AdjustCheckBox(ID_DIPSWITCH3, &re);
	re_box.right = (re_box.right > re.right ? re_box.right : re.right);

	re_box.right += margin;
	re_box.bottom = re.bottom + margin;
	AdjustControl(IDC_STATIC_1, &re_box, re_box.right - re_box.left, re_box.bottom - re_box.top);
	lay_r_left = (lay_r_left > re_box.right ? lay_r_left : re_box.right);

	// FDD type select
	re_box.top = re_box.bottom + margin;
	re_box.left = margin + margin;
	re_box.right = 0;
	re_box.bottom = 0;

	re.top = re_box.top + h + padding;
	re.left = re_box.left + margin;
	AdjustStatic(IDC_STATIC_ALLOW, &re);
	int aw_right = re.right;
	re.left = re.right + padding;
	AdjustCheckBox(IDC_RADIO_NOFDD, &re);
	int aw_top = re.top + 2;
	int aw_height = re.bottom - re.top;
	re_box.right = (re_box.right > re.right ? re_box.right : re.right);

	re.top = re.bottom;
	AdjustCheckBox(IDC_RADIO_3FDD, &re);
	re_box.right = (re_box.right > re.right ? re_box.right : re.right);
	re.top = re.bottom;
	AdjustCheckBox(IDC_RADIO_5FDD, &re);
	re_box.right = (re_box.right > re.right ? re_box.right : re.right);
	re.top = re.bottom;
	AdjustCheckBox(IDC_RADIO_8FDD, &re);
	re_box.right = (re_box.right > re.right ? re_box.right : re.right);

	re_box.right += margin;
	re_box.bottom = re.bottom + margin;
	AdjustControl(IDC_STATIC_2, &re_box, re_box.right - re_box.left, re_box.bottom - re_box.top);
	lay_r_left = (lay_r_left > re_box.right ? lay_r_left : re_box.right);
	int aw_bottom = re_box.bottom;

	hCtrl = GetDlgItem(hDlg, IDC_STATIC_ALLOW);
	re.left = re_box.left + margin; re.top = (config.fdd_type) * aw_height + aw_top;
	re.right = aw_right; re.bottom = re.top + aw_height;
	SetWindowPos(hCtrl, 0, re.left, re.top, 1, 1, SWP_NOSIZE | SWP_NOZORDER);

	re_box.top = re_box.bottom + margin;
	AdjustCheckBox(IDC_CHK_POWEROFF, &re_box);
	lay_r_left = (lay_r_left > re_box.right ? lay_r_left : re_box.right);
	aw_bottom = re_box.bottom;

	// I/O port address
	re_box.top = re_tab.top;
	re_box.left = lay_r_left + margin;
	re_box.right = 0;
	re_box.bottom = 0;

	re.bottom = re_box.top + h + padding;
	for(int i=IOPORT_STARTNUM; i<IOPORT_NUMS; i++) {
		re.top = re.bottom + padding;
		re.left = re_box.left + margin;
		AdjustStatic(IDC_STATIC_IOPORT1 + i, &re);
		re.left = re.right + padding;
		AdjustCheckBox(IDC_CHK_IOPORT1 + i, &re);
		re_box.right = (re_box.right > re.right ? re_box.right : re.right);
	}

	re_box.right += margin;
	re_box.bottom = re.bottom + margin;
	AdjustControl(IDC_STATIC_3, &re_box, re_box.right - re_box.left, re_box.bottom - re_box.top);
	aw_bottom = (re_box.bottom > aw_bottom ? re_box.bottom : aw_bottom);

	// text
	re.top = aw_bottom + padding;
	re.left = re_tab.left;
	AdjustStatic(IDC_STATIC_RESTART, &re);

	// 1:Screen
	lay_r_left = 0;

	// DirectX
	re_box.top = re_tab.top;
	re_box.left = re_tab.left;
	re_box.right = 0;
	re_box.bottom = 0;

	re.top = re_box.top + h + padding;
	re.left = re_box.left + margin;
#ifdef USE_DIRECT3D
	AdjustStatic(IDC_STATIC_D3D_USE, &re);
#endif
#ifdef USE_OPENGL
	AdjustStatic(IDC_STATIC_OPENGL_USE, &re);
#endif
	re_box.right = (re_box.right > re.right ? re_box.right : re.right);
	re.top = re.bottom + padding;
#ifdef USE_DIRECT3D
	AdjustStatic(IDC_STATIC_D3D_FILTER, &re);
#endif
#ifdef USE_OPENGL
	AdjustStatic(IDC_STATIC_OPENGL_FILTER, &re);
#endif
	re_box.right = (re_box.right > re.right ? re_box.right : re.right);

	re.top = re_box.top + h + padding;
	re.left = re_box.right + padding;
#ifdef USE_DIRECT3D
	AdjustComboBox(IDC_COMBO_D3D_USE, 6, &re);
#endif
#ifdef USE_OPENGL
	AdjustComboBox(IDC_COMBO_OPENGL_USE, 6, &re);
#endif
	re_box.right = (re_box.right > re.right ? re_box.right : re.right);
	re.top = re.bottom + padding;
#ifdef USE_DIRECT3D
	AdjustComboBox(IDC_COMBO_D3D_FILTER, 6, &re);
#endif
#ifdef USE_OPENGL
	AdjustComboBox(IDC_COMBO_OPENGL_FILTER, 6, &re);
#endif
	re_box.right = (re_box.right > re.right ? re_box.right : re.right);

	re_box.right += margin;
	re_box.bottom = re.bottom + margin;
	AdjustControl(IDC_STATIC_7, &re_box, re_box.right - re_box.left, re_box.bottom - re_box.top);
	lay_r_left = (lay_r_left > re_box.right ? lay_r_left : re_box.right);

	// crtc
	re_box.top = re_tab.top;
	re_box.left = lay_r_left + margin;
	re_box.right = 0;
	re_box.bottom = 0;

	re.top = re_box.top + h + padding;
	re.left = re_box.left + margin;
	AdjustStatic(IDC_STATIC_DISPTMG, &re);
	re_box.right = (re_box.right > re.right ? re_box.right : re.right);
	re.top = re.bottom + padding;
	AdjustStatic(IDC_STATIC_CURDISP, &re);
	re_box.right = (re_box.right > re.right ? re_box.right : re.right);

	re.top = re_box.top + h + padding;
	re.left = re_box.right + padding;
	AdjustComboBox(IDC_COMBO_DISPTMG, 2, &re);
	re_box.right = (re_box.right > re.right ? re_box.right : re.right);
	re.top = re.bottom + padding;
	AdjustComboBox(IDC_COMBO_CURDISP, 2, &re);
	re_box.right = (re_box.right > re.right ? re_box.right : re.right);

	re_box.right += margin;
	re_box.bottom = re.bottom + margin;
	AdjustControl(IDC_STATIC_6, &re_box, re_box.right - re_box.left, re_box.bottom - re_box.top);

	// LED
	re.top = re_box.bottom + margin;
	re.left = re_tab.left;
	AdjustStatic(IDC_STATIC_LED_SHOW, &re);
	re.left = re.right + margin;
	AdjustComboBox(IDC_COMBO_LED_SHOW, 8, &re);
	re.left = re.right + margin;
	AdjustStatic(IDC_STATIC_LED_POS, &re);
	re.left = re.right + margin;
	AdjustComboBox(IDC_COMBO_LED_POS, 8, &re);

	// Capture Type
	re.top = re.bottom + padding;
	re.left = re_tab.left;
	AdjustStatic(IDC_STATIC_CAPTURE_TYPE, &re);
	re.left = re.right + margin;
	AdjustComboBox(IDC_COMBO_CAPTURE_TYPE, 8, &re);

	re_box.bottom = re.bottom;

	// Snapshot path , Font file , Message font
	lay_r_left = 0;
	re.top = re_box.bottom + padding;
	re.left = re_tab.left;
	AdjustStatic(IDC_STATIC_SNAP_PATH, &re);
	lay_r_left = (lay_r_left > re.right ? lay_r_left : re.right);
	re.top = re.bottom + padding;
	AdjustStatic(IDC_STATIC_FONT_FILE, &re);
	lay_r_left = (lay_r_left > re.right ? lay_r_left : re.right);
	re.top = re.bottom + padding;
	AdjustStatic(IDC_STATIC_MSG_FONT_NAME_S, &re);
	lay_r_left = (lay_r_left > re.right ? lay_r_left : re.right);
	re.top = re.bottom + padding;
	AdjustStatic(IDC_STATIC_MSG_FONT_NAME_L, &re);
	lay_r_left = (lay_r_left > re.right ? lay_r_left : re.right);

	//
	re.top = re_box.bottom + padding;
	re.left = lay_r_left + padding;
	AdjustEditBox(IDC_SNAP_PATH, 30, &re);
	re.left = re.right + padding;
	AdjustButton(IDC_BTN_SNAP_PATH, 5, &re);

	re.top = re.bottom + padding;
	re.left = lay_r_left;
	AdjustEditBox(IDC_FONT_FILE, 30, &re);
	re.left = re.right + padding;
	AdjustButton(IDC_BTN_FONT_FILE, 5, &re);

	re.top = re.bottom + padding;
	re.left = lay_r_left;
	AdjustEditBox(IDC_MSG_FONT_NAME_S, 10, &re);
	re.left = re.right + margin;
	AdjustEditBoxWithStatic(IDC_STATIC_MSG_FONT_SIZE_S, IDC_MSG_FONT_SIZE_S, 3, &re);
	re.left = re.right + padding;
	AdjustButton(IDC_BTN_FONT_NAME_S, 5, &re);

	re.top = re.bottom + padding;
	re.left = lay_r_left;
	AdjustEditBox(IDC_MSG_FONT_NAME_L, 10, &re);
	re.left = re.right + margin;
	AdjustEditBoxWithStatic(IDC_STATIC_MSG_FONT_SIZE_L, IDC_MSG_FONT_SIZE_L, 3, &re);
	re.left = re.right + padding;
	AdjustButton(IDC_BTN_FONT_NAME_L, 5, &re);

	// text
	re.top = re.bottom + padding;
	re.left = re_tab.left;
	AdjustStatic(IDC_STATIC_RESTART_2, &re);

	// 2:Tape, FDD
	lay_r_left = 0;

	// Load wav file
	re_box.top = re_tab.top;
	re_box.left = re_tab.left;
	re_box.right = 0;
	re_box.bottom = 0;

	re.top = re_box.top + h + padding;
	re.left = re_box.left + margin;
	AdjustCheckBox(IDC_CHK_REVERSE, &re);
	re.left = re.right + margin;
	AdjustCheckBox(IDC_CHK_HALFWAVE, &re);
	re_box.right = (re_box.right > re.right ? re_box.right : re.right);
	re.top = re.bottom;
	re.left = re_box.left + margin;
	AdjustCheckBox(IDC_CHK_CORRECT, &re);
	re.left = re.right + margin;
	AdjustCheckBox(IDC_RADIO_COSW, &re);
	re.left = re.right + margin;
	AdjustCheckBox(IDC_RADIO_SINW, &re);
	re_box.right = (re_box.right > re.right ? re_box.right : re.right);

	re_box.right += margin;
	re_box.bottom = re.bottom + margin;
	AdjustControl(IDC_STATIC_4, &re_box, re_box.right - re_box.left, re_box.bottom - re_box.top);
	lay_r_left = (lay_r_left > re_box.right ? lay_r_left : re_box.right);
	int lay_bottom = re_box.bottom;

	// Save wav file
	re_box.top = re_tab.top;
	re_box.left = lay_r_left + margin;
	re_box.right = 0;
	re_box.bottom = 0;

	re.top = re_box.top + h + padding;
	re.left = re_box.left + margin;
	AdjustStatic(IDC_STATIC_SRATE, &re);
	re_box.right = (re_box.right > re.right ? re_box.right : re.right);
	re.top = re.bottom + padding;
	AdjustStatic(IDC_STATIC_SBITS, &re);
	re_box.right = (re_box.right > re.right ? re_box.right : re.right);

	re.top = re_box.top + h + padding;
	re.left = re_box.right + padding;
	AdjustComboBox(IDC_COMBO_SRATE, 4, &re);
	re_box.right = (re_box.right > re.right ? re_box.right : re.right);
	re.top = re.bottom + padding;
	AdjustComboBox(IDC_COMBO_SBITS, 4, &re);
	re_box.right = (re_box.right > re.right ? re_box.right : re.right);

	re_box.right += margin;
	re_box.bottom = re.bottom + margin;
	AdjustControl(IDC_STATIC_5, &re_box, re_box.right - re_box.left, re_box.bottom - re_box.top);
	lay_bottom = (re_box.bottom > lay_bottom ? re_box.bottom : lay_bottom);

	// FDD
	re_box.top = lay_bottom + margin;
	re_box.left = re_tab.left;
	re_box.right = 0;
	re_box.bottom = 0;

	// mount fdd
	re.top = re_box.top + h + padding;
	re.left = re_box.left + margin;
	AdjustStatic(IDC_STATIC_FDD_MOUNT, &re);
	re.top -= 2;
	for(int i=0; i<MAX_DRIVE; i++) {
		re.left = re.right + padding;
		AdjustCheckBox(IDC_CHK_FDD_MOUNT0 + i, &re);
	}
	re.bottom += 2;
	re_box.right = (re_box.right > re.right ? re_box.right : re.right);

	re.top = re.bottom + padding;
	re.left = re_box.left + margin;
	AdjustCheckBox(IDC_CHK_DELAYFD1, &re);
	re_box.right = (re_box.right > re.right ? re_box.right : re.right);
	re.top = re.bottom + padding;
	AdjustCheckBox(IDC_CHK_DELAYFD2, &re);
	re_box.right = (re_box.right > re.right ? re_box.right : re.right);

	re_box.right += margin;
	re_box.bottom = re.bottom + margin;
	AdjustControl(IDC_STATIC_8, &re_box, re_box.right - re_box.left, re_box.bottom - re_box.top);
	lay_bottom = (re_box.bottom > lay_bottom ? re_box.bottom : lay_bottom);

	// 3:Network
	lay_r_left = 0;

	// LPT0 .. COM2
	re.bottom = re_tab.top;
	re.left = re_tab.left;
	for(int i=0; i<6; i++) {
		re.top = re.bottom + padding;
		AdjustStatic(IDC_STATIC_NET0 + i, &re);
		lay_r_left = (lay_r_left > re.right ? lay_r_left : re.right);
	}
#ifdef USE_DEBUGGER
	re.top = re.bottom + padding;
	AdjustStatic(IDC_STATIC_DBGR, &re);
	int re_bottom_dbgr = re.bottom;
#endif
	re.bottom = re_tab.top;
	re.left = lay_r_left + padding;
	for(int i=0; i<6; i++) {
		re.top = re.bottom + padding;
		if (i == 0) {
			AdjustStatic(IDC_HOSTNAME_NET0 + i, &re);
		} else {
			AdjustEditBox(IDC_HOSTNAME_NET0 + i, 12, &re);
		}
		lay_r_left = (lay_r_left > re.right ? lay_r_left : re.right);
	}
#ifdef USE_DEBUGGER
	re.top = re_bottom_dbgr + padding;
	AdjustEditBox(IDC_HOSTNAME_DBGR, 12, &re);
#endif
	re.bottom = re_tab.top;
	for(int i=0; i<6; i++) {
		re.top = re.bottom + padding;
		re.left = lay_r_left + padding;
		if (i == 0) {
			AdjustStatic(IDC_PORT_NET0 + i, &re);
		} else {
			AdjustEditBox(IDC_PORT_NET0 + i, 6, &re);
		}
		if (i >= 4) {
			re.left = re.right + padding;
			AdjustComboBox(IDC_COMBO_COM0 + i - 4, 10, &re);
		}
	}
#ifdef USE_DEBUGGER
	re.top = re_bottom_dbgr + padding;
	re.left = lay_r_left + padding;
	AdjustEditBox(IDC_PORT_DBGR, 6, &re);
#endif

	// 4:CPU, Memory
	re.top = re_tab.top;
	re.left = re_tab.left;
	AdjustEditBoxWithStatic(IDC_STATIC_ROM_PATH, IDC_ROM_PATH, 30, &re);
	re.left = re.right + padding;
	AdjustButton(IDC_BTN_ROM_PATH, 5, &re);

	// ex memory
	re.top = re.bottom + padding;
	re.left = re_tab.left;
	AdjustCheckBox(IDC_CHK_EXMEM, &re);

	// undef opcode
	re.top = re.bottom + padding;
	re.left = re_tab.left;
	AdjustCheckBox(IDC_CHK_UNDEFOP, &re);

	// text
	re.top = re.bottom + margin;
	re.left = re_tab.left;
	AdjustStatic(IDC_STATIC_RESTART_3, &re);


	// tab control size
	re_box.top = margin;
	re_box.left = margin;
	AdjustControl(IDC_TAB1, &re_box, GetRight() + margin, GetBottom() + margin);
	re_tab.top = re_tab.top - margin + 1;
	re_tab.left = re_box.left + 2;
	re_tab.bottom = re_box.bottom - re_tab.top - 3;
	re_tab.right = re_box.right - re_tab.left - 4;
	AdjustControl(IDC_STATIC_0, &re_tab, re_tab.right, re_tab.bottom);

	// button
	re.top = GetBottom() + padding;
	re.right = GetRight();
	AdjustButton(IDCANCEL, 8, &re, true);
	re.right = re.left - margin;
	AdjustButton(IDOK, 8, &re, true);

	AdjustWindow(margin);

	select_tabctrl(selected_tabctrl);

	return (INT_PTR)TRUE;
}

INT_PTR ConfigBox::onCommand(UINT message, WPARAM wParam, LPARAM lParam)
{
#ifdef USE_IOPORT_FDD
	HWND hCtrl;
#endif
	WORD wId = LOWORD(wParam);

	if (wId == IDOK) {
		onOK(message, wParam, lParam);
	}
	else if (wId == IDC_RADIO_NOFDD) {
		CheckDlgButton(hDlg, IDC_RADIO_3FDD, 0);
		CheckDlgButton(hDlg, IDC_RADIO_5FDD, 0);
		CheckDlgButton(hDlg, IDC_RADIO_8FDD, 0);
#ifdef USE_IOPORT_FDD
		// I/O port
		CheckDlgButton(hDlg, IDC_CHK_IOPORT1, 0);
		hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT1);
		EnableWindow(hCtrl, true);
		CheckDlgButton(hDlg, IDC_CHK_IOPORT2, 0);
		hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT2);
		EnableWindow(hCtrl, true);
#endif
		return (INT_PTR)FALSE;
	}
	else if (wId == IDC_RADIO_3FDD) {
		CheckDlgButton(hDlg, IDC_RADIO_NOFDD, 0);
		CheckDlgButton(hDlg, IDC_RADIO_5FDD, 0);
		CheckDlgButton(hDlg, IDC_RADIO_8FDD, 0);
#ifdef USE_IOPORT_FDD
		// I/O port
		CheckDlgButton(hDlg, IDC_CHK_IOPORT1, 0);
		hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT1);
		EnableWindow(hCtrl, false);
		CheckDlgButton(hDlg, IDC_CHK_IOPORT2, 1);
		hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT2);
		EnableWindow(hCtrl, false);
#endif
		return (INT_PTR)FALSE;
	}
	else if (wId == IDC_RADIO_5FDD) {
		CheckDlgButton(hDlg, IDC_RADIO_NOFDD, 0);
		CheckDlgButton(hDlg, IDC_RADIO_3FDD, 0);
		CheckDlgButton(hDlg, IDC_RADIO_8FDD, 0);
#ifdef USE_IOPORT_FDD
		// I/O port
		CheckDlgButton(hDlg, IDC_CHK_IOPORT1, 1);
		hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT1);
		EnableWindow(hCtrl, false);
		CheckDlgButton(hDlg, IDC_CHK_IOPORT2, 0);
		hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT2);
		EnableWindow(hCtrl, false);
#endif
		return (INT_PTR)FALSE;
	}
	else if (wId == IDC_RADIO_8FDD) {
		CheckDlgButton(hDlg, IDC_RADIO_NOFDD, 0);
		CheckDlgButton(hDlg, IDC_RADIO_3FDD, 0);
		CheckDlgButton(hDlg, IDC_RADIO_5FDD, 0);
#ifdef USE_IOPORT_FDD
		// I/O port
		CheckDlgButton(hDlg, IDC_CHK_IOPORT1, 1);
		hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT1);
		EnableWindow(hCtrl, false);
		CheckDlgButton(hDlg, IDC_CHK_IOPORT2, 0);
		hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT2);
		EnableWindow(hCtrl, false);
#endif
		return (INT_PTR)FALSE;
	}
	else if (wId == IDC_RADIO_COSW)
	{
		CheckDlgButton(hDlg, IDC_RADIO_SINW, 0);
		return (INT_PTR)FALSE;
	}
	else if (wId == IDC_RADIO_SINW)
	{
		CheckDlgButton(hDlg, IDC_RADIO_COSW, 0);
		return (INT_PTR)FALSE;
	}
//	else if (wId == IDC_CHK_CORRECT)
//	{
//		if (IsDlgButtonChecked(hDlg, IDC_CHK_CORRECT)) {
//			hCtrl = GetDlgItem(hDlg, IDC_RADIO_COSW);
//			EnableWindow(hCtrl, true);
//			hCtrl = GetDlgItem(hDlg, IDC_RADIO_SINW);
//			EnableWindow(hCtrl, true);
//		} else {
//			hCtrl = GetDlgItem(hDlg, IDC_RADIO_COSW);
//			EnableWindow(hCtrl, false);
//			hCtrl = GetDlgItem(hDlg, IDC_RADIO_SINW);
//			EnableWindow(hCtrl, false);
//		}
//		return (INT_PTR)FALSE;
//	}
#ifdef USE_IOPORT_FDD
	else if (wId == IDC_CHK_IOPORT1)
	{
		if (IsDlgButtonChecked(hDlg, IDC_CHK_IOPORT1)) {
			CheckDlgButton(hDlg, IDC_CHK_IOPORT2, 0);
		}
		return (INT_PTR)FALSE;
	}
	else if (wId == IDC_CHK_IOPORT2)
	{
		if (IsDlgButtonChecked(hDlg, IDC_CHK_IOPORT2)) {
			CheckDlgButton(hDlg, IDC_CHK_IOPORT1, 0);
		}
		return (INT_PTR)FALSE;
	}
#endif
	else if (wId == IDC_CHK_IOPORT6)
	{
		if (IsDlgButtonChecked(hDlg, IDC_CHK_IOPORT6)) {
			CheckDlgButton(hDlg, IDC_CHK_IOPORT7, 0);
		}
		return (INT_PTR)FALSE;
	}
	else if (wId == IDC_CHK_IOPORT7)
	{
		if (IsDlgButtonChecked(hDlg, IDC_CHK_IOPORT7)) {
			CheckDlgButton(hDlg, IDC_CHK_IOPORT6, 0);
		}
		return (INT_PTR)FALSE;
	}
	else if (wId == IDC_BTN_SNAP_PATH)
	{
		_TCHAR buf[_MAX_PATH];
		FolderBox fbox(hDlg);
		GetDlgItemText(hDlg, IDC_SNAP_PATH, buf, _MAX_PATH);
		if (buf[0] == _T('\0')) emu->application_path(buf, _MAX_PATH);
		if (fbox.Show(_("Select a folder to save snapshot images."), buf)) {
			SetDlgItemText(hDlg, IDC_SNAP_PATH, buf);
		}
		return (INT_PTR)FALSE;
	}
	else if (wId == IDC_BTN_FONT_FILE)
	{
#ifdef USE_WIN
		const _TCHAR *filter[] =
			{ _("Supported Files (*.ttf;*.otf)"), _("All Files (*.*)"), NULL };
		_TCHAR buf[_MAX_PATH];
		FileBox fbox(hDlg);
		GetDlgItemText(hDlg, IDC_FONT_FILE, buf, _MAX_PATH);
		if (fbox.Show(
			filter,
			_("Select a font file for messages."),
			NULL,
			_T("ttf"),
			false,
			buf)) {
			SetDlgItemText(hDlg, IDC_FONT_FILE, buf);
		}
#endif
#if defined(USE_SDL) || defined(USE_SDL2)
		_TCHAR buf[_MAX_PATH];
		FolderBox fbox(hDlg);
		GetDlgItemText(hDlg, IDC_FONT_FILE, buf, _MAX_PATH);
		if (buf[0] == _T('\0')) emu->application_path(buf, _MAX_PATH);
		if (fbox.Show(_("Select a font folder for showing messages."), buf)) {
			SetDlgItemText(hDlg, IDC_FONT_FILE, buf);
		}
#endif
		return (INT_PTR)FALSE;
	}
	else if (wId == IDC_BTN_FONT_NAME_L)
	{
		_TCHAR buf[_MAX_PATH];
		double font_size;
		BOOL rc;
		FontBox fbox(hDlg);
		GetDlgItemText(hDlg, IDC_MSG_FONT_NAME_L, buf, _MAX_PATH);
		font_size = (double)GetDlgItemInt(hDlg, IDC_MSG_FONT_SIZE_L, &rc, TRUE);
		if (fbox.Show(
			_("Select a font."),
			NULL, buf, &font_size)) {
			SetDlgItemText(hDlg, IDC_MSG_FONT_NAME_L, buf);
			SetDlgItemInt(hDlg, IDC_MSG_FONT_SIZE_L, (UINT)font_size, TRUE);
		}
		return (INT_PTR)FALSE;
	}
	else if (wId == IDC_BTN_FONT_NAME_S)
	{
		_TCHAR buf[_MAX_PATH];
		double font_size;
		BOOL rc;
		FontBox fbox(hDlg);
		GetDlgItemText(hDlg, IDC_MSG_FONT_NAME_S, buf, _MAX_PATH);
		font_size = (double)GetDlgItemInt(hDlg, IDC_MSG_FONT_SIZE_S, &rc, TRUE);
		if (fbox.Show(
			_("Select a font."),
			NULL, buf, &font_size)) {
			SetDlgItemText(hDlg, IDC_MSG_FONT_NAME_S, buf);
			SetDlgItemInt(hDlg, IDC_MSG_FONT_SIZE_S, (UINT)font_size, TRUE);
		}
		return (INT_PTR)FALSE;
	}
	else if (wId == IDC_BTN_ROM_PATH)
	{
		_TCHAR buf[_MAX_PATH];
		FolderBox fbox(hDlg);
		GetDlgItemText(hDlg, IDC_ROM_PATH, buf, _MAX_PATH);
		if (buf[0] == _T('\0')) emu->application_path(buf, _MAX_PATH);
		if (fbox.Show(_("Select a folder containing the rom images."), buf)) {
			SetDlgItemText(hDlg, IDC_ROM_PATH, buf);
		}
		return (INT_PTR)FALSE;
	}

	if (wId == IDOK || wId == IDCANCEL) {
		::EndDialog(hDlg, wId);
		return (INT_PTR)TRUE;
	}
	return (INT_PTR)FALSE;
}

INT_PTR ConfigBox::onNotify(UINT message, WPARAM wParam, LPARAM lParam)
{
	// ^u؂ւ
	LPNMHDR lpNmHdr = (NMHDR *)lParam;
	int i;
	if (lpNmHdr->idFrom = IDC_TAB1) {
		switch (lpNmHdr->code) {
		case TCN_SELCHANGE:
			i = TabCtrl_GetCurSel(lpNmHdr->hwndFrom);
			select_tabctrl(i);
			break;
		}
	}
	return (INT_PTR)TRUE;
}

INT_PTR ConfigBox::onOK(UINT message, WPARAM wParam, LPARAM lParam)
{
	_TCHAR buf[_MAX_PATH];

	// power off
	config.use_power_off = (IsDlgButtonChecked(hDlg, IDC_CHK_POWEROFF) == BST_CHECKED);

	// MODE switch
	if (IsDlgButtonChecked(hDlg, ID_DIPSWITCH3)) {
		config.dipswitch |= 4;
	} else {
		config.dipswitch &= ~4;
	}

	// fdd type
	fdd_type = (IsDlgButtonChecked(hDlg, IDC_RADIO_3FDD) ? 1 : 0)
			 + (IsDlgButtonChecked(hDlg, IDC_RADIO_5FDD) ? 2 : 0)
			 + (IsDlgButtonChecked(hDlg, IDC_RADIO_8FDD) ? 3 : 0);
#ifndef USE_IOPORT_FDD
	io_port &= ~IOPORT_MSK_FDDALL;
	io_port |= (fdd_type == 1 ? IOPORT_MSK_3FDD : (fdd_type >= 2 ? IOPORT_MSK_5FDD : 0));
#endif

	config.mount_fdd = 0;
	for (int i=0; i<MAX_DRIVE; i++) {
		config.mount_fdd |= (IsDlgButtonChecked(hDlg, IDC_CHK_FDD_MOUNT0 + i) == BST_CHECKED ? 1 << i : 0);
	}
	config.delay_fdd = (IsDlgButtonChecked(hDlg, IDC_CHK_DELAYFD1) == BST_CHECKED ? MSK_DELAY_FDSEARCH : 0)
					| (IsDlgButtonChecked(hDlg, IDC_CHK_DELAYFD2) == BST_CHECKED ? MSK_DELAY_FDSEEK : 0);

	config.wav_reverse = (IsDlgButtonChecked(hDlg, IDC_CHK_REVERSE) == BST_CHECKED);
	config.wav_half = (IsDlgButtonChecked(hDlg, IDC_CHK_HALFWAVE) == BST_CHECKED);
	config.wav_correct = (IsDlgButtonChecked(hDlg, IDC_CHK_CORRECT) == BST_CHECKED);
	config.wav_correct_type = (IsDlgButtonChecked(hDlg, IDC_RADIO_SINW) == BST_CHECKED ? 1 : 0);

	config.wav_sample_rate = (uint8_t)SendDlgItemMessage(hDlg, IDC_COMBO_SRATE, CB_GETCURSEL, 0, 0);
	config.wav_sample_bits = (uint8_t)SendDlgItemMessage(hDlg, IDC_COMBO_SBITS, CB_GETCURSEL, 0, 0);

	// I/O port address
	for (int i=IOPORT_STARTNUM; i<IOPORT_NUMS; i++) {
		if (IsDlgButtonChecked(hDlg, IDC_CHK_IOPORT1 + i))  {
			io_port |= (1 << i);
		} else {
			io_port &= ~(1 << i);
		}
	}
	// crtc
	config.disptmg_skew = (uint8_t)SendDlgItemMessage(hDlg, IDC_COMBO_DISPTMG, CB_GETCURSEL, 0, 0);
	config.curdisp_skew = (uint8_t)SendDlgItemMessage(hDlg, IDC_COMBO_CURDISP, CB_GETCURSEL, 0, 0);

#ifdef USE_DIRECT3D
	// d3d use
	config.use_direct3d = (uint8_t)SendDlgItemMessage(hDlg, IDC_COMBO_D3D_USE, CB_GETCURSEL, 0, 0);

	// d3d filter type
	config.d3d_filter_type = (uint8_t)SendDlgItemMessage(hDlg, IDC_COMBO_D3D_FILTER, CB_GETCURSEL, 0, 0);
#endif
#ifdef USE_OPENGL
	// opengl use
	config.use_opengl = (uint8_t)SendDlgItemMessage(hDlg, IDC_COMBO_OPENGL_USE, CB_GETCURSEL, 0, 0);

	// opengl filter type
	config.gl_filter_type = (uint8_t)SendDlgItemMessage(hDlg, IDC_COMBO_OPENGL_FILTER, CB_GETCURSEL, 0, 0);
#endif

	// led show
	int led_show = (int)SendDlgItemMessage(hDlg, IDC_COMBO_LED_SHOW, CB_GETCURSEL, 0, 0);
	// led pos
	config.led_pos = (uint8_t)SendDlgItemMessage(hDlg, IDC_COMBO_LED_POS, CB_GETCURSEL, 0, 0);

	// capture type
	config.capture_type = (uint8_t)SendDlgItemMessage(hDlg, IDC_COMBO_CAPTURE_TYPE, CB_GETCURSEL, 0, 0);

	// snapshot path
	GetDlgItemText(hDlg, IDC_SNAP_PATH, buf, _MAX_PATH);
	config.snapshot_path.Set(buf);
	// font file / path
	GetDlgItemText(hDlg, IDC_FONT_FILE, buf, _MAX_PATH);
	config.font_path.Set(buf);
	// message font name
	GetDlgItemText(hDlg, IDC_MSG_FONT_NAME_S, buf, _MAX_PATH);
	config.msgboard_msg_fontname.Set(buf);
	GetDlgItemText(hDlg, IDC_MSG_FONT_NAME_L, buf, _MAX_PATH);
	config.msgboard_info_fontname.Set(buf);
	// message font size
	int valuel;
	BOOL rc;
	valuel = GetDlgItemInt(hDlg, IDC_MSG_FONT_SIZE_L, &rc, TRUE);
	if (rc == TRUE && 1 <= valuel && valuel <= 60) {
		config.msgboard_info_fontsize = (uint8_t)valuel;
	}
	valuel = GetDlgItemInt(hDlg, IDC_MSG_FONT_SIZE_S, &rc, TRUE);
	if (rc == TRUE && 1 <= valuel && valuel <= 60) {
		config.msgboard_msg_fontsize = (uint8_t)valuel;
	}

	// hostname, port
	for(int i=0; i<MAX_PRINTER; i++) {
		GetDlgItemText(hDlg, IDC_HOSTNAME_LPT0 + i, buf, _MAX_PATH);
		config.printer_server_host[i].Set(buf);
		valuel = GetDlgItemInt(hDlg, IDC_PORT_LPT0 + i, &rc, TRUE);
		if (rc == TRUE && 0 <= valuel && valuel <= 65535) {
			config.printer_server_port[i] = valuel;
		}
	}
	for(int i=0; i<MAX_COMM; i++) {
		GetDlgItemText(hDlg, IDC_HOSTNAME_COM0 + i, buf, _MAX_PATH);
		config.comm_server_host[i].Set(buf);
		valuel = GetDlgItemInt(hDlg, IDC_PORT_COM0 + i, &rc, TRUE);
		if (rc == TRUE && 0 <= valuel && valuel <= 65535) {
			config.comm_server_port[i] = valuel;
		}
		valuel = (int)SendDlgItemMessage(hDlg, IDC_COMBO_COM0 + i, CB_GETCURSEL, 0, 0);
		if (valuel >= 0 && valuel <= 3) {
			config.comm_dipswitch[i] = valuel + 1;
		}
	}
#ifdef USE_DEBUGGER
	GetDlgItemText(hDlg, IDC_HOSTNAME_DBGR, buf, _MAX_PATH);
	config.debugger_server_host.Set(buf);
	valuel = GetDlgItemInt(hDlg, IDC_PORT_DBGR, &rc, TRUE);
	if (rc == TRUE && 0 <= valuel && valuel <= 65535) {
		config.debugger_server_port = valuel;
	}
#endif

	// rom path
	GetDlgItemText(hDlg, IDC_ROM_PATH, buf, _MAX_PATH);
	config.rom_path.Set(buf);

	// extended memory
	config.exram_size_num = IsDlgButtonChecked(hDlg, IDC_CHK_EXMEM) ? 1 : 0;

	// undef opcode
	config.show_dlg = (IsDlgButtonChecked(hDlg, IDC_CHK_UNDEFOP) == BST_CHECKED ? config.show_dlg | MSK_SHOWMSG_UNDEFOP : config.show_dlg & ~MSK_SHOWMSG_UNDEFOP);

	emu->set_parami(VM::ParamFddType, fdd_type);
	emu->set_parami(VM::ParamIOPort, io_port);

	// message font
#ifdef USE_MESSAGE_BOARD
	MsgBoard *msgboard = emu->get_msgboard();
	if (msgboard) {
		msgboard->SetFont();
	}
#endif

	gui->ChangeLedBox(led_show);
	gui->ChangeLedBoxPosition(config.led_pos);

	config.save();
#ifdef USE_OPENGL
	emu->change_opengl_attr();
#endif
	return (INT_PTR)TRUE;
}

#if 0
INT_PTR ConfigBox::onControlColorStatic(UINT message, WPARAM wParam, LPARAM lParam)
{
	HANDLE h = (HANDLE)GetStockObject(NULL_BRUSH);
	SetBkMode((HDC)wParam, TRANSPARENT);
	return (INT_PTR)h;
}
#endif

static int ctrl_ids0[] = {
	IDC_CHK_POWEROFF, IDC_STATIC_MODE_SW, ID_DIPSWITCH3, IDC_STATIC_1,
	IDC_STATIC_ALLOW, IDC_RADIO_NOFDD, IDC_RADIO_3FDD, IDC_RADIO_5FDD, IDC_RADIO_8FDD, IDC_STATIC_2,
#ifdef USE_IOPORT_FDD
	IDC_STATIC_IOPORT1, IDC_STATIC_IOPORT2,
#endif
	IDC_STATIC_IOPORT3, IDC_STATIC_IOPORT4, IDC_STATIC_IOPORT5, IDC_STATIC_IOPORT6, IDC_STATIC_IOPORT7,
#ifdef USE_IOPORT_FDD
	IDC_CHK_IOPORT1, IDC_CHK_IOPORT2,
#endif
	IDC_CHK_IOPORT3, IDC_CHK_IOPORT4, IDC_CHK_IOPORT5, IDC_CHK_IOPORT6, IDC_CHK_IOPORT7, IDC_STATIC_3,
	IDC_STATIC_RESTART,
	-1
};
static int ctrl_ids1[] = {
#ifdef USE_DIRECT3D
	IDC_STATIC_D3D_USE, IDC_COMBO_D3D_USE, IDC_STATIC_D3D_FILTER, IDC_COMBO_D3D_FILTER, IDC_STATIC_7,
#endif
#ifdef USE_OPENGL
	IDC_STATIC_OPENGL_USE, IDC_COMBO_OPENGL_USE, IDC_STATIC_OPENGL_FILTER, IDC_COMBO_OPENGL_FILTER, IDC_STATIC_7,
#endif
	IDC_STATIC_LED_SHOW, IDC_COMBO_LED_SHOW, IDC_STATIC_LED_POS, IDC_COMBO_LED_POS,
	IDC_STATIC_CAPTURE_TYPE, IDC_COMBO_CAPTURE_TYPE,
	IDC_STATIC_DISPTMG, IDC_STATIC_CURDISP, IDC_COMBO_DISPTMG, IDC_COMBO_CURDISP, IDC_STATIC_6,
	IDC_STATIC_SNAP_PATH, IDC_SNAP_PATH, IDC_BTN_SNAP_PATH, IDC_STATIC_FONT_FILE, IDC_FONT_FILE, IDC_BTN_FONT_FILE,
	IDC_STATIC_MSG_FONT_NAME_L, IDC_MSG_FONT_NAME_L, IDC_STATIC_MSG_FONT_SIZE_L, IDC_MSG_FONT_SIZE_L, IDC_BTN_FONT_NAME_L,
	IDC_STATIC_MSG_FONT_NAME_S, IDC_MSG_FONT_NAME_S, IDC_STATIC_MSG_FONT_SIZE_S, IDC_MSG_FONT_SIZE_S, IDC_BTN_FONT_NAME_S,
	IDC_STATIC_RESTART_2,
	-1
};
static int ctrl_ids2[] = {
	IDC_CHK_REVERSE, IDC_CHK_HALFWAVE, IDC_CHK_CORRECT, IDC_RADIO_COSW, IDC_RADIO_SINW, IDC_STATIC_4,
	IDC_STATIC_SRATE, IDC_STATIC_SBITS, IDC_COMBO_SRATE, IDC_COMBO_SBITS, IDC_STATIC_5,
	IDC_STATIC_FDD_MOUNT, IDC_CHK_FDD_MOUNT0, IDC_CHK_FDD_MOUNT1, IDC_CHK_FDD_MOUNT2, IDC_CHK_FDD_MOUNT3,
	IDC_CHK_DELAYFD1, IDC_CHK_DELAYFD2, IDC_STATIC_8,
	-1
};
static int ctrl_ids3[] = {
	IDC_STATIC_NET0, IDC_STATIC_LPT0, IDC_STATIC_LPT1, IDC_STATIC_LPT2, IDC_STATIC_COM0, IDC_STATIC_COM1,
	IDC_HOSTNAME_NET0, IDC_HOSTNAME_LPT0, IDC_HOSTNAME_LPT1, IDC_HOSTNAME_LPT2, IDC_HOSTNAME_COM0, IDC_HOSTNAME_COM1,
	IDC_PORT_NET0, IDC_PORT_LPT0, IDC_PORT_LPT1, IDC_PORT_LPT2, IDC_PORT_COM0, IDC_PORT_COM1,
	IDC_COMBO_COM0, IDC_COMBO_COM1,
#ifdef USE_DEBUGGER
	IDC_STATIC_DBGR, IDC_HOSTNAME_DBGR, IDC_PORT_DBGR,
#endif
	-1
};
static int ctrl_ids4[] = {
	IDC_STATIC_ROM_PATH, IDC_ROM_PATH, IDC_BTN_ROM_PATH, IDC_CHK_EXMEM,
	IDC_CHK_UNDEFOP,
	IDC_STATIC_RESTART_3,
	-1
};

void ConfigBox::select_tabctrl(int tab_num)
{
	HWND hCtrl;
	int cmdShow;

	selected_tabctrl = tab_num;
	for(int i=0; i<5; i++) {
		cmdShow = (selected_tabctrl == i ? SW_SHOW : SW_HIDE);
		switch(i) {
		case 0:
			// Mode
			for(int j=0; ctrl_ids0[j] > 0; j++) {
				hCtrl = GetDlgItem(hDlg, ctrl_ids0[j]);
				ShowWindow(hCtrl, cmdShow);
			}
			if (selected_tabctrl == i) {
				// Mode switch
				hCtrl = GetDlgItem(hDlg, IDC_STATIC_MODE_SW);
				ShowWindow(hCtrl, (config.dipswitch & 4) != 0 ? SW_SHOWNORMAL : SW_HIDE);
				// I/O port address
				for (int i=IOPORT_STARTNUM; i<IOPORT_NUMS; i++) {
					CheckDlgButton(hDlg, IDC_CHK_IOPORT1 + i, (io_port & (1 << i)) != 0);
					hCtrl = GetDlgItem(hDlg, IDC_STATIC_IOPORT1 + i);
					ShowWindow(hCtrl, (config.io_port & (1 << i)) != 0 ? SW_SHOWNORMAL : SW_HIDE);
				}
#ifdef USE_IOPORT_FDD
				if (fdd_type == 1) {
					// I/O port (3inch)
					CheckDlgButton(hDlg, IDC_CHK_IOPORT1, 0);
					hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT1);
					EnableWindow(hCtrl, false);
					CheckDlgButton(hDlg, IDC_CHK_IOPORT2, 1);
					hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT2);
					EnableWindow(hCtrl, false);
				}
				else if ((fdd_type & 2) == 2) {
					// I/O port (5inch)
					CheckDlgButton(hDlg, IDC_CHK_IOPORT1, 1);
					hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT1);
					EnableWindow(hCtrl, false);
					CheckDlgButton(hDlg, IDC_CHK_IOPORT2, 0);
					hCtrl = GetDlgItem(hDlg, IDC_CHK_IOPORT2);
					EnableWindow(hCtrl, false);
				}
#endif
			}
			break;
		case 1:
			// Screen
			for(int j=0; ctrl_ids1[j] > 0; j++) {
				hCtrl = GetDlgItem(hDlg, ctrl_ids1[j]);
				ShowWindow(hCtrl, cmdShow);
			}
			break;
		case 2:
			// Tape
			for(int j=0; ctrl_ids2[j] > 0; j++) {
				hCtrl = GetDlgItem(hDlg, ctrl_ids2[j]);
				ShowWindow(hCtrl, cmdShow);
			}
			break;
		case 3:
			// Network
			for(int j=0; ctrl_ids3[j] > 0; j++) {
				hCtrl = GetDlgItem(hDlg, ctrl_ids3[j]);
				ShowWindow(hCtrl, cmdShow);
			}
			break;
		case 4:
			// Misc
			for(int j=0; ctrl_ids4[j] > 0; j++) {
				hCtrl = GetDlgItem(hDlg, ctrl_ids4[j]);
				ShowWindow(hCtrl, cmdShow);
			}
			break;
		}
	}
}

}; /* namespace GUI_WIN */
