﻿/** @file win_dialogbox.cpp

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2015.01.21

	@brief [ dialog box ]
*/

#include "win_dialogbox.h"
#include "win_gui.h"

namespace GUI_WIN
{

CDialogBox::CDialogBox(HINSTANCE new_inst, DWORD new_id, CFont *new_font, EMU *new_emu, GUI *new_gui)
{
	hInstance = new_inst;
	dialogId  = new_id;
	hDlg      = NULL;
	font      = new_font;
	emu       = new_emu;
	gui       = new_gui;
	isModeless = false;

	padding   = 2;
	margin    = 10;

	// size of dialog
	re_max.left = 0;
	re_max.top = 0;
	re_max.right = 0;
	re_max.bottom = 0;

//	hToolTips = NULL;
}

CDialogBox::~CDialogBox()
{
	Close();
}

/// create modal dialog box
INT_PTR CDialogBox::Show(HWND hWnd)
{
	return ::DialogBoxParam(hInstance, MAKEINTRESOURCE(dialogId), hWnd, Proc, (LPARAM)this);
}

/// create modeless dialog box
HWND CDialogBox::Create(HWND hWnd)
{
	if (hDlg == NULL) {
		hDlg = ::CreateDialogParam(hInstance, MAKEINTRESOURCE(dialogId), hWnd, Proc, (LPARAM)this);
		if (hDlg != NULL) {
			isModeless = true;
		}
	}
	return hDlg;
}

/// close modeless dialog box
void CDialogBox::Close()
{
	if (isModeless && hDlg != NULL) {
		::DestroyWindow(hDlg);
		isModeless = false;
	}
	hDlg = NULL;
}

INT_PTR CALLBACK CDialogBox::Proc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	UNREFERENCED_PARAMETER(lParam);
	CDialogBox *myObj = (CDialogBox *)GetWindowLongPtr(hDlg, DWLP_USER);

	switch (message) {
		case WM_INITDIALOG:
			SetWindowLongPtr(hDlg, DWLP_USER, lParam);
			myObj = (CDialogBox *)lParam;
			myObj->hDlg = hDlg;

			return myObj->onInitDialog(message, wParam, lParam);
		case WM_COMMAND:
			return myObj->onCommand(message, wParam, lParam);
		case WM_NOTIFY:
			return myObj->onNotify(message, wParam, lParam);
		case WM_MOUSEWHEEL:
			return myObj->onMouseWheel(message, wParam, lParam);
		case WM_HSCROLL:
			return myObj->onHScroll(message, wParam, lParam);
		case WM_VSCROLL:
			return myObj->onVScroll(message, wParam, lParam);
		case WM_CLOSE:
			return myObj->onClose(message, wParam, lParam);
		case WM_HELP:
			return myObj->onHelp(message, wParam, lParam);
		case WM_CTLCOLORSTATIC:
			return myObj->onControlColorStatic(message, wParam, lParam);
		case WM_CTLCOLORDLG:
			return myObj->onControlColorDialog(message, wParam, lParam);
		case WM_SIZE:
			return myObj->onSize(message, wParam, lParam);
		break;

	}
	return (INT_PTR)FALSE;
}

INT_PTR CDialogBox::onInitDialog(UINT message, WPARAM wParam, LPARAM lParam)
{
	// get client size on window
//	::GetClientRect(hDlg, &re_max);
	memset(&re_max, 0, sizeof(re_max));

	// set font and translate text
	SetFontAndTranslateText();

	// create tooltip
//	DWORD dwStyle = WS_POPUP
//		| TTS_NOPREFIX; // | TTS_BALLOON | TTS_ALWAYSTIP | TTS_NOANIMATE | TTS_NOFADE;
//	hToolTips = CreateWindowEx(WS_EX_TOPMOST, TOOLTIPS_CLASS, NULL, dwStyle
//		, 0, 0, 0, 0
//		, hDlg, NULL, hInstance, NULL);

	return 0;
}

INT_PTR CDialogBox::onCommand(UINT message, WPARAM wParam, LPARAM lParam)
{
	if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) {
		::EndDialog(hDlg, LOWORD(wParam));
		hDlg = NULL;
		return (INT_PTR)TRUE;
	}
	return (INT_PTR)FALSE;
}

INT_PTR CDialogBox::onNotify(UINT message, WPARAM wParam, LPARAM lParam)
{
	return (INT_PTR)TRUE;
}

INT_PTR CDialogBox::onMouseWheel(UINT message, WPARAM wParam, LPARAM lParam)
{
	return (INT_PTR)TRUE;
}

INT_PTR CDialogBox::onHScroll(UINT message, WPARAM wParam, LPARAM lParam)
{
	return (INT_PTR)FALSE;
}

INT_PTR CDialogBox::onVScroll(UINT message, WPARAM wParam, LPARAM lParam)
{
	return (INT_PTR)FALSE;
}

INT_PTR CDialogBox::onOK(UINT message, WPARAM wParam, LPARAM lParam)
{
	return (INT_PTR)TRUE;
}

INT_PTR CDialogBox::onClose(UINT message, WPARAM wParam, LPARAM lParam)
{
	return (INT_PTR)FALSE;
}

INT_PTR CDialogBox::onHelp(UINT message, WPARAM wParam, LPARAM lParam)
{
	return (INT_PTR)FALSE;
}

INT_PTR CDialogBox::onSize(UINT message, WPARAM wParam, LPARAM lParam)
{
	return (INT_PTR)FALSE;
}

INT_PTR CDialogBox::onControlColorStatic(UINT message, WPARAM wParam, LPARAM lParam)
{
	return (INT_PTR)FALSE;
}

INT_PTR CDialogBox::onControlColorDialog(UINT message, WPARAM wParam, LPARAM lParam)
{
	return (INT_PTR)FALSE;
}

void CDialogBox::SetFontAndTranslateText()
{
	// set font
	SendMessage(hDlg, WM_SETFONT, (WPARAM)font->GetFont(), MAKELPARAM(TRUE, 0));
#ifdef USE_GETTEXT
	// translate title string on window
	_TCHAR buf[_MAX_PATH];
	if (GetWindowText(hDlg, buf, sizeof(buf))) {
		SetWindowText(hDlg, _tgettext(buf));
	}
#endif
	// set font and translate text string in child controls
	EnumChildWindows(hDlg, ChildProc, (LPARAM)this);
}

BOOL CALLBACK CDialogBox::ChildProc(HWND hCtrl, LPARAM lParam)
{
	CDialogBox *myObj = (CDialogBox *)lParam;
	// set font
	SendMessage(hCtrl, WM_SETFONT, (WPARAM)myObj->font->GetFont(), MAKELPARAM(TRUE, 0));
#ifdef USE_GETTEXT
	// translate text string
	_TCHAR buf[_MAX_PATH];
	if (GetWindowText(hCtrl, buf, sizeof(buf))) {
		SetWindowText(hCtrl, _tgettext(buf));
	}
#endif
	return TRUE;
}

#if 0
void CDialogBox::AddToolTipText(const t_tooltip_text *texts)
{
	if (!hToolTips) return;

	TOOLINFO ti;

	memset(&ti, 0, sizeof(ti));
	ti.cbSize = sizeof(TOOLINFO);
	ti.hwnd = hDlg;
	ti.uFlags = TTF_SUBCLASS | TTF_IDISHWND;
	for(int n = 0; texts[n].text != NULL; n++) {
		ti.uId = (UINT_PTR)GetDlgItem(hDlg, texts[n].id);
		ti.lpszText = (LPTSTR)_tgettext(texts[n].text);
	    SendMessage(hToolTips, TTM_ADDTOOL, 0, (LPARAM)&ti);
	}
}
#endif

void CDialogBox::Larger(RECT *re)
{
	if (re_max.right < re->right) {
		re_max.right = re->right;
	}
	if (re_max.bottom < re->bottom) {
		re_max.bottom = re->bottom;
	}
}

void CDialogBox::Larger(LONG width, LONG height)
{
	if (re_max.right < width) {
		re_max.right = width;
	}
	if (re_max.bottom < height) {
		re_max.bottom = height;
	}
}

HWND CDialogBox::CreateStatic(int nItemId, const _TCHAR *label, RECT *re, bool align_right)
{
	DWORD dwStyle = WS_CHILD | WS_VISIBLE | SS_LEFT;
	HWND hCtrl = CreateWindowEx(0, WC_STATIC, label, dwStyle, 0, 0, 10, 10, hDlg, (HMENU)nItemId, hInstance, NULL);
	if (hCtrl) {
		SendMessage(hCtrl, WM_SETFONT, (WPARAM)font->GetFont(), MAKELPARAM(TRUE, 0));
		AdjustStatic(nItemId, re, align_right);
	}
	return hCtrl;
}

HWND CDialogBox::CreateComboBox(int nItemId, const _TCHAR **list, int selnum, int nMinSize, RECT *re, bool align_right, bool translate)
{
	DWORD dwStyle = WS_CHILD | WS_VISIBLE | CBS_DROPDOWNLIST | CBS_HASSTRINGS;
	HWND hCtrl = CreateWindow(WC_COMBOBOX, "", dwStyle , 0, 0, 10, 10, hDlg, (HMENU)nItemId, hInstance, NULL);
	if (hCtrl) {
		SendMessage(hCtrl, WM_SETFONT, (WPARAM)font->GetFont(), MAKELPARAM(TRUE, 0));
		SetComboBoxItems(hCtrl, list, selnum, translate);
		AdjustComboBox(nItemId, nMinSize, re, align_right);
	}
	return hCtrl;
}

void CDialogBox::SetComboBoxItems(int nItemId, const _TCHAR **list, int selnum, bool translate)
{
	SetComboBoxItems(GetDlgItem(hDlg, nItemId), list, selnum, translate);
}

void CDialogBox::SetComboBoxItems(HWND combo, const _TCHAR **list, int selnum, bool translate)
{
	for(int n=0; list[n] != NULL; n++) {
		const _TCHAR *label = list[n];
		if (translate) label = _tgettext(label);
		SendMessage(combo, CB_ADDSTRING, n, (LPARAM)(LPTSTR)label);
	}
	SendMessage(combo, CB_SETCURSEL, selnum, 0);
}

HWND CDialogBox::CreateDefaultButton(int nItemId, const _TCHAR *caption, int nSize, RECT *re, bool align_right)
{
	DWORD dwStyle = WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_DEFPUSHBUTTON;
	HWND hCtrl = CreateWindowEx(0, WC_BUTTON, caption, dwStyle, 0, 0, 10, 10, hDlg, (HMENU)nItemId, hInstance, NULL);
	if (hCtrl) {
		SendMessage(hCtrl, WM_SETFONT, (WPARAM)font->GetFont(), MAKELPARAM(TRUE, 0));
		AdjustButton(nItemId, nSize, re, align_right);
	}
	return hCtrl;
}

HWND CDialogBox::CreateButton(int nItemId, const _TCHAR *caption, int nSize, RECT *re, bool align_right)
{
	DWORD dwStyle = WS_CHILD | WS_VISIBLE | WS_TABSTOP | BS_PUSHBUTTON;
	HWND hCtrl = CreateWindowEx(0, WC_BUTTON, caption, dwStyle, 0, 0, 10, 10, hDlg, (HMENU)nItemId, hInstance, NULL);
	if (hCtrl) {
		SendMessage(hCtrl, WM_SETFONT, (WPARAM)font->GetFont(), MAKELPARAM(TRUE, 0));
		AdjustButton(nItemId, nSize, re, align_right);
	}
	return hCtrl;
}

void CDialogBox::AdjustButton(int nItemId, int nSize, RECT *re, bool align_right)
{
	_TCHAR buf[_MAX_PATH];
	SIZE siz = { 0, 0 };

	if (!GetDlgItemText(hDlg, nItemId, buf, sizeof(buf))) {
		_tcscpy(buf, _T("0"));
	}
	font->GetTextSize(hDlg, buf, &siz);

	siz.cx += (siz.cy * 2);
	if (siz.cx < siz.cy * nSize) {
		siz.cx = siz.cy * nSize;
	}
	if (align_right) {
		re->left = re->right - siz.cx;
	} else {
		re->right = siz.cx + re->left;
	}
	re->bottom = siz.cy + padding * 4 + re->top;
	MoveWindow(GetDlgItem(hDlg, nItemId), re->left, re->top, re->right - re->left, re->bottom - re->top, TRUE);

	Larger(re);
}


void CDialogBox::AdjustStatic(int nItemId, RECT *re, bool align_right)
{
	_TCHAR buf[_MAX_PATH];
	SIZE siz = { 0, 0 };

	// static text
	if (GetDlgItemText(hDlg, nItemId, buf, sizeof(buf))) {
		font->GetTextSize(hDlg, buf, &siz);
	}
	if (align_right) {
		re->left = re->right - siz.cx;
	} else {
		re->right = siz.cx + re->left;
	}
	LONG top = re->top + padding;
	re->bottom = siz.cy + padding * 2 + top;
	MoveWindow(GetDlgItem(hDlg, nItemId), re->left, top, re->right - re->left, re->bottom - top, TRUE);
	re->bottom += padding;

	Larger(re);
}

void CDialogBox::AdjustCheckBox(int nItemId, RECT *re, bool align_right)
{
	_TCHAR buf[_MAX_PATH];
	SIZE siz = { 0, 0 };

	// checkbox
	if (GetDlgItemText(hDlg, nItemId, buf, sizeof(buf))) {
		font->GetTextSize(hDlg, buf, &siz);
	}
	if (align_right) {
		re->left = re->right - siz.cx - 20;
	} else {
		re->right = siz.cx + 20 + re->left;
	}
	LONG top = re->top + padding;
	re->bottom = siz.cy + padding * 3 + top;
	MoveWindow(GetDlgItem(hDlg, nItemId), re->left, top, re->right - re->left, re->bottom - top, TRUE);

	Larger(re);
}

void CDialogBox::AdjustEditBox(int nItemId, int nMaxSize, RECT *re, bool align_right, const _TCHAR ch)
{
	_TCHAR buf[2] = { ch, _T('\0') };
	SIZE siz = { 0, 0 };

	// edit box
	font->GetTextSize(hDlg, buf, &siz);

	if (align_right) {
		re->left = re->right - padding - siz.cx * nMaxSize - padding;
	} else {
		re->right = padding + siz.cx * nMaxSize + padding + re->left;
	}
	re->bottom = siz.cy + padding * 4 + re->top;
	MoveWindow(GetDlgItem(hDlg, nItemId), re->left, re->top, re->right - re->left, re->bottom - re->top, TRUE);

	Larger(re);
}

void CDialogBox::AdjustEditBox(int nItemId, RECT *re, const _TCHAR *cLenStr, bool align_right)
{
	SIZE siz = { 0, 0 };

	// edit box
	font->GetTextSize(hDlg, cLenStr, &siz);

	if (align_right) {
		re->left = re->right - padding - siz.cx - padding;
	} else {
		re->right = padding + siz.cx + padding + re->left;
	}
	re->bottom = siz.cy + padding * 3 + re->top;
	MoveWindow(GetDlgItem(hDlg, nItemId), re->left, re->top, re->right - re->left, re->bottom - re->top, TRUE);

	Larger(re);
}

void CDialogBox::AdjustEditBoxWithStatic(int nStaticItemId, int nEditItemId, int nEditMaxSize, RECT *re, bool align_right)
{
	LONG left = re->left;
	LONG right = re->right;

	// static text
	AdjustStatic(nStaticItemId, re, align_right);
	// edit box
	if (align_right) {
		re->right = re->left - padding;
	} else {
		re->left = re->right + padding;
	}
	AdjustEditBox(nEditItemId, nEditMaxSize, re, align_right);
	if (align_right) {
		re->right = right;
	} else {
		re->left = left;
	}
}

void CDialogBox::AdjustComboBox(int nItemId, int nMinSize, RECT *re, bool align_right)
{
	_TCHAR buf[_MAX_PATH];
	SIZE siz = { 0, 0 };

	// combo box
	int count = (int)SendDlgItemMessage(hDlg, nItemId, CB_GETCOUNT, 0, 0);
	int max_len = 0;
	for(int n = 0; n < count; n++) {
		int len = (int)SendDlgItemMessage(hDlg, nItemId, CB_GETLBTEXTLEN, (WPARAM)n, 0);
		if (len > max_len) {
			SendDlgItemMessage(hDlg, nItemId, CB_GETLBTEXT, (WPARAM)n, (LPARAM)buf);
			max_len = len;
		}
	}
	if (max_len == 0) {
		_tcscpy(buf, _T("m"));
	}
	font->GetTextSize(hDlg, buf, &siz);

	if (siz.cx < siz.cy * nMinSize) {
		siz.cx = siz.cy * nMinSize;
	}
	if (align_right) {
		re->left = re->right - padding - siz.cx - padding - siz.cy - padding;
	} else {
		re->right = padding + siz.cx + padding + siz.cy + padding + re->left;
	}
	re->bottom = siz.cy + padding * 4 + re->top;
	MoveWindow(GetDlgItem(hDlg, nItemId), re->left, re->top, re->right - re->left, re->bottom - re->top, TRUE);

	Larger(re);
}

void CDialogBox::AdjustComboBoxWithStatic(int nStaticItemId, int nComboItemId, int nComboMinSize, RECT *re, bool align_right)
{
	LONG left = re->left;
	LONG right = re->right;

	// static text
	AdjustStatic(nStaticItemId, re, align_right);
	// combo box
	if (align_right) {
		re->right = re->left - padding;
	} else {
		re->left = re->right + padding;
	}
	AdjustComboBox(nComboItemId, nComboMinSize, re, align_right);
	if (align_right) {
		re->right = right;
	} else {
		re->left = left;
	}
}

void CDialogBox::AdjustControl(int nItemId, RECT *re, int w, int h, bool align_right, int style)
{
	if (style != 0) {
		int val = GetSystemMetrics(style);
		switch(style) {
		case SM_CXVSCROLL:
			w += val;
			break;
		case SM_CYHSCROLL:
			h += val;
			break;
		}
	}
	if (align_right) {
		re->left = re->right - w;
	} else {
		re->right = w + re->left;
	}
	re->bottom = h + re->top;
	MoveWindow(GetDlgItem(hDlg, nItemId), re->left, re->top, re->right - re->left, re->bottom - re->top, TRUE);

	Larger(re);
}


void CDialogBox::AdjustWindow(int lmargin)
{
	WINDOWINFO wi;
	GetWindowInfo(hDlg, &wi);

//	RECT scn_re;
//	GetWindowRect(GetDesktopWindow(), &scn_re);
	re_max.right = re_max.right + lmargin;
	re_max.bottom = re_max.bottom + lmargin;

	RECT win_re;
	win_re.left = 0;
	win_re.top = 0;
	win_re.right = re_max.right;
	win_re.bottom = re_max.bottom;
	// get window size included border size
	AdjustWindowRectEx(&win_re, wi.dwStyle, FALSE, wi.dwExStyle);
	// resize window
	win_re.right = win_re.right - win_re.left;	// width
	win_re.bottom = win_re.bottom - win_re.top;	// height
//	win_re.left = (scn_re.right - win_re.right) / 2;
//	win_re.top  = (scn_re.bottom - win_re.bottom) / 2;
	win_re.left = wi.rcWindow.left;	// position on screen
	win_re.top  = wi.rcWindow.top;	// position on screen
	MoveWindow(hDlg, win_re.left, win_re.top, win_re.right, win_re.bottom, TRUE);

}

void CDialogBox::AdjustControlSize(int nItemId, int diff_x, int diff_y)
{
	HWND hCtrl;
	if (nItemId > 0) {
		hCtrl = GetDlgItem(hDlg, nItemId);
	} else {
		hCtrl = hDlg;
	}
	if (hCtrl == NULL) return;
	WINDOWINFO wi;
	GetWindowInfo(hCtrl, &wi);
	int width = wi.rcWindow.right - wi.rcWindow.left + diff_x;
	int height = wi.rcWindow.bottom - wi.rcWindow.top + diff_y;
	SetWindowPos(hCtrl, NULL, 0, 0, width, height, SWP_NOMOVE | SWP_NOZORDER);

	if (nItemId > 0) {
		RECT re;
		GetControlPos(nItemId, &re);
		Larger(&re);
	}
}

void CDialogBox::AdjustControlPos(int nItemId, int diff_x, int diff_y)
{
	HWND hCtrl = GetDlgItem(hDlg, nItemId);
	if (hCtrl == NULL) return;
	RECT re;
	GetControlPos(nItemId, &re);
	re.left += diff_x;
	re.top += diff_y;
	SetWindowPos(hCtrl, NULL, re.left, re.top, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
}

int CDialogBox::GetControlXdiff(int nItemId, int new_width)
{
	WINDOWINFO wi;
	HWND hCtrl = GetDlgItem(hDlg, nItemId);
	GetWindowInfo(hCtrl, &wi);
	return new_width - wi.rcClient.right + wi.rcClient.left;
}

/// コントロールのウィンドウ上の相対位置を得る
void CDialogBox::GetControlPos(int nItemId, RECT *re)
{
	HWND hCtrl = GetDlgItem(hDlg, nItemId);
	if (hCtrl == NULL) return;
	WINDOWINFO wiDlg;
	GetWindowInfo(hDlg, &wiDlg);
	WINDOWINFO wi;
	GetWindowInfo(hCtrl, &wi);
	if (re != NULL) {
		re->left = wi.rcWindow.left - wiDlg.rcClient.left;
		re->top =  wi.rcWindow.top  - wiDlg.rcClient.top;
		re->right  = wi.rcWindow.right - wiDlg.rcClient.left;
		re->bottom = wi.rcWindow.bottom  - wiDlg.rcClient.top;
	}
}

/// コントロールのサイズを得る
void CDialogBox::GetControlSize(int nItemId, SIZE *size)
{
	HWND hCtrl = GetDlgItem(hDlg, nItemId);
	if (hCtrl == NULL) return;
	RECT re;
	::GetClientRect(hCtrl, &re);
	if (size != NULL) {
		size->cx = re.right - re.left;
		size->cy = re.bottom - re.top;
	}
}

void CDialogBox::EnableDlgItem(int nItemId, bool value)
{
	::EnableWindow(::GetDlgItem(hDlg, nItemId), value);
}

}; /* namespace GUI_WIN */
