﻿/** @file win_dialogbox.h

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2015.01.21

	@brief [ dialog box ]
*/

#ifndef _DIALOGBOX_H_
#define _DIALOGBOX_H_

#include <windows.h>
#include <commctrl.h>
#include "../../clocale.h"
#include "winfont.h"
#include "../../common.h"
#include "../../emu.h"
#include "../../res/resource.h"

class GUI;

namespace GUI_WIN
{

/**
	@brief dialog box template
*/
class CDialogBox
{
protected:
	HINSTANCE   hInstance;
	DWORD       dialogId;
	HWND        hDlg;
	bool        isModeless;
	CFont      *font;
//	HWND		hToolTips;
	EMU        *emu;
	GUI        *gui;

	int padding;
	int margin;
	RECT re_max;

	virtual INT_PTR onInitDialog(UINT, WPARAM, LPARAM);
	virtual INT_PTR onCommand(UINT, WPARAM, LPARAM);
	virtual INT_PTR onNotify(UINT, WPARAM, LPARAM);
	virtual INT_PTR onMouseWheel(UINT, WPARAM, LPARAM);
	virtual INT_PTR onHScroll(UINT, WPARAM, LPARAM);
	virtual INT_PTR onVScroll(UINT, WPARAM, LPARAM);
	virtual INT_PTR onSize(UINT, WPARAM, LPARAM);
	virtual INT_PTR onOK(UINT, WPARAM, LPARAM);
	virtual INT_PTR onClose(UINT, WPARAM, LPARAM);
	virtual INT_PTR onHelp(UINT, WPARAM, LPARAM);
	virtual INT_PTR onControlColorStatic(UINT, WPARAM, LPARAM);
	virtual INT_PTR onControlColorDialog(UINT, WPARAM, LPARAM);

	static INT_PTR CALLBACK Proc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);

//	virtual void AddToolTipText(const t_tooltip_text *texts);

private:
	static BOOL CALLBACK ChildProc(HWND hCtrl, LPARAM lParam);

	void Larger(RECT *re);
	void Larger(LONG width, LONG height);

public:
	CDialogBox(HINSTANCE new_inst, DWORD new_id, CFont *new_font, EMU *new_emu, GUI *new_gui);
	virtual ~CDialogBox();

	virtual INT_PTR Show(HWND hWnd);
	virtual HWND Create(HWND hWnd);
	virtual void Close();
	virtual void SetFontAndTranslateText();

	virtual HWND CreateStatic(int nItemId, const _TCHAR *label, RECT *re, bool align_right = false);
	virtual HWND CreateComboBox(int nItemId, const _TCHAR **list, int selnum, int nMinSize, RECT *re, bool align_right = false, bool translate = false);
	virtual void SetComboBoxItems(int nItemId, const _TCHAR **list, int selnum, bool translate = false);
	virtual void SetComboBoxItems(HWND combo, const _TCHAR **list, int selnum, bool translate = false);

	virtual HWND CreateDefaultButton(int nItemId, const _TCHAR *caption, int nSize, RECT *re, bool align_right = false);
	virtual HWND CreateButton(int nItemId, const _TCHAR *caption, int nSize, RECT *re, bool align_right = false);
	virtual void AdjustButton(int nItemId, int nSize, RECT *re, bool align_right = false);
	virtual void AdjustStatic(int nItemId, RECT *re, bool align_right = false);
	virtual void AdjustCheckBox(int nItemId, RECT *re, bool align_right = false);
	virtual void AdjustEditBox(int nItemId, int nMaxSize, RECT *re, bool align_right = false, const _TCHAR ch = _T('m'));
	virtual void AdjustEditBox(int nItemId, RECT *re, const _TCHAR *cLenStr, bool align_right = false);
	virtual void AdjustEditBoxWithStatic(int nStaticItemId, int nEditItemId, int nEditMaxSize, RECT *re, bool align_right = false);
	virtual void AdjustComboBox(int nItemId, int nMinSize, RECT *re, bool align_right = false);
	virtual void AdjustComboBoxWithStatic(int nStaticItemId, int nComboItemId, int nComboMinSize, RECT *re, bool align_right = false);
	virtual void AdjustControl(int nItemId, RECT *re, int w, int h, bool align_right = false, int style = 0);

	virtual void AdjustWindow(int lmargin);

	virtual void AdjustControlSize(int nItemId, int diff_x, int diff_y);
	virtual void AdjustControlPos(int nItemId, int diff_x, int diff_y);
	virtual int  GetControlXdiff(int nItemId, int new_width);
	virtual void GetControlPos(int nItemId, RECT *re);
	virtual void GetControlSize(int nItemId, SIZE *size);

	void EnableDlgItem(int nItemId, bool value);

	int GetRight() { return re_max.right; }
	int GetBottom() { return re_max.bottom; }
};

}; /* namespace GUI_WIN */

#endif /* _DIALOGBOX_H_ */
