/** @file win_filebox.cpp

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2015.03.21 -

	@brief [ file box ]
*/

#include "win_filebox.h"
#include "../../common.h"
#include "../../utility.h"

namespace GUI_WIN
{

FileBox::FileBox(HWND parent_window)
{
	hWnd = parent_window;
	memset(selected_file, 0, sizeof(selected_file));
}

FileBox::~FileBox()
{
}

bool FileBox::Show(const _TCHAR *filter[], const _TCHAR *title, const _TCHAR *dir, const _TCHAR *ext, bool save, _TCHAR *path)
{
	_TCHAR tmp[_MAX_PATH] = _T("");
	_TCHAR app[_MAX_PATH];
	_TCHAR fil[_MAX_PATH];
	_TCHAR subext[_MAX_PATH];

	OPENFILENAME OpenFileName;
	int rc = 0;

	memset(&OpenFileName, 0, sizeof(OpenFileName));
	OpenFileName.lStructSize = sizeof(OPENFILENAME);
	OpenFileName.hwndOwner = hWnd;
	if (filter) {
		int pos = 0;
		memset(fil, 0, sizeof(fil));
		for(int i=0; filter[i] != NULL; i++) {
			_tcsncat(&fil[pos], filter[i], _MAX_PATH - 2);
			pos += (int)_tcslen(filter[i]);
			pos++;
			if (pos >= _MAX_PATH) break;
			if (UTILITY::substr_in_bracket(filter[i], subext) <= 0) continue;

			_tcsncat(&fil[pos], subext, _MAX_PATH - 2);
			pos += (int)_tcslen(subext);
			pos++;
			if (pos >= _MAX_PATH) break;
		}
		OpenFileName.lpstrFilter = fil;
	}
	OpenFileName.lpstrFile = tmp;
	OpenFileName.nMaxFile = _MAX_PATH;
	OpenFileName.lpstrTitle = title;
	if(ext != NULL && ext[0]) {
		OpenFileName.lpstrDefExt = ext;
	}
	if(dir != NULL && dir[0]) {
		OpenFileName.lpstrInitialDir = dir;
	} else if (path != NULL && path[0]) {
		_tcscpy(app, path);
		UTILITY::get_parent_dir(app);
		OpenFileName.lpstrInitialDir = app;
	} else {
		GetModuleFileName(NULL, app, _MAX_PATH);
		UTILITY::get_parent_dir(app);
		OpenFileName.lpstrInitialDir = app;
	}
	if (save) {
		OpenFileName.Flags = OFN_OVERWRITEPROMPT;
		rc = GetSaveFileName(&OpenFileName);
	} else {
		OpenFileName.Flags = 0;
		rc = GetOpenFileName(&OpenFileName);
	}

	flags = OpenFileName.Flags;

	if (rc) {
		_tcsncpy(selected_file, OpenFileName.lpstrFile, _MAX_PATH);
		if (path) _tcscpy(path, selected_file);
		return true;
	}
	return false;
}

}; /* namespace GUI_WIN */
