/** @file win_keybindbox.cpp

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2012.03.31 -

	@brief [ keybind box ]
*/

#include "win_keybindbox.h"
#include "../../emu.h"
#include "../../clocale.h"
#include "win_gui.h"

namespace GUI_WIN
{

KeybindBox::KeybindBox(HINSTANCE hInst, CFont *new_font, EMU *new_emu, GUI *new_gui)
	: CDialogBox(hInst, IDD_KEYBIND, new_font, new_emu, new_gui)
{
}

KeybindBox::~KeybindBox()
{
}

#if 0
void KeybindBox::SetTitleLabel(const _TCHAR *vm, const _TCHAR *vk, const _TCHAR *vj)
{
	_tcscpy(vm_label, vm);
	_tcscpy(vk_label, vk);
	_tcscpy(vj_label, vj);
}
#endif

INT_PTR KeybindBox::onInitDialog(UINT message, WPARAM wParam, LPARAM lParam)
{
	HWND hCtrl;
	TCITEM tcitm;

	CDialogBox::onInitDialog(message, wParam, lParam);

	// create dialog
	selected_tabctrl = 0;

	SIZE siz;
	font->GetTextSize(hDlg, NULL, &siz);
	for(int tab_num=0; tab_num<KEYBIND_MAX_NUM; tab_num++) {
		hCtrl = GetDlgItem(hDlg, IDC_CUSTOM0 + tab_num);
		kbctl[tab_num] = KeybindControl::GetPtr(hCtrl);

		kbctl[tab_num]->Init(emu, tab_num, font->GetFont());

		kbctl[tab_num]->SetCellSize(siz.cx * 16 + padding * 2, siz.cy + padding * 2);

		kbctl[tab_num]->MapDefaultVmKey();

		kbctl[tab_num]->Update();
		if (tab_num == 1) {
			CheckDlgButton(hDlg, IDC_CHK_COMBI1, kbctl[tab_num]->GetCombi() != 0);
		}
	}
	kbctl[0]->SetTitleLabel(_("Level3 Key"), _("Bind"));
	kbctl[1]->SetTitleLabel(_("Level3 Key"), _("Joypad"));
#if (KEYBIND_MAX_NUM > 2)
	kbctl[2]->SetTitleLabel(_("PIA on L3"), _("Joypad"));
#else
	ShowWindow(GetDlgItem(hDlg, IDC_CUSTOM2), SW_HIDE);
#endif

	// ^uRg[
	HWND hTabCtrl = GetDlgItem(hDlg, IDC_TAB1);

	tcitm.mask = TCIF_TEXT;

	tcitm.pszText = (LPSTR)_("Keyboard");
	TabCtrl_InsertItem(hTabCtrl , 0 , &tcitm);
	tcitm.pszText = (LPSTR)_("Joypad");
	TabCtrl_InsertItem(hTabCtrl , 1 , &tcitm);
#if (KEYBIND_MAX_NUM > 2)
	tcitm.pszText = (LPSTR)_("Joypad (PIA Type)");
	TabCtrl_InsertItem(hTabCtrl , 2 , &tcitm);
#endif
	TabCtrl_SetCurSel(hTabCtrl, selected_tabctrl);

	//
	// adjust control size
	//
	RECT re;
	RECT re_tab;
	GetClientRect(hTabCtrl, &re_tab);
	TabCtrl_AdjustRect(hTabCtrl, FALSE, &re_tab);
	re_tab.top += margin + margin;
	re_tab.left += margin + margin;

	// kb control
	re.top = re_tab.top;
	re.left = re_tab.left;
	AdjustControl(IDC_CUSTOM0, &re, kbctl[0]->GetWidth(), 340, false, SM_CXVSCROLL);
	AdjustControl(IDC_CUSTOM1, &re, kbctl[0]->GetWidth(), 340, false, SM_CXVSCROLL);
#if (KEYBIND_MAX_NUM > 2)
	AdjustControl(IDC_CUSTOM2, &re, kbctl[0]->GetWidth(), 340, false, SM_CXVSCROLL);
#endif
	int kb_top = re.top;
	int kb_right = re.right;

	// checkbox size
	re.top = re.bottom + padding;
	AdjustCheckBox(IDC_CHK_COMBI1, &re);

	// buttons
	re.bottom = kb_top - margin;
	re.left = kb_right + margin + margin;
	for(int i=IDC_BTN_LOAD_DEFAULT; i<=IDC_BTN_SAVE_PRESET4; i++) {
		re.top = re.bottom + margin;
		if (i == IDC_BTN_LOAD_PRESET1 || i == IDC_BTN_SAVE_PRESET1) {
			re.top += margin;
		}
		AdjustButton(i, 8, &re);
	}

	// tab control size
	re.top = margin;
	re.left = margin;
	AdjustControl(IDC_TAB1, &re, GetRight() + margin, GetBottom() + margin);
	re_tab.top = re_tab.top - margin + padding;
	re_tab.left = re_tab.left - margin;
	re_tab.bottom = re.bottom - re_tab.top - padding - 4;
	re_tab.right = re.right - re_tab.left - padding - 4;
	AdjustControl(IDC_STATIC_0, &re_tab, re_tab.right, re_tab.bottom);

	// ok cancel
	re.top = GetBottom() + margin;
	re.right = GetRight();
	AdjustButton(IDCANCEL, 8, &re, true);
	re.right = re.left - margin;
	AdjustButton(IDOK, 8, &re, true);

	// dialog size
	AdjustWindow(margin);

	select_tabctrl(selected_tabctrl);

	return (INT_PTR)TRUE;
}

INT_PTR KeybindBox::onCommand(UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(LOWORD(wParam)) {
	case IDOK:
	case IDCANCEL:
		if (LOWORD(wParam) == IDOK) {
			onClickOk();
		}
		::EndDialog(hDlg, LOWORD(wParam));
		return (INT_PTR)TRUE;

	case IDC_BTN_LOAD_DEFAULT:
		return onClickLoadDefault();

	case IDC_BTN_LOAD_PRESET1:
	case IDC_BTN_LOAD_PRESET2:
	case IDC_BTN_LOAD_PRESET3:
	case IDC_BTN_LOAD_PRESET4:
		return onClickLoadPreset(LOWORD(wParam) - IDC_BTN_LOAD_PRESET1);

	case IDC_BTN_SAVE_PRESET1:
	case IDC_BTN_SAVE_PRESET2:
	case IDC_BTN_SAVE_PRESET3:
	case IDC_BTN_SAVE_PRESET4:
		return onClickSavePreset(LOWORD(wParam) - IDC_BTN_SAVE_PRESET1);

	}
	return (INT_PTR)FALSE;
}

INT_PTR KeybindBox::onNotify(UINT message, WPARAM wParam, LPARAM lParam)
{
	// ^u؂ւ
	LPNMHDR lpNmHdr = (NMHDR *)lParam;
	int i;
	if (lpNmHdr->idFrom = IDC_TAB1) {
		switch (lpNmHdr->code) {
		case TCN_SELCHANGE:
			i = TabCtrl_GetCurSel(lpNmHdr->hwndFrom);
			select_tabctrl(i);
			break;
		}
	}
	return (INT_PTR)TRUE;
}

INT_PTR KeybindBox::onMouseWheel(UINT message, WPARAM wParam, LPARAM lParam)
{
	SendDlgItemMessage(hDlg, IDC_CUSTOM0 + selected_tabctrl, message, wParam, lParam);
	return (INT_PTR)TRUE;
}

#if 0
INT_PTR KeybindBox::onControlColorStatic(UINT message, WPARAM wParam, LPARAM lParam)
{
	HANDLE h = (HANDLE)GetStockObject(NULL_BRUSH);
	SetBkMode((HDC)wParam, TRANSPARENT);
	return (INT_PTR)h;
}
#endif

INT_PTR KeybindBox::onClickOk()
{
	for(int tab_num=0; tab_num<KEYBIND_MAX_NUM; tab_num++) {
		kbctl[tab_num]->SetData();
		if (tab_num == 1) {
			kbctl[tab_num]->SetCombi(IsDlgButtonChecked(hDlg, IDC_CHK_COMBI1) ? 1 : 0);
		}
	}
	return (INT_PTR)TRUE;
}

INT_PTR KeybindBox::onClickLoadDefault()
{
	int tab_num = selected_tabctrl;

	kbctl[tab_num]->LoadDefaultPreset();
	if (tab_num == 1) {
		CheckDlgButton(hDlg, IDC_CHK_COMBI1, kbctl[tab_num]->GetCombi() != 0);
	}
	return (INT_PTR)TRUE;
}

INT_PTR KeybindBox::onClickLoadPreset(int idx)
{
	int tab_num = selected_tabctrl;

	kbctl[tab_num]->LoadPreset(idx);
	if (tab_num == 1) {
		CheckDlgButton(hDlg, IDC_CHK_COMBI1, kbctl[tab_num]->GetCombi() != 0);
	}
	return (INT_PTR)TRUE;
}

INT_PTR KeybindBox::onClickSavePreset(int idx)
{
	int tab_num = selected_tabctrl;

	kbctl[tab_num]->SavePreset(idx);
	if (tab_num == 1) {
		kbctl[tab_num]->SetCombi(IsDlgButtonChecked(hDlg, IDC_CHK_COMBI1) ? 1 : 0);
	}
	return (INT_PTR)TRUE;
}

void KeybindBox::select_tabctrl(int tab_num)
{
	HWND hCtrl;

	selected_tabctrl = tab_num;
	for(int i=0; i<KEYBIND_MAX_NUM; i++) {
		hCtrl = GetDlgItem(hDlg, IDC_CUSTOM0 + i);
		ShowWindow(hCtrl, i == selected_tabctrl ? SW_SHOW : SW_HIDE);
	}
	hCtrl = GetDlgItem(hDlg, IDC_CHK_COMBI1);
	ShowWindow(hCtrl, selected_tabctrl == 1 ? SW_SHOW : SW_HIDE);
}

}; /* namespace GUI_WIN */
