/** @file win_recvidbox.cpp

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2015.05.30 -

	@brief [ record video box ]
*/
#include "win_recvidbox.h"
#include "../../emu.h"
#include "../../video/rec_video.h"
#include "win_gui.h"

namespace GUI_WIN
{

// list
static const char *video_type_label[] = {
#ifdef USE_REC_VIDEO
#ifdef USE_REC_VIDEO_VFW
	_TX("video for windows"),
#endif
#ifdef USE_REC_VIDEO_MMF
	_TX("media foundation"),
#endif
#ifdef USE_REC_VIDEO_FFMPEG
	_TX("ffmpeg"),
#endif
#endif
	NULL };
static const int video_type_ids[] = {
#ifdef USE_REC_VIDEO
#ifdef USE_REC_VIDEO_VFW
	RECORD_VIDEO_TYPE_VFW,
#endif
#ifdef USE_REC_VIDEO_MMF
	RECORD_VIDEO_TYPE_MMF,
#endif
#ifdef USE_REC_VIDEO_FFMPEG
	RECORD_VIDEO_TYPE_FFMPEG,
#endif
#endif
	0 };

RecVideoBox::RecVideoBox(HINSTANCE hInst, CFont *new_font, EMU *new_emu, GUI *new_gui)
	: CDialogBox(hInst, IDD_RECVIDEOBOX, new_font, new_emu, new_gui)
{
	typnum = 0;
	memset(codnums, 0, sizeof(codnums));
	memset(quanums, 0, sizeof(quanums));
	memset(cod_ids, 0, sizeof(cod_ids));
	memset(qua_ids, 0, sizeof(qua_ids));
	memset(enables, 0, sizeof(enables));
	cont = false;
}

RecVideoBox::~RecVideoBox()
{
}

/// create modal dialog box
INT_PTR RecVideoBox::Show(HWND hWnd, bool continuous)
{
	cont = continuous;
	return (video_type_ids[0] ? ::DialogBoxParam(hInstance, MAKEINTRESOURCE(dialogId), hWnd, Proc, (LPARAM)this) : IDCANCEL);
}

INT_PTR RecVideoBox::onInitDialog(UINT message, WPARAM wParam, LPARAM lParam)
{
	CDialogBox::onInitDialog(message, wParam, lParam);

	int codnum = emu->get_parami(VM::ParamRecVideoCodec);
	int quanum = emu->get_parami(VM::ParamRecVideoQuality);
	int i;
	typnum = 0;
	for(i=0; video_type_ids[i] != 0; i++) {
		if (video_type_ids[i] == emu->get_parami(VM::ParamRecVideoType)) {
			typnum = i;
			break;
		}
	}

	// layout
	HWND hTabCtrl = GetDlgItem(hDlg, IDC_TAB1);

	TCITEM tcitm;
	tcitm.mask = TCIF_TEXT;

	int h = font->GetHeight();
	RECT re_tab;
	GetClientRect(hTabCtrl, &re_tab);
	TabCtrl_AdjustRect(hTabCtrl, FALSE, &re_tab);
	re_tab.top += margin + margin;
	re_tab.left += margin + margin;

	int item_id = IDC_TAB1 + 1;

	RECT re = {0,0,0,0};
	DWORD dwStyle = WS_CHILD | WS_VISIBLE;

	//
	for(i=0; video_type_ids[i] != 0; i++) {
		tcitm.pszText = (LPSTR)_tgettext(video_type_label[i]);
		TabCtrl_InsertItem(hTabCtrl , i , &tcitm);
		enables[i] = emu->rec_video_enabled(video_type_ids[i]);

		switch(video_type_ids[i]) {
			case RECORD_VIDEO_TYPE_VFW:
			{
				item_range[i].first = item_id;
				re.left = re_tab.left;
				re.top = re_tab.top + margin + margin;
				CreateStatic(item_id, _("You can set properties after pressing start button."), &re);
				item_id++;
				item_range[i].last = item_id - 1;
				break;
			}
			default:
			{
				item_range[i].first = item_id;
				re.left = re_tab.left;
				re.top = re_tab.top + margin + margin;
				CreateStatic(item_id, _("Codec Type"), &re);
				item_id++;

				const char **codlbl = emu->get_rec_video_codec_list(video_type_ids[i]);
				re.left = re.right + padding;
				CreateComboBox(item_id, codlbl, codnums[i], 12, &re, false, true);
				cod_ids[i] = item_id;
				item_id++;

				re.left = re_tab.left;
				re.top = re.bottom + margin;
				CreateStatic(item_id, _("Quality"), &re);
				item_id++;

				const char **qualbl = emu->get_rec_video_quality_list(video_type_ids[i]);
				re.left = re.right + padding;
				CreateComboBox(item_id, qualbl, quanums[i], 12, &re, false, true);
				qua_ids[i] = item_id;
				item_id++;

				if (!enables[i]) {
					re.left = re_tab.left;
					re.top = re.bottom + margin;
					CreateStatic(item_id, _("! Need install library."), &re);
					item_id++;
				}

				item_range[i].last = item_id - 1;
				break;
			}
		}
	}
	// select tab
	TabCtrl_SetCurSel(hTabCtrl, typnum);

	// tab control size
	re.top = margin;
	re.left = margin;
	AdjustControl(IDC_TAB1, &re, GetRight() + margin, GetBottom() + margin);
	re_tab.top = re.top + h + margin + 2;
	re_tab.left = re.left + 2;
	re_tab.bottom = re.bottom - re_tab.top - 3;
	re_tab.right = re.right - re_tab.left - 4;
	AdjustControl(IDC_STATIC_0, &re_tab, re_tab.right, re_tab.bottom);

	// button
	if (cont) {
		SetDlgItemText(hDlg, IDOK, _("Next"));
	}
	re.top = GetBottom() + padding;
	re.right = GetRight();
	AdjustButton(IDCANCEL, 8, &re, true);
	re.right = re.left - margin;
	AdjustButton(IDOK, 8, &re, true);

	AdjustWindow(margin);

	select_tabctrl(typnum);

	return (INT_PTR)TRUE;
}

void RecVideoBox::select_tabctrl(int tab_num)
{
	HWND hCtrl;
	int cmdShow;

	typnum = tab_num;
	for(int i=0; video_type_ids[i] != 0 && i < WIN_RECVIDEO_LIBS; i++) {
		cmdShow = (tab_num == i ? SW_SHOW : SW_HIDE);

		for(int id=item_range[i].first; id<=item_range[i].last; id++) {
			hCtrl = GetDlgItem(hDlg, id);
			ShowWindow(hCtrl, cmdShow);
		}
	}
	hCtrl = GetDlgItem(hDlg, IDOK);
	EnableWindow(hCtrl, enables[typnum]);
}

INT_PTR RecVideoBox::onCommand(UINT message, WPARAM wParam, LPARAM lParam)
{
	WORD wId = LOWORD(wParam);

	if (wId == IDOK) {
		onOK(message, wParam, lParam);
	}
	return CDialogBox::onCommand(message, wParam, lParam);
}

INT_PTR RecVideoBox::onNotify(UINT message, WPARAM wParam, LPARAM lParam)
{
	LPNMHDR lpNmHdr = (NMHDR *)lParam;
	int i;
	if (lpNmHdr->idFrom = IDC_TAB1) {
		switch (lpNmHdr->code) {
		case TCN_SELCHANGE:
			i = TabCtrl_GetCurSel(lpNmHdr->hwndFrom);
			select_tabctrl(i);
			break;
		}
	}
	return (INT_PTR)TRUE;
}
INT_PTR RecVideoBox::onOK(UINT message, WPARAM wParam, LPARAM lParam)
{
	if (cod_ids[typnum]) codnums[typnum] = (int)SendDlgItemMessage(hDlg, cod_ids[typnum], CB_GETCURSEL, 0, 0);
	if (qua_ids[typnum]) quanums[typnum] = (int)SendDlgItemMessage(hDlg, qua_ids[typnum], CB_GETCURSEL, 0, 0);
	emu->set_parami(VM::ParamRecVideoType, video_type_ids[typnum]);
	emu->set_parami(VM::ParamRecVideoCodec, codnums[typnum]);
	emu->set_parami(VM::ParamRecVideoQuality, quanums[typnum]);

	return (INT_PTR)TRUE;
}

}; /* namespace GUI_WIN */
