/** @file win_volumebox.cpp

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2013.10.30 -

	@brief [ volume box ]
*/
#include "win_volumebox.h"
#include "../../emu.h"
#include "win_gui.h"

namespace GUI_WIN
{

VolumeBox::VolumeBox(HINSTANCE hInst, CFont *new_font, EMU *new_emu, GUI *new_gui)
	: CDialogBox(hInst, IDD_VOLUME, new_font, new_emu, new_gui)
{
	int i = 0;
	memset(volumes, 0, sizeof(volumes));
	volumes[i++] = &config.volume;
	volumes[i++] = &config.beep_volume;
	volumes[i++] = &config.psg6_volume;
	volumes[i++] = &config.psg9_volume;
	volumes[i++] = &config.relay_volume;
	volumes[i++] = &config.cmt_volume;
	volumes[i++] = &config.fdd_volume;

	i = 0;
	memset(mutes, 0, sizeof(mutes));
	mutes[i++] = &config.mute;
	mutes[i++] = &config.beep_mute;
	mutes[i++] = &config.psg6_mute;
	mutes[i++] = &config.psg9_mute;
	mutes[i++] = &config.relay_mute;
	mutes[i++] = &config.cmt_mute;
	mutes[i++] = &config.fdd_mute;
}

VolumeBox::~VolumeBox()
{
}

INT_PTR VolumeBox::onInitDialog(UINT message, WPARAM wParam, LPARAM lParam)
{
	CDialogBox::onInitDialog(message, wParam, lParam);

	for(int i=0; i<VOLUMEBOX_NUMS; i++) {
		if (volumes[i]) {
			SendDlgItemMessage(hDlg, IDC_SLIDER_VOL1+i, TBM_SETRANGE, TRUE, MAKELPARAM(0, 100));
			SendDlgItemMessage(hDlg, IDC_SLIDER_VOL1+i, TBM_SETTICFREQ, 25, 0);
			SendDlgItemMessage(hDlg, IDC_SLIDER_VOL1+i, TBM_SETPOS  , TRUE, 100 - (*volumes[i]));
		}
		if (mutes[i]) {
			CheckDlgButton(hDlg, IDC_CHK_MUTE1+i, *mutes[i]);
		}
	}

	// layout
	RECT re;
	SIZE siz;
	font->GetTextSize(hDlg, _T("0000000"), &siz);
	SIZE sli;
	GetControlSize(IDC_SLIDER_VOL1, &sli);

	int left = margin + margin;
	re.top = margin + siz.cy * 2 + padding;
	re.left = left + (siz.cx - sli.cx) / 2;
	AdjustControl(IDC_SLIDER_VOL1, &re, sli.cx, 100);
	re.top = re.bottom + padding;
	re.left = left;
	AdjustCheckBox(IDC_CHK_MUTE1, &re);
	re.top = margin;
	re.left = margin;
	AdjustControl(IDC_STATIC_1, &re, siz.cx + margin + margin, re.bottom - re.top + margin);

	for(int i=1; i<VOLUMEBOX_NUMS; i++) {
		re.top = margin;
		left = GetRight() + margin;
		re.left = left;
		AdjustControl(IDC_STATIC_1 + i, &re, siz.cx, siz.cy * 2);
		re.top = re.bottom + padding;
		re.left = left + (siz.cx - sli.cx) / 2;
		AdjustControl(IDC_SLIDER_VOL1 + i, &re, sli.cx, 100);
		re.top = re.bottom + padding;
		re.left = left;
		AdjustCheckBox(IDC_CHK_MUTE1 + i, &re);
	}

	// button
	re.top = GetBottom() + margin;
	re.right = GetRight();
	AdjustButton(IDOK, 8, &re, true);

	AdjustWindow(margin);

	return (INT_PTR)TRUE;
}

INT_PTR VolumeBox::onHScroll(UINT message, WPARAM wParam, LPARAM lParam)
{
	SetVolume();
	return (INT_PTR)TRUE;
}

INT_PTR VolumeBox::onVScroll(UINT message, WPARAM wParam, LPARAM lParam)
{
	SetVolume();
	return (INT_PTR)TRUE;
}

INT_PTR VolumeBox::onCommand(UINT message, WPARAM wParam, LPARAM lParam)
{
	SetVolume();

	return CDialogBox::onCommand(message, wParam, lParam);
}

void VolumeBox::SetVolume()
{
	for(int i=0; i<VOLUMEBOX_NUMS; i++) {
		if (volumes[i]) {
			*volumes[i] = (int)SendDlgItemMessage(hDlg, IDC_SLIDER_VOL1 + i, TBM_GETPOS, 0, 0);
			*volumes[i] = 100 - (*volumes[i]);
		}
		if (mutes[i]) {
			*mutes[i] = (IsDlgButtonChecked(hDlg, IDC_CHK_MUTE1 + i) != 0);
		}
	}

	// callback
	if (emu) {
		emu->set_volume(0);
	}

	return;
}

}; /* namespace GUI_WIN */
