/** @file loadlibrary.h

	Skelton for retropc emulator

	@author Sasaji
	@date   2016.11.10 -

	@brief [ load dynamic library ]
*/

#ifndef _LOADLIBRARY_H_
#define _LOADLIBRARY_H_

#ifndef _WIN32
#include <dlfcn.h>
#endif

#if defined(_WIN32)

#define LOAD_LIB(handle, lib_base, version) { \
	_TCHAR libname[64]; \
	_tcscpy(libname, _T(lib_base)); \
	if (version) { \
		_tcscat(libname, _T("-")); \
		_tcscat(libname, version); \
	} \
	_tcscat(libname, _T(".dll")); \
	handle = LoadLibrary(libname); \
	if (!handle) { \
		emu->out_logf(LOG_DEBUG, _T("Cannot load %s."), libname); \
		continue; \
	} else { \
		emu->out_logf(LOG_DEBUG, _T("Loaded %s."), libname); \
	} \
}
#define GET_ADDR(func, func_type, handle, func_name) { \
	func = (func_type) GetProcAddress(handle, func_name); \
	if (!func) { \
		emu->out_logf(LOG_DEBUG, _T("Cannot get address of %s."), _T(func_name)); \
		continue; \
	} \
}
#define GET_ADDR_OPTIONAL(func, func_type, handle, func_name) { \
	func = (func_type) GetProcAddress(handle, func_name); \
	if (!func) { \
		emu->out_logf(LOG_DEBUG, _T("Cannot get address of %s."), _T(func_name)); \
	} \
}
#define UNLOAD_LIB(handle) { \
	if (handle) FreeLibrary(handle); \
	handle = NULL; \
}

#else /* !_WIN32 */

#if defined(__APPLE__) && defined(__MACH__)
#define LOAD_LIB(handle, lib_base, version) { \
	char libname[64]; \
	strcpy(libname, "lib"); \
	strcat(libname, lib_base); \
	if (version) { \
		strcat(libname, "."); \
		strcat(libname, version); \
	} \
	strcat(libname, ".dylib"); \
	handle = dlopen(libname, RTLD_NOW | RTLD_GLOBAL); \
	if (!handle) { \
		emu->out_logf(LOG_DEBUG, _T("Cannot load %s."), libname); \
		continue; \
	} else { \
		emu->out_logf(LOG_DEBUG, _T("Loaded %s."), libname); \
	} \
}
#else
#define LOAD_LIB(handle, lib_base, version) { \
	char libname[64]; \
	strcpy(libname, "lib"); \
	strcat(libname, lib_base); \
	strcat(libname, ".so"); \
	if (version) { \
		strcat(libname, "."); \
		strcat(libname, version); \
	} \
	handle = dlopen(libname, RTLD_NOW | RTLD_GLOBAL); \
	if (!handle) { \
		emu->out_logf(LOG_DEBUG, _T("Cannot load %s."), libname); \
		continue; \
	} else { \
		emu->out_logf(LOG_DEBUG, _T("Loaded %s."), libname); \
	} \
}
#endif
#define GET_ADDR(func, func_type, handle, func_name) { \
	func = (func_type)dlsym(handle, func_name); \
	if (!func) { \
		emu->out_logf(LOG_DEBUG, _T("Cannot get address of %s."), func_name); \
		continue; \
	} \
}
#define GET_ADDR_OPTIONAL(func, func_type, handle, func_name) { \
	func = (func_type)dlsym(handle, func_name); \
	if (!func) { \
		emu->out_logf(LOG_DEBUG, _T("Cannot get address of %s."), func_name); \
	} \
}
#define UNLOAD_LIB(handle) { \
	if (handle) dlclose(handle); \
	handle = NULL; \
}

#endif

#endif /* _LOADLIBRARY_H_ */
