/** @file logging.h

	Skelton for retropc emulator

	@author Sasaji
	@date   2015.02.18 -

	@brief [ logging ]
*/

#ifndef _LOGGING_H_
#define _LOGGING_H_

#include "depend.h"
#include "fileio.h"
#include "cmutex.h"

// for log
// #define _LOG_CONSOLE
#define _LOG_FILE

#define LOG_INFO	0
#define LOG_WARN	1
#define LOG_ERROR	2
#define LOG_DEBUG	3

// for debug
//#define _DEBUG_LOG
#ifdef _DEBUG_LOG
	// output debug log to console
//	#define _DEBUG_CONSOLE
	// output debug log to file
	#define _DEBUG_FILE

	// output cpu debug log
//	#define _CPU_DEBUG_LOG
	// output fdc debug log
//	#define _FDC_DEBUG_LOG
	// output i/o debug log
//	#define _IO_DEBUG_LOG
#endif

class Logging
{
private:
#ifdef _LOG_FILE
	FILEIO *fplog;
#endif
	CMutex *mux;

public:
	Logging();
	virtual ~Logging();

	void open_logfile(const _TCHAR *path);
	void close_logfile();

	// for narrow char
	virtual void out_log(int level, const char* msg);
	virtual void out_logf(int level, const char* format, ...);
	virtual void out_logv(int level, const char* format, va_list ap);
	/// debug log
	virtual void out_debug(const char* msg);
	virtual void out_debugf(const char* format, ...);
	// system error log
	virtual void out_syserrlog(int level, int err_num, const char* premsg);

	virtual void send_log_message(int level, const char *levelstr, const char *msg) {};

#ifdef _UNICODE

	virtual void out_log(int level, const wchar_t* msg);
	virtual void out_logf(int level, const wchar_t* format, ...);
	virtual void out_logv(int level, const wchar_t* format, va_list ap);
	/// debug log
	virtual void out_debug(const wchar_t* msg);
	virtual void out_debugf(const wchar_t* format, ...);
	// system error log
	virtual void out_syserrlog(int level, int err_num, const wchar_t* premsg);

	virtual void send_log_message(int level, const wchar_t *levelstr, const wchar_t *msg) {};

#endif

};

#endif
