/** @file sdl_debugger.h

	Skelton for retropc emulator

	@author Sasaji
	@date   2016.02.01 -

	@brief [ debugger console ]

	@note Original author is Takeda.Toshiya at 2006.08.18 -
*/

#ifndef _SDL_DEBUGGER_H_
#define _SDL_DEBUGGER_H_

#include "vm/vm.h"

#ifdef USE_DEBUGGER

#include "vm/device.h"
#include "vm/debugger.h"
//#include <fcntl.h>
//#include "res/resource.h"
//#include "fileio.h"

#ifdef _WIN32
#define USE_CONSOLE_WINDOW 1
#endif

#define USE_TELNET_SERVER 1

#undef USE_STANDARD_INOUT

#ifdef USE_CONSOLE_WINDOW
#include <conio.h>
#include <io.h>
#endif

#ifdef USE_STANDARD_INOUT
#ifndef _WIN32
#include <sys/select.h>
#endif
#endif

#if defined(_BML3MK5) || defined(_MBS1)
#undef  USE_IOPORT_ACCESS
#else
#define USE_IOPORT_ACCESS
#endif

#define DC_MAX_COMMAND_LEN	128
#define DC_MAX_BUFFER_LEN 1024

class FILEIO;

///
///
///
class DebuggerConsole
{
private:
	volatile debugger_thread_t *dp;
	DEVICE *cpu;
	DEVICE *mem;
	DEBUGGER *debugger;

	uint32_t prog_addr_mask;
	uint32_t data_addr_mask;

	int phys_type;
	uint32_t dump_addr;
	uint32_t dump_phys_addr;

	uint32_t dasm_addr;
	uint32_t dasm_phys_addr;

#ifdef USE_CONSOLE_WINDOW
	HANDLE hStdIn;
	HANDLE hStdOut;
#endif
	bool cp932;
	bool echo_bak;

	FILEIO* logfile;
	_TCHAR logfilename[_MAX_PATH];

	_TCHAR buffer[DC_MAX_BUFFER_LEN];
	
	_TCHAR command[DC_MAX_COMMAND_LEN + 1];
	_TCHAR prev_command[DC_MAX_COMMAND_LEN + 1];
	int commandptr;

	_TCHAR *params[32];
	int paramnum;

	uint8_t key_stat_tmp[KEY_STATUS_SIZE];

private:
	uint32_t HexaToInt(const _TCHAR *str);
	uint8_t  HexaToByte(char *value);
	uint16_t HexaToWord(char *value);
	bool HexaToInt(const _TCHAR *str, uint32_t *addr, int *len);
	break_point_t *GetBreakPoint(const _TCHAR *command);
//	bool IsConsoleActive();

	bool SetCommandLine(_TCHAR chr, bool echo);

	bool ReadInput();
	int ParseInput();
	bool EscapePressed();
	bool NowDisable();

	int GetAddressType(int num);
	void UsageAddressType(int num);

	void CommandDumpMemory(int num);
	void UsageDumpMemory(bool s, int num);
	void CommandEditMemoryBinary(int n, int num);
	void CommandEditMemoryAscii(int num);
	void UsageEditMemory(bool s, int num);
	void CommandInputIOBinary(int n);
	void CommandOutputIOBinary(int n);
	void UsageEditIO(bool s);
	void CommandAccessRegister();
	void UsageAccessRegister(bool s);
	void CommandAccessDevice();
	void UsageAccessDevice(bool s);
#ifdef USE_EMU_INHERENT_SPEC
	void CommandShowMemoryMap();
	void UsageShowMemoryMap(bool s);
#endif
#ifdef _MBS1
	void CommandAccessAddressMap();
	void UsageAccessAddressMap(bool s);
#endif
	void CommandSearch(int num);
	void UsageSearch(bool s, int num);
	void CommandUnassemble(int num);
	void UsageUnassemble(bool s, int num);

	void CommandCalculateHexa();
	void UsageCalculateHexa(bool s);
	void CommandSetFileName();
	void UsageSetFileName(bool s);
	void CommandLoadBinaryFile(int num);
	void CommandSaveBinaryFile(int num);
	void UsageBinaryFile(bool s, int num);
	void CommandSaveImageFile(int num);
	void UsageImageFile(bool s, int num);

	void CommandSetBreakPoint(int num);
	void UsageSetBreakPoint(bool s, int num);
	void PrintBreakPointInfo(int num, break_point_t *bp, int i, const _TCHAR *iname);
	void CommandClearBreakPoint(int num);
	void CommandChangeBreakPoint(int num);
	void UsageChangeBreakPoint(bool s, int num);
	void CommandListBreakPoint(int num);
	void UsageListBreakPoint(bool s, int num);

	void CommandExecute();
	void UsageExecute(bool s);
	void PrintBreakedReason(uint32_t reason);
	void CommandTrace();
	void UsageTrace(bool s);
	void CommandTraceBack();
	void UsageTraceBack(bool s);

	void CommandShowClock();
	void UsageShowClock(bool s);

	void CommandQuit();
	void UsageQuit(bool s);

	void CommandOutputLogFile();
	void UsageOutputLogFile(bool s);

	void CommandSendToEmulator();
	void CommandSendResetToEmulator();
	void CommandSendKeyToEmulator();
	void UsageSendToEmulator(bool s);

	void CommandUsage();
	void Usage();
	void UsageCmdStr1(bool s, const _TCHAR *cmd1, const _TCHAR *args1, const _TCHAR *desc1);
	void UsageCmdStr2(bool s, const _TCHAR *cmd1, const _TCHAR *args1, const _TCHAR *desc1
		, const _TCHAR *cmd2, const _TCHAR *args2, const _TCHAR *desc2);
	void UsageCmdStr3(bool s, const _TCHAR *cmd1, const _TCHAR *args1, const _TCHAR *desc1
		, const _TCHAR *cmd2, const _TCHAR *args2, const _TCHAR *desc2
		, const _TCHAR *cmd3, const _TCHAR *args3, const _TCHAR *desc3);

	void TraceFirst();
	void TraceCurrent();
	void TraceNext();

	void SetKeyStatus();

public:
	DebuggerConsole(volatile debugger_thread_t *p);
	~DebuggerConsole();

	enum enTextColor {
		Black = 0,
		Blue,
		Red,
		Magenta,
		Green,
		Cyan,
		Yellow,
		White,
		EndColor
	};

	void SetTextColor(enTextColor color);
	void Printf(enTextColor color, const _TCHAR *format, ...);
	void Printf(const _TCHAR *format, ...);
	void Vprintf(const _TCHAR *format, va_list ap);
	void Print(enTextColor color, const _TCHAR *str, bool cr = true);
	void Print(const _TCHAR *str, bool cr = true);
	void Out(enTextColor color, bool cr = true);
	void Out(bool cr = true);
	void PutCh(_TCHAR c);
	void Cr();
	void Flush();

	void Process();
	void PrintPrompt();

	_TCHAR *GetBuffer(bool clear = false);
	int GetBufferSize() { return DC_MAX_BUFFER_LEN; }
	const _TCHAR *GetParam(int idx);
	int GetParamNums() { return paramnum; }

};

#endif
#endif

