/** @file utility.h

	Skelton for retropc emulator

	@author Sasaji
	@date   2014.04.01 -

	@brief [ utility ]
*/

#ifndef _UTILITY_H_
#define _UTILITY_H_

#include "common.h"
#include <stdarg.h>

namespace UTILITY
{
	// ----------------------------------------------------------------------
	// file path conversion

	bool check_file_extension(const _TCHAR *filename, const _TCHAR *ext);

	void convert_path_separator(_TCHAR *path);
	void add_path_separator(_TCHAR *path, size_t maxlen = _MAX_PATH);
	void get_parent_dir(_TCHAR *path);
	void get_ancestor_dir(_TCHAR *path, int ancestor_num);

	void get_dir_and_basename(const _TCHAR *path, _TCHAR *dir, _TCHAR *name);
	void get_dir_and_basename(_TCHAR *path, _TCHAR *name);

	_TCHAR *trim_center(const _TCHAR *str, int maxlen);

	bool make_relative_path(const _TCHAR *base_path, _TCHAR *path);
	bool make_absolute_path(const _TCHAR *base_path, _TCHAR *path);
	void get_long_full_path_name(const _TCHAR *src, _TCHAR *dst);

	_TCHAR *slim_path(const _TCHAR *path, int *parent_num = NULL);

	int copy_path(_TCHAR *dst, const _TCHAR *src, int maxSize);

	int chomp_crlf(char *str);
	int chomp_crlf(wchar_t *str);

	// ----------------------------------------------------------------------
	// string conversion

#ifdef _UNICODE
	/* _UNICODE */
//	bool conv_wchar_to_utf8(const wchar_t *srcStart, int srcMaxSize, char *dstStart, int dstMaxSize);
//	bool conv_utf8_to_wchar(const char *srcStart, int srcMaxSize, wchar_t *dstStart, int dstMaxSize);

	/// native string to UTF-8 string
	void wconv_from_native_path(const wchar_t *src, wchar_t *dst, int maxSize = _MAX_PATH);
	/// native string to UTF-8 string
	void cconv_from_native_path(const wchar_t *src, char *dst, int maxSize = _MAX_PATH);
	/// native string to UTF-8 string
	int      wcs_to_mbs(char *dst, const wchar_t *src, int maxSize = _MAX_PATH);
	/// UTF-8 string to native string
	void conv_to_native_path(const wchar_t *src, wchar_t *dst, int maxSize = _MAX_PATH);
	/// UTF-8 string to native string
	void conv_to_native_path(const char *src, wchar_t *dst, int maxSize = _MAX_PATH);

#define conv_from_native_path wconv_from_native_path

	/// native string to wide string
	int   mbs_to_wcs(wchar_t *dst, const char *src, int maxSize = _MAX_PATH);

#else	/* !_UNICODE */

	bool conv_mbs_to_utf8(const char *srcStart, int srcMaxSize, char *dstStart, int dstMaxSize);
	bool conv_utf8_to_mbs(const char *srcStart, int srcMaxSize, char *dstStart, int dstMaxSize);
	/// native string to UTF-8 string
	void cconv_from_native_path(const char *src, char *dst, int maxSize = _MAX_PATH);
	/// native string to UTF-8 string
	int   wcs_to_mbs(char *dst, const char *src, int maxSize = _MAX_PATH);
	/// UTF-8 string to native string
	void conv_to_native_path(const char *src, char *dst, int maxSize = _MAX_PATH);

#define conv_from_native_path cconv_from_native_path

#endif	/* _UNICODE */

	// ----------------------------------------------------------------------
	// string format

#ifdef _UNICODE
	const wchar_t *concat(const wchar_t *src, ...);
	void concatv(wchar_t *dst, size_t max_len, const wchar_t *src, va_list ap);
	void concat(wchar_t *dst, size_t max_len, const wchar_t *src, ...);
#endif /* _UNICODE */
	const char *concat(const char *src, ...);
	void concatv(char *dst, size_t max_len, const char *src, va_list ap);
	void concat(char *dst, size_t max_len, const char *src, ...);

	void tcscpy(_TCHAR *dst, size_t max_len, const _TCHAR *src);

	int stprintf(_TCHAR *dest, size_t count, const _TCHAR *format, ...);
	int vstprintf(_TCHAR *dest, size_t count, const _TCHAR *format, va_list ap);

	int sprintf_utf8(char *dest, size_t count, const char *format, ...);
	int vsprintf_utf8(char *dest, size_t count, const char *format, va_list ap);

	int utf8firstpos(const char *str);

	/// substring in last circle bracket
	int substr_in_bracket(const _TCHAR *src, _TCHAR *dst, int maxSize = _MAX_PATH);
	void rtrim(_TCHAR *str, const _TCHAR *sep = NULL);
	int  get_token(const _TCHAR *str, int start_pos, int max_len, const _TCHAR sep, int &st, int &ed);
	int  get_token(const _TCHAR *str, int start_pos, int max_len, _TCHAR *word, int word_max_len, const _TCHAR sep = _T(','), int *word_len = NULL);
	int  tcstok(_TCHAR *str, int start_pos, int max_len, const _TCHAR sep = _T(','));
	int  get_parameters(_TCHAR *str, int max_len, _TCHAR **params, int max_params);
	int  get_parameters(const _TCHAR *str, int max_len, const _TCHAR **params, int max_params, int *paramslen);
};

#endif /* _UTILITY_H_ */
