/** @file avk_rec_video.h

 Skelton for retropc emulator

 @author Sasaji
 @date   2016.11.03 -

 @brief [ record video using AVFoundation ]
 */

#ifndef _AVK_RECORD_VIDEO_H_
#define _AVK_RECORD_VIDEO_H_

#ifdef __OBJC__
#import <AVFoundation/AVFoundation.h>
#endif
#include "../../common.h"

#if defined(USE_REC_VIDEO) && defined(USE_REC_VIDEO_AVKIT)

class EMU;
class REC_VIDEO;
class CSurface;

class AVK_REC_VIDEO
{
private:
	EMU *emu;
	REC_VIDEO *vid;

	int rec_fps;
	int frame_count;

	int bpc;
	int bpp;
	int bpr;
	
	CSurface *rec_surface;

#ifdef __OBJC__
	AVAssetWriter *asset;
	AVAssetWriterInput *input;
	AVAssetWriterInputPixelBufferAdaptor *input_adaptor;
	NSString *rec_file;
	CGColorSpaceRef color_space;
#ifdef USE_AVK_CREATE_PIXEL_BUFFER
	CVPixelBufferRef pixel_buffer;
#endif
#else
	void *asset;
	void *input;
	void *input_adaptor;
	void *rec_file;
	void *color_space;
#ifdef USE_AVK_CREATE_PIXEL_BUFFER
	void *pixel_buffer;
#endif
#endif
	
public:
	AVK_REC_VIDEO(EMU *new_emu, REC_VIDEO *new_vid);
	~AVK_REC_VIDEO();

	bool IsEnabled();

	/// true:OK false:ERROR
	bool Start(const _TCHAR *path, int fps, CSurface *surface, bool show_dialog);
	void Stop();
	bool Restart();
	bool Record();

	const _TCHAR **GetCodecList();
	const _TCHAR **GetQualityList();
};

#endif /* USE_REC_VIDEO && USE_REC_VIDEO_AVKIT */

#endif /* _AVK_RECORD_VIDEO_H_ */
