/** @file display.cpp

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2011.06.08 -

	@brief [ display ]
*/

#include "display.h"
#include "../../fileio.h"
#include "../../config.h"

#define CURSOR_MAXPOS 0xffffff

#ifdef _DEBUG
//#define DEBUG_CRTMON
//#define DEBUG_CRTMON2
#endif

void DISPLAY::initialize()
{
	memset(font, 0, sizeof(font));
	font_rom_loaded = false;
	font_rom_loaded_at_first = false;

	vm_pause = emu->get_pause_ptr();

	// load rom images
	load_font_rom_file();

	emu->get_rgbformat(&Rmask, &Gmask, &Bmask, &Rshift, &Gshift, &Bshift);

	// create pc palette
#if 1
	for(int i = 0; i < 8; i++) {
//		palette_pc[i] = RGB_COLOR((i & 2) ? 255 : 0, (i & 4) ? 255 : 0, (i & 1) ? 255 : 0);
		palette_pc[i] = emu->map_rgbcolor((i & 2) ? 255 : 0, (i & 4) ? 255 : 0, (i & 1) ? 255 : 0);
		emu->out_debugf(_T("palette_pc[%d]:%x"),i,palette_pc[i]);
	}
#else
	palette_pc[0] = emu->map_rgbcolor(0x1f,0x1f,0x1f);
	palette_pc[1] = emu->map_rgbcolor(0x3f,0x3f,0x3f);
	palette_pc[2] = emu->map_rgbcolor(0x5f,0x5f,0x5f);
	palette_pc[3] = emu->map_rgbcolor(0x7f,0x7f,0x7f);
	palette_pc[4] = emu->map_rgbcolor(0x9f,0x9f,0x9f);
	palette_pc[5] = emu->map_rgbcolor(0xbf,0xbf,0xbf);
	palette_pc[6] = emu->map_rgbcolor(0xdf,0xdf,0xdf);
	palette_pc[7] = emu->map_rgbcolor(0xff,0xff,0xff);
#endif

	memset(scrntop, 0, sizeof(scrntop));

	screen_left = 0;
	screen_top = 0;
	screen_right = SCREEN_WIDTH + screen_left;
	screen_bottom = SCREEN_HEIGHT + screen_top;
	scrnline1_offset = 0;

	interlace_sel = 0;

	mode_sel = 0;
	ig_enable = 0;
	ig_mask = true;

	cursor_addr = 0;
	disp_cursor = false;
	for(int i=0; i<32; i++) {
		cur_left[i] = 0;
		cur_right[i] = 0;
	}
	cur_top = 0;
	cur_bottom = 0;

	crtc_chr_clocks = 0;
	crtc_ppch = 16;

	bg_color = 0;
	hireso = 0;
	width_sel = 0;

	top_raster = 0;
	bottom_raster = 0;
	vsync_top_raster = 0;
	vsync_bottom_raster = 0;
	crt_mon_now_raster = 0;

	// synchronizable vertical range
	v_total_min[0] = (LINES_PER_FRAME - 20);
	v_total_max[0] = (LINES_PER_FRAME + 8);
	v_total_min[1] = v_total_min[0] / 2;
	v_total_max[1] = v_total_max[0] / 2;
	v_total_min[2] = v_total_min[0] * 2;
	v_total_max[2] = v_total_max[0] * 2;

//	crt_mon_start_raster = 0;
	for(int i=0; i<VRAM_BUF_HEIGHT; i++) {
		crt_mon_vline[i].left_col = 0;
		crt_mon_vline[i].right_col = 0;
		crt_mon_vline[i].hsync_left_col = 0;
		crt_mon_vline[i].hsync_right_col = 0;
		crt_mon_vline[i].raster = 0;
	}
	left_diff = 0;
	// synchronizable horizontal range
	h_total_min[0] = CHARS_PER_LINE - 3;	// w.40
	h_total_max[0] = CHARS_PER_LINE + 3;
	h_total_min[1] = CHARS_PER_LINE * 2 - 4;	// w.80
	h_total_max[1] = CHARS_PER_LINE * 2 + 4;

	sw = 0;

	//
	memset(crt_mon, 0, sizeof(crt_mon));
	crt_mon_col = 0;
	crt_mon_row = 0;
	crt_mon_start_row = 0;
	crt_mon_addr = ADDR_VRAM_START;
	crt_mon_page = 0;

	crt_mon_cursor_col = 0;
	crt_mon_cursor_row = 0;

	skip_frame_count = 0;

	for(int i=0; i<8; i++) {
		ppd[i]=(mode_sel & 0x80) ? 1 : 2;	// pixel per 1dot
		dof[i]=(0x80 >> i);
	}
	// pixel per 1char
	crtc_ppch = (mode_sel & 0x80) ? 8 : 16;

	update_config();

	// register event (unused)
//	register_frame_event(this);
}

void DISPLAY::reset()
{
	load_font_rom_file();

	ig_enable = 0;
	ig_mask = true;
}

void DISPLAY::update_config()
{
	scanline = config.scan_line;
	afterimage = config.afterimage;
#ifdef USE_KEEPIMAGE
	keepimage = config.keepimage;
#endif
	buf_stepcols = (scanline <= 2 ? 1 : 2);
	crt_mon_stepcols = (scanline <= 2 ? 1 : 2);

	// scanline
	dws_ys = (scanline ? 2 : 1);	// skip step

	// color filter for afterimage
	if (afterimage == 0) {
		memset(scrnline1, 0x00, sizeof(scrntype) * SCREEN_WIDTH);
		memset(scrnline2, 0xff, sizeof(scrnline2));
		scrnline1_offset = 0;
	} else if (afterimage == 1) {
		memset(scrnline2, 0xff, sizeof(scrnline2));
		scrnline1_offset = SCREEN_WIDTH;

		Rloss = 64 * (scanline <= 1 ? 1 : scanline);
		Gloss = 32 * (scanline <= 1 ? 1 : scanline);
		Bloss = 96 * (scanline <= 1 ? 1 : scanline);
		if (Rloss > 255) Rloss = 255;
		if (Gloss > 255) Gloss = 255;
		if (Bloss > 255) Bloss = 255; 
		Rloss <<= Rshift;
		Gloss <<= Gshift;
		Bloss <<= Bshift;
	} else if (afterimage == 2) {
		memset(scrnline2, 0xbf, sizeof(scrnline2));
		scrnline1_offset = SCREEN_WIDTH;
	}

#ifdef USE_KEEPIMAGE
	if (keepimage > 0) {
		memset(scrntop, 0, sizeof(scrntop));
	}
#endif
}

void DISPLAY::load_font_rom_file()
{
	const _TCHAR *app_path, *rom_path[2];

	rom_path[0] = config.rom_path;
	rom_path[1] = emu->application_path();

	for(int i=0; i<2; i++) {
		app_path = rom_path[i];

		if (!font_rom_loaded) {
			font_rom_loaded = emu->load_data_from_file(app_path, _T("FONT.ROM"), font, sizeof(font), (const uint8_t *)"\x00\x00\x00\x00", 4);
		}
	}

	if (!font_rom_loaded) {
		if (!font_rom_loaded_at_first) {
			emu->out_log(LOG_WARN, _("FONT.ROM couldn't be loaded."));
		}
		memset(chrfont, 0x10, sizeof(chrfont));
	} else {
		// convert font data to useful array data
		for(int i = 0; i < 2; i++) {
			for(int c = 0; c < 256; c++) {
				for(int l = 0; l < 16; l++) {
					chrfont[i][c][l] = get_font_data(i, c, l);
				}
			}
		}
	}
	font_rom_loaded_at_first = true;
}

void DISPLAY::write_io8(uint32_t addr, uint32_t data)
{
	switch(addr & 0xffff) {
	case 0xffd0:
		// mode sel
		mode_sel = (data & 0xff);

		bg_color = mode_sel & 0x07;				// bg color
		hireso = (mode_sel & 0x40) ? 0 : 1;		// 0:normal 1:hireso
		width_sel = (mode_sel & 0x80) ? 1 : 0;	// 0:w.40  1:w.80

		for(int i=0; i<8; i++) {
			ppd[i]=width_sel ? 1 : 2;	// pixel per 1dot
			dof[i]=(0x80 >> i);
		}
		// pixel per 1char
		crtc_ppch = (mode_sel & 0x80) ? 8 : 16;

		crtc_chr_clocks = width_sel;

		break;
	case 0xffd6:
		// interlace sel
		interlace_sel = (data & 0x08) ? 1 : 0;	// 0:noninterlace char  1:interlace char
		break;
	case 0xffe9:
		// ig mode reg
		ig_enable = (data & 0x01);
		ig_mask &= ((data & 0x100) == 0);
		break;
	}
}

void DISPLAY::write_signal(int id, uint32_t data, uint32_t mask)
{
	switch(id) {
		case SIG_DISPLAY_LPSTB:
			{
			int width = regs[1];
			int dotx = (data >> 16) & 0xffff;
			int doty = (data & 0xffff);
			int x = 0;
			int y = 0;
			int offset = (width_sel ? 5 : 4);
			uint32_t newaddr;
			t_crt_mon_vline *vline = &crt_mon_vline[doty >> 1];

			if (dotx < vline->left_col) { dotx = vline->right_col - 1; doty-=16; }
			if (dotx >= vline->right_col) { dotx = vline->left_col; doty+=16; }
			if (doty < top_raster) doty = top_raster;
			if (doty >= bottom_raster) doty = bottom_raster - 1;

			x = (dotx - vline->left_col) >> (width_sel ? 3 : 4);
			y = (doty - top_raster) >> 4;
			newaddr = ((regs[12] << 8)+regs[13]) + y * width + x + offset;
			regs[16] = (newaddr >> 8) & 0x3f;
			regs[17] = newaddr & 0xff;

//			emu->out_debugf("lp dx:%03d dy:%03d x:%02d y:%02d addr:%02x%02x"
//				,dotx,doty,x,y,regs[16],regs[17]);
			}
			break;

		case SIG_DISPLAY_VSYNC:
			if ((data & mask) == 0) {
				// synchronize display vertical raster
				for(int i=0; i<3; i++) {
					if (v_total_min[i] <= (*crtc_vt_total) && (*crtc_vt_total) <= v_total_max[i]) {
						crt_mon_now_raster = 0;
						break;
					}
				}

#ifdef DEBUG_CRTMON
				emu->out_debugf(_T("DISPLAY:SIG_DISPLAY_VSYNC_OFF: crtc:%d nras:%d cmr:%d cmc:%d cmras:%d top:%d btm:%d vtop:%d vbtm:%d")
					,(*crtc_vt_count), crt_mon_now_raster
					,crt_mon_row, crt_mon_col, (*crtc_ra)
					,top_raster, bottom_raster, vsync_top_raster, vsync_bottom_raster
				);
#endif
			}
			break;
	}
}

void DISPLAY::update_cursor()
{
	int cur_left_0;
	uint8_t cur_st_l;
	uint8_t cur_ed_l;

	if(crt_mon_cursor_col >= CURSOR_MAXPOS || crt_mon_cursor_row >= CURSOR_MAXPOS) {
		disp_cursor = false;
	} else {
		disp_cursor = ((*crtc_curdisp) != 0);
	}
#ifdef DEBUG_CRTMON2
	emu->out_debugf(_T("update_cursor: sw:%d crt_mon_cursor_col:%d crt_mon_cursor_row:%d crtc_curdisp:%d")
		,sw , crt_mon_cursor_col, crt_mon_cursor_row, (*crtc_curdisp)
	);
#endif

	if (disp_cursor) {
		if ((*videomode) == 1) {
			cur_st_l = (regs[10] & 0x0f);
			cur_ed_l = (regs[11] & 0x0f);
		} else {
			cur_st_l = ((regs[10] & 0x0f) << 1);
			cur_ed_l = ((regs[11] & 0x0f) << 1) + 1;
		}

		// cursor draw area
		cur_top = top_raster + (crt_mon_cursor_row << 1) - (crt_mon_vline[crt_mon_cursor_row].raster & 0x7e);
		cur_bottom = cur_top + cur_ed_l;
		cur_top += cur_st_l;

		cur_left_0 = crt_mon_vline[crt_mon_cursor_row].left_col + (crt_mon_cursor_col + (*curdisp_skew) + config.curdisp_skew - 2 - (*disptmg_skew) + config.disptmg_skew) * crtc_ppch;
		for(int i=0; i < ((*crtc_max_ra) << (1 - (*videomode))); i++) {
			cur_left[i] = cur_left_0;
			cur_right[i] = cur_left[i] + crtc_ppch;
		}
	}
	crt_mon_cursor_col = CURSOR_MAXPOS;
	crt_mon_cursor_row = CURSOR_MAXPOS;
}

void DISPLAY::set_display_size(int left, int top, int right, int bottom)
{
	screen_left = left;
	screen_right = right;
	screen_top = top;
	screen_bottom = bottom;
}

void DISPLAY::draw_screen()
{
#if 0 // skip_frame_count update in update_vram()
	if (emu->now_skip_frame()) {
		skip_frame_count++;
	} else {
		skip_frame_count = 0;
	}
#endif
	if ((afterimage == 0 && skip_frame_count >= 1)
	||  (afterimage > 0 && skip_frame_count >= 2)) {
		// clear cursor
//		disp_cursor = false;
//		crt_mon_cursor_col = CURSOR_MAXPOS;
//		crt_mon_cursor_row = CURSOR_MAXPOS;
#ifdef DEBUG_CRTMON2
		emu->out_debugf(_T("draw_screen:skip sw:%d skip_frame_count:%d afterimage:%d pause:%d")
			,sw , skip_frame_count, afterimage, (*vm_pause)
		);
#endif
		return;
	}
#ifdef DEBUG_CRTMON2
		emu->out_debugf(_T("draw_screen:draw sw:%d skip_frame_count:%d afterimage:%d pause:%d")
			,sw , skip_frame_count, afterimage, (*vm_pause)
		);
#endif

	if (*vm_pause) return;

	// update cursor
	update_cursor();

	// render screen
	if (afterimage == 0) draw_screen_sub();
	else if (afterimage == 1) draw_screen_sub_afterimage1();
	else draw_screen_sub_afterimage2();

	// sw bit0 : even/odd line (y axis)  bit1: even/odd char (x axis)
//	if (!(*vm_pause) || scanline >= 2)
	sw = ((sw + 1) & 3);

	if (scanline <= 1 && *crtc_odd_line && skip_frame_count == 0) {
		sw = *crtc_odd_line;
	}
}

uint8_t DISPLAY::get_font_data(int interlace, int chr_code, int y)
{
	int code = 0;
	int offset = 0;
	if (interlace == 1) {
		// interlace mode char
//		if (videomode == 0) {
//			y = y >> 1;	//	double size
//		}
		if (chr_code < 0x80) {
			code = chr_code << 4;	// 16byte per 1char
			offset = (y & 0x0e);	// 0 0 2 2 4 4 6 6 8 8 a a c c e e
		} else {
			code = chr_code << 4;	// 16byte per 1char
			offset = (y & 0x0f);	// 0 1 2 3 4 5 6 7 8 9 a b c d e f
		}
	} else {
		// non interlace mode char
		if (chr_code < 0x80) {
			// ascii code
			code = chr_code << 4;	// 16byte per 1char
		} else {
			code = ((chr_code - 0x80) << 4) + 1;
		}
//		if (videomode == 0) {
//			offset = (y & 0x0e);		// 0 0 2 2 4 4 6 6 8 8 a a c c e e
//		} else {
			offset = ((y << 1) & 0x0e);	// 0 2 4 6 8 a c e 0 2 4 6 8 a c e
//		}
	}
	return font[code + offset];
}

#define COLOR_REVERSE(fg,bg) { \
	uint8_t c; \
	c = (bg); \
	(bg) = (fg); \
	(fg) = c; \
}

#define SET_SCRN_DOT(xx,dd) ((dd & scrnline2[xx]) | scrnline1p[xx])

#define DISP_OUT_OF_DISPTMG(p_vline,xx) { \
	dws_disp_cursor_char = (dws_disp_cursor_line && cur_left[dws_l] <= xx && xx < cur_right[dws_l]); \
\
	if (p_vline->hsync_left_col <= xx && xx < p_vline->hsync_right_col) { \
		dws_c_f = 0; \
		dws_c_b = 0; \
	} else { \
		dws_c_f = dws_cram & 0x07; \
		dws_c_b = bg_color; \
	} \
	if (dws_disp_cursor_char) COLOR_REVERSE(dws_c_f, dws_c_b); \
	dws_scrn[xx] = SET_SCRN_DOT(xx, palette_pc[dws_c_b]); \
}

#define DISP_TEXT_CHAR_ONE_DOT(pos, code, c_f, c_b, scrn1, palette) { \
	dws_fg = (code & dof[pos]) ? c_f : c_b; \
	dws_dtm = dws_x+dws_bi+ppd[pos]; \
	if (dws_dtm > SCREEN_WIDTH) dws_dtm = SCREEN_WIDTH; \
	for(dws_dt = dws_x+dws_bi; dws_dt < dws_dtm; dws_dt++) { \
		scrn1 = (palette); \
	} \
}

#define DISP_TEXT_CHAR_MAIN(code, c_f, c_b, scrn1, palette) { \
	dws_bi = 0; \
	for(dws_bt = 0; dws_bt < 8; dws_bt++) { \
		DISP_TEXT_CHAR_ONE_DOT(dws_bt, code, c_f, c_b, scrn1, palette); \
		dws_bi += ppd[dws_bt]; \
	} \
}

#define DISP_TEXT_CHAR { \
	dws_p = chrfont[interlace_sel][dws_code][(dws_l >> (1 - (*videomode))) & 0xf]; \
\
	if (dws_disp_cursor_char) COLOR_REVERSE(dws_c_f, dws_c_b); \
	DISP_TEXT_CHAR_MAIN(dws_p, dws_c_f, dws_c_b, dws_scrn[dws_dt], SET_SCRN_DOT(dws_dt, palette_pc[dws_fg])) \
	if (dws_ki) { \
		DISP_TEXT_CHAR_MAIN(dws_p, dws_ck_f, dws_ck_b, scrntop[dws_y][dws_dt], palette_pc[dws_fg]) \
	} \
}

#define DISP_IG_PATTERN_MAIN(c_f, c_b, scrn1, palette) { \
	dws_bi = 0; \
	for(dws_bt = 0; dws_bt < 8; dws_bt++) { \
		dws_p = ((dws_i_g & dof[dws_bt]) ? 4 : 0) | ((dws_i_r & dof[dws_bt]) ? 2 : 0) | ((dws_i_b & dof[dws_bt]) ? 1 : 0); \
		if (dws_p == 0) dws_p = c_b; \
		dws_dtm = dws_x+dws_bi+ppd[dws_bt]; \
		if (dws_dtm > SCREEN_WIDTH) dws_dtm = SCREEN_WIDTH; \
		for(dws_dt = dws_x+dws_bi; dws_dt < dws_dtm; dws_dt++) { \
			scrn1 = (palette); \
		} \
		dws_bi += ppd[dws_bt]; \
	} \
}

#define DISP_IG_PATTERN { \
	dws_ll = dws_l >> dws_int_vid; \
	dws_i_b = ig_ram[dws_code*8+dws_ll]; \
	dws_i_r = ig_ram[dws_code*8+dws_ll+IGRAM_SIZE]; \
	dws_i_g = ig_ram[dws_code*8+dws_ll+IGRAM_SIZE+IGRAM_SIZE]; \
\
	if (FLG_ORIG_CURIG && dws_disp_cursor_char) { \
		dws_i_b = ~dws_i_b; \
		dws_i_r = ~dws_i_r; \
		dws_i_g = ~dws_i_g; \
	} \
	DISP_IG_PATTERN_MAIN(dws_c_f, dws_c_b, dws_scrn[dws_dt], SET_SCRN_DOT(dws_dt, palette_pc[dws_p])) \
	if (dws_ki) { \
		DISP_IG_PATTERN_MAIN(0, 0, scrntop[dws_y][dws_dt], palette_pc[dws_p]) \
	} \
}

#define DISP_WHITE_SPACE_MAIN(scrn1, palette) { \
	for(dws_dt = dws_x; dws_dt < dws_dtm; dws_dt++) { \
		scrn1 = (palette); \
	} \
}

#define DISP_WHITE_SPACE { \
	dws_p = 0xff; \
	dws_c_f = 7; \
	dws_c_b = bg_color; \
	dws_ck_f = 7; \
	dws_ck_b = 0; \
\
	if (dws_buf_x == 0 && dws_buf_y_h == 0) { \
		dws_bi = 0; \
		int btt = (1 << (rand() % 4)) | 0x20; \
		for(dws_bt = 0; dws_bt < 8; dws_bt++) { \
			dws_p = (btt & (1 << (7-dws_bt))) != 0 ? 3 : 7; \
			dws_dtm = dws_x+dws_bi+ppd[dws_bt]; \
			if (dws_dtm > SCREEN_WIDTH) dws_dtm = SCREEN_WIDTH; \
			for(dws_dt = dws_x+dws_bi; dws_dt < dws_dtm; dws_dt++) { \
				dws_scrn[dws_dt] = SET_SCRN_DOT(dws_dt, palette_pc[dws_p]); \
			} \
			dws_bi += ppd[dws_bt]; \
		} \
	} else { \
		dws_dtm = dws_x+crtc_ppch; \
		if (dws_dtm > SCREEN_WIDTH) dws_dtm = SCREEN_WIDTH; \
		DISP_WHITE_SPACE_MAIN(dws_scrn[dws_dt], SET_SCRN_DOT(dws_dt, palette_pc[dws_c_f])) \
	} \
	if (dws_ki) { \
		DISP_WHITE_SPACE_MAIN(scrntop[dws_y][dws_dt], palette_pc[dws_ck_f]) \
	} \
}

#define	DISP_GRAPHIC_HIRESO_MAIN(code, c_f, c_b, scrn1, palette) { \
	dws_bi = 0; \
	for(dws_bt = 0; dws_bt < 8; dws_bt++) { \
		dws_fg = (code & dof[dws_bt]) ? c_f : c_b; \
		dws_dtm = dws_x+dws_bi+ppd[dws_bt]; \
		if (dws_dtm > SCREEN_WIDTH) dws_dtm = SCREEN_WIDTH; \
		for(dws_dt = dws_x+dws_bi; dws_dt < dws_dtm; dws_dt++) { \
			scrn1 = (palette); \
		} \
		dws_bi += ppd[dws_bt]; \
	} \
}

#define	DISP_GRAPHIC_HIRESO { \
	DISP_GRAPHIC_HIRESO_MAIN(dws_code, dws_c_f, dws_c_b, dws_scrn[dws_dt], SET_SCRN_DOT(dws_dt, palette_pc[dws_fg])) \
	if (dws_ki) { \
		DISP_GRAPHIC_HIRESO_MAIN(dws_code, dws_ck_f, dws_ck_b, scrntop[dws_y][dws_dt], palette_pc[dws_fg]) \
	} \
}

#define DISP_GRAPHIC_NORMAL_MAIN(code, c_f, c_b, scrn1, palette) { \
	dws_bi = 0; \
	for (dws_bt = 0; dws_bt < 8; dws_bt += 4) { \
		dws_fg = (code & ((0x10 >> dws_bt) << ((dws_l >> (1 + dws_int_vid)) & 3))) ? c_f : c_b; \
		dws_dtm = dws_x+dws_bi+ppd[dws_bt]+ppd[dws_bt+1]+ppd[dws_bt+2]+ppd[dws_bt+3]; \
		if (dws_dtm > SCREEN_WIDTH) dws_dtm = SCREEN_WIDTH; \
		for(dws_dt = dws_x+dws_bi; dws_dt < dws_dtm; dws_dt++) { \
			scrn1 = (palette); \
		} \
		dws_bi += ppd[dws_bt] + ppd[dws_bt + 1] + ppd[dws_bt + 2] + ppd[dws_bt + 3]; \
	} \
}

#define DISP_GRAPHIC_NORMAL { \
	DISP_GRAPHIC_NORMAL_MAIN(dws_code, dws_c_f, dws_c_b, dws_scrn[dws_dt], SET_SCRN_DOT(dws_dt, palette_pc[dws_fg])) \
	if (dws_ki) { \
		DISP_GRAPHIC_NORMAL_MAIN(dws_code, dws_ck_f, dws_ck_b, scrntop[dws_y][dws_dt], palette_pc[dws_fg]) \
	} \
}

#define SET_YAXIS_POSITION { \
	if ((*disptmg_skew) == 3) { \
		dws_y0[0] = (screen_top | dws_yd); dws_y1[0] = screen_bottom;	/* no drawing */ \
		dws_y0[1] = 0; dws_y1[1] = 0; \
		dws_y0[2] = 0; dws_y1[2] = 0; \
		dws_ii[0] = 1; dws_ii[1] = 1; \
	} \
	else if (top_raster <  bottom_raster) { \
		dws_y0[0] = (screen_top | dws_yd); dws_y1[0] = (screen_bottom < top_raster ? screen_bottom : top_raster);	/* top padding */ \
		dws_y0[1] = (bottom_raster | dws_yd); dws_y1[1] = screen_bottom;	/* bottom padding */ \
		dws_y0[2] = ((top_raster > screen_top ? top_raster : screen_top) | dws_yd); dws_y1[2] = (screen_bottom < bottom_raster ? screen_bottom : bottom_raster); \
		dws_ii[0] = 2; dws_ii[1] = 3; \
	} \
	else if (top_raster >= bottom_raster) { \
		dws_y0[0] = ((bottom_raster > screen_top ? bottom_raster : screen_top) | dws_yd); dws_y1[0] = (screen_bottom < top_raster ? screen_bottom : top_raster); \
		dws_y0[1] = (screen_top | dws_yd); dws_y1[1] = (screen_bottom < bottom_raster ? screen_bottom : bottom_raster);	/* top drawing */ \
		dws_y0[2] = (top_raster | dws_yd); dws_y1[2] = screen_bottom;	/* bottom drawing */ \
		dws_ii[0] = 1; dws_ii[1] = 3; \
	} \
	else { \
		dws_y0[0] = (screen_top | dws_yd); dws_y1[1] = screen_bottom; \
		dws_y0[1] = 0; dws_y1[1] = 0; \
		dws_y0[2] = 0; dws_y1[2] = 0; \
		dws_ii[0] = 0; dws_ii[1] = 1; \
	} \
}

#undef SET_SCRN_DOT

#define INIT_SCRN_DOT(yy) dws_scrn = dws_scrn0 + dws_scrn_offset * (yy)
#define INIT_SCRNLINE1(yy)
#define SET_SCRN_DOT(xx,dd) (dd)
#define ADD_SCRN_DOT dws_scrn += dws_scrn_offset * dws_ys
#define ADD_SCRNLINE1

void DISPLAY::draw_screen_sub()
{
	dws_raster_per_char = ((*crtc_max_ra) << (1 - (*videomode)));
	dws_disp_cursor_line = false;
	dws_disp_cursor_char = false;
	dws_scrn_offset = emu->screen_buffer_offset();
	dws_scrn0 = emu->screen_buffer(0);
	dws_dot = 0;
	dws_buf_x = 0;
	dws_cram = 0;
	
#include "displaysub.h"
}

#undef INIT_SCRNLINE1
#undef SET_SCRN_DOT
#undef ADD_SCRNLINE1

#define INIT_SCRNLINE1(yy)  dws_scrnline1p = scrnline1[0] + scrnline1_offset * (yy)
#define SET_SCRN_DOT(xx,dd) ((dd & scrnline2[xx]) | dws_scrnline1p[xx])
#define ADD_SCRNLINE1       dws_scrnline1p += scrnline1_offset * dws_ys

void DISPLAY::draw_screen_sub_afterimage1()
{
	dws_raster_per_char = ((*crtc_max_ra) << (1 - (*videomode)));
	dws_disp_cursor_line = false;
	dws_disp_cursor_char = false;
	dws_scrn_offset = emu->screen_buffer_offset();
	dws_scrn0 = emu->screen_buffer(0);
	dws_dot = 0;
	dws_buf_x = 0;
	dws_cram = 0;

	// color filter for afterimage
	dws_scrn = dws_scrn0 + dws_scrn_offset * screen_top;
	for(dws_y = screen_top; dws_y < screen_bottom; dws_y++) {
		for(dws_x = screen_left; dws_x < screen_right; dws_x++) {
			dws_dot = dws_scrn[dws_x];
			dws_r = (dws_dot & Rmask);
			dws_g = (dws_dot & Gmask);
			dws_b = (dws_dot & Bmask);

			if (dws_r > Rloss) dws_r = dws_r - Rloss;
			else dws_r = 0;
			if (dws_g > Gloss) dws_g = dws_g - Gloss;
			else dws_g = 0;
			if (dws_b > Bloss) dws_b = dws_b - Bloss;
			else dws_b = 0;

			dws_dot = dws_r | dws_g | dws_b;

			scrnline1[dws_y][dws_x] = dws_dot;
		}
		dws_scrn += dws_scrn_offset;
	}

#include "displaysub.h"
}

#undef INIT_SCRNLINE1
#undef SET_SCRN_DOT
#undef ADD_SCRNLINE1

#define INIT_SCRNLINE1(yy)  dws_scrnline1p = scrnline1[0] + scrnline1_offset * (yy)
#define SET_SCRN_DOT(xx,dd) ((dd & scrnline2[xx]) | dws_scrnline1p[xx])
#define ADD_SCRNLINE1       dws_scrnline1p += scrnline1_offset * dws_ys

void DISPLAY::draw_screen_sub_afterimage2()
{
	dws_raster_per_char = ((*crtc_max_ra) << (1 - (*videomode)));
	dws_disp_cursor_line = false;
	dws_disp_cursor_char = false;
	dws_scrn_offset = emu->screen_buffer_offset();
	dws_scrn0 = emu->screen_buffer(0);
	dws_dot = 0;
	dws_buf_x = 0;
	dws_cram = 0;

	// color filter for afterimage2
	dws_scrn = dws_scrn0 + dws_scrn_offset * screen_top;
	for(dws_y = screen_top; dws_y < screen_bottom; dws_y++) {
		for(dws_x = screen_left; dws_x < screen_right; dws_x++) {
			// 0xff or 0xdf -> 0xe0 -> 0x00
//			dws_dot = dws_scrn[x] & 0x10101010;
//			dws_dot = (dws_dot << 3) | (dws_dot << 2) | (dws_dot << 1);
//			scrnline2[dws_x] = 0xdfdfdfdf;
			// 0xff or 0xbf -> 0xc0 -> 0x00
			dws_dot = dws_scrn[dws_x] & 0x20202020;
			dws_dot = (dws_dot << 2) | (dws_dot << 1);

			scrnline1[dws_y][dws_x] = dws_dot;
		}
		dws_scrn += dws_scrn_offset;
	}

#include "displaysub.h"
}

void DISPLAY::update_vram(int v, int clock)
{
	static int left_col;
	static int addr_offset;

	if ((*crtc_vt_count) != crt_mon_row) {
		crt_mon_row = (*crtc_vt_count) % VRAM_BUF_HEIGHT;
		crt_mon_col = (crt_mon_stepcols - 1) * ((crt_mon_row + sw) & 1);
		int row_zero = (*crtc_vt_count) % LINES_PER_FRAME;

		if (row_zero == 0) {
			// disptmg on

			// pixel of height
//			crt_mon_start_raster = ((*crtc_ra) - (*crtc_odd_line)) << (1 - (*videomode));

			if (*crtc_vt_count) {
				// no sync with vsync
				crt_mon_start_row = crt_mon_row;
				top_raster = 0;
				bottom_raster = top_raster + (regs[6] * (*crtc_max_ra) << (1 - (*videomode))) - (*crtc_vt_count) * 2;

				vsync_top_raster = top_raster + (regs[7] * (*crtc_max_ra) << (1 - (*videomode))) - (*crtc_vt_count) * 2;
				vsync_bottom_raster = vsync_top_raster + (((regs[3] & 0xf0) ? (regs[3] >> 4) : 16) << 1);
			} else {
				// if disptmg turn on then top_raster is crt_mon_now_raster.
				crt_mon_start_row = 0;
				top_raster = crt_mon_now_raster * 2;
				top_raster += (SCREEN_HEIGHT - 488) / 2;
				bottom_raster = top_raster + (regs[6] * (*crtc_max_ra) << (1 - (*videomode)));

				vsync_top_raster = top_raster + (regs[7] * (*crtc_max_ra) << (1 - (*videomode)));
				vsync_bottom_raster = vsync_top_raster + (((regs[3] & 0xf0) ? (regs[3] >> 4) : 16) << 1);
			}

			// synchronize horizontal total
			if (h_total_min[crtc_chr_clocks] <= regs[0] && regs[0] <= h_total_max[crtc_chr_clocks]) {
				left_diff = 0;
			}
#ifdef USE_KEEPIMAGE
			// top most page ?
			crt_mon_page = ((keepimage == 1 && (*crtc_ma) < 0x0800) || (keepimage == 2 && (*crtc_ma) >= 0x0800)) ? 0x80 : 0;
#else
			crt_mon_page = 0;
#endif
		}

		addr_offset = hireso * (((*crtc_ra) >> interlace_sel) & 0x7) << (10 + width_sel);
		crt_mon_addr = *crtc_ma;
		crt_mon_addr += crt_mon_col;
		crt_mon_addr += addr_offset;
		crt_mon_addr &= 0x3fff;
		if (crt_mon_addr < ADDR_VRAM_START) {
			crt_mon_addr += VRAM_SIZE;
		}

		// cursor
		cursor_addr = (((regs[14] << 8) | regs[15]) + (*disptmg_skew) - config.disptmg_skew);
		cursor_addr += addr_offset;
		cursor_addr &= 0x3fff;
		if (cursor_addr < ADDR_VRAM_START) {
			cursor_addr += VRAM_SIZE;
		}

#ifdef DEBUG_CRTMON
		if (crt_mon_row < 2) {
			emu->out_debugf(_T("DISPLAY:update_vram: sw:%d crtc:cnt:%d ra:%d cmnr:%d cmr:%d cmsr:%d cmc:%d top:%d btm:%d vtop:%d vbtm:%d")
				,sw
				,(*crtc_vt_count), (*crtc_ra), crt_mon_now_raster
				,crt_mon_row, crt_mon_start_row,  crt_mon_col
				,top_raster, bottom_raster, vsync_top_raster, vsync_bottom_raster
			);
		}
#endif

		// pixel of width
		if (crt_mon_row < VRAM_BUF_HEIGHT) {
			dws_p_vline = &crt_mon_vline[crt_mon_row];

			dws_p_vline->raster = (*crtc_ra) << (1 - (*videomode));

			left_col = regs[0]-regs[2]-(regs[3] & 0x0f)-(crtc_chr_clocks ? 14 : 5) + (*disptmg_skew) - config.disptmg_skew;
			left_col *= crtc_ppch;
			left_col += ((SCREEN_WIDTH - 768) / 2);
			left_col += left_diff;

			dws_p_vline->left_col = left_col;
			dws_p_vline->right_col = left_col + crtc_ppch * regs[1];
			if (dws_p_vline->right_col < 0) dws_p_vline->right_col = 0;

			dws_p_vline->hsync_left_col = left_col + regs[2] * crtc_ppch;
			dws_p_vline->hsync_right_col = dws_p_vline->hsync_left_col + (regs[3] & 0x0f) * crtc_ppch;
		}
		// diff horizontal total display and crtc
		if (regs[0] < h_total_min[crtc_chr_clocks] || h_total_max[crtc_chr_clocks] < regs[0]) {
			left_diff += (regs[0] + 1 - (1 << (crtc_chr_clocks + 6))) * crtc_ppch;

			if (left_diff >= SCREEN_WIDTH) left_diff = -SCREEN_WIDTH + 1;
			else if (left_diff <= -SCREEN_WIDTH) left_diff = SCREEN_WIDTH - 1;
		}

		crt_mon_now_raster++;
		if (crt_mon_now_raster >= LINES_PER_FRAME) crt_mon_now_raster = 0;
	}

	// now skip frame ?
	if (v == 0) {
		if (emu->now_skip_frame()) {
			skip_frame_count++;
		} else {
			skip_frame_count = 0;
		}
		if ((afterimage == 0 && skip_frame_count >= 1)
		||  (afterimage > 0 && skip_frame_count >= 2)) {
			// skip copy vram data
			return;
		}
	}

	// copy vram data to buffer
	if (crt_mon_row < VRAM_BUF_HEIGHT && *crtc_vt_count < *crtc_vt_disp) {
		for(int i=0; i < (clock << width_sel); i++) {
			dws_p_crt_mon = &crt_mon[crt_mon_row][crt_mon_col];

			if (crt_mon_addr >= ADDR_VRAM_END) {
				crt_mon_addr -= VRAM_SIZE;
			}
			if (crt_mon_col < regs[1]) {
				dws_p_crt_mon->text = vram[crt_mon_addr];
				// bit 0-5: color ram   bit 6: hireso  bit 7:top_most
				dws_p_crt_mon->attr = (color_ram[crt_mon_addr - ADDR_VRAM_START] | (hireso << 6) | crt_mon_page) & (ig_mask ? 0xdf : 0xff);
				// cursor
				if (crt_mon_addr == cursor_addr && (*crtc_ra) <= 3) {
					if (crt_mon_col < crt_mon_cursor_col) crt_mon_cursor_col = crt_mon_col;
					if (crt_mon_row < crt_mon_cursor_row) crt_mon_cursor_row = crt_mon_row;
				}
			} else if (crt_mon_col < VRAM_BUF_WIDTH) {
				dws_p_crt_mon->text = 0;
				dws_p_crt_mon->attr = 0;
			}
			crt_mon_col += crt_mon_stepcols;
			crt_mon_addr += crt_mon_stepcols;
		}
	}
}

// ----------------------------------------------------------------------------
// event handler
// ----------------------------------------------------------------------------

void DISPLAY::event_frame()
{
}

// ----------------------------------------------------------------------------

void DISPLAY::save_state(FILEIO *fio)
{
	struct vm_state_st vm_state;

	//
	vm_state_ident.version = Uint16_LE(2);
	vm_state_ident.size = Uint32_LE(sizeof(vm_state_ident) + sizeof(vm_state));

	// copy values
	memset(&vm_state, 0, sizeof(vm_state));
	vm_state.ig_enable = Int32_LE(ig_enable);
	vm_state.ig_mask = ig_mask ? 1 : 0;

	vm_state.chr_clocks = (crtc_chr_clocks & 0xff);
	fio->Fwrite(&vm_state_ident, sizeof(vm_state_ident), 1);
	fio->Fwrite(&vm_state, sizeof(vm_state), 1);
}

void DISPLAY::load_state(FILEIO *fio)
{
	vm_state_ident_t vm_state_i;
	struct vm_state_st vm_state;

	READ_STATE_CHUNK(fio, vm_state_i, vm_state);

	ig_enable = Int32_LE(vm_state.ig_enable);
	ig_mask = vm_state.ig_mask ? true : false;

	if (Uint16_LE(vm_state_i.version) >= 2) {
		crtc_chr_clocks = vm_state.chr_clocks;
		crtc_ppch = (crtc_chr_clocks ? 8 : 16);
	}

	// clear
	crt_mon_row = 9999;
	crt_mon_col = 0;

}

// ----------------------------------------------------------------------------

#ifdef USE_DEBUGGER

bool DISPLAY::get_debug_graphic_memory_size(int type, int *width, int *height)
{
	switch(type) {
	case 0:
	case 1:
	case 2:
	case 3:
		// IG
		*width = (16 * 8);
		*height = (16 * 8);
		break;
	default:
		return false;
	}
	return true;
}

bool DISPLAY::debug_draw_graphic(int type, int width, int height, scrntype *buffer)
{

	int size = width * height;
	switch (type) {
	case 0:
	case 1:
	case 2:
	case 3:
		// IG
		for(int x=0; x<16; x++) {
		for(int y=0; y<16; y++) {
			for(int li=0; li<8; li++) {
			for(int bt=0; bt<8; bt++) {
				int pos = bt + (y + (li + x * 8) * 16) * 8;
				if (pos < size) {
					int rampos = (y*16 + x) * 8 + li;
					uint8_t msk = (0x80 >> bt);
					buffer[pos]=emu->map_rgbcolor(
							((type & 3) == 0 || (type & 3) == 2) && (ig_ram[rampos+0x0800] & msk) ? 0xff : 0,	// r
							((type & 3) == 0 || (type & 3) == 3) && (ig_ram[rampos+0x1000] & msk) ? 0xff : 0,	// g
							((type & 3) == 0 || (type & 3) == 1) && (ig_ram[rampos] & msk) ? 0xff : 0	// b
						);
				}
			}
			}
		}
		}
		break;
	default:
		return false;
	}
	return true;
}

#endif

