/** @file display.h

	HITACHI BASIC MASTER LEVEL3 Mark5 Emulator 'EmuB-6892'
	Skelton for retropc emulator

	@author Sasaji
	@date   2011.06.08 -

	@brief [ display ]
*/

#ifndef _DISPLAY_H_
#define _DISPLAY_H_

#include "../vm.h"
#include "../../emu.h"
#include "../device.h"

#define ADDR_VRAM_START		0x0400
#define ADDR_VRAM_END		0x4400
#define VRAM_SIZE			0x4000

#define ADDR_IGRAM_START	0xa000
#define ADDR_IGRAM_END		0xa800

#define IGRAM_SIZE			0x800

#define SIG_DISPLAY_LPSTB	1
#define SIG_DISPLAY_VSYNC	2

#define VRAM_BUF_HEIGHT	524
#define VRAM_BUF_WIDTH	128

/**
	@brief Display Monitor
*/
class DISPLAY : public DEVICE
{
private:
	DEVICE* d_crtc;

	uint8_t* regs;
	uint16_t cursor_addr;
	bool disp_cursor;
	int *crtc_curdisp;
	int cur_left[32];
	int cur_right[32];
	int cur_top;
	int cur_bottom;

	int   mode_sel;					///< ffd0
	int   interlace_sel;			///< ffd6
	int   ig_enable;				///< ffe9
	bool  ig_mask;					///< compatible for MB-6890/6891

	int   crtc_chr_clocks;			///< crtc clocks 0:1MHz 1:2MHz
	int   crtc_ppch;				///< crtc dot per char w.40:16 w.80:8

	// crtc reg8
	int  *videomode;				///< 1:videomode(interlace only)
	int  *disptmg_skew;				///< display timing skew
	int  *curdisp_skew;				///< cursor disp skew

	int   bg_color;					///< bg color (mode-sel bit0-bit2)
	int   hireso;					///< 0:normal 1:hireso (mode-sel bit6)
	int   width_sel;				///< 0:w.40 1:w.80 (mode-sel bit7)

	// vertical
	int top_raster;
	int bottom_raster;
	int vsync_top_raster;
	int vsync_bottom_raster;
//	int raster_per_char;			///< raster lines per 1 char (16 generally)
	// synchronizable vertical range
	int v_total_min[3];
	int v_total_max[3];

	int left_diff;
	// synchronizable horizontal range
	int h_total_min[2];
	int h_total_max[2];

	int sw;

	uint32_t Rmask;
	uint32_t Gmask;
	uint32_t Bmask;
	uint8_t  Rshift;
	uint8_t  Gshift;
	uint8_t  Bshift;
	uint32_t Rloss;
	uint32_t Gloss;
	uint32_t Bloss;

	uint8_t font[0x1000];
	uint8_t chrfont[2][256][16];
	uint8_t* vram;
	uint8_t* color_ram;
	uint8_t* ig_ram;

	// CRT monitor
	typedef struct st_crt_mon {
		uint8_t text;	///< vram
		uint8_t attr; ///< color ram
	} t_crt_mon;
	t_crt_mon crt_mon[VRAM_BUF_HEIGHT][VRAM_BUF_WIDTH];
	int crt_mon_start_row;
//	int crt_mon_start_raster;
	int crt_mon_col;
	int crt_mon_row;
	int crt_mon_addr;
	int crt_mon_page;
	int crt_mon_now_raster;
	int crt_mon_cursor_col;
	int crt_mon_cursor_row;
	int crt_mon_stepcols;

	typedef struct st_crt_mon_vline {
		// horizontal
		int left_col;					///< display timing area left side
		int right_col;					///< display timing area right side
		int hsync_left_col;
		int hsync_right_col;
		uint8_t raster;					///< bit0-5:rastar line  bit7:reserved
	} t_crt_mon_vline;
	t_crt_mon_vline crt_mon_vline[VRAM_BUF_HEIGHT];

	int screen_left;
	int screen_right;
	int screen_top;
	int screen_bottom;
	int scrnline1_offset;
	scrntype scrnline1[SCREEN_HEIGHT][SCREEN_WIDTH];
	scrntype scrnline2[SCREEN_WIDTH];
	scrntype scrntop[SCREEN_HEIGHT][SCREEN_WIDTH];
	scrntype palette_pc[8];
	int  ppd[8];	// pixel per 1dot
//	int  ppch;		// pixel per 1char
	uint8_t dof[8];	// dot mask

	uint8_t scanline;
	int  afterimage;
#ifdef USE_KEEPIMAGE
	int  keepimage;
#endif
	int  buf_stepcols;

	int skip_frame_count;
	int *crtc_vt_total;		///< crtc total rows (dot unit)
	int *crtc_vt_count;		///< crtc now line
	int *crtc_vt_disp;		///< crtc can display rows (disptmg)
	int *crtc_ma;			///< crtc memory address
	int *crtc_ra;			///< crtc raster address
	int *crtc_max_ra;		///< crtc max raster address
	int *crtc_odd_line;		///< crtc odd line (videomode only)

	int *vm_pause;

	//for resume
#pragma pack(1)
	struct vm_state_st {
		int   crtc_addr;
		int   ig_enable;
		uint8_t ig_mask;

		uint8_t chr_clocks;

		char  reserved[6];
	};
#pragma pack()

	bool font_rom_loaded;
	bool font_rom_loaded_at_first;

	// for draw_screen and update_vram
	uint8_t dws_p;
	uint8_t dws_code;
	uint8_t dws_c_f;
	uint8_t dws_c_b;
	uint8_t dws_ck_f;
	uint8_t dws_ck_b;
	uint8_t dws_cram;
	uint8_t dws_fg;
	uint8_t dws_i_r,dws_i_g,dws_i_b;
	int dws_bt;
	int dws_bi;
	int dws_dt, dws_dtm;
	int dws_x,dws_x0,dws_x1,dws_x2,dws_x3,dws_x4,dws_xodd;
	int dws_y,dws_y0[3],dws_y1[3],dws_ys,dws_yd;
	int dws_l,dws_ll;
	int dws_int_vid;
	bool dws_disp_cursor_line;
	bool dws_disp_cursor_char;
	bool dws_ki;
//	bool dws_skip_line;
	int dws_buf_x;
	int dws_buf_y;
	int dws_buf_y_h;
	int dws_i,dws_ii[2];
	t_crt_mon *dws_p_crt_mon;
	t_crt_mon_vline *dws_p_vline;

	int dws_scrn_offset;
	scrntype *dws_scrn0;
	scrntype *dws_scrn;
	scrntype dws_dot;
	scrntype *dws_scrnline1p;
	uint32_t dws_r,dws_g,dws_b;

	int dws_raster_per_char;


	void load_font_rom_file();
	uint8_t get_font_data(int, int, int);
	void update_cursor();

	void draw_screen_sub();
	void draw_screen_sub_afterimage1();
	void draw_screen_sub_afterimage2();
public:
	DISPLAY(VM* parent_vm, EMU* parent_emu, const char* identifier) : DEVICE(parent_vm, parent_emu, identifier)
	{
		set_class_name("DISPLAY");
	}
	~DISPLAY() {}

	// common functions
	void initialize();
	void reset();
	void update_config();
	void write_io8(uint32_t addr, uint32_t data);
	void write_signal(int id, uint32_t data, uint32_t mask);
	void event_frame();
	void update_vram(int v, int clock);

	// unique function
	void set_context_crtc(DEVICE* device) {
		d_crtc = device;
	}
	void set_vram_ptr(uint8_t* ptr) {
		// vram is main ram!
		vram = ptr;
	}
	void set_color_ram_ptr(uint8_t* ptr) {
		color_ram = ptr;
	}
	void set_ig_ram_ptr(uint8_t* ptr) {
		ig_ram = ptr;
	}
	void set_regs_ptr(uint8_t* ptr) {
		regs = ptr;
	}
	void set_crtc_vt_ptr(int* ptrt, int *ptrc, int *ptrd) {
		crtc_vt_total = ptrt;
		crtc_vt_count = ptrc;
		crtc_vt_disp  = ptrd;
	}
	void set_crtc_ma_ra_ptr(int *ptrma, int *ptrra) {
		crtc_ma = ptrma;
		crtc_ra = ptrra;
	}
	void set_crtc_max_ra_ptr(int *ptr) {
		crtc_max_ra = ptr;
	}
	void set_crtc_odd_line_ptr(int *ptr) {
		crtc_odd_line = ptr;
	}
	void set_crtc_reg8_ptr(int *ptrv, int *ptrds, int *ptrcs) {
		videomode = ptrv;
		disptmg_skew = ptrds;
		curdisp_skew = ptrcs;
	}
	void set_crtc_curdisp_ptr(int *ptr) {
		crtc_curdisp = ptr;
	}
	void set_display_size(int left, int top, int right, int bottom);
	void draw_screen();

	void save_state(FILEIO *fio);
	void load_state(FILEIO *fio);

#ifdef USE_DEBUGGER
	bool get_debug_graphic_memory_size(int type, int *width, int *height);
	bool debug_draw_graphic(int type, int width, int height, scrntype *buffer);
#endif
};

#endif

