/// @file parsewav.cpp
///
/// @brief wavef[^ <-> gf[^ <-> VAf[^@ϊNX
///
/// @author Sasaji
/// @date   2011.7.1
///
#define _USE_MATH_DEFINES 1
#include <math.h>
#include "parsewav.h"

//#define MAX_CARRIER_PATTERN	1
static const char	*carrier_pattern[4][4]={
	{
		"11001100",	// 0 (600 baud)
		"00110011",	// 0 (600 baud frip)
		"10101010",	// 1 (600 baud)
		"01010101",	// 1 (600 baud frip)
	},{
		"1100",		// 0 (1200 baud)
		"0011",		// 0 (1200 baud frip)
		"1010",		// 1 (1200 baud)
		"0101",		// 1 (1200 baud frip)
	},{
		"11",		// 0 (2400 baud)
		"00",		// 0 (2400 baud frip)
		"10",		// 1 (2400 baud)
		"01",		// 1 (2400 baud frip)
	},{
		"1100110011001100",	// 0 (300 baud)
		"0011001100110011",	// 0 (300 baud frip)
		"1010101010101010",	// 1 (300 baud)
		"0101010101010101",	// 1 (300 baud frip)
	}
};
#define IDX_PTN_600		0
#define IDX_PTN_1200	1
#define IDX_PTN_2400	2
#define IDX_PTN_300		3
// 2400{[̎̓GbWD
static const char *carrier_edge_pattern[]={
	"0010",		// 01 (2400 baud)
	"1101",		// 01 (2400 baud frip)
	"0100",		// 10 (2400 baud)
	"1011",		// 10 (2400 baud frip)
	NULL
};

//static const int c_baud_min[4]={ 3, 0, 1, 2 };

//static const int c_baud_ridx[4]={ 1, 2, 3, 0 };

//static const uint8_t cmt_ident[4] = { 0xff, 0x01, 0x3c, 0x00 };

/// @brief RXgN^
///
PARSEWAV::PARSEWAV()
{
	// p[^
	param.range1200 = 20;
	param.range2400 = 20;
	param.freq1200 = 1200;
	param.freq2400 = 2400;
	param.reverse = 0;

	param.half_wave = 1;
	param.correct = 0;
	param.correct_type = 0;

	param.baud = 0;	// 0:600band 1:1200band 2:2400baud 3:300baud
	param.auto_baud = 1;
	param.word_select = 0x04;

	param.debug_log = false;

	param.sample_rate = 3;	// 48000
	param.sample_bits = 0;	// 8bit

	param.file_split = 0;
	param.del_mhead = 0;

	//
	wave_data = new Buf_t<uint8_t>(false);
	wave_correct_data = new Buf_t<uint8_t>(false);
	carrier_data = new Buf_t<char>(false, false, 36);
	freq_data = new Buf_t<int>[2];

	err_num = 0;

	// wavt@Cwb_(o͗p)
	memcpy(outwav.head.RIFF, "RIFF", 4);
	outwav.head.file_len = 0;
	memcpy(outwav.head.WAVE, "WAVE", 4);

	memcpy(outwav.fmt.fmt, "fmt ", 4);
	outwav.fmt.format_id = 1;
	outwav.fmt.channels = 1;
	outwav.fmt.sample_rate = 48000;
	outwav.fmt.data_speed = 48000;
	outwav.fmt.block_size = 1;
	outwav.fmt.sample_bits = 8;
	outwav.fmt.fmt_size = 16;

	memcpy(outwav.data.data, "data", 4);
	outwav.data.data_len = 0;

	//
	util = new PARSEWAV_UTIL();

	if (param.debug_log) {
		flog = new FILEIO();
		flog->Fopen(_T("d:\\parsewav.log"), FILEIO_WRITE_BINARY);
	}
}

/// @brief fXgN^
///
PARSEWAV::~PARSEWAV()
{
	if (param.debug_log) {
		flog->Fclose();
		delete flog;
	}
	delete util;
	delete [] freq_data;
	delete carrier_data;
	delete wave_correct_data;
	delete wave_data;
}

/// @brief G[bZ[Wo
///
/// @return G[bZ[Wւ̃|C^
///
///
///
_TCHAR* PARSEWAV::Errmsg()
{
	static _TCHAR str[1024];

	switch(err_num) {
		case 0:
			// no error
			_tcscpy(str,_T(""));
			break;
		case 1:
			_tcscpy(str,_T("File not found."));
			break;
		case 2:
			_tcscpy(str,_T("This is not the wav file which is formatted linear PCM."));
			break;
		case 3:
			_tcscpy(str,_T("This is not the frequency between 11025 and 48000Hz."));
			break;
		case 4:
			_tcscpy(str,_T("Cannot write to file."));
			break;
		default:
			_stprintf(str,_T("Unknown error: %d"),err_num);
			break;
	}
	return str;
}

/// @brief ̓t@C̃tH[}bg`FbN
///
/// @param[in] fp        ̓t@C
/// @param[in] file_type t@C 0:wavet@C 1:l3ct@C
/// @return    0: 1..4:G[
int PARSEWAV::CheckFileFormat(FILEIO *fp, int file_type) {
	int rc = 0;

	// check wave header
	if (file_type == 0) {
		rc = util->CheckWavFormat(fp, &inwav.head, &inwav.fmt, &inwav.data, &sample_num);
		err_num = rc;
		if (rc == 0) {
			sample_num_orig = sample_num;
			sample_pos = 0;
			sample_ms = 0;
		}
	} else if (file_type == 1) {
		rc = check_l3c_format(fp);
	}

	return rc;
}

/// @brief 
///
/// @param[in] fp        ̓t@C
/// @param[in] file_type t@C 0:wavet@C 1:l3ct@C
/// @return 0
int PARSEWAV::InitData(FILEIO *fp, int file_type)
{
	fio = fp;
	infile_type = file_type;

	wave_data->init();
	wave_correct_data->init();
	carrier_data->init();

//	wav_data = 0;
	half_wave = param.half_wave;

	// wavet@C̃wb_o
	outwav.fmt.channels = 1;
	outwav.fmt.sample_rate = get_sample_rate(param.sample_rate);
	outwav.fmt.sample_bits = (param.sample_bits == 0 ? 8 : 16);
	outwav.fmt.block_size = (param.sample_bits == 0 ? 1 : 2);
	outwav.fmt.data_speed = outwav.fmt.sample_rate * outwav.fmt.block_size;

	util->InitConvSampleData();

	if (infile_type == 1) {
		phase1 = 11;
		start_phase = 11;
	} else {
		phase1 = 0;
		start_phase = 0;
	}
	phase2 = 0;
	phase2_frip = 0;
	baud24_frip = 0;

	samples = inwav.fmt.sample_rate / 1200;

	st_pa_carr.x0_prev = 0.0;
	st_pa_carr.sample_cnt = 0;
	st_pa_carr.wav_prev = 0;
	st_pa_carr.carr_prev = 0;
	st_pa_carr.odd = 0;

	st_lamda.us_delta = 1000000.0 / (double)inwav.fmt.sample_rate;
	st_lamda.us_range = 1000000.0 / 2400.0;
	st_lamda.us1200 = 1000000.0 / (double)param.freq1200 / (double)(1 << half_wave);
	st_lamda.us1800 = 1000000.0 / 1800.0 / (double)(1 << half_wave);
	st_lamda.us2400 = 1000000.0 / (double)param.freq2400 / (double)(1 << half_wave);
	st_lamda.us1200avg = st_lamda.us1200;
	st_lamda.us1800avg = st_lamda.us1800;
	st_lamda.us2400avg = st_lamda.us2400;

	st_lamda.us1200min = st_lamda.us1200 - st_lamda.us_range * param.range1200 / 100.0;
	st_lamda.us1200max = st_lamda.us1200 + st_lamda.us_range * param.range1200 / 100.0;
	st_lamda.us2400min = st_lamda.us2400 - st_lamda.us_range * param.range2400 / 100.0;
	st_lamda.us2400max = st_lamda.us2400 + st_lamda.us_range * param.range2400 / 100.0;

	if (infile_type == 0 && param.correct == 1) {
		init_dft(param.correct_type);
	}

	return 0;
}

/// @brief wavet@C̃wb_obt@ɏo͂
///
/// @param[out] buffer obt@
/// @return wb_
///
int PARSEWAV::InitFileHeader(uint8_t *buffer)
{
	int len = 0;
	int len_all = 0;
	len = sizeof(outwav.head);
	memcpy(buffer + len_all, &outwav.head, len);
	len_all += len;
	len = sizeof(outwav.fmt);
	memcpy(buffer + len_all, &outwav.fmt, len);
	len_all += len;
	len = sizeof(outwav.data);
	memcpy(buffer + len_all, &outwav.data, len);
	len_all += len;

	return len_all;
}

/// @brief wavet@CɃt@CTCY
///
/// @param[in] fp ̓t@C
///
///
void PARSEWAV::SetFileHeader(FILEIO *fp)
{
	uint32_t file_size = 0;
	uint32_t riff_size = 0;
	uint32_t data_size = 0;

	fp->Fseek(0, FILEIO_SEEK_END);
	file_size = fp->Ftell();

	riff_size = file_size - 8;

	fp->Fseek(4, FILEIO_SEEK_SET);
	fp->Fwrite((uint32_t *)&riff_size, sizeof(uint32_t), 1);

	data_size = file_size - 44;

	fp->Fseek(40, FILEIO_SEEK_SET);
	fp->Fwrite((uint32_t *)&data_size, sizeof(uint32_t), 1);

	return;
}

/// @brief Tv[gԂ
///
/// @param[in] num 0:11025Hz 1:22050Hz 2:44100Hz
/// @return Tv[g
int PARSEWAV::get_sample_rate(int num) {
	int rate = 48000;

	switch(num) {
		case 0:
			rate = 11025;
			break;
		case 1:
			rate = 22050;
			break;
		case 2:
			rate = 44100;
			break;
	}
	return rate;
}

/// @brief l3ct@C̃tH[}bg`FbNi`FbNĂȂj
///
/// @param[in] fp ̓t@C
/// @return 0
///
int PARSEWAV::check_l3c_format(FILEIO *fp)
{
	fp->Fseek(0, FILEIO_SEEK_END);
	sample_num = fp->Ftell();
	fp->Fseek(0, FILEIO_SEEK_SET);

	sample_pos = 0;

	return 0;
}

/// @brief wavt@CPTvǂ
///
/// @param[in] size    1: uint8_tŕԂ  2: int16_tŕԂ
/// @param[in] reverse 1: g`𔽓]
/// @return ϊPTvf[^ or EOF
///
int PARSEWAV::get_wav_sample(int size, int reverse)
{
	int l = 0;
	int h = 0;

	// 1Tv𓾂
	l = fio->Fgetc();
	if (l == EOF) {
		return l;
	}
	if(inwav.fmt.sample_bits == 16) {
		// int16_t̏ꍇ
		h = fio->Fgetc();
		if (h >= 128) {
			h = h - 256;
		}
		if (reverse) {
			h *= -1;
			if (h >= 128) h = 127;
			l *= -1;
		}
	} else {
		if (reverse) {
			l = 256 - l;
			if (l >= 256) l = 255;
		}
	}
	// XeȈꍇA𖳎
	for(int i = 2; i <= inwav.fmt.channels; i++) {
		fio->Fgetc();
		if(inwav.fmt.sample_bits == 16) {
			fio->Fgetc();
		}
	}
	sample_pos++;
	sample_ms = (uint32_t)(1000.0 * (double)sample_pos / (double)inwav.fmt.sample_rate);

	if (size == 1) {
		if(inwav.fmt.sample_bits == 16) {
			// int16_t -> uint8_t
			return ((h + 128) & 0xff);
		} else {
			return (l & 0xff);
		}
	} else {
		if(inwav.fmt.sample_bits == 16) {
			return h * 256 + l;
		} else {
			// uint8_t -> int16_t
			return (l - 128) * 256;
		}
	}
}
/// @brief wavt@CPTvǂ(int16_tŕԂ)
///
/// @param[in] reverse 1: g`𔽓]
/// @return ϊPTvf[^
int PARSEWAV::get_wav_sample(int reverse)
{
	return get_wav_sample(2, reverse);
}

/// @brief wavt@CPTvǂŃobt@ɒǋL
///
/// @param[in,out] w_data Tvf[^p̃obt@(ǋLĂ)
/// @param[in] reverse 1: g`𔽓]
/// @return ǂݍ񂾃f[^̒
///
size_t PARSEWAV::get_wav_sample(Buf_t<uint8_t> *w_data, int reverse)
{
	int l;

	while(w_data->isFull(0) != true && sample_pos < sample_num) {
		l = get_wav_sample(1, reverse);

		w_data->add((uint8_t)l);
	}
	if (sample_pos >= sample_num) {
		w_data->last = true;
	}
	return w_data->w_pos;
}

/// @brief l3ct@C(grbgf[^)Pf[^ǂŃobt@ɒǋL
///
/// @param[in,out] c_data gf[^p̃obt@(ǋLĂ)
/// @return ǂݍ񂾃f[^̒
///
///
size_t PARSEWAV::get_l3c_sample(Buf_t<char> *c_data)
{
//	int pos = 0;
	int l;

//	pos = strlen(c_data);
	while(c_data->isFull(0) != true && sample_pos < sample_num) {
		l = fio->Fgetc();
		l &= 0xff;

		sample_pos++;
		sample_ms = (uint32_t)(1000.0 * (double)sample_pos / 4800.0);
		if (l == '\r' || l == '\n') {
			continue;
		}
		c_data->add(l, sample_ms);
	}
	if (sample_pos >= sample_num) {
		c_data->last = true;
	}
	return c_data->w_pos;
}

/// @brief ␳pdfťW쐬
///
/// @param[in] type 0:cos 1:sin
///
void PARSEWAV::init_dft(int type)
{
	int half = samples / 2;
	int n, k;

	for(k = 0; k < 2; k++) {
		for(n = 0; n < samples; n++) {
			if (type == 0) {
				h[k][n] = cos(-2.0 * M_PI * (k + 1) * (n - half) / samples);
			} else {
				h[k][n] = sin(-2.0 * M_PI * (k + 1) * (n - half) / samples);
			}
		}
	}
	return;
}


/// @brief t[Gϊ
/// 1200Hz, 2400Hz̐gor]gĕ␳
///
/// @param[in] w_data    ̔g`f[^
/// @param[out] wc_data  ␳̔g`f[^
/// @return ϊf[^
size_t PARSEWAV::calcrate_dft(Buf_t<uint8_t> *w_data, Buf_t<uint8_t> *wc_data)
{
	int k;
	int n;
	int p;
	int wc;
	double dat;
	Buf_t<int> *f_data;

	int half = samples / 2;

	for (k = 0; k < 2; k++) {
		f_data = &freq_data[k];
		f_data->init();
	}
	for (p = half; p < (w_data->w_pos - half); p++) {
		for (k = 0; k < 2; k++) {
			f_data = &freq_data[k];

			dat = 0.0;
			for (n = p - half; n < (p + half); n++) {
				dat += (double)((int)w_data->data[n] - 128) * h[k][n - p + half];
			}
			f_data->add((int)(dat * 1000.0 / samples), 0);
		}
		wc = (freq_data[0].data[p - half] + freq_data[1].data[p - half]) / 1000;
		if (wc >= 128) wc = 127;
		if (wc < -128) wc = -128;
		wc_data->add((uint8_t)(wc + 128), w_data->get_ms(0));
		w_data->r_pos++;
		w_data->total_r_pos++;
		if (wc_data->isFull(0)) {
			break;
		}
	}
#if 0
	for (p = 0; p < (wc_data->w_pos); p++) {
//		if (param.debug_log) {
		sprintf(cbuff,"%d,%d,%d,%d,%d,%d\n"
			, (w_data->data[p + half] - 128) * 1000
			, freq_data[0].re[p]
			, freq_data[1].re[p]
			, freq_data[2].re[p]
			, (w_data->data[p + half] - 128)
			, (wc_data->data[p + wc_data->start_pos] - 128)
		);
		if (fiolog) fwrite((void *)cbuff, strlen(cbuff), 1, fiolog);
//		}
	}
#endif
	return wc_data->w_pos;
}




/// @brief 2400Hz1200Hz̔gărbgf[^ɕϊ
///
/// @param[in]   w_data    WAVTvf[^
/// @param[out]  c_data  "10": 2400Hz "1100":1200Hz "?HHLL":Too long
/// @return WAV1Tvf[^
int PARSEWAV::decode_to_carrier_wave_bn(Buf_t<uint8_t> *w_data, Buf_t<char> *c_data)
{
	char bit_data[10];

	int data = 0;
	uint8_t  dat;
	uint32_t ms;
	bool hi = false;
	bool found = false;

	double x0,lamda;

	// find next trigger point
	while(w_data->isEOB(0) != true) {
		w_data->get(dat, ms);
		data = (int)dat - 128;

		if (st_pa_carr.wav_prev < 0 && data >= 0) {
			// l -> h ɂȂʒug(us)߂
			hi = true;
			x0 = - (st_pa_carr.wav_prev * st_lamda.us_delta / (data - st_pa_carr.wav_prev));
			lamda = st_pa_carr.x0_prev + (st_pa_carr.sample_cnt * st_lamda.us_delta) + x0;
			// gZꍇ̓mCYƔfďp
			if (lamda >= 105.0) {
				st_pa_carr.x0_prev = st_lamda.us_delta - x0;
				st_pa_carr.odd = 1 - st_pa_carr.odd;
				found = true;
				break;
			}
		} else if (st_pa_carr.wav_prev > 0 && data <= 0 && half_wave == 1) {
			// h -> l ɂȂʒug(us)߂
			hi = false;
			x0 = - (st_pa_carr.wav_prev * st_lamda.us_delta / (data - st_pa_carr.wav_prev));
			lamda = st_pa_carr.x0_prev + (st_pa_carr.sample_cnt * st_lamda.us_delta) + x0;
			// gZꍇ̓mCYƔfďp
			if (lamda >= 105.0) {
				st_pa_carr.x0_prev = st_lamda.us_delta - x0;
				st_pa_carr.odd = 1 - st_pa_carr.odd;
				found = true;
				break;
			}
		}
		st_pa_carr.sample_cnt++;
		st_pa_carr.wav_prev = data;
	}
	if (found != true) {
		return data;
	}

	if (half_wave == 0) {
		// Sg
		if (st_lamda.us1200min <= lamda && lamda <= st_lamda.us1200max) {
			// long (half wave)
			strcpy(bit_data,"1100");
			st_pa_carr.carr_prev = 4;
			st_lamda.us1200avg = (st_lamda.us1200avg + lamda) / 2.0;

			rep_p1_sample_num[0]++;
		}
		else if (st_lamda.us2400min <= lamda && lamda <= st_lamda.us2400max) {
			// short (half wave)
			strcpy(bit_data,"10");
			st_pa_carr.carr_prev = 2;
			st_lamda.us2400avg = (st_lamda.us2400avg + lamda) / 2.0;

			rep_p1_sample_num[1]++;
		}
		else if (st_lamda.us1800 <= lamda && lamda < st_lamda.us1200min) {
			// middle long
			strcpy(bit_data,"?1100");
			st_pa_carr.carr_prev = 4;
			st_lamda.us1800avg = (st_lamda.us1800avg + lamda) / 2.0;

			rep_p1_sample_num[2]++;
		}
		else if (st_lamda.us2400max < lamda && lamda < st_lamda.us1800) {
			// middle short
			strcpy(bit_data,"?10");
			st_pa_carr.carr_prev = 2;
			st_lamda.us1800avg = (st_lamda.us1800avg + lamda) / 2.0;

			rep_p1_sample_num[2]++;
		}
		else if (st_lamda.us1200max < lamda) {
			// too long
			strcpy(bit_data,"?HHLL");
			st_pa_carr.carr_prev = 5;
			rep_p1_sample_num[3]++;
		}
		else if (lamda < st_lamda.us2400min) {
			// too short
			strcpy(bit_data,"?HL");
			st_pa_carr.carr_prev = 1;
			rep_p1_sample_num[4]++;
		}
		else {
			// error
			strcpy(bit_data,"??");
			st_pa_carr.carr_prev = 0;
			rep_p1_sample_num[5]++;
		}
	} else {
		// g
		if (st_lamda.us1200min <= lamda && lamda <= st_lamda.us1200max) {
			// long (half wave)
			if (hi) {
				strcpy(bit_data,"00");
				if (st_pa_carr.carr_prev != 4) st_pa_carr.odd = 1;
				st_pa_carr.carr_prev = -4;
			} else {
				strcpy(bit_data,"11");
				if (st_pa_carr.carr_prev != -4) st_pa_carr.odd = 1;
				st_pa_carr.carr_prev = 4;
			}
			st_lamda.us1200avg = (st_lamda.us1200avg + lamda) / 2.0;

			rep_p1_sample_num[0]++;
		}
		else if (st_lamda.us2400min <= lamda && lamda <= st_lamda.us2400max) {
			// short (half wave)
			if (hi) {
				strcpy(bit_data,"0");
				if (st_pa_carr.carr_prev != 2) st_pa_carr.odd = 1;
				st_pa_carr.carr_prev = -2;
			} else {
				strcpy(bit_data,"1");
				if (st_pa_carr.carr_prev != -2) st_pa_carr.odd = 1;
				st_pa_carr.carr_prev = 2;
			}
			st_lamda.us2400avg = (st_lamda.us2400avg + lamda) / 2.0;

			rep_p1_sample_num[1]++;
		}
		else if (st_lamda.us2400max < lamda && lamda < st_lamda.us1200min) {
			// middle
			if (hi) {
				if ((st_pa_carr.odd == 0 && st_pa_carr.carr_prev == 4)
				 || (st_pa_carr.odd == 1 && st_pa_carr.carr_prev == 2)) {
					strcpy(bit_data,"?00");
					st_pa_carr.carr_prev = -4;
				} else {
					strcpy(bit_data,"?0");
					st_pa_carr.carr_prev = -2;
				}
			} else {
				if ((st_pa_carr.odd == 0 && st_pa_carr.carr_prev == -4)
				 || (st_pa_carr.odd == 1 && st_pa_carr.carr_prev == -2)) {
					strcpy(bit_data,"?11");
					st_pa_carr.carr_prev = 4;
				} else {
					strcpy(bit_data,"?1");
					st_pa_carr.carr_prev = 2;
				}
			}
			st_lamda.us1800avg = (st_lamda.us1800avg + lamda) / 2.0;

			rep_p1_sample_num[2]++;
		}
		else if (st_lamda.us1200max < lamda) {
			// too long
			if (hi) {
				strcpy(bit_data,"?LL");
				st_pa_carr.carr_prev = -5;
			} else {
				strcpy(bit_data,"?HH");
				st_pa_carr.carr_prev = 5;
			}
			rep_p1_sample_num[3]++;
		}
		else if (lamda < st_lamda.us2400min) {
			// too short
			if (hi) {
				strcpy(bit_data,"?L");
				st_pa_carr.carr_prev = -1;
			} else {
				strcpy(bit_data,"?H");
				st_pa_carr.carr_prev = 1;
			}
			rep_p1_sample_num[4]++;
		}
		else {
			// error
			strcpy(bit_data,"??");
			st_pa_carr.carr_prev = 0;

			rep_p1_sample_num[5]++;
		}
	}
	// f[^ǉ
	if (bit_data[0] != '?') {
		// OK data
		c_data->adds(bit_data, ms);
	} else {
		// NG data
		c_data->adds(&bit_data[1], ms);
#if 0
		if (param.debug_log > 1) {
			// fobOO
			sprintf(cbuff, "p1 w:%12d(%s) c:%12d: %s\n"
				, w_data->total_r_pos
				, get_time_cstr(ms)
				, c_data->total_w_pos
				, bit_data);
				if (fiolog) fwrite((void *)cbuff, strlen(cbuff), 1, fiolog);
		}
#endif
	}

	st_pa_carr.wav_prev = data;
	st_pa_carr.sample_cnt = 0;

	return data;
}

/// @brief vp^[T
///
/// @param[in]   c_data g
/// @param[out]  s_data VAf[^
/// @param[in]   baud   600baud:0 1200baud:1 2400baud:2 300baud:3
/// @param[out]  frip   1->0->1->0: 0 0->1->0->1:1
/// @param[out]  c_onedata NULL or VAf[^̔gf[^
/// @return X^[gʒu / -1 Ȃ
int PARSEWAV::find_start_carrier_bit(Buf_t<char> *c_data, char *s_data, int baud, int *frip, char *c_onedata)
{
	int pos1[4];
	int pos = -1;
	int idx_ptn = (baud & 3);
	int best_num = -1;
	int best_pos = (int)(c_data->size);
	int i;
//	int ptn;
    char bit[4];
//	uint32_t ms;

	strcpy(bit, "?");

	// 2400{[̎̓GbWT
	if (idx_ptn == IDX_PTN_2400) {
		for(i=0; i<4; i++) {
			pos1[i]=c_data->find(0, carrier_edge_pattern[i]);
		}
		for(i=0; i<4; i++) {
			if (pos1[i] >= 0 && best_pos > pos1[i]) {
				best_pos = pos1[i];
				best_num = i;
			}
		}
		if (best_num >= 0) {
			// 
			pos = best_pos + (int)strlen(carrier_edge_pattern[best_num]);

			if ((best_num % 2) == 0) {
				(*frip) = 0;
			} else {
				(*frip) = 1;
			}
			if ((best_num / 2) == 0) {
				strcpy(bit, "01");
			} else {
				strcpy(bit, "10");
			}
		}
	}

	// vp^[T
	if (best_num < 0) {
		for(i=0; i<4; i++) {
			pos1[i]=c_data->find(0, carrier_pattern[idx_ptn][i]);
		}
		for(i=0; i<4; i++) {
			if (pos1[i] >= 0 && best_pos > pos1[i]) {
				best_pos = pos1[i];
				best_num = i;
			}
		}

		if (best_num >= 0) {
			pos = best_pos + (int)strlen(carrier_pattern[idx_ptn][best_num]);

			if ((best_num % 2) == 0) {
				(*frip) = 0;
			} else {
				(*frip) = 1;
			}
			if ((best_num / 2) == 0) {
				strcpy(bit, "0");
			} else {
				strcpy(bit, "1");
			}
		}
	}
	if (pos >= 0) {
		// f[^L
#if 0
		ms = c_data->get_ms(best_pos);

		if (param.debug_log > 1) {
			// fobOO
			sprintf(cbuff, "p2 fst c:%12d(%s) w_pos:%d pos:%d frip:%d ("
				, c_data->total_r_pos
				, get_time_cstr(ms)
				, s_data->total_w_pos
				, best_pos, *frip);
			strncat(cbuff, c_data->curptr(best_pos), pos - best_pos);
			strcat(cbuff, ")\n");
				if (fiolog) fwrite((void *)cbuff, strlen(cbuff), 1, fiolog);
		}
#endif
		if (c_onedata != NULL) {
			strncat(c_onedata, &c_data->data[c_data->r_pos], pos);
		}
		c_data->r_pos += pos;
		c_data->total_r_pos += pos;

	} else {
		// f[^Ȃ
		if (c_onedata != NULL) {
			strncat(c_onedata, &c_data->data[c_data->r_pos], c_data->w_pos - c_data->r_pos);
		}
		c_data->total_r_pos += c_data->w_pos - c_data->r_pos;
		c_data->r_pos = c_data->w_pos;
	}

	strcpy(s_data, bit);

	return pos;
}

/// @brief g(2400/1200Hz)̃f[^VAf[^Prbgo
///
/// @param[in]  c_data  g
/// @param[out] s_data    VAf[^
/// @param[in]  baud    600baud:0 1200baud:1 2400baud:2 300baud:3
/// @param[in,out]  frip    1->0->1->0: 0 0->1->0->1:1
/// @param[out] c_onedata NULL or VAf[^̔gf[^
/// @return ̈ʒu
int PARSEWAV::decode_to_serial_data(Buf_t<char> *c_data, char *s_data, int baud, int *frip, char *c_onedata)
{
	int pos = -1;
	int idx_ptn = (baud & 3);
	int len;
//	int ptn;
	char bit_data[4];
	char *pptn;

	strcpy(bit_data, "");

	len = (int)strlen(carrier_pattern[idx_ptn][*frip]);
	pptn = (char *)carrier_pattern[idx_ptn][*frip];
	if (c_data->cmp(0, pptn, len) == 0) {	// 0
		// 0
		strcpy(bit_data, "0");
		pos = len;
		if (idx_ptn == IDX_PTN_2400) {
			// 2400{[0̏ꍇAfrip
			*frip = (1 - *frip);
		}
	}
	if (pos < 0) {
		len = (int)strlen(carrier_pattern[idx_ptn][2 + *frip]);
		pptn = (char *)carrier_pattern[idx_ptn][2 + *frip];
		if (c_data->cmp(0, pptn, len) == 0)	{ // 1
			// 1
			strcpy(bit_data, "1");
			pos = len;
		}
	}

//	ms = c_data->get_ms(0);

	if (pos >= 0) {
		// f[^ǉ
		if (c_onedata != NULL) {
			strncat(c_onedata, &c_data->data[c_data->r_pos], pos);
		}
		c_data->r_pos += pos;
		c_data->total_r_pos += pos;
	} else {
		// vp^[Ȃ
		strcpy(bit_data, "?");

		if (c_onedata != NULL) {
			strncat(c_onedata, &c_data->data[c_data->r_pos], 1);
		}
		c_data->r_pos += 1;
		c_data->total_r_pos += 1;
#if 0
		rep_p2_error_num++;

		if (param.debug_log > 1) {
			// fobOO
			sprintf(cbuff,"p2 c2s c:%12d(%s) s:%8d: error ("
				, c_data->total_r_pos
				, get_time_cstr(ms)
				, s_data->total_w_pos
				);
			strncat(cbuff, c_data->curptr(0)
				, c_data->sublen() >= len ? len : c_data->sublen());
			strcat((char *)cbuff, ")\n");
				if (fiolog) fwrite((void *)cbuff, strlen(cbuff), 1, fiolog);
		}
#endif
	}

	strcpy(s_data, bit_data);

	return pos;
}

/// @brief waveTvf[^g(1200/2400Hz)͂
/// @brief rbgf[^ɕϊB(FSK)
///
/// @param[in]  w_data wavef[^
/// @param[out] wc_data wave␳f[^
/// @param[out] c_data gf[^
/// @param[in]  start_phase JntF[Y
/// @param[in]  corret_type ␳@(gp)
/// @param[in]  reverse g`]
/// @return     f[^I true
bool PARSEWAV::decode_phase1(Buf_t<uint8_t> *w_data, Buf_t<uint8_t> *wc_data, Buf_t<char> *c_data, int start_phase, int corret_type, int reverse)
{
	bool last = false;
	Buf_t<uint8_t> *wn_data;

	if (infile_type == 0 && param.correct == 1) {
		wn_data = wc_data;
	} else {
		wn_data = w_data;
	}

	while(phase1 >= 0 && last == false) {
		switch(phase1) {
			case 0:
				// WAVt@Cǂݍ
				get_wav_sample(w_data, reverse);
				phase1 = 1;
				break;
			case 1:
				// WAVTvf[^␳
				if (infile_type == 0 && param.correct == 1) {
					calcrate_dft(w_data, wc_data);
				}
				// f[^Ō܂œǂ񂾏ꍇ
				if (param.correct == 1) {
					if (w_data->last && w_data->isEOB(samples)) {
						wc_data->last = true;
					}
				}
				phase1 = 2;
				break;
			case 2:
				// L3C֕ϊ
				if (c_data->isFull(1) != true) {
					decode_to_carrier_wave_bn(wn_data, c_data);
				}
				if (c_data->isFull(8)) {
					// cobt@ςɂȂfo
					phase1 = 21;
				} else if (wn_data->last != true && wn_data->isEOB(samples)) {
					// cobt@ςɂȂ܂ŌJԂ
					phase1 = 21;
				} else if (wn_data->last && wn_data->isEOB(0)) {
					// Ō̃f[^
					c_data->last = true;
					phase1 = -1;
				}
				break;
			case 11:
				// L3Ct@Cǂݍ
				get_l3c_sample(c_data);
				phase1 = 21;
				break;
			case 21:
				// cobt@̍ŏɃRs[̃^[ŉ͂邽
				w_data->shift();
				wc_data->shift();

				phase1 = start_phase;
				last = true;
				break;
		}
	}
	return c_data->last;
}

/// @brief g(1200/2400Hz)f[^VAf[^ɕϊ
/// Ń{[[gɂăVAf[^ςĂ
/// @param[in] c_data gf[^
/// @param[out] s_data VAf[^
/// @param[out] c_onedata NULL or VAf[^1rbg̔g
/// @return tF[Yԍ
int PARSEWAV::decode_phase2(Buf_t<char> *c_data, char *s_data, char *c_onedata)
{
	int step = 0;
	bool break_data = false;
	int baud = param.baud;

	switch (phase2) {
		case 0:
			step = find_start_carrier_bit(c_data, s_data, baud, &phase2_frip, c_onedata);
			if (c_data->last && c_data->isEOB(0)) {
				phase2 = -1;
				break;
			}
			if (step >= 0) {
				phase2 = 1;
			}
			break_data = true;
			break;
		case 1:
			step = decode_to_serial_data(c_data, s_data, baud, &phase2_frip, c_onedata);
			if (c_data->last && c_data->isEOB(0)) {
				phase2 = -1;
				break;
			}
			if (step < 0) {
				phase2 = 0;
			}
			break_data = true;
			break;
	}
	if (sample_pos >= sample_num) {
		phase2 = -1;
	}

	// obt@̖ɋ߂Â
	// c̖̓f[^obt@̐擪ɃRs[
	// ̃^[ŉ͂sB
	if (c_data->isEOB(8)) {
		c_data->shift();
	}

	return phase2;
}

/// @brief l3b -> l3c VAf[^grbgf[^ɕϊ
///
/// @param[in] data VAf[^i1rbgj
/// @param[out] buffer gf[^
/// @param[out] c_len  gf[^̒
/// @return bufferɏ񂾃TCY
int PARSEWAV::encode_phase2(uint8_t data, uint8_t *buffer, int *c_len)
{
	int idx_ptn = (param.baud & 3);
	int len = 0;

	if (data & 0x01) {
		// 1
		len = (int)strlen(carrier_pattern[idx_ptn][2 + baud24_frip]);
		strncpy((char *)buffer, carrier_pattern[idx_ptn][2 + baud24_frip], len);
	} else {
		// 0
		len = (int)strlen(carrier_pattern[idx_ptn][baud24_frip]);
		strncpy((char *)buffer, carrier_pattern[idx_ptn][baud24_frip], len);
		if (idx_ptn == IDX_PTN_2400) {
			baud24_frip = (1-baud24_frip);
		}
	}
	if (c_len != NULL) {
		*c_len = len;
	}
	return len;
}

/// @brief VAf[^grbgf[^ɕϊwaveTvf[^ɕϊ
///
/// @param[in] data VAf[^i1rbgj
/// @param[out] buffer wavef[^
/// @param[out] c_len  gf[^̒
/// @return bufferɏ񂾃TCY
int PARSEWAV::encode_phase1(uint8_t data, uint8_t *buffer, int *c_len)
{
	int len = 0;
	int pos = 0;
	uint8_t wav_samples[10];

	len = encode_phase2(data, (uint8_t *)carrier_data->data, c_len);

	// convert to wave sample data
	memset(wav_samples, 0xf0, 10);
	wave_data->clear();

	for(int j=0; j < len; j++) {
		if (j > 0 && (carrier_data->data[j-1] & 1) == (carrier_data->data[j] & 1)) {
			wav_samples[0] = 0xf0;
		} else {
			wav_samples[0] = 0xec;
		}
		if (j < len - 1 && (carrier_data->data[j] & 1) == (carrier_data->data[j+1] & 1)) {
			wav_samples[9] = 0xf0;
		} else {
			wav_samples[9] = 0xec;
		}
		for(int i=0; i<10; i++) {
			wave_data->add((carrier_data->data[j] & 1) ? wav_samples[i] : 256 - wav_samples[i]);
		}
	}
	// TvO[gϊ
	pos = (int)util->OutConvSampleData(wave_data->data, 48000, 8, 1, wave_data->w_pos
						, buffer, outwav.fmt.sample_rate, outwav.fmt.sample_bits, DATA_BUFFER_SIZE, 0);

	return pos;
}

/// @brief t@Cf[^ǂ݃VAf[^1bitɕϊďo͂
/// @param[out] s_data  VAf[^ 1-2bit
/// @param[out] c_onedata NULL or VAf[^1-2rbg̔g
/// @return  / f[^I: -1
int PARSEWAV::GetData(char *s_data, char *c_onedata)
{
	int retry = 1;

	if (sample_pos >= sample_num) {
		return -1;
	}

	do {
		decode_phase1(wave_data, wave_correct_data, carrier_data, start_phase, param.correct_type, param.reverse);
		decode_phase2(carrier_data, s_data, c_onedata);
		retry--;
	} while (s_data[0] == '?' && retry > 0);

	if (s_data[0] == '?') {
		strcpy(s_data, "1");
	}

	return (int)strlen(s_data);
}

/// @brief VAf[^1bitϊăobt@ɏo͂
///
/// @param[in] data ̓f[^
/// @param[out] buffer ϊ̃f[^
/// @param[out] c_len  c_dataɕϊ̒
/// @return bufferɏ񂾃TCY
int PARSEWAV::SetData(uint8_t data, uint8_t *buffer, int *c_len)
{
	if (infile_type == 1) {
		// l3cf[^ɕϊ
		return encode_phase2(data, buffer, c_len);
	} else {
		// waveTvf[^ɕϊ
		return encode_phase1(data, buffer, c_len);
	}
}

/// @brief RXgN^
///
PARSEWAV_UTIL::PARSEWAV_UTIL()
{
	InitConvSampleData();
}

/// @brief fXgN^
///
PARSEWAV_UTIL::~PARSEWAV_UTIL()
{
}

/// @brief waveTvOf[^̃[gƃrbgϊp̕ϐ
///
void PARSEWAV_UTIL::InitConvSampleData()
{
	divide = 0;
	surplus = 0;
	remain_buf_len = 0;
	memset(remain_buf, 0, sizeof(remain_buf));
}

/// @brief waveTvOf[^̃[gƃrbgϊăt@Cɏo
///
/// @param[in] in_buf  ̓f[^
/// @param[in] in_rate in_buf̃Tv[g
/// @param[in] in_bits in_buf̃Tvrbg(8 or 16)
/// @param[in] in_ch   in_buf̃`l(1 or 2)
/// @param[in] in_len  in_buf
/// @param[out] outbuf o̓f[^
/// @param[in] outrate outbuf̃Tv[g
/// @param[in] outbits outbuf̃Tvrbg(8 or 16)
/// @param[in] outlen  outbuf
/// @param[in] outpos  outbufɏoŏ̈ʒu
/// @return outpos + ooCg
size_t PARSEWAV_UTIL::OutConvSampleData(void *in_buf, uint32_t in_rate, int in_bits, int in_ch, size_t in_len
							,  uint8_t *outbuf, uint32_t outrate, int outbits, size_t outlen, size_t outpos)
{
	int16_t *src_buf = new int16_t[outlen];
	int16_t *new_buf = new int16_t[outlen];

	size_t src_len = 0;
	size_t pos = 0;
	int new_data = 0;
	size_t new_len = 0;
	size_t i = 0;

	memset(src_buf, 0, outlen * sizeof(int16_t));
	memset(new_buf, 0, outlen * sizeof(int16_t));
	for (i=0; i<(size_t)remain_buf_len; i++) {
		src_buf[i]=remain_buf[i];
	}
	src_len = remain_buf_len;

	for(i=0; i<in_len; i+=in_ch) {
		if (in_bits == 8) {
			// 8rbg
			src_buf[src_len]=(int16_t)*((uint8_t *)in_buf + i);
		} else if (in_bits == 16) {
			// 16rbg
			src_buf[src_len]=(int16_t)*((int16_t *)in_buf + i);
		}
		src_len++;
	}

	memset(remain_buf, 0, sizeof(remain_buf));
	remain_buf_len = 0;

	if (in_rate > outrate) {
		// rate ꍇ
		pos = 0;
		while (pos < src_len) {
			divide = (in_rate + surplus) / outrate;
			if (pos + divide + 2 >= src_len) {
				break;
			}
			int n_surplus = (in_rate + surplus) % outrate;

			// ω
			new_data = 0;
			if (surplus >= 0) {
				new_data += (int)((double)src_buf[pos] * (double)((int)outrate - surplus) / (double)outrate);
				pos++;
			}
			for(i = 1; i < (size_t)divide; i++) {
				new_data += src_buf[pos];
				pos++;
			}
			if (n_surplus > 0) {
				new_data += (int)((double)src_buf[pos] * (double)n_surplus / (double)outrate);
			}
			new_data = (int)((double)new_data * (double)outrate / (double)in_rate);

			surplus = n_surplus;
			new_buf[new_len] = (int16_t)new_data;
			new_len++;
		}
		if (pos < src_len) {
			// ]f[^͎ɂ܂킷
			for(i=pos; i<src_len; i++) {
				remain_buf[i-pos]=src_buf[i];
			}
			remain_buf_len = (int)(src_len - pos);
		}
	} else if (in_rate < outrate) {
		// rate グꍇ
		pos = 0;
		while (pos < src_len - 1) {
			divide = (outrate + surplus) / in_rate;
			surplus = (outrate + surplus) % in_rate;
			// TvԂ`ŕ⊮
			for(i=0; i<(size_t)divide; i++) {
				new_data = (int)src_buf[pos] + (int)i * ((int)src_buf[pos+1] - (int)src_buf[pos]) / divide;
				new_buf[new_len] = (int16_t)new_data;
				new_len++;
				// obt@ςȂ̂Ńt@Cɏo
				if (new_len >= outlen - 1) {
					outpos = out_sample_buffer(new_buf, new_len, in_bits, outbuf, outlen, outpos, outbits);
					memset(new_buf, 0, sizeof(int16_t) * outlen);
					new_len = 0;
				}
			}
			pos++;
		}
		// Ō̃f[^ɉ
		remain_buf[0]=src_buf[src_len-1];
		remain_buf_len = 1;
	} else {
		// [g
		memcpy(new_buf, src_buf, src_len * sizeof(int16_t));
		new_len = src_len;
	}

	outpos = out_sample_buffer(new_buf, new_len, in_bits, outbuf, outlen, outpos, outbits);

	delete[] new_buf;
	delete[] src_buf;

	return outpos;
}

/// @brief wavet@CTvO[g/Tvrbgϊăobt@ɓǂݍ
///
/// @param[in] fin     wavt@C|C^
/// @param[in] in_fmt  wavt@Cfmt`N
/// @param[in] in_len  wavt@C̃f[^̒
/// @param[out] outbuf o̓f[^
/// @param[in] outrate outbuf̃Tv[g
/// @param[in] outbits outbuf̃Tvrbg(8 or 16)
/// @param[in] outlen  outbuf̒
/// @return ooCg
size_t PARSEWAV_UTIL::ReadWavData(FILEIO *fin, wav_fmt_chank_t *in_fmt, size_t in_len, uint8_t *outbuf, uint32_t outrate, int outbits, size_t outlen)
{
	uint8_t buf[32];
	size_t outpos = 0;
	size_t len = 0;
	size_t siz = 0;

	InitConvSampleData();
	while ((siz = fin->Fread(buf, sizeof(uint8_t), 32)) > 0 && len < in_len && outpos < outlen) {
		outpos = OutConvSampleData(buf, in_fmt->sample_rate, in_fmt->sample_bits, in_fmt->channels, siz * 8 / in_fmt->sample_bits
									, outbuf, outrate, outbits, outlen, outpos);
		len += siz;
	}
	return outpos;
}

/// @brief waveTvOf[^̃rbgϊ
///
/// @param[in] in_buf ̓f[^
/// @param[in] in_len  in_buf
/// @param[in] in_bits in_buf̃Tvrbg(8 or 16)
/// @param[out] outbuf o̓f[^
/// @param[in] outlen  outbuf
/// @param[in] outpos  outbufɏoŏ̈ʒu
/// @param[in] outbits outbuf̃Tvrbg(8 or 16)
/// @return outpos + ooCg
size_t PARSEWAV_UTIL::out_sample_buffer(int16_t *in_buf, size_t in_len, int in_bits, uint8_t *outbuf, size_t outlen, size_t outpos, int outbits)
{
	int16_t new_data = 0;
	size_t i = 0;

	// o
	for(i=0; i<in_len; i++) {
		if (in_bits > outbits) {
			// rbg ꍇ
			new_data = (in_buf[i] / 256) + 128;
		} else if (in_bits < outbits) {
			// rbg グꍇ
			new_data = (in_buf[i] - 128) * 256;
		} else {
			new_data = in_buf[i];
		}
		if (outbits == 16) {
			outbuf[outpos++] = (new_data & 0xff);
			outbuf[outpos++] = ((new_data >> 8) & 0xff);
		} else {
			outbuf[outpos++] = (new_data & 0xff);
		}
		if (outpos >= outlen) {
			break;
		}
	}
	return outpos;
}

/// @brief WAVEt@C̃tH[}bg`FbN
///
/// @param[in]   fp     ̓t@C
/// @param[out]  head   wavewb_
/// @param[out]  fmt    wavetH[}bg^Cv
/// @param[out]  data   wavef[^
/// @param[out]  data_len Tv
/// @return 0: OK  1:t@CȂ() 2: PCMłȂ 3:Ή[głȂ
int PARSEWAV_UTIL::CheckWavFormat(FILEIO *fp, wav_header_t *head, wav_fmt_chank_t *fmt, wav_data_chank_t *data, size_t *data_len)
{
	char buf[10];
	long offset = 0;
	long fpos_data = 0;
	wav_unknown_chank_t unk;

	memset(head, 0, sizeof(wav_header_t));
	memset(fmt, 0, sizeof(wav_fmt_chank_t));
	memset(data, 0, sizeof(wav_data_chank_t));
	*data_len = 0;

	offset = (long)fp->Fread(head, sizeof(wav_header_t), 1);
	if (offset == 0) {
		// empty file
		return 1;
	}

	if(memcmp(head->RIFF,"RIFF",4) != 0 || memcmp(head->WAVE,"WAVE",4) != 0) {
		// this is not wave format !!!
		return 2;
	}

	for (int i=0; i<10; i++) {
		fp->Fread(buf, sizeof(char), 4);
		if (memcmp(buf, "fmt ", 4) == 0) {
			// fmt chank
			fp->Fseek(-4, SEEK_CUR);
			fp->Fread(fmt, sizeof(wav_fmt_chank_t), 1);
			if (fmt->format_id != 1) {
				// this is not pcm format !!!
				return 2;
			}

			// 11025 ` 48000Hz
			if (fmt->sample_rate < 11025 || 48000 < fmt->sample_rate) {
				return 3;
			}

			if (fpos_data != 0) break;

			offset = 8 + fmt->fmt_size - sizeof(wav_fmt_chank_t);

		} else if (memcmp(buf, "data", 4) == 0) {
			// data chank
			fp->Fseek(-4, SEEK_CUR);
			fp->Fread(data, sizeof(wav_data_chank_t), 1);
			fpos_data = fp->Ftell();

			if (fmt->format_id != 0) break;

			offset = data->data_len;

		} else {
			// unknown chank
			fp->Fseek(-4, SEEK_CUR);
			fp->Fread(&unk, sizeof(wav_unknown_chank_t), 1);

			offset = unk.len;
		}
		fp->Fseek(offset, SEEK_CUR);
	}

	if (fpos_data == 0 || fmt->format_id != 1) {
		// this is not pcm format !!!
		return 2;
	}

	fp->Fseek(fpos_data, SEEK_SET);

	if (data_len != NULL) {
		*data_len = (data->data_len) / fmt->channels;
		if(fmt->sample_bits == 16) {
			*data_len /= 2;
		}
	}

	return 0;
}
