/// @file parsewav.h
///
/// @brief wavef[^ <-> gf[^ <-> VAf[^@ϊNX
///
/// @author Sasaji
/// @date   2011.7.1
///

#ifndef _PARSEWAV_H_
#define _PARSEWAV_H_

#include "../../common.h"
#include "../../fileio.h"
#include "parsewav_param.h"
#include "parsewav_buffer.h"
#include "parsewav_format.h"

typedef struct pos_st {
	int    pos;
	uint32_t ms;
} pos_t;

class FILEIO;

/// @brief WAVt@CNX
class PARSEWAV_UTIL
{
private:
	int remain_buf[100];
	int remain_buf_len;
	int divide;
	int surplus;

	size_t out_sample_buffer(int16_t *in_buf, size_t in_len, int in_bsiz, uint8_t *outbuf, size_t outlen, size_t outpos, int outbsiz);

public:
	PARSEWAV_UTIL();
	~PARSEWAV_UTIL();

	void   InitConvSampleData();
	size_t OutConvSampleData(void *in_buf, uint32_t in_rate, int in_bits, int in_ch, size_t in_len
						, uint8_t *outbuf, uint32_t outrate, int outbits, size_t outlen, size_t outpos);
	size_t ReadWavData(FILEIO *fin, wav_fmt_chank_t *in_fmt, size_t in_len, uint8_t *outbuf, uint32_t outrate, int outbits, size_t outlen);
	int    CheckWavFormat(FILEIO *fp, wav_header_t *head, wav_fmt_chank_t *fmt, wav_data_chank_t *data, size_t *data_len = NULL);

};


/// @brief f[^R[_̕ϒg(FSK)͂ăVAf[^ɕϊNX
///
/// wavet@C`ŕۑꂽf[^R[_̕ϒg(FSK)1200Hz2400Hz̔g
/// ͂Ĕgf[^ƂɃVAf[^oB
/// tɃVAf[^ϒgwavet@C쐬B
class PARSEWAV
{
private:
	FILEIO *fio;
	FILEIO *flog;
	PARSEWAV_PARAM param;

	int infile_type;

	// input file info
	size_t sample_pos;
	size_t sample_num;
	size_t sample_num_orig;
	uint32_t sample_ms;
	int  samples;


	int	phase1;
	int phase2;
	int start_phase;

	int phase2_frip;

	int baud24_frip;

	int half_wave;

	struct parse_carrier_st {
		double x0_prev;
		int    wav_prev;
		int    sample_cnt;
		int    odd;
		int    carr_prev;
	} st_pa_carr;

	struct lamda_st {
		double us_delta;	// 1Tv̒(us)
		double us_range;
		double us1200;
		double us1200min;
		double us1200max;
		double us1200avg;
		double us1800;
		double us1800avg;
		double us2400;
		double us2400min;
		double us2400max;
		double us2400avg;
	} st_lamda;

	Buf_t<uint8_t> *wave_data;
	Buf_t<uint8_t> *wave_correct_data;
	Buf_t<char> *carrier_data;
	Buf_t<int>   *freq_data;

	int err_num;
	int rep_p1_sample_num[9];
	struct inwav_st {
		wav_header_t	 head;
		wav_fmt_chank_t  fmt;
		wav_data_chank_t data;
	} inwav;

	struct outwav_st {
		wav_header_t	 head;
		wav_fmt_chank_t  fmt;
		wav_data_chank_t data;
	} outwav;

	double h[2][50];

	int check_l3c_format(FILEIO *fp);

	int get_wav_sample(int);
	int get_wav_sample(int, int);
	size_t get_wav_sample(Buf_t<uint8_t> *w_data, int reverse);
	size_t get_l3c_sample(Buf_t<char> *c_data);
	void init_dft(int type);
	size_t calcrate_dft(Buf_t<uint8_t> *w_data, Buf_t<uint8_t> *wc_data);
	int   decode_to_carrier_wave_bn(Buf_t<uint8_t> *w_data, Buf_t<char> *c_data);

	int   find_start_carrier_bit(Buf_t<char> *c_data, char *s_data, int baud, int *frip, char *c_onedata);
	int   decode_to_serial_data(Buf_t<char> *c_data, char *s_data, int baud, int *frip, char *c_onedata);
	bool  decode_phase1(Buf_t<uint8_t> *w_data, Buf_t<uint8_t> *wc_data, Buf_t<char> *c_data, int start_phase, int corret_type, int reverse);
	int   decode_phase2(Buf_t<char> *c_data, char *s_data, char *c_onedata);

	int   encode_phase2(uint8_t data, uint8_t *buffer, int *c_len);
	int   encode_phase1(uint8_t data, uint8_t *buffer, int *c_len);

	int   get_sample_rate(int);

public:
	PARSEWAV_UTIL *util;

	// prototype define
	PARSEWAV();
	~PARSEWAV();

	int   CheckFileFormat(FILEIO *fp, int file_type);

	int   InitData(FILEIO *fp, int file_type);
	int   GetData(char *s_data, char *c_onedata = NULL);
	int   SetData(uint8_t data, uint8_t *buffer, int *c_len);
	int   InitFileHeader(uint8_t *buffer);
	void  SetFileHeader(FILEIO *fp);

	_TCHAR* Errmsg();

	void SetSampleRate(int value) { param.sample_rate = value; }
	void SetSampleBits(int value) {	param.sample_bits = value; }
	void SetBaud(int value)		{ param.baud = value; }
	void SetReverseWave(int value) { param.reverse = (value ? 1 : 0); }
	void SetHalfWave(int value) { param.half_wave = (value ? 1 : 0); }
	void SetCorrectWave(int value) { param.correct = (value ? 1 : 0); }
	void SetCorrectType(int value) { param.correct_type = value; }

	int GetSampleRate(void) { return param.sample_rate; }
	int GetSampleBits(void) { return param.sample_bits; }
	int GetBaud(void)		{ return param.baud; }
	int GetReverseWave(void) { return param.reverse; }
	int GetHalfWave(void)	{ return param.half_wave; }
	int GetCorrectWave(void) { return param.correct; }
	int GetCorrectType(void) { return param.correct_type; }

};

#endif
